/*****************************************************************************/
/*   M-Control Program Include File					     */
/*   Copyright (c) 1992 IBM Corporation.  All Rights Reserved.		     */
/*									     */
/*   M/CP AUDIO SUPPORT FOR APPLICATIONS WITHOUT MULTIMEDIA EXTENSIONS	     */
/*									     */
/*   Dependencies: Assumes WINDOWS.H has been previously included.	     */
/*****************************************************************************/


typedef WORD VERSION;

//*****************************************************************************
//	       MULTIMEDIA SYSTEMS WINDOW MESSAGES
//*****************************************************************************

#define MM_WOM_OPEN	    0x3BB
#define MM_WOM_CLOSE	    0x3BC
#define MM_WOM_DONE	    0x3BD

#define MM_WIM_OPEN	    0x3BE
#define MM_WIM_CLOSE	    0x3BF
#define MM_WIM_DATA	    0x3C0


//*****************************************************************************
//  MMTIME UNION OF VARIOUS STRUCTS
//*****************************************************************************

typedef struct mmtime_tag {
    WORD    wType;		// The contents of the union
    union {
	DWORD ms;		// Milliseconds
	DWORD sample;		// Samples
	DWORD cb;		// Byte count
	struct {		// SMPTE
	    BYTE hour;		// Hours
	    BYTE min;		// Minutes
	    BYTE sec;		// Seconds
	    BYTE frame; 	// Frames
	    BYTE fps;		// Frames per second (24, 25, 29(30 drop) or 30)
	    BYTE dummy;
	    } smpte;
	struct {		// MIDI
	    DWORD songptrpos;	// Song pointer position
	    } midi;
	} u;
    } MMTIME;

typedef MMTIME FAR *LPMMTIME;

// types for MMTIME struct

#define TIME_MS 	0x0001		// Time counted in Milliseconds
#define TIME_SAMPLES	0x0002		// Number of wave samples
#define TIME_BYTES	0x0004		// Current byte offset


//*****************************************************************************
//  STRING RESOURCE NUMBER BASES
//*****************************************************************************

#define MAXPNAMELEN		32	// Maximum length of szPname in
					//  Devcaps structures
#define MAXERRORLENGTH	       128	// Maximum error text length (including
					//  Terminating null )
#define MMSYSERR_BASE	       0
#define WAVERR_BASE	       32
#define MCIERR_BASE	       256


//*****************************************************************************
//  GENERAL ERROR RETURN VALUES
//*****************************************************************************

#define MMSYSERR_NOERROR       0		    // No error
#define MMSYSERR_ERROR	       (MMSYSERR_BASE + 1)  // Unspecified error
#define MMSYSERR_BADDEVICEID   (MMSYSERR_BASE + 2)  // Device id is out of range
#define MMSYSERR_NOTENABLED    (MMSYSERR_BASE + 3)  // Driver failed its enable
#define MMSYSERR_ALLOCATED     (MMSYSERR_BASE + 4)  // Device already allocated
#define MMSYSERR_INVALHANDLE   (MMSYSERR_BASE + 5)  // Device handle is invalid
#define MMSYSERR_NODRIVER      (MMSYSERR_BASE + 6)  // No device driver present
#define MMSYSERR_NOMEM	       (MMSYSERR_BASE + 7)  // Memory allocation error
#define MMSYSERR_NOTSUPPORTED  (MMSYSERR_BASE + 8)  // Function isn't supported
#define MMSYSERR_BADERRNUM     (MMSYSERR_BASE + 9)  // Error number out of range
#define MMSYSERR_LASTERROR     (MMSYSERR_BASE + 9)  // Last error in range


//*****************************************************************************
//  DRIVER CALLBACK
//  The CALLBACK_* flags are used with WaveOutOpen() and WaveInOpen() to
//  determine the type of the dwCallback parameter
//*****************************************************************************

#define CALLBACK_TYPEMASK   0x00070000l 	    // Callback type mask
#define CALLBACK_NULL	    0x00000000l 	    // Unknown callback type
#define CALLBACK_WINDOW     0x00010000l 	    // dwCallback is a HWND
#define CALLBACK_FUNCTION   0x00030000l 	    // dwCallback is a FARPROC


typedef void ( FAR PASCAL DRVCALLBACK ) ( HANDLE h, WORD msg, DWORD dwUser, DWORD dw1, DWORD dw2 );
typedef DRVCALLBACK FAR *LPDRVCALLBACK;

// Flags for waveOutOpen and waveInOpen

#define  WAVE_FORMAT_QUERY     0x0001


//*****************************************************************************
//  WAVEFORM
//*****************************************************************************

typedef HANDLE	HWAVEIN;
typedef HANDLE	HWAVEOUT;
typedef HWAVEIN FAR *LPHWAVEIN;
typedef HWAVEOUT FAR *LPHWAVEOUT;

// Flags for waveOutOpen() and waveInOpen()

#define  WAVE_FORMAT_QUERY     0x0001


//*****************************************************************************
//  WAVE ERROR RETURN VALUES
//*****************************************************************************

#define WAVERR_BADFORMAT      (WAVERR_BASE + 0)  // unsupported wave format
#define WAVERR_STILLPLAYING   (WAVERR_BASE + 1)  // still something playing
#define WAVERR_UNPREPARED     (WAVERR_BASE + 2)  // header hasn't been prepared
#define WAVERR_LASTERROR      (WAVERR_BASE + 2)  // last error in range


//*****************************************************************************
//  WAVE CALLBACK
//*****************************************************************************

typedef DRVCALLBACK  WAVECALLBACK;
typedef WAVECALLBACK FAR *LPWAVECALLBACK;

// Wave Callback Messages

#define WOM_OPEN	MM_WOM_OPEN
#define WOM_CLOSE	MM_WOM_CLOSE
#define WOM_DONE	MM_WOM_DONE

#define WIM_OPEN	MM_WIM_OPEN
#define WIM_CLOSE	MM_WIM_CLOSE
#define WIM_DATA	MM_WIM_DATA


//*****************************************************************************
//  WAVE DATA BLOCK HEADER
//*****************************************************************************

typedef struct wavehdr_tag {
    LPSTR	lpData; 		// Pointer to locked data buffer
    DWORD	dwBufferLength; 	// Length of data buffer
    DWORD	dwBytesRecorded;	// Used for input only
    DWORD	dwUser; 		// For client's use
    DWORD	dwFlags;		// Assorted flags (see defines)
    DWORD	dwLoops;		// Loop control counter
    struct wavehdr_tag far *lpNext;	// Reserved for driver
    DWORD	reserved;		// Reserved for driver
} WAVEHDR;

typedef WAVEHDR FAR *LPWAVEHDR;

// equates for WAVEHDR flag bits

#define WHDR_DONE	0x00000001	// Done bit
#define WHDR_PREPARED	0x00000002	// Set if this header has been prepared
#define WHDR_BEGINLOOP	0x00000004	// Loop start block
#define WHDR_ENDLOOP	0x00000008	// Loop end block
#define WHDR_INQUEUE	0x00000010	// Reserved for driver


//*****************************************************************************
//  WAVE DEVICE CAPS STRUCTURES
//*****************************************************************************

typedef struct waveoutcaps_tag {
    WORD    wMid;		      // Manufacturer id
    WORD    wPid;		      // Product id
    VERSION vDriverVersion;	      // Version of the driver
    char    szPname[MAXPNAMELEN];     // Product name (NULL terminated string)
    DWORD   dwFormats;		      // Formats supported
    WORD    wChannels;		      // Number of sources supported.
    DWORD   dwSupport;		      // What functionality the driver supports
} WAVEOUTCAPS;

typedef WAVEOUTCAPS FAR *LPWAVEOUTCAPS;

// Flags for dwSupport

#define WAVECAPS_PITCH		0x0001	 // Supports pitch control
#define WAVECAPS_PLAYBACKRATE	0x0002	 // Supports playback rate control
#define WAVECAPS_VOLUME 	0x0004	 // Supports volume control
#define WAVECAPS_LRVOLUME	0x0008	 // Separate left-right volume control

typedef struct waveincaps_tag {
    WORD    wMid;		       // Manufacturer id
    WORD    wPid;		       // Product id
    VERSION vDriverVersion;	       // Version of the driver
    char    szPname[MAXPNAMELEN];      // Product name (NULL terminated string)
    DWORD   dwFormats;		       // Formats supported
    WORD    wChannels;		       // Number of streams supported.
} WAVEINCAPS;

typedef WAVEINCAPS FAR *LPWAVEINCAPS;

// Defines for WAVEXXXCAPS format field

#define WAVE_INVALIDFORMAT     0x00000000	// Invalid format
#define WAVE_FORMAT_1M08       0x00000001	// 11.025 kHz, Mono,   8-bit
#define WAVE_FORMAT_1S08       0x00000002	// 11.025 kHz, Stereo, 8-bit
#define WAVE_FORMAT_1M16       0x00000004	// 11.025 kHz, Mono,   16-bit
#define WAVE_FORMAT_1S16       0x00000008	// 11.025 kHz, Stereo, 16-bit
#define WAVE_FORMAT_2M08       0x00000010	// 22.05  kHz, Mono,   8-bit
#define WAVE_FORMAT_2S08       0x00000020	// 22.05  kHz, Stereo, 8-bit
#define WAVE_FORMAT_2M16       0x00000040	// 22.05  kHz, Mono,   16-bit
#define WAVE_FORMAT_2S16       0x00000080	// 22.05  kHz, Stereo, 16-bit
#define WAVE_FORMAT_4M08       0x00000100	// 44.1   kHz, Mono,   8-bit
#define WAVE_FORMAT_4S08       0x00000200	// 44.1   kHz, Stereo, 8-bit
#define WAVE_FORMAT_4M16       0x00000400	// 44.1   kHz, Mono,   16-bit
#define WAVE_FORMAT_4S16       0x00000800	// 44.1   kHz, Stereo, 16-bit


//*****************************************************************************
// WAVE FORMAT STRUCTURES
//*****************************************************************************

typedef struct waveformat_tag {
    WORD    wFormatTag; 	// what format type is this?
    WORD    nChannels;		// number of channels (i.e. mono, stereo, etc.)
    DWORD   nSamplesPerSec;	// sample rate
    DWORD   nAvgBytesPerSec;	// for buffer estimation
    WORD    nBlockAlign;	// block size of data
} WAVEFORMAT;

typedef WAVEFORMAT NEAR *NPWAVEFORMAT;
typedef WAVEFORMAT FAR *LPWAVEFORMAT;



#define WAVE_FORMAT_PCM     1			// defines for wFormatTag


// Use if wFormatTag == WAVE_FORMAT_PCM

typedef struct pcmwaveformat_tag {
    WAVEFORMAT	wf;
    WORD	wBitsPerSample;
} PCMWAVEFORMAT;

typedef PCMWAVEFORMAT NEAR *NPPCMWAVEFORMAT;
typedef PCMWAVEFORMAT FAR *LPPCMWAVEFORMAT;


//*****************************************************************************
//	      WAVE SUPPORT FUNCTIONS
//*****************************************************************************

WORD FAR PASCAL waveOutGetNumDevs( void );
WORD FAR PASCAL waveOutGetDevCaps( WORD wDeviceID,
				   LPWAVEOUTCAPS lpCaps, WORD wSize );
WORD FAR PASCAL waveOutGetVolume( WORD wDeviceID, LPDWORD lpdwVolume );
WORD FAR PASCAL waveOutSetVolume( WORD wDeviceID, DWORD dwVolume );
WORD FAR PASCAL waveOutGetErrorText(WORD wError, LPSTR lpText, WORD wSize );
WORD FAR PASCAL waveOutOpen( LPHWAVEOUT lphWaveOut, WORD wDeviceID,
			     LPWAVEFORMAT lpFormat, DWORD dwCallback,
			     DWORD dwInstance, DWORD dwFlags );
WORD FAR PASCAL waveOutClose(HWAVEOUT hWaveOut);
WORD FAR PASCAL waveOutPrepareHeader( HWAVEOUT hWaveOut,
				      LPWAVEHDR lpWaveOutHdr, WORD wSize );
WORD FAR PASCAL waveOutUnprepareHeader( HWAVEOUT hWaveOut,
					LPWAVEHDR lpWaveOutHdr, WORD wSize );
WORD FAR PASCAL waveOutWrite( HWAVEOUT hWaveOut,
			      LPWAVEHDR lpWaveOutHdr, WORD wSize );
WORD FAR PASCAL waveOutPause( HWAVEOUT hWaveOut );
WORD FAR PASCAL waveOutRestart( HWAVEOUT hWaveOut );
WORD FAR PASCAL waveOutReset( HWAVEOUT hWaveOut );
WORD FAR PASCAL waveOutBreakLoop( HWAVEOUT hWaveOut );
WORD FAR PASCAL waveOutGetPosition( HWAVEOUT hWaveOut,
				    LPMMTIME lpInfo, WORD wSize );
WORD FAR PASCAL waveOutGetPitch( HWAVEOUT hWaveOut, LPDWORD lpdwPitch );
WORD FAR PASCAL waveOutSetPitch( HWAVEOUT hWaveOut, DWORD dwPitch );
WORD FAR PASCAL waveOutGetPlaybackRate( HWAVEOUT hWaveOut, LPDWORD lpdwRate );
WORD FAR PASCAL waveOutSetPlaybackRate( HWAVEOUT hWaveOut, DWORD dwRate );
WORD FAR PASCAL waveOutGetID( HWAVEOUT hWaveOut, LPWORD lpwDeviceID );

WORD FAR PASCAL waveInGetNumDevs( void );
WORD FAR PASCAL waveInGetDevCaps( WORD wDeviceID,
				  LPWAVEINCAPS lpCaps, WORD wSize );
WORD FAR PASCAL waveInGetErrorText( WORD wError, LPSTR lpText, WORD wSize );
WORD FAR PASCAL waveInOpen( LPHWAVEIN lphWaveIn, WORD wDeviceID,
			    LPWAVEFORMAT lpFormat, DWORD dwCallback,
			    DWORD dwInstance, DWORD dwFlags );
WORD FAR PASCAL waveInClose( HWAVEIN hWaveIn );
WORD FAR PASCAL waveInPrepareHeader( HWAVEIN hWaveIn,
				     LPWAVEHDR lpWaveInHdr, WORD wSize );
WORD FAR PASCAL waveInUnprepareHeader( HWAVEIN hWaveIn,
				       LPWAVEHDR lpWaveInHdr, WORD wSize );
WORD FAR PASCAL waveInAddBuffer( HWAVEIN hWaveIn,
				 LPWAVEHDR lpWaveInHdr, WORD wSize );
WORD FAR PASCAL waveInStart( HWAVEIN hWaveIn );
WORD FAR PASCAL waveInStop( HWAVEIN hWaveIn );
WORD FAR PASCAL waveInReset( HWAVEIN hWaveIn );
WORD FAR PASCAL waveInGetPosition( HWAVEIN hWaveIn, LPMMTIME lpInfo, WORD wSize );
WORD FAR PASCAL waveInGetID( HWAVEIN hWaveIn, LPWORD lpwDeviceID );


//*****************************************************************************
//		    AUXILIARY OUTPUT DEVICE
//*****************************************************************************

//*****************************************************************************
//  AUX DEVICE CAPS STRUCTURES
//*****************************************************************************

typedef struct auxcaps_tag {
    WORD    wMid;		      // Manufacturer id
    WORD    wPid;		      // Product id
    VERSION vDriverVersion;	      // Version of the driver
    char    szPname[MAXPNAMELEN];     // Product name (NULL terminated string)
    WORD    wTechnology;	      // Type of device
    DWORD   dwSupport;		      // What functionality the driver supports
} AUXCAPS;

typedef AUXCAPS FAR *LPAUXCAPS;

// Defines for AUXCAPS technology field

#define AUXCAPS_CDAUDIO    1	      // Audio from internal CD-ROM drive
#define AUXCAPS_AUXIN	   2	      // Audio from auxiliary input jacks

// Flags for dwSupport

#define AUXCAPS_VOLUME		0x0001	// supports volume control
#define AUXCAPS_LRVOLUME	0x0002	// separate left-right volume control

//*****************************************************************************
//  AUX SUPPORT FUNCTIONS
//*****************************************************************************

WORD FAR PASCAL auxGetNumDevs( void );
WORD FAR PASCAL auxGetDevCaps( WORD wDeviceID, LPAUXCAPS lpCaps, WORD wSize );
WORD FAR PASCAL auxSetVolume( WORD wDeviceID, DWORD dwVolume );
WORD FAR PASCAL auxGetVolume( WORD wDeviceID, LPDWORD lpdwVolume );


//*****************************************************************************
//  DRIVER SEND MESSAGE FUNCTIONS
//*****************************************************************************

#define DRV_RESERVED	       0x0800
#define DRV_MCI_FIRST	       DRV_RESERVED
#define DRV_MCI_LAST	       ( DRV_RESERVED + 0xFFF )
#define DRV_USER	       0x4000

LONG   FAR PASCAL DrvClose( HANDLE hDriver, LONG lParam1, LONG lParam2 );
HANDLE FAR PASCAL DrvOpen( LPSTR szDriverName, LPSTR szSectionName, LONG lParam2 );
LONG   FAR PASCAL DrvSendMessage( HANDLE hDriver, WORD message, LONG lParam1, LONG lParam2 );

