
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  MVPCDefs.h        (Multimedia Videodisc Player Control Defines)
// VERSION: 1.00
//
// DESCRIPTION:
//
//    This module contains Defines used by the Multimedia videodisc control
//    class (MVPC).
//
//    This header file is imbedded by MVPC.H.  It is used EXTERNALLY by toolkit
//    developers that are including the Multimedia videodisc control class
//    object in their applications.
//
//*****************************************************************************

//*****************************************************************************
// GENERAL DEFINES:
//*****************************************************************************
// Window Style bits:
#define WS_VDP1               0x00000000L
#define WS_VDP2               0x00000001L
#define WS_VDP3               0x00000002L

// Window Style Mask to retrieve player
#define WS_PLAYERMASK         0x00000003L

// Maximum number of outstanding match frames:
#define MAX_MATCHES           5


//*****************************************************************************
// Defines used as mpAdvParam1 in SEEK advises as the reason for the SEEK
//*****************************************************************************

#ifndef MVDP_UNLOAD
#define MVDP_UNLOAD           0     // Unloading player
#define MVDP_INIT             1     // Init ( Loading player )
#define MVDP_PLAYFT           3     // Play From-To
#define MVDP_RESET            6     // Reset
#define MVDP_STILL            9     // Still Frame
#define MVDP_STOP             10    // Stopping player
#endif

#define MVDP_STEPF            2     // Step forward
#define MVDP_STEPB            4     // Step backward

#ifndef MVDP_NOERROR
#define  MVDP_NOERROR           0               /* Successful Completion    */
#define  MVDP_NOTLOADED       901  /* =0x0385 *//* VDP Not Loaded           */
#define  MVDP_MATCHERROR      902  /* =0x0386 *//* Unmatchable Match Frame  */
#define  MVDP_MAXMATCHES      903  /* =0x0387 *//* Max Match Frames Already */
                                                /*  Pending                 */
#define  MVDP_NOPLAYER        904  /* =0x0388 *//* Invalid Player Number    */
#define  MVDP_PLAYERERROR     905  /* =0x0389 *//* Player Error             */
#define  MVDP_BUSY            906  /* =0x038A *//* Destination is Busy      */
#define  MVDP_CANCELLED       907  /* =0x038B *//* Command Was Cancelled    */
#define  MVDP_INVALIDFRAME    910  /* =0x038E *//* Can't Find Frame:        */
#endif                                          /*  More Than 5 Seconds     */
                                                /*  Elapsed During a Search */

#ifndef MVDP_NOTPLAYING                         /* VdpIsPlaying() Ret Codes */
#define  MVDP_NOTPLAYING      0                 /*  VDP Is Not Playing      */
#define  MVDP_PLAYFORWARD     1                 /*  VDP Is Playing Forward  */
#define  MVDP_PLAYREVERSE     2                 /*  VDP Is Playing Reverse  */
#endif


//*****************************************************************************
// DEFINES for MCOMMAND structure:
//*****************************************************************************
// Field: usCommand -----------------------------------------------------------
// Supported Commands for MVDP operations
#define MVPC_CMD_AUDIO              0x0100   // =256 Requires the TOKEN
#define MVPC_CMD_INFODISPLAY        0x0101   // =257 Requires the TOKEN
#define MVPC_CMD_INIT               0x0102   // =258 Requires the TOKEN
#define MVPC_CMD_MATCHFRAME         0x0103   // =259 Requires the TOKEN
#define MVPC_CMD_PLAYFROMTO         0x0104   // =260 Requires the TOKEN
#define MVPC_CMD_RESET              0x0105   // =261 Requires the TOKEN
#define MVPC_CMD_SPEED              0x0106   // =262 Requires the TOKEN
#define MVPC_CMD_STEPBACKWARD       0x0107   // =263 Requires the TOKEN
#define MVPC_CMD_STEPFORWARD        0x0108   // =264 Requires the TOKEN
#define MVPC_CMD_STILLFRAME         0x0109   // =265 Requires the TOKEN
#define MVPC_CMD_STOP               0x010A   // =266 Requires the TOKEN
#define MVPC_CMD_UNLOAD             0x010B   // =267 Requires the TOKEN

// Supported Commands for MVPC operations
#define MVPC_CMD_ACTIVATE           0x0210   // =528 Requires the TOKEN
#define MVPC_CMD_SETFRAMERANGE      0x0211   // =529 Requires the TOKEN

#define MVPC_CMD_SETSTEPINCREMENT   0x0280   // =640 Does not require TOKEN

// These Commands result in MVPC ADVISE messages to the specified hwnd:
#define MVPC_CMD_ADVISEACTIVE       0x0300   // =768 Does not require TOKEN
#define MVPC_CMD_ADVISEFRAME        0x0301   // =769 Does not require TOKEN
#define MVPC_CMD_ADVISESEEK         0x0302   // =770 Does not require TOKEN
#define MVPC_CMD_ADVISEPLAYERERROR  0x0304   // =772 Does not require TOKEN

// define User Interface Command range
#define MVPC_CMD_UI                 0x1000   // command numbers >= this
                                             // and less than MVPC_QRY_UI
                                             // are reserved for UI commands

// Field: usReturnCode --------------------------------------------------------
#define MVPC_ERR_NOERROR            0

// ERRORS (returned by MVPC)
#define MVPC_ERR_NOTLOADED          MVDP_NOTLOADED     // 0x0385 =901
#define MVPC_ERR_MATCHERROR         MVDP_MATCHERROR    // 0x0386 =902
#define MVPC_ERR_MAXMATCHES         MVDP_MAXMATCHES    // 0x0387 =903
#define MVPC_ERR_VDPERROR           MVDP_PLAYERERROR   // 0x0389 =905
#define MVPC_ERR_BUSY               MVDP_BUSY          // 0x038A =906
#define MVPC_ERR_CANCELLED          MVDP_CANCELLED     // 0x038B =907
#define MVPC_ERR_INVALIDFRAME       MVDP_INVALIDFRAME  // 0x038E =910

// internal MVPC ERRORS
#define MVPC_ERR_INVALIDRANGE       0x0201   // =513
#define MVPC_ERR_RETURNED           0x0203   // =515
#define MVPC_ERR_INVALIDQRY         0x0205   // =517

//*****************************************************************************
// DEFINES for MVPCQRY structure:
//*****************************************************************************
// Field: usQuery -------------------------------------------------------------
#define MVPC_QRY_ACTIVE          0x0100   // =256 Active (Token holder) or not
#define MVPC_QRY_STATUS          0x0102   // =258 ALL MVPC and MVDP status info

#define MVPC_QRY_UI              0x2000   // Queries to the user interface

// Field: usPlayStatus --------------------------------------------------------
// Note: These are defined above
// #define  MVDP_NOTPLAYING      0              /*  VDP Is Not Playing      */
// #define  MVDP_PLAYFORWARD     1              /*  VDP Is Playing Forward  */
// #define  MVDP_PLAYREVERSE     2              /*  VDP Is Playing Reverse  */


//*****************************************************************************
// DEFINES for MVPCADV structure:
//*****************************************************************************
// Field: usAdvise ------------------------------------------------------------
// These ADVISEs go to the hwnd specified in the MVPC_CMD_ADVISETOKEN cmd:
#define MVPC_ADV_TOKENCHANGE        0x0100
#define MVPC_ADV_LOSINGTOKEN        0x0101   // =257 Giving up the Focus (Token)
#define MVPC_ADV_REQUESTINGTOKEN    0x0102   // =258 Requesting the Focus (Token)
#define MVPC_ADV_TOKENNOTCAPTURED   0x0103   // =259 The Focus Request failed (Token)
#define MVPC_ADV_TOKENCAPTURED      0x0104   // =260 Focus processing complete (Token)

// This ADVISE goes to the hwnd specified in the MVPC_CMD_ADVISEFRAME cmd:
#define MVPC_ADV_FRAMECHANGE        0x0110   // =272 Advise of a frame change

// These ADVISES go to the hwnd specified in the MVPC_CMD_ADVISESEEK cmd:
#define MVPC_ADV_SEEK               0x0120
#define MVPC_ADV_SEEKBEGIN          0x0121   // =289 Seek is beginning
#define MVPC_ADV_SEEKEND            0x0122   // =290 Seek has ended
#define MVPC_ADV_SEEKFAILED         0x0123   // =291 Seek attempt failed

// These values will be in mpAdvParam1 for an MVPC_ADV_SEEKBEGIN
#define MVPC_SEEK_UNLOAD            MVDP_UNLOAD       // 0
#define MVPC_SEEK_INIT              MVDP_INIT         // 1
#define MVPC_SEEK_STEPF             MVDP_STEPF        // 2
#define MVPC_SEEK_PLAYFT            MVDP_PLAYFT       // 3
#define MVPC_SEEK_STEPB             MVDP_STEPB        // 4
#define MVPC_SEEK_RESET             MVDP_RESET        // 6
#define MVPC_SEEK_STILL             MVDP_STILL        // 9
#define MVPC_SEEK_STOP              MVDP_STOP         // 10
#define MVPC_SEEK_TOKENCHANGE       5

#define MVPC_ADV_PLAYERERROR        0x0151

// ADDITIONAL ADVISES FOR UI'S ONLY:
#define MVPC_ADV_FRAMERANGE         0x0111
#define MVPC_ADV_STEPINCREMENT      0x0112

#define MVPC_ADV_MOTION             0x0130      // PUIADVISE->usPlayDirection

#define MVPC_ADV_AUDIO              0x0140
#define MVPC_ADV_INFODISPLAY        0x0141
#define MVPC_ADV_SPEED              0x0142

#define MVPC_ADV_LOADED             0x0150
#define MVPC_ADV_BUSY               0x0152


//*****************************************************************************
// DEFINES for MVPCCTL structure:
//*****************************************************************************
// Field: usReturnCode --------------------------------------------------------
#define MVPC_RC_CREATE_OK           0
#define MVPC_RC_CREATE_NOVDP        1
#define MVPC_RC_CREATE_FAILED       2


//*****************************************************************************
// END OF FILE
//****************************************************************************
