
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  MVPCType.h        (Multimedia Videodisc Player Control Typedefs)
// VERSION: 1.00
//
// DESCRIPTION:
//
//    This module contains Typedefs for structures used by the Multimedia
//    videodisc control object (MVPC).
//
//    This header file is imbedded by MVPC.H.  It is used EXTERNALLY by toolkit
//    developers that are including the Multimedia videodisc contorl ojbect
//    in their applications.
//
//    The following structures are declared:
//
//       MVPCDATA    -- The VDP Data structure
//       MVPCCMD     -- The VDP Command structure
//       MVPCQRY     -- The VDP Query structure
//       MVPCADV     -- The VDP Advise structure
//       MVPCCTL     -- The VDP Control Data structure
//
//    -------------------------------------------------------------------------
//    Here is an example of how to send a command to an MVPC object:
//
//    PMVPCCMD pmvpccmd;
//
//    if (MVPCCreateCmd (&pmvpccmd, MVPC_CMD_PLAYFROMTO))
//       {
//       pmvpccmd->data.mpCmdParam1 = MPFROMLONG (1000L);  // From: 1000
//       pmvpccmd->data.mpCmdParam2 = MPFROMLONG (2000L);  // To:   2000
//
//       WinPostMsg (hwndMVPC, msgMVPC_COMMAND, hwnd, pmvpccmd);
//       }
//
//    -------------------------------------------------------------------------
//    Here is an example of how to send a query to an MVPC object:
//
//    PMVPCQRY pmvpcqry;
//    MRESULT  rc;
//    LONG     lCurrentFrame;
//
//    if (MVPCCreateQry (&pmvpcqry, MVPC_QRY_STATUS))
//       {
//       rc = WinSendMsg (hwndMVPC, msgMVPC_QUERY, hwnd, pmvpcqry);
//       if (!rc)
//          {
//          lCurrentFrame = pmvpcqry->lCurrentFrame;
//          }
//       }
//
//*****************************************************************************

//*****************************************************************************
typedef struct _MVPCDATA               // Videodisc Player Control Data struct
   {
   MPARAM      mpCmdParam1;            // Command Parameter #1
   MPARAM      mpCmdParam2;            // Command Parameter #2
   MPARAM      mpCmdParam3;            // Command Parameter #3
   MPARAM      mpCmdParam4;            // Command Parameter #4
   MPARAM      mpCmdParam5;            // Command Parameter #5
   } MVPCDATA;

typedef MVPCDATA FAR *PMVPCDATA;


//*****************************************************************************
#ifdef MVPC
typedef struct _MVPCCMD                // Used to send an MVPC_CMD message
   {
   MPREFIX     prefix;                 // Multimedia Prefix structure
   MCOMMAND    command;                // Multimedia Command Structure
   MVPCDATA    data;                   // Videodisc Player Control Data struct
   } MVPCCMD;

typedef MVPCCMD FAR *PMVPCCMD;
#endif


//*****************************************************************************
typedef struct _MVPCQRY                // Query structure
   {
   // INPUT:
   ULONG       cbData;                 // cbData = sizeof (MVPCQRY)
   USHORT      usQuery;                // Query item (ie. MVPC_QRY_*)

   // OUTPUT (everything from here down is Output only):
   // The following fields will always be returned:
   USHORT      usReturnCode;           // Return code (ie. MVPC_ERR_*)
   BOOL        fActive;                // TRUE = MVPC is ACTIVE (Token holder)
   USHORT      usPlayer;               // The VDP # (1, 2, or 3)

   // MVDP Information (this will be information returned for the VDP):
   BOOL        fInfoDisplay;           // TRUE = Info Display is ON
   USHORT      usAudio;                // AUDIO state
   USHORT      usSpeed;                // Speed for Play
   USHORT      usPlayStatus;           // MVDP_STOPPED, MVDP_FORWARD, or
                                       //    MVDP_REVERSE
   LONG        lCurrentFrame;          // The Current frame (Still or Playing)
   LONG        lFromFrame;             // The From frame (Playing only)
   LONG        lToFrame;               // The To frame (Playing only)
   LONG        alMatchFrame[MAX_MATCHES]; // Match Frame array

   // MVPC Settings:
   USHORT      usStepIncrement;        // The current Step Increment
   LONG        lLowerBound;            // Lower bound of the frame range
   LONG        lUpperBound;            // Upper bound of the frame range
   HWND        hwndActive;             // HWND to notify of Focus changes
   HWND        hwndFrame;              // HWND to notify of Frame changes
   HWND        hwndSeek;               // HWND to notify of VDP Seeks
   HWND        hwndError;              // HWND to notify of player errors

   // Additional MVDP information:
   LONG        lMaxFrame;              // Current maximum possible frame number
   } MVPCQRY;

typedef MVPCQRY FAR *PMVPCQRY;


//*****************************************************************************
typedef struct _MVPCADV                // Used to send an MVPC_ADV message
   {
   ULONG       cbData;                 // cbData = sizeof (MVPCADV);
   USHORT      usAdvise;               // Advise Item (ie. MVPC_ADV_*)

   USHORT      usPlayer;               // The VDP # associated with this MVPC
   MPARAM      mpAdvParam1;            // Advise Parameter #1
   MPARAM      mpAdvParam2;            // Advise Parameter #2
   MPARAM      mpAdvParam3;            // Advise Parameter #3
   } MVPCADV;

typedef MVPCADV FAR *PMVPCADV;


//**************************************************************************<f>
// MVPCCTL = Videodisc Player Control Data Block (used at WM_CREATE time):
typedef struct _MVPCCTL
   {
   ULONG       cbData;                 // Length of this data structure
   USHORT      usReturnCode;           // Return code (ie. MVPC_RC_CREATE_*)
   USHORT      usStepIncrement;        // The amount to increment/decrement the
                                       //    frame number on a step command.
   HWND        hwndAdviseActive;       // HWND to notify of TOKEN changes.
   HWND        hwndAdviseFrame;        // HWND to notify of VDP frame changes
   HWND        hwndAdviseSeek;         // HWND to notify when the VDP Seeks
   HWND        hwndAdvisePlayerError;  // HWND to notify of player errors
   } MVPCCTL;

typedef MVPCCTL FAR *PMVPCCTL;

//**************************************************************************<f>
// UIADVISE = Advise structure used when MVPC advises the user interface
typedef struct _UIADVISE               // User interface advise structure
   {
   USHORT      usDataLen;
   USHORT      usError;
   BOOL        fBusy;
   BOOL        fLoaded;
   USHORT      usFocus;
   USHORT      usSeek;
   USHORT      usAudio;
   USHORT      usInfoDisplay;
   USHORT      usSpeed;
   USHORT      usStepIncrement;
   USHORT      usPlayDirection;
   LONG        lCurrentFrame;
   LONG        lLowerBound;
   LONG        lUpperBound;
   } UIADVISE;

typedef UIADVISE FAR *PUIADVISE;

//*****************************************************************************
// END OF FILE
//*****************************************************************************
