// Public WINDOWS header
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1990,1992
//
// MODULE:  MVPCWIN.H	       (Multimedia Videodisc Player Control Imbed File)
// VERSION: 2.00
//
// DESCRIPTION:
//
//    This module imbeds the header files that contain Defines, Typedefs, and
//    Exported function prototypes for the Multimedia Videodisc Player Control
//    (MVPC).
//
//    The application developer that wishes to include the MVPC class in
//    his/her application need only "#include" this module.  The others will
//    automatically be imbedded, as necessary.
//
//*****************************************************************************
//    -------------------------------------------------------------------------
//    Here is an example of how to send a command to an MVPC object:
//
//    HANDLE	 hmvpccmd;
//    LPMVPCCMD  lpmvpccmd;
//    HWND	 hWndMVPC;
//    HWND	 hWnd;
//    WORD	 msgMVPC_COMMAND;   // filled in by earlier MVPCInitialize call
//
//    if (hmvpccmd = MVPCCreateCmd (MVPC_CMD_PLAYFROMTO))
//	 {
//	 if (lpmvpccmd = (LPMVPCCMD)GlobalLock (hmvpccmd))
//	    {
//	    lpmvpccmd->data.mpCmdParam1 = MPFROMLONG (1000L);  // From: 1000
//	    lpmvpccmd->data.mpCmdParam2 = MPFROMLONG (2000L);  // To:	2000
//	    GlobalUnlock (hmvpccmd);
//
//	    WinPostMsg (hWndMVPC, msgMVPC_COMMAND, hWnd, (DWORD)hmvpccmd);
//	    }
//	 }
//
//    -------------------------------------------------------------------------
//    Here is an example of how to send a query to an MVPC object:
//
//    HANDLE	hmvpcqry;
//    LPMVPCQRY lpmvpcqry;
//    MRESULT	rc;
//    LONG	lCurrentFrame;
//    HWND	hWndMVPC;
//    HWND	hWnd;
//    WORD	msgMVPC_QUERY;	 // filled in by earlier MVPCInitialize call
//
//    if (hmvpcqry = MVPCCreateQry (MVPC_QRY_STATUS))
//	 {
//	 rc = WinSendMsg (hWndMVPC, msgMVPC_QUERY, hWnd, (DWORD)hmvpcqry);
//	 if (!rc)
//	    {
//	    lpmvpcqry = (LPMVPCQRY)GlobalLock (hmvpcqry);
//	    lCurrentFrame = lpmvpcqry->lCurrentFrame;
//	    GlobalUnlock (hmvpcqry);
//	    }
//	 }
//
//*****************************************************************************

//*****************************************************************************
// INCLUDES
//*****************************************************************************

#ifndef MTYPE_MVPC
#include "MDEFSWIN.H"      // Defines  for shared Multimedia classes
#include "MTYPEWIN.H"      // Typedefs for shared Multimedia classes
#endif

//*****************************************************************************
// GENERAL DEFINES:
//*****************************************************************************
// Window Style bits:
#define WS_VDP1 	      0x00000000L
#define WS_VDP2 	      0x00000001L

// Window Style Mask to retrieve player
#define WS_PLAYERMASK	      0x00000001L

// Maximum number of outstanding match frames:
#define MAX_MATCHES	      3


//*****************************************************************************
// Defines used as mpAdvParam1 in SEEK advises as the reason for the SEEK
//*****************************************************************************

#ifndef MVDP_UNLOAD
#define MVDP_UNLOAD	      0     // Unloading player
#define MVDP_INIT	      1     // Init ( Loading player )
#define MVDP_PLAYFT	      3     // Play From-To
#define MVDP_RESET	      6     // Reset
#define MVDP_STILL	      9     // Still Frame
#define MVDP_STOP	      10    // Stopping player
#endif

#define MVDP_STEPF	      2     // Step forward
#define MVDP_STEPB	      4     // Step backward

#ifndef MVDP_NOERROR
#define  MVDP_NOERROR		0		// Successful Completion
#define  MVDP_NOTLOADED       901  // =0x0385	   VDP Not Loaded
#define  MVDP_MATCHERROR      902  // =0x0386	   Unmatchable Match Frame
#define  MVDP_MAXMATCHES      903  // =0x0387	   Max Match Frames Already
						//  Pending
#define  MVDP_NOPLAYER	      904  // =0x0388	   Invalid Player Number
#define  MVDP_PLAYERERROR     905  // =0x0389	   Player Error
#define  MVDP_BUSY	      906  // =0x038A	   Destination is Busy
#define  MVDP_CANCELLED       907  // =0x038B	   Command Was Cancelled
#define  MVDP_INVALIDFRAME    910  // =0x038E	   Can't Find Frame:
#endif						//  More Than 5 Seconds
						//  Elapsed During a Search

#ifndef MVDP_NOTPLAYING
#define  MVDP_NOTPLAYING      0 		//  VDP Is Not Playing
#define  MVDP_PLAYFORWARD     1 		//  VDP Is Playing Forward
#define  MVDP_PLAYREVERSE     2 		//  VDP Is Playing Reverse
#endif


//*****************************************************************************
// DEFINES for MCOMMAND structure:
//*****************************************************************************
// Field: wCommand ------------------------------------------------------------
// Supported Commands for MVDP operations
#define MVPC_CMD_AUDIO		    0x0100   // =256 Requires the TOKEN
#define MVPC_CMD_INFODISPLAY	    0x0101   // =257 Requires the TOKEN
#define MVPC_CMD_INIT		    0x0102   // =258 Requires the TOKEN
#define MVPC_CMD_MATCHFRAME	    0x0103   // =259 Requires the TOKEN
#define MVPC_CMD_PLAYFROMTO	    0x0104   // =260 Requires the TOKEN
#define MVPC_CMD_RESET		    0x0105   // =261 Requires the TOKEN
#define MVPC_CMD_SPEED		    0x0106   // =262 Requires the TOKEN
#define MVPC_CMD_STEPBACKWARD	    0x0107   // =263 Requires the TOKEN
#define MVPC_CMD_STEPFORWARD	    0x0108   // =264 Requires the TOKEN
#define MVPC_CMD_STILLFRAME	    0x0109   // =265 Requires the TOKEN
#define MVPC_CMD_STOP		    0x010A   // =266 Requires the TOKEN
#define MVPC_CMD_UNLOAD 	    0x010B   // =267 Requires the TOKEN

// Supported Commands for MVPC operations
#define MVPC_CMD_ACTIVATE	    0x0210   // =528 Requires the TOKEN
#define MVPC_CMD_SETFRAMERANGE	    0x0211   // =529 Requires the TOKEN

#define MVPC_CMD_SETSTEPINCREMENT   0x0280   // =640 Does not require TOKEN

// These Commands result in MVPC ADVISE messages to the specified hwnd:
#define MVPC_CMD_ADVISEACTIVE	    0x0300   // =768 Does not require TOKEN
#define MVPC_CMD_ADVISEFRAME	    0x0301   // =769 Does not require TOKEN
#define MVPC_CMD_ADVISESEEK	    0x0302   // =770 Does not require TOKEN
#define MVPC_CMD_ADVISEPLAYERERROR  0x0304   // =772 Does not require TOKEN


// define User Interface Command range
#define MVPC_CMD_UI		    0x1000   // command numbers >= this
					     // and less than MVPC_QRY_UI
					     // are reserved for UI commands

// Field: wReturnCode ---------------------------------------------------------
#define MVPC_ERR_NOERROR	    0

// ERRORS (returned by MVPC)
#define MVPC_ERR_NOTLOADED	    MVDP_NOTLOADED     // 0x0385 =901
#define MVPC_ERR_MATCHERROR	    MVDP_MATCHERROR    // 0x0386 =902
#define MVPC_ERR_MAXMATCHES	    MVDP_MAXMATCHES    // 0x0387 =903
#define MVPC_ERR_VDPERROR	    MVDP_PLAYERERROR   // 0x0389 =905
#define MVPC_ERR_BUSY		    MVDP_BUSY	       // 0x038A =906
#define MVPC_ERR_CANCELLED	    MVDP_CANCELLED     // 0x038B =907
#define MVPC_ERR_INVALIDFRAME	    MVDP_INVALIDFRAME  // 0x038E =910

// Internal MVPC ERRORS
#define MVPC_ERR_INVALIDRANGE	    0x0201   // =513
#define MVPC_ERR_RETURNED	    0x0203   // =515
#define MVPC_ERR_INVALIDQRY	    0x0205   // =517
#define MVPC_ERR_INSUFFICIENTMEMORY 0x0207   // =519

//*****************************************************************************
// DEFINES for MVPCQRY structure:
//*****************************************************************************
// Field: wQuery --------------------------------------------------------------
#define MVPC_QRY_ACTIVE 	 0x0100   // =256 Active (Token holder) or not
#define MVPC_QRY_STATUS 	 0x0102   // =258 ALL MVPC and MVDP status info

#define MVPC_QRY_UI		 0x2000   // Queries to the user interface

// Field: wPlayStatus ---------------------------------------------------------
// Note: These are defined above
// #define  MVDP_NOTPLAYING	 0		//  VDP Is Not Playing
// #define  MVDP_PLAYFORWARD	 1		//  VDP Is Playing Forward
// #define  MVDP_PLAYREVERSE	 2		//  VDP Is Playing Reverse


//*****************************************************************************
// DEFINES for MVPCADV structure:
//*****************************************************************************
// Field: wAdvise -------------------------------------------------------------
// These ADVISEs go to the hwnd specified in the MVPC_CMD_ADVISETOKEN cmd:
#define MVPC_ADV_TOKENCHANGE	    0x0100
#define MVPC_ADV_LOSINGTOKEN	    0x0101   // =257 Giving up the Focus (Token)
#define MVPC_ADV_REQUESTINGTOKEN    0x0102   // =258 Requesting the Focus (Token)
#define MVPC_ADV_TOKENNOTCAPTURED   0x0103   // =259 The Focus Request failed (Token)
#define MVPC_ADV_TOKENCAPTURED	    0x0104   // =260 Focus processing complete (Token)

// This ADVISE goes to the hwnd specified in the MVPC_CMD_ADVISEFRAME cmd:
#define MVPC_ADV_FRAMECHANGE	    0x0110   // =272 Advise of a frame change

// These ADVISES go to the User Interface hwnd automatically
#define MVPC_ADV_FRAMERANGE	    0x0111
#define MVPC_ADV_STEPINCREMENT	    0x0112

// These ADVISES go to the hwnd specified in the MVPC_CMD_ADVISESEEK cmd:
#define MVPC_ADV_SEEK		    0x0120
#define MVPC_ADV_SEEKBEGIN	    0x0121   // =289 Seek is beginning
#define MVPC_ADV_SEEKEND	    0x0122   // =290 Seek has ended
#define MVPC_ADV_SEEKFAILED	    0x0123   // =291 Seek attempt failed

// These values will be in mpAdvParam1 for an MVPC_ADV_SEEKBEGIN
#define MVPC_SEEK_UNLOAD	    MVDP_UNLOAD       // 0
#define MVPC_SEEK_INIT		    MVDP_INIT	      // 1
#define MVPC_SEEK_STEPF 	    MVDP_STEPF	      // 2
#define MVPC_SEEK_PLAYFT	    MVDP_PLAYFT       // 3
#define MVPC_SEEK_STEPB 	    MVDP_STEPB	      // 4
#define MVPC_SEEK_RESET 	    MVDP_RESET	      // 6
#define MVPC_SEEK_STILL 	    MVDP_STILL	      // 9
#define MVPC_SEEK_TOKENCHANGE	    5

#define MVPC_ADV_MOTION 	    0x0130   // UI's check
					     // LPUIADVISE->wPlayDirection

#define MVPC_ADV_AUDIO		    0x0140
#define MVPC_ADV_INFODISPLAY	    0x0141
#define MVPC_ADV_SPEED		    0x0142

#define MVPC_ADV_LOADED 	    0x0150
#define MVPC_ADV_PLAYERERROR	    0x0151
#define MVPC_ADV_BUSY		    0x0152


//*****************************************************************************
// DEFINES for MVPCCTL structure:
//*****************************************************************************
// Field: wReturnCode ---------------------------------------------------------
#define MVPC_RC_CREATE_OK	    0
#define MVPC_RC_CREATE_NOVDP	    1
#define MVPC_RC_CREATE_FAILED	    2




// Typedefs for the Multimedia Videodisc Player Control
//*****************************************************************************
typedef struct _MVPCDATA	       // Videodisc Player Control Data struct
   {
   DWORD       dwCmdParam1;	       // Command Parameter #1
   DWORD       dwCmdParam2;	       // Command Parameter #2
   DWORD       dwCmdParam3;	       // Command Parameter #3
   DWORD       dwCmdParam4;	       // Command Parameter #4
   DWORD       dwCmdParam5;	       // Command Parameter #5
   } MVPCDATA;

typedef MVPCDATA FAR *LPMVPCDATA;


#ifndef EXCLUDE_MVPCCMD
//*****************************************************************************
typedef struct _MVPCCMD 	       // Used to send an MVPC_CMD message
   {
   MPREFIX     prefix;		       // Multimedia Prefix structure
   MCOMMAND    command; 	       // Multimedia Command Structure
   MVPCDATA    data;		       // Videodisc Player Control Data struct
   } MVPCCMD;

typedef MVPCCMD FAR *LPMVPCCMD;
#endif


//*****************************************************************************
typedef struct _MVPCQRY 	       // Query structure
   {
   // INPUT:
   DWORD       dwData;		       // dwData = sizeof (MVPCQRY)
   WORD        wQuery;		       // Query item (ie. MVPC_QRY_*)

   // OUTPUT (everything from here down is Output only):
   // The following fields will always be returned:
   WORD        wReturnCode;	       // Return code (ie. MVPC_ERR_*)
   BOOL        bActive; 	       // TRUE = MVPC is ACTIVE (Token holder)
   WORD        wPlayer; 	       // The VDP # (1 or 2)

   // MVDP Information (this will be information returned for the VDP):
   BOOL        bInfoDisplay;	       // TRUE = Info Display is ON
   WORD        wAudio;		       // AUDIO state
   WORD        wSpeed;		       // Speed for Play
   WORD        wPlayStatus;	       // MVDP_STOPPED, MVDP_FORWARD, or
				       //    MVDP_REVERSE
   LONG        lCurrentFrame;	       // The Current frame (Still or Playing)
   LONG        lFromFrame;	       // The From frame (Playing only)
   LONG        lToFrame;	       // The To frame (Playing only)
   LONG        alMatchFrame[MAX_MATCHES]; // Match Frame array

   // MVPC Settings:
   WORD        wStepIncrement;	       // The current Step Increment
   LONG        lLowerBound;	       // Lower bound of the frame range
   LONG        lUpperBound;	       // Upper bound of the frame range
   HWND        hWndActive;	       // HWND to notify of Focus changes
   HWND        hWndFrame;	       // HWND to notify of Frame changes
   HWND        hWndSeek;	       // HWND to notify of VDP Seeks
   HWND        hWndError;	       // HWND to notify of player errors

   LONG        lMaxPosition;	       // Maximum player position (FRAME)
   } MVPCQRY;

typedef MVPCQRY FAR *LPMVPCQRY;


//*****************************************************************************
typedef struct _MVPCADV 	       // Used to send an MVPC_ADV message
   {
   DWORD       dwData;		       // dwData = sizeof (MVPCADV);
   WORD        wAdvise; 	       // Advise Item (ie. MVPC_ADV_*)

   WORD        wPlayer; 	       // The VDP # associated with this MVPC
   DWORD       dwAdvParam1;	       // Advise Parameter #1
   DWORD       dwAdvParam2;	       // Advise Parameter #2
   DWORD       dwAdvParam3;	       // Advise Parameter #3
   } MVPCADV;

typedef MVPCADV FAR *LPMVPCADV;


//**************************************************************************<f>
// MVPCCTL = Videodisc Player Control Data Block (used at WM_CREATE time):
typedef struct _MVPCCTL
   {
   DWORD       dwData;		       // Length of this data structure
   WORD        wReturnCode;	       // Return code (ie. MVPC_RC_CREATE_*)
   WORD        wStepIncrement;	       // The amount to increment/decrement the
				       //    frame number on a step command.
   HWND        hWndAdviseActive;       // HWND to notify of TOKEN changes.
   HWND        hWndAdviseFrame;        // HWND to notify of VDP frame changes
   HWND        hWndAdviseSeek;	       // HWND to notify when the VDP Seeks
   HWND        hWndAdvisePlayerError;  // HWND to notify of player errors
   } MVPCCTL;

typedef MVPCCTL FAR *LPMVPCCTL;

//**************************************************************************<f>
// UIADVISE = Advise structure used when MVPC advises the user interface
typedef struct _UIADVISE	       // User interface advise structure
   {
   WORD        wDataLen;
   WORD        wError;
   BOOL        bBusy;
   BOOL        bLoaded;
   WORD        wFocus;
   WORD        wSeek;
   WORD        wAudio;
   WORD        wInfoDisplay;
   WORD        wSpeed;
   WORD        wStepIncrement;
   WORD        wPlayDirection;
   LONG        lCurrentFrame;
   LONG        lLowerBound;
   LONG        lUpperBound;
   } UIADVISE;

typedef UIADVISE FAR *LPUIADVISE;


//*****************************************************************************
// EXPORTED FUNCTION PROTOTYPES
//*****************************************************************************

BOOL   FAR PASCAL MVPCInitialize (LPWORD  lpwMVPC_COMMAND,
				  LPWORD  lpwMVPC_QUERY,
				  WORD	  wMVPC_ACK,
				  WORD	  wMVPC_RESPONSE,
				  WORD	  wMVPC_ADVISE,
				  LPSTR   lpszUIDLLName);

VOID   FAR PASCAL MVPCTerminate (VOID);

HANDLE FAR PASCAL MVPCCreateCtl  (VOID);
BOOL   FAR PASCAL MVPCDestroyCtl (HANDLE hMvpcCtl);

HANDLE FAR PASCAL MVPCCreateCmd  (WORD wCmd);
BOOL   FAR PASCAL MVPCDestroyCmd (HANDLE hMvpcCmd, HWND hWnd, WORD wCmd);

HANDLE FAR PASCAL MVPCCreateQry  (WORD wQuery);
BOOL   FAR PASCAL MVPCDestroyQry (HANDLE hMvpcQry);

HANDLE FAR PASCAL MVPCCreateAdv  (WORD wAdvise, WORD wPlayer,
				  HWND hWndRecipient);
BOOL   FAR PASCAL MVPCDestroyAdv (HANDLE hMvpcAdv);

WORD   FAR PASCAL MVPCGetVersion (VOID);

BOOL   FAR PASCAL MVPCSetUserPtr   (HWND hWnd, LPVOID lpUserPtr);
LPVOID FAR PASCAL MVPCQueryUserPtr (HWND hWnd);

DWORD  FAR PASCAL MVPCUserRangeToFrame (HWND hWnd, WORD wLow, WORD wHigh,
					WORD wPosControl);

BOOL   FAR PASCAL MVPCPlayerExists (WORD wPlayer, LPSTR lpszNameBuffer,
				    int nSize);


//*****************************************************************************
// END OF FILE
//*****************************************************************************
