
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  MVWNDefs.h        (Multimedia Video Window Defines)
// VERSION: 1.00
//
// DESCRIPTION:
//
//    This include file contains the defines required for
//    a Multimedia Video Window (MVWN) application.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES for MCOMMAND structure
//*****************************************************************************
// Field: usCommand------------------------------------------------------------
// Supported Commands for MMVA operations
#define MVWN_CMD_SCALE              0x0100   // =256
#define MVWN_CMD_VIDEOSOURCE        0x0101   // =257
#define MVWN_CMD_BLACKWHITE         0x0102   // =258
#define MVWN_CMD_FREEZE             0x0103   // =259
#define MVWN_CMD_SCANMODE           0x0104   // =260
#define MVWN_CMD_BRIGHTNESS         0x0105   // =261
#define MVWN_CMD_SHARPNESS          0x0106   // =262
#define MVWN_CMD_CONTRAST           0x0107   // =263
#define MVWN_CMD_COLOR              0x0108   // =264
#define MVWN_CMD_TINT               0x0109   // =265

#define MVWN_CMD_MUTE               0x0110   // =272
#define MVWN_CMD_VOLUME             0x0111   // =273
#define MVWN_CMD_BALANCE            0x0112   // =274
#define MVWN_CMD_BASS               0x0113   // =275
#define MVWN_CMD_TREBLE             0x0114   // =276
#define MVWN_CMD_SOURCEDIM          0x0115   // =277

#define MVWN_CMD_FADE               0x0120   // =288
#define MVWN_CMD_CLEAR              0x0121   // =289

#define MVWN_CMD_CAPTUREBITMAP      0x0130   // =304
#define MVWN_CMD_RESTOREBITMAP      0x0131   // =305
#define MVWN_CMD_RELEASEBITMAP      0x0132   // =306

#define MVWN_CMD_CAPTUREIMAGE       0x0140   // =320
#define MVWN_CMD_RESTOREIMAGE       0x0141   // =321
#define MVWN_CMD_RELEASEIMAGE       0x0142   // =322

// Supported Commands for MVWN operations
#define MVWN_CMD_ACTIVATE           0x0200   // =512
#define MVWN_CMD_SETINACTIVESTYLE   0x0201   // =513
#define MVWN_CMD_USERACTIVATE       0x0202   // =514
#define MVWN_CMD_SETPOINTERS        0x0203   // =515
#define MVWN_CMD_SETFILLCOLOR       0x0204   // =516

// These Commands result in MVWN_ADV messages to the specified hwnd:
#define MVWN_CMD_ADVISEACTIVE       0x0300   // =768


// Field: usReturnCode---------------------------------------------------------
#define MVWN_ERR_NOERROR            0

// ERRORS for MVWN API calls: (returned by MVWN Video Window)
#define MVWN_ERR_SIZE               0x0084   // =132
#define MVWN_ERR_INVALIDCOORDINATES 0x0088   // =136
#define MVWN_ERR_BUSY               0x0090   // =144
#define MVWN_ERR_FORMAT             0x00A0   // =160

// ERRORS for MVWN Video Window calls
#define MVWN_ERR_UNKNOWNCMD         0x0200   // =512 Unknown Command
#define MVWN_ERR_UNKNOWNQRY         0x0200   // =512 Unknown Query
#define MVWN_ERR_MMVA               0x0201   // =513 An MMVA error occurred
#define MVWN_ERR_INVALIDSIZE        0x0202   // =514 Capture size is different
                                             //       from actual VW coords
#define MVWN_ERR_INVALIDPARAM       0x0203   // =515
#define MVWN_ERR_NOBUFFER           0x0204   // =516
#define MVWN_ERR_NOHBITMAP          0x0205   // =517


//*****************************************************************************
// DEFINES for MVWNDATA structure
//*****************************************************************************

// for usCommand == MVWN_CMD_SCANMODE, valid values for scan mode type
#define MVWN_SCANM_DEFAULT          0x0000
#define MVWN_SCANM_PROGRESSIVE      0x0001   // Default scan method
#define MVWN_SCANM_LINEDOUBLING     0x0002
#define MVWN_SCANM_SINGLEFIELD      0x0004

// for usCommand == MVWN_CMD_SCANMODE, valid values for scan algorithm
#define MVWN_SCANA_DEFAULT          0x0000
#define MVWN_SCANA_ALGORITHMA       0x0001   // Default for Progressive
#define MVWN_SCANA_ALGORITHMB       0x0002   // Default for other two methods
#define MVWN_SCANA_ALGORITHMC       0x0004

// for usCommand == MVWN_CMD_SETINACTIVESTYLE, valid values are
#define MVWN_INACTIVE_DEFAULTSTYLE  0x0001   // Mono Bitmap
#define MVWN_INACTIVE_MONOBITMAP    0x0001
#define MVWN_INACTIVE_VIDEOIMAGE    0x0002
#define MVWN_INACTIVE_COLORFILL     0x0003


//*****************************************************************************
// DEFINES for MVWNQRY structure
//*****************************************************************************
// Field: usQuery -------------------------------------------------------------
// Supported queries for MVWN Information (MVWN_QUERY):
#define MVWN_QRY_STATUS             0x0200   // =512 MVWN status


//*****************************************************************************
// DEFINES for MVWNADV structure:
//*****************************************************************************
// Field: usAdvise ------------------------------------------------------------
// These ADVISEs go to the hwnd specified in the MVWN_CMD_ADVISEACTIVE cmd:
#define MVWN_ADV_LOSINGTOKEN        0x0100   // =256 Giving up the Token
#define MVWN_ADV_REQUESTINGTOKEN    0x0101   // =257 Requesting the Token
#define MVWN_ADV_TOKENNOTCAPTURED   0x0102   // =258 The Token Request failed
#define MVWN_ADV_TOKENCAPTURED      0x0103   // =259 Token processing complete


//*****************************************************************************
// DEFINES for MVWNCTL structure:
//*****************************************************************************
// Field: usReturnCode --------------------------------------------------------
#define MVWN_RC_CREATE_OK           0
#define MVWN_RC_CREATE_FAILED       1


//*****************************************************************************
// DEFINES for MVWNVideoSourceType () function return code:
//*****************************************************************************
#define MVWN_VST_UNUSED             0
#define MVWN_VST_NTSC               1
#define MVWN_VST_PAL                2
#define MVWN_VST_SNTSC              3        // Y/C NTSC (ie. Super VHS, etc.)
#define MVWN_VST_SPAL               4        // Y/C PAL

// Special Force Paint Video Window Style

#define  WS_HIDDENVIDEO    0x400000L


//*****************************************************************************
// END OF FILE
//*****************************************************************************
