//*****************************************************************************
// (C) Copyright IBM Corporation 1989, 1990, 1991, 1992
//
// MODULE:  MVWNWIN.H	      (Multimedia Video Window Imbed File)
// VERSION: Version 2.00
//
// DESCRIPTION:
//
//    This module imbeds the header files that contain Defines, Typedefs, and
//    Exported function prototypes for the Multimedia Video Window (MVWN).
//
//    Application developers that wish to include the MVWN class in their
//    application need only "#include" this module.  The others will
//    automatically be imbedded, as necessary.
//
//*****************************************************************************

//*****************************************************************************
// INCLUDES
//*****************************************************************************

#define MVWN

#ifndef MTYPE_MVWN
#include "MDefswin.h"               // Defines  for shared Multimedia classes
#include "MTypewin.h"               // Typedefs for shared Multimedia classes
#endif

//*****************************************************************************
// DEFINES for MCOMMAND structure:
//*****************************************************************************

// Field: usAck ---------------------------------------------------------------
#define MACK_NOTPROCESSEDYET	    0
#define MACK_CMDACCEPTED	    1
#define MACK_CMDNOTACCEPTED	    2

// Field: usResponse ----------------------------------------------------------
#define MRESP_NOTPROCESSEDYET	    0
#define MRESP_NORESPONSEPENDING     1
#define MRESP_RESPONSEPENDING	    2

//*****************************************************************************
// DEFINES for MCOMMAND structure
//*****************************************************************************
// Field: wCommand------------------------------------------------------------
// Supported Commands for MMVA operations
#define MVWN_CMD_SCALE		    0x0100   // =256
#define MVWN_CMD_VIDEOSOURCE	    0x0101   // =257
#define MVWN_CMD_BLACKWHITE	    0x0102   // =258
#define MVWN_CMD_FREEZE 	    0x0103   // =259
#define MVWN_CMD_SCANMODE	    0x0104   // =260
#define MVWN_CMD_BRIGHTNESS	    0x0105   // =261
#define MVWN_CMD_SHARPNESS	    0x0106   // =262
#define MVWN_CMD_CONTRAST	    0x0107   // =263
#define MVWN_CMD_COLOR		    0x0108   // =264
#define MVWN_CMD_TINT		    0x0109   // =265

#define MVWN_CMD_MUTE		    0x0110   // =272
#define MVWN_CMD_VOLUME 	    0x0111   // =273
#define MVWN_CMD_BALANCE	    0x0112   // =274
#define MVWN_CMD_BASS		    0x0113   // =275
#define MVWN_CMD_TREBLE 	    0x0114   // =276
#define MVWN_CMD_SOURCEDIM	    0x0115   // =277

#define MVWN_CMD_FADE		    0x0120   // =288
#define MVWN_CMD_CLEAR		    0x0121   // =289

#define MVWN_CMD_CAPTUREBITMAP	    0x0130   // =304
#define MVWN_CMD_RELEASEBITMAP	    0x0132   // =306

#define MVWN_CMD_CAPTUREIMAGE	    0x0140   // =320
#define MVWN_CMD_RESTOREIMAGE	    0x0141   // =321
#define MVWN_CMD_RELEASEIMAGE	    0x0142   // =322

#define MVWN_CMD_ACTIVATE	    0x0200   // =512
#define MVWN_CMD_SETINACTIVESTYLE   0x0201   // =513
#define MVWN_CMD_USERACTIVATE	    0x0202   // =514
#define MVWN_CMD_SETPOINTERS	    0x0203   // =515
#define MVWN_CMD_SETFILLCOLOR	    0x0204   // =516

// These Commands result in MVWN_ADV messages to the specified hwnd:
#define MVWN_CMD_ADVISEACTIVE	    0x0300   // =768


// Field: wReturnCode---------------------------------------------------------
#define MVWN_ERR_NOERROR	    0

// ERRORS for MVWN API calls: (returned by MVWN Video Window)
#define MVWN_ERR_CLASS		    0x0001   // =001
#define MVWN_ERR_REG		    0x0002   // =002
#define MVWN_ERR_CONFIG 	    0x0003   // =003
#define MVWN_ERR_SIZE		    0x0084   // =132
#define MVWN_ERR_INVALIDCOORDINATES 0x0088   // =136
#define MVWN_ERR_BUSY		    0x0090   // =144
#define MVWN_ERR_FORMAT 	    0x00A0   // =160

// ERRORS for MVWN Video Window calls
#define MVWN_ERR_UNKNOWNCMD	    0x0200   // =512 Unknown Command
#define MVWN_ERR_UNKNOWNQRY	    0x0200   // =512 Unknown Query
#define MVWN_ERR_MMVA		    0x0201   // =513 An MMVA error occurred
#define MVWN_ERR_INVALIDSIZE	    0x0202   // =514 Capture size is different
					     //       from actual VW coords
#define MVWN_ERR_INVALIDPARAM	    0x0203   // =515
#define MVWN_ERR_NOBUFFER	    0x0204   // =516
#define MVWN_ERR_NOHBITMAP	    0x0205   // =517

//*****************************************************************************
// DEFINES for MVWNDATA structure
//*****************************************************************************

// for wCommand == MVWN_CMD_SCANMODE, valid values for scan mode type
#define MVWN_SCANM_DEFAULT	    0x0000
#define MVWN_SCANM_PROGRESSIVE	    0x0001   // Default scan method
#define MVWN_SCANM_LINEDOUBLING     0x0002
#define MVWN_SCANM_SINGLEFIELD	    0x0004

// for wCommand == MVWN_CMD_SCANMODE, valid values for scan algorithm
#define MVWN_SCANA_DEFAULT	    0x0000
#define MVWN_SCANA_ALGORITHMA	    0x0001   // Default for Progressive
#define MVWN_SCANA_ALGORITHMB	    0x0002   // Default for other two methods
#define MVWN_SCANA_ALGORITHMC	    0x0004

// for wCommand == MVWN_CMD_SETINACTIVESTYLE, valid values are
#define MVWN_INACTIVE_DEFAULTSTYLE  0x0001   // Mono Bitmap
#define MVWN_INACTIVE_MONOBITMAP    0x0001
#define MVWN_INACTIVE_VIDEOIMAGE    0x0002
#define MVWN_INACTIVE_COLORFILL     0x0003

#define MVWN_DFT_VIDEOSOURCE	    1
#define MVWN_CLR_INACTIVEDEFAULT    0

//*****************************************************************************
// DEFINES for MVWNQRY structure
//*****************************************************************************
// Field: wQuery -------------------------------------------------------------
// Supported queries for MVWN Information (MVWN_QUERY):
#define MVWN_QRY_STATUS 	    0x0200   // =512 MVWN status


//*****************************************************************************
// DEFINES for MVWNADV structure:
//*****************************************************************************
// Field: wAdvise ------------------------------------------------------------
// These ADVISEs go to the hwnd specified in the MVWN_CMD_ADVISEACTIVE cmd:
#define MVWN_ADV_LOSINGTOKEN	    0x0100   // =256 Giving up the Token
#define MVWN_ADV_REQUESTINGTOKEN    0x0101   // =257 Requesting the Token
#define MVWN_ADV_TOKENNOTCAPTURED   0x0102   // =258 The Token Request failed
#define MVWN_ADV_TOKENCAPTURED	    0x0103   // =259 Token processing complete


//*****************************************************************************
// DEFINES for MVWNCTL structure:
//*****************************************************************************
// Field: wReturnCode --------------------------------------------------------
#define MVWN_RC_CREATE_OK	    0
#define MVWN_RC_CREATE_FAILED	    1


//*****************************************************************************
// DEFINES for MVWNVideoSourceType () function return code:
//*****************************************************************************
#define MVWN_VST_UNUSED 	    0
#define MVWN_VST_NTSC		    1
#define MVWN_VST_PAL		    2
#define MVWN_VST_SNTSC		    3	     // Y/C NTSC (ie. Super VHS, etc.)
#define MVWN_VST_SPAL		    4	     // Y/C PAL

//*****************************************************************************
// DEFINES for SetVideoWindowPos () function:
//*****************************************************************************

#define TOP			    1
#define BOTTOM			    2
#define STEPBACK		    3
#define STEPFORWARD		    4

//*****************************************************************************
typedef struct _MVWNDATA	    // Video Window Data Structure
{
 DWORD dwCmdParam1;		    // Command Parameter #1
 DWORD dwCmdParam2;		    // Command Parameter #2
 DWORD dwCmdParam3;		    // Command Parameter #3
 DWORD dwCmdParam4;		    // Command Parameter #4
 DWORD dwCmdParam5;		    // Command Parameter #5
} MVWNDATA;
typedef MVWNDATA FAR *LPMVWNDATA;


//*****************************************************************************
typedef struct _MVWNCMD 	    // Used to send an MVWN_CMD message
{
 MPREFIX    prefix;		    // Multimedia Prefix structure
 MCOMMAND   command;		    // Multimedia Command Structure
 MVWNDATA   data;		    // Video Window Data structure
} MVWNCMD;
typedef MVWNCMD FAR *LPMVWNCMD;


//*****************************************************************************
typedef struct _MVWNVIDIMAGE	    // MVWN Video Image Structure
{
 short	  sLenX;		    // X-Length of the Video Image
 short	  sLenY;		    // Y-Length of the Video Image
 DWORD	  dwImageSize;		    // Size (in bytes) of the Video Image
 HANDLE   hImageBuffer; 	    // handle to the Video Image Buffer
} MVWNVIDIMAGE;

typedef MVWNVIDIMAGE FAR *LPMVWNVIDIMAGE;

//*****************************************************************************
typedef struct _MVWNVIDST	    // Video Status Structure
{
 WORD wVideoSource;		    // Video Source #1, 2, or 3.
 BOOL bBlackWhite;		    // TRUE: Video is in Black/White
 BOOL bFreeze;			    // TRUE: Video is frozen
 WORD wScanMethod;		    // MVWN_SCANM_*
 WORD wScanAlgorithm;		    // MVWN_SCANA_*
 WORD wBrightness;		    // 0 to 100%
 WORD wSharpness;		    // 0 to 100%
 WORD wContrast;		    // 0 to 100%
 WORD wColor;			    // 0 to 100%
 short sTint;			    // -100 to 100
 WORD wFade;			    // 0 to 100%
} MVWNVIDST;
typedef MVWNVIDST FAR *LPMVWNVIDST;


//*****************************************************************************
typedef struct _MVWNAUDST	    // Audio Status Structure
{
 BOOL	 bMuted;		    // TRUE: Audio is muted
 WORD	 wVolume;		    // 0 to 100%
 short	 sBalance;		    // -100 to 100
 short	 sBass; 		    // -100 to 100
 short	 sTreble;		    // -100 to 100
 WORD	 wFade; 		    // 0 to 100%
} MVWNAUDST;
typedef MVWNAUDST FAR *LPMVWNAUDST;


//*****************************************************************************
typedef struct _MVWNQRY 	    // Query structure
{
 DWORD	    dwData;		    // sizeof (MVWNQRY)
 WORD	    wQuery;		    // Query item (MVWN_QRY_*)
 WORD	    wReturnCode;	    // Return code (MVWN_ERR_*)
 BOOL	    bActive;		    // TRUE = this video window is active
 BOOL	    bScale;		    // TRUE = video window is scaled
				    // (compressed)
 MVWNVIDST  mvidst;		    // VIDEO STATUS STRUCTURE
 MVWNAUDST  maudst;		    // AUDIO Status STRUCTURE
 HWND	    hWndAdviseActive;	    // Advise window for token changes
 BOOL	    bUserActivate;	    // TRUE: Mouse Button 1 activates
 WORD	    wInactiveStyle;	    // MVWN_UPD_* values
 LONG	    lInactiveColor;	    // CLR_* value for COLORFILL
 HCURSOR    hCursorSrc1;	    // Pointer when Video Source == 1
 HCURSOR    hCursorSrc2;	    // Pointer when Video Source == 2
 HCURSOR    hCursorSrc3;	    // Pointer when Video Source == 3
 HCURSOR    hCursorInactive;	    // Pointer when inactive
 WORD	    wSourceHorg ;
 WORD	    wSourceVorg ;
 WORD	    wSourceHlen ;
 WORD	    wSourceVlen ;
 WORD	    wDestHorg ;
 WORD	    wDestVorg ;
 WORD	    wDestHlen ;
 WORD	    wDestVlen ;
} MVWNQRY;
typedef MVWNQRY FAR *LPMVWNQRY;


//*****************************************************************************
typedef struct _MVWNADV 	    // Used to send an MVWN_ADV message
{
 DWORD	 dwData;		    // sizeof (MVWNADV);
 WORD	 wAdvise;		    // Advise Item (MVWN_ADV_*)
 DWORD	 dwAdvParam1;		    // Advise parameter
} MVWNADV;
typedef MVWNADV FAR *LPMVWNADV;


//*****************************************************************************
typedef struct _MVWNCTL
{
 DWORD	    dwData;		    // Length of this data structure
 BOOL	    bScale;		    // TRUE = video window is scaled (compressed)
 MVWNVIDST  mvidst;		    // VIDEO STATUS STRUCTURE
 MVWNAUDST  maudst;		    // AUDIO STATUS STRUCTURE
 HWND	    hWndAdviseActive;	    // hWnd notified when video win is active/inactive.
 BOOL	    bUserActivate;	    // TRUE if left mouse button activates
 WORD	    wInactiveStyle;	    // MVWN_UPD_* values
 LONG	    lInactiveColor;	    // CLR_* value for COLORFILL
 HCURSOR    hCursorSrc1;	    // cursor when Video Source == 1
 HCURSOR    hCursorSrc2;	    // cursor when Video Source == 2
 HCURSOR    hCursorSrc3;	    // cursor when Video Source == 3
 HCURSOR    hCursorInactive;	    // cursor when inactive
} MVWNCTL;
typedef MVWNCTL FAR *LPMVWNCTL;

// Special Force Paint Video Window Style

#define  WS_HIDDENVIDEO    0x8000L

//*****************************************************************************
// EXPORTED FUNCTION PROTOTYPES
//*****************************************************************************

BOOL   FAR PASCAL MVWNInitialize( LPWORD,
				  LPWORD,
				  WORD,
				  WORD,
				  WORD ) ;
VOID   FAR PASCAL MVWNTerminate( VOID ) ;
HANDLE FAR PASCAL MVWNCreateCtl( VOID ) ;
BOOL   FAR PASCAL MVWNDestroyCtl( HANDLE ) ;
HANDLE FAR PASCAL MVWNCreateCmd( WORD ) ;
BOOL   FAR PASCAL MVWNDestroyCmd( HANDLE, HWND, WORD ) ;
HANDLE FAR PASCAL MVWNCreateQry( WORD ) ;
BOOL   FAR PASCAL MVWNDestroyQry( HANDLE ) ;
BOOL   FAR PASCAL MVWNDestroyAdv( HANDLE ) ;
WORD   FAR PASCAL MVWNGetVersion( VOID ) ;
LPVOID FAR PASCAL MVWNQueryUserPtr( HWND );
BOOL   FAR PASCAL MVWNSetUserPtr( HWND, LPVOID ) ;
WORD   FAR PASCAL MVWNVideoSourceType( WORD ) ;
VOID   FAR PASCAL SetVideoWindowPos( HWND, WORD, WORD, WORD, WORD ) ;

//*****************************************************************************
// END OF FILE
//*****************************************************************************
