// Public WINDOWS header
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1990,1992
//
// MODULE:  VSBWIN.H
// VERSION: 2.00
//
// DESCRIPTION:
//
//	    This header contains DEFINEs, TYPEDEFs, and function
//	    prototypes necessary for using the Videodisc Scroll Bar (VSB)
//	    User Interface (UI) Dynamic Link Library (DLL) - VSB.DLL
//
//	    The VSB is a sample UI for the MVPC (Videodisc Player Control
//	    Class).  The source code for this UI is provided as a starting
//	    point for those application developers who wish to create
//	    their own UIs for the MVPC object.
//
//	    If you wish to provide specific support for this UI in your
//	    application, then #include this file in your source code
//	    where you wish to make VSB-specific commands, queries, and
//	    advises.
//
//*****************************************************************************

//*****************************************************************************
// User Interface Command IDs must be greater than MVPC_CMD_UI, which is
//    defined in MVPCDEFS.H
//*****************************************************************************

#ifndef MVPC_CMD_UI
#define MVPC_CMD_UI   0x1000
#endif

#ifndef MVPC_QRY_UI
#define MVPC_QRY_UI   0x2000
#endif


//*****************************************************************************
// DEFINES: VSB Command     (Used with MVPCCMD structure)
//*****************************************************************************
// Field: wCommand ------------------------------------------------------------
#define VSB_CMD_SETPOINTERS   MVPC_CMD_UI
#define VSB_CMD_ADVISEUSER    MVPC_CMD_UI + 1


//*****************************************************************************
// DEFINES: VSB Query	    (Used with VSBQRY structure)
//*****************************************************************************
// Field: wQuery --------------------------------------------------------------
#define VSB_QRY_STATUS	      MVPC_QRY_UI

// Field: wReturnCode ---------------------------------------------------------
#define VSB_OKQRY	      1
#define VSB_INVALIDQRY	      0


//*****************************************************************************
// DEFINES: VSB Advise
//*****************************************************************************
// Field: wAdvise -------------------------------------------------------------
#define VSB_ADV_USER	      0x1500
#define VSB_ADV_SLIDERTRACK   0x1501


//*****************************************************************************
// TYPEDEFS: VSB Query Structure
//*****************************************************************************
typedef struct _VSBQRY			  // VSB Query Structure
  {
  DWORD     dwData;			  // Length of this structure
  WORD	    wQuery;			 // ie. VSB_QRY_STATUS
  WORD	    wReturnCode;		 // ie. VSB_OKQRY
  HPOINTER  hptrActive; 		  // Active POINTER
  HPOINTER  hptrBusy;			  // Busy POINTER
  HWND	    hWndAdviseSlider;		  // Advise window handle
  } VSBQRY;

typedef VSBQRY FAR *LPVSBQRY;

//*****************************************************************************
// EXPORTED FUNCTION PROTOTYPES:
//
// Important!  The method used by the MVPC to know which function to
//	       call from a UI DLL.
//
//	       The functions must be EXPORTed in the following manner:
//	       (where XXX is the name of your UI.  It is not limited to
//	       three letters.)
//
//		  XXXInitialize     @1
//		  XXXTerminate	    @2
//		  XXXCreateQry	    @3
//		  XXXDestroyQry     @4
//
//	       Keep in mind that your window procedure must also be
//	       exported for PM, although the MVPC does not call it.
//
//*****************************************************************************

BOOL FAR PASCAL  VSBInitialize (WORD   msgCmd,
				WORD   msgAck,
				WORD   msgResp,
				WORD   msgQuery,
				WORD   msgAdvise,
				LPSTR  lpszUIClassName);

VOID FAR PASCAL  VSBTerminate (VOID);

HANDLE FAR PASCAL  VSBCreateQry  (WORD wQuery);
BOOL   FAR PASCAL  VSBDestroyQry (HANDLE hMmvpcQry);

//*****************************************************************************
// END OF FILE
//*****************************************************************************
