//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : ap.c
//
// Description :
//      This program will playback line audio from an IBM MMotion Audio
// file.  It also permits the user to change the playback volume, indicate
// when to stop playback, display the current file location, and pause the
// playback.
//
//      To run:
//              1. Compile
//              2. Determine the name of the audio file to be played,
//                      SONG.AUD for example
//              3. Type "AP SONG.AUD"
//              4. Playback starts as so as the audio file is loaded
//              5. Change the playback (see below for options) as desired
//              6. Press the ESC key to stop playback and return to
//                      DOS or wait until the end of the audio file is reached
//
//      Keyboard Options:
//         ESC      Exit the audio file before reaching the end
//         u or U   Raise the playback volume by 5% (100% maximum)
//         d or D   Lower the playback volume by 5% (0% minimum)
//         p or P   Pause or resume audio output
//         l or L   Display location in audio file (milliseconds)
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile AP.C with IBM C/2 version 1.1 using the following parms
//
//         cl /AL /G2s /Zp ap.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//
//
//    CHANGES:
//     01.01  09-09-91  Changed cpaud_Close() segment parameter from 0L
//                      to &seg_array so cpaud_Close would free memory
//
//**********************************************************************

/****************************/
/* Standard C include files */
/****************************/
#include <graph.h>
#include <dos.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <string.h>

/**********************************/
/* M-Control Program include file */
/**********************************/
#include "mcalls.h"

/***********************/
/* Function Prototypes */
/***********************/
int verify_audio_file (char *Filename);
void set_play_vol (void);

/*************/
/* Variables */
/*************/
BYTE mybuff[24000];
extern BYTE   cpaud_repchange_ahand;
extern BYTE   cpaud_repchange_seg;
extern ULONG  cpaud_repchange_pos;
extern USHORT cpaud_in_play;
char   play_vol = 50;
char textline[100];

/*****************************************************************************
*  Function: verify_audio_file
*
*  Command Used: NONE - utility function
*
*  Description:
*      Opens and evaluates the specifed file to find audio segment at the
*         beginning of the file.
*
*  Input Parms:
*       Ptr to ascii string for filename to be searched.
*
*  Returns:
*       0 : if not an audio file
*       1 : if header found in file
*****************************************************************************/
int verify_audio_file (char *Filename)
{
  int  filehdl;
  UINT bytesread;
  char Header[8];

  /************************************************/
  /* Attempt to open the file, start at beginning */
  /************************************************/
  filehdl = open (Filename, O_RDONLY | O_BINARY, S_IREAD );

  if (filehdl == 0xFFFF)
    return (0);

  bytesread = read(filehdl, Header, sizeof(Header));
  if (!bytesread || (bytesread == 0xFFFF) ||
       strncmp (Header, "PCM08M08",8))
     {
     close (filehdl);
     return (0);
     }
  close (filehdl);
  return (1);
  }


//*************************************************************
// Function : set_play_vol
//
// Description :
//    Sets the NEW playback volume (changed by the user) and
// displays the new level.
//*************************************************************
void set_play_vol ( )
  {
  cpaud_PlayVolume (FALSE, play_vol);
  sprintf (textline,  "Digital Playback Volume is %d%%  ", play_vol);
  _settextposition ( 15, 10);
  _outtext (textline);
  }


//*************************************************************
// Function : main
//
// Description :
//    Checks the incoming parms for a playback file, then starts
// the playback process.
//
//    Once playback is started the keyboard is read to allow control
// of the playback.  See above for for keyboard controls.
//
// Input Parms :
//      1) filename to be used to hold the audio data
//      2) start time (in milliseconds) position in the audio file
//      3) end time (in milliseconds) position in the audio file
//      4) number of audio segment to play
//
//*************************************************************
main ( int argc, char *argv[] )

{
  BYTE ahand;
  LONG beginms;
  LONG endms;
  BYTE control;
  CP_FRCVBUF prc;
  BYTE subs;
  BYTE select_aseg = 1;
  AUDIO_SEGMENTS *seg_array = 0L;
  int   action;
  BOOL paused = FALSE;
  BOOL show_position = FALSE;

/*************************************************************/
/* Check to make sure that the M-Motion code has been loaded */
/*************************************************************/
  if ( !cp_loaded ( ) )
  {
    printf ( "M-Motion not yet active, install M-Motion and try again" );
    return ( 0 );
  }

  control = 0x00;   // no output??
  control = 0x02;   // left speaker
  control = 0x01;   // right speaker
  control = 0x03;   // both speakers

  switch ( argc )
  {
  case 1:
/********************************************************/
/* There must be a filename provided to play audio from */
/********************************************************/
     printf ("AP will play audio from a file, and provides controls to raise\n");
     printf ("and lower the playback volume.n\n");
     printf ("To use AP type:\n");
     printf ("    AP <filename> <begin> <end> <R/P> \n");
     printf ("where <filename> is the name of the file you wish to use. For example,\n");
     printf ("    AR SONG.AUD\n\n");
     printf ("Only the playback audio filename required.\n");
     exit ( -1 );
     break;
  case 2:
//   only the filename is supplied use the whole recording
     beginms = 0L;
     endms = 0xFFFFFFFFL;
     break;
  case 3:
//   the filename and a begin time has been given, go from the begin time
     beginms = atol ( argv[2] );
     endms = 0xFFFFFFFFL;
     break;
  case 4:
     beginms = atol ( argv[2] );
     endms = atol ( argv[3] );
     break;
  default:
     beginms = atol ( argv[2] );
     endms = atol ( argv[3] );
     select_aseg = (BYTE)atoi ( argv[4] );
  }

  if (!verify_audio_file (argv[1]) )
     {
     printf ("%s does not exist or start with an Audio file header.\n", argv[1] );
     printf ("Check your filename to verify it is an audio file.\n");
     exit ( -1 );
     }

  /***********************/
  /* Start graphics mode */
  /***********************/
  _setvideomode ( _VRES16COLOR );
  _settextcolor ( 9 );
  _settextposition ( 5, 17);
  _outtext ("M-Motion Digital Audio Playback Sample Program");

  /****************************/
  /* Display the instructions */
  /****************************/
  _settextcolor (11);
  _settextposition (  8, 10);
  _outtext ("Press the 'U' key to raise the playback volume");
  _settextposition (  9, 10);
  _outtext ("           or 'D' to lower the playback volume");
  _settextposition ( 10, 10);
  _outtext ("           or 'P' to pause/resume the playback");
  _settextposition ( 11, 10);
  _outtext ("           or 'L' to display/remove the audio file location");
  _settextposition ( 13, 10);
  _outtext ("Press ESC to stop playback and return to DOS");
  _settextposition ( 14, 10);
  _outtext ("                                                                  ");
  sprintf (textline,  "Digital Playback Volume is %d%%  ", play_vol);
  _settextposition ( 15, 10);
  _outtext (textline);
  _settextposition ( 16, 10);
  _outtext ("                                                                  ");
  _settextposition ( 17, 10);
  _outtext ("                                                                  ");
  _settextposition ( 18, 10);
  _outtext ("                                                                  ");
  _settextposition ( 19, 10);
  _outtext ("                                                                  ");
  _settextcolor ( 15 );
  _settextposition ( 15, 10);
  _outtext (textline);
  cp_init ( );

  cpaud_Reset ();

  cpaud_Mute ( 2 );     // mute line audio, to let our digital come thru

/* Init and activate the audio buffer */

  prc = cpaud_BufferInit ( mybuff, sizeof ( mybuff ) );

  prc = cpaud_BufferActivate ( mybuff );

/* If the buffer is OK, try to play the file */
  if ( !prc->data[0])
     {

     prc = cpaud_OpenPlay ( argv[1], &ahand, &seg_array, &subs );

     if (!prc->data[0])
        {
        sprintf (textline,
                 "Audio file with %d segment(s) %c",
                 subs,(subs==1) ? ' ' : 's' );
        _settextcolor (11);
        _settextposition ( 18, 10);
        _outtext (textline);
        _settextcolor ( 15 );

        /***********************/
        /* PLAY the audio file */
        /***********************/

        cpaud_Play ( control, ahand, select_aseg, beginms, endms );

        paused = FALSE;

        while ( cpaud_in_play || paused )
           {

           cpaud_ReportChange ( );

           if (show_position)
              {
              sprintf (textline,
                       "Audio Play : Audio Handle %d Segment # %d Position %ld",
                       (int)cpaud_repchange_ahand, (int)cpaud_repchange_seg, cpaud_repchange_pos );
_settextposition ( 16, 10); _outtext (textline);
              }
           if ( kbhit () )
              {
              action = getch ( );

/************************/
/* ESC = quit playback  */
/************************/
              if ( action == 0x1B )
                 {
                 cpaud_Stop ( );
                 break;
                 }

/**************************/
/* u or U = UP the volume */
/**************************/
              else if ( action == 'u' || action == 'U' )
                 {
                 // raise the playback volume
                 play_vol += 5;
                 if ( play_vol > 100 )
                    play_vol = 100;
                 set_play_vol ( );
                 }

/****************************/
/* d or D = DOWN the volume */
/****************************/
              else if ( action == 'd' || action == 'D' )
                 {
                 // lower the playback volume
                 play_vol -= 5;
                 if ( play_vol < 0 )
                    play_vol = 0;
                 set_play_vol ( );
                 }

/****************************/
/* p or P = Pause playback  */
/****************************/
              else if ( action == 'p' ||  action == 'P'  )
                 {
                 // pause the output
                 if (!paused)
                    {
                    cpaud_Stop ( );
                    paused = TRUE;
                    }
                 else
                    {
                    prc = cpaud_PlayCurrent ( control, ahand );
                    paused = FALSE;
                    }
                 }
/*****************************/
/* l or L = Location Display */
/*****************************/
              else if ( action == 'l' ||  action == 'L'  )
                 {
                 // toggle show_position
                 if ( show_position )
                    {
                    _settextposition ( 16, 10);
                    _outtext ("Location Display Ended                                            ");
                    }
                 show_position = !show_position;
                 }
              action = 0;
              }
           }
        _settextposition ( 18, 10);
        _outtext ("Audio has stopped                                                 ");
        _settextposition ( 19, 10);
        _outtext ("Press a key to return to DOS                                      ");
        action = getch ( );

        _setvideomode ( _DEFAULTMODE );

        cpaud_Close ( ahand, &seg_array );                         /* 01.01 */

        }
     else
        printf ( "%s could not be opened as an Audio file\n", argv[1] );

     cpaud_BufferClose ( );

     }

  cpaud_Mute ( 0 );

  cp_exit ( );

  return ( 0 );
}
