//**********************************************************************
// Copyright IBM Corporation 1989, 1992
/**********************************************************************
 * FILE : AVCVIDEO.C
 **********************************************************************
 * Description :
 *    Loads 640x480 256 color images in AVC format into the MMotion Video
 * Adapter video buffer.  Each image is thereby converted to a video
 * still frame.  Further VGA graphics could then be displayed over the
 * still video image.
 *
 * Note : ASSUMES the AVC image is 640x480.  This sample application is
 *        not intended to support all capabilities of the AVC; therefore
 *        some images may not be displayed perfectly.
 *
 * Compilation Instructions:
 *      1) First, compile MCALLS.C into MCALLS.OBJ.
 *
 *      2) Compile AVCVIDEO.C with IBM C/2 version 1.1 using the
 *         following parms:
 *
 *              cl /AL /G2s /Zp avcvideo.c mcalls.obj /link graphics
 *
 *         If you created combined libraries with graphics you will not
 *         need to link graphics at this stage.
 *
 *         Do NOT select/use ANSI compatibility for your compiler.
 ********************************************************************/

/****************************/
/* Standard C include files */
/****************************/
#include "stdio.h"
#include "conio.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "string.h"
#include "io.h"
#include "fcntl.h"
#include "stdlib.h"
#include "graph.h"
#include "dos.h"
#include "memory.h"

/******************************/
/* AVC & MCALLS include files */
/******************************/
#include "mcalls.h"
#include "avcvideo.h"

/***********/
/* DEFINES */
/***********/
#define UINT    unsigned int
#define TRUE    1
#define FALSE   0

/***********************/
/* Function Prototypes */
/***********************/
int  ReadLoops (VOID);
void ReadTransLine (VOID);
void ReadRunLengthLine (VOID);
int  DoLoad (VOID);
int  ck_for_avc (VOID);

/********************/
/* Global Variables */
/********************/
BYTE SourceFname[100];  /* ASCII string name of AVC file */
int  srcfile;           /* handle of opened AVC file */
BYTE palette256[768];   /* 256 color palette, RGBRGBRGB order */

BYTE VGALine[640];      /* One horz line of pels AVC decompressed */

BYTE AVCTable[256][16]; /* Full AVC Transistion table */
BYTE AVCTableRow[16];   /* single line of AVC transition table */

IMAGEHEADER *im_hdr;    /* ptr to AVC file header info */

int  bNewByte = TRUE;   /* Flag - TRUE if a new data byte must be started */
BYTE TableColumn = 0;   /* The current column in the transition table */
BYTE DataByte;          /* The current data byte from the file */

/*******************************************************************
 * Function : main
 *
 * Description :
 *    Ensures that a parm was passed, checks to determine if it
 * represents an AVC file, then calls the AVC load function.
 *******************************************************************/
void main( int argc, char *argv[] )
   {
   if (argc < 2)
      {
      printf ("AVCVIDEO will display a 640x480 AVC graphics image\n");
      printf ("as a still-video image on the MMotion Video Adapter\n\n");
      printf ("To run AVCVIDEO, type: \n");
      printf ("    AVCVIDEO <filename>\n");
      printf ("where <filename> is the name of the AVC picture file\n");
      return;
      }

   strcpy (SourceFname, argv[1]);       /* AVC file */

   srcfile = open (SourceFname, O_RDONLY | O_BINARY);
   if (srcfile == 0xFFFF)
      {
      printf ("Could not open AVC file requested.\n");
      return;
      }


   if (!ck_for_avc ())
      {
      printf ("Input file is NOT an AVC image w/256 colors\n");
      close (srcfile);
      return;
      }


   DoLoad ();

   close (srcfile);
   _setvideomode (_DEFAULTMODE);
   }

/********************************************************************
 * Function : ck_for_avc                Author : S.Swix / A.Pasho
 *                                      Date   : 8/21/89
 * Description :
 *      Determines if the file is in valid AVC picture file.
 * Must also include a image component.
 *
 * Input Parms  : None
 * Return Code  :
 *    TRUE if a valid AVC file
 *    FALSE if not a valid file
 ********************************************************************/
int ck_for_avc()
{
   DIRHEADER   *dir_header;   /* header of AVC files */
                              /* only about three are active */
   /* allocate memory for the directory header */
   dir_header = (DIRHEADER *)calloc(1, sizeof(DIRHEADER));

   /* Determine if this is a valid AVC file */
   /* by reading in the header information */
   read(srcfile, (char *)dir_header, sizeof(DIRHEADER));

   /* check for avc image type file */
   /* quit if it is not */
   if(dir_header->typ != 0x0200) {
       printf ("NOT an AVC image file!\n");
       free((char *)dir_header);
       return (FALSE);
       }

   free((char *)dir_header);

   /* If it made it this far, it's a valid AVC file */
   return (TRUE);
}

//***********************************************************
// SPECIAL FUNCTIONS
//***********************************************************
void setBIOSbordercolor (void)
  {
  union REGS myregs;

  // just set the border color to  0
  myregs.x.ax = 0x1001;
  myregs.x.bx = 0x0000;

  int86 ( 0x10, &myregs, &myregs );
  }


/**/
/********************************************************************
 * Function : DoLoad                    Author : Scott R. Swix
 *                                      Date   : 02/07/90
 * Function :
 *      Loads an AVC picture file into the MMotion Video Frame Buffer
 *
 * Input Parms  : None
 * Return Code  :
 *    TRUE     if valid AVC file was loaded
 *    FALSE    if an error was encountered
 ********************************************************************/
int DoLoad()
{
   struct videoconfig vc;
   DIRHEADER   *dir_header;
   DIRENTRY    *dir_entry;
   MAPHEADER   *map_hdr; /* header infront of the palette data */
   BYTE image_found,
        colormap_found;
   int status,
       read_flag=0,
       i;
   long int image_off,
            map_off,
            img_header_siz,
            map_header_siz,
            count = 0;


   image_found = FALSE;
   colormap_found = FALSE;


   /* allocate memory for the directory header */
   dir_header = (DIRHEADER *)calloc(1, sizeof(DIRHEADER));

   lseek(srcfile, 0L, SEEK_SET);

   /* Read in the header information */
   status = read(srcfile, (char *)dir_header, sizeof(DIRHEADER));
   if (status == 0xFFFF)
      {
      printf ("Read error on AVC file\n");
      return (FALSE);
      }

   /* allocate memory for one directory entry */
   dir_entry = (DIRENTRY *)calloc(1, sizeof(DIRENTRY));

   /* read through the active directory entries */
   for(i=0; i<dir_header->dactiv; i++) {
      read(srcfile, (char *)dir_entry, sizeof(DIRENTRY));

      /* find the type 0x200 entry for image data */
      if (dir_entry->typ == 0x200)
         {
         /* offset of the image entry */
         image_off = (long)dir_entry->off;

         /* size of the image entry header */
         img_header_siz = (long)dir_entry->sfx;

         image_found = TRUE;
         }

      /* type 0x201 = color map data */
      else if(dir_entry->typ == 0x201)
         {
         /* offset of the color map entry */
         map_off = (long)dir_entry->off;

         /* size of the map entry header */
         map_header_siz = (long)dir_entry->sfx;

         colormap_found = TRUE;
         }

      if (image_found && colormap_found)
         break;
      }

   free((char *)dir_entry);
   free((char *)dir_header);

   /* allocate memory for the image header */
   im_hdr = (IMAGEHEADER *)calloc(1, (UINT)img_header_siz);

   /* read the image entry */
   lseek(srcfile, image_off, SEEK_SET);
   read(srcfile, (char *)im_hdr, (UINT)img_header_siz);

   /* Test code to limit reading to ONLY 640x480 256 color images */
   if ((im_hdr->pln_pel != 640) ||
       (im_hdr->pln_row != 480) ||
       (im_hdr->pel_bit != 8))
      {
      free((char *)im_hdr);
      printf ("This file is NOT an AVC 640x480 256 color image\n");
      return (FALSE);   /* An error has occured - invalid file */
      }


   /*******************************************/
   /* Read in the palette for the image  */
   /*******************************************/
      /* allocate memory for the map header */
      map_hdr = (MAPHEADER *)calloc(1, sizeof(MAPHEADER));

      /* read the map entry */
      lseek (srcfile, map_off, SEEK_SET);
      read(srcfile, (char *)map_hdr, sizeof(MAPHEADER));

      /* read the palette data */
      read(srcfile, (char *)palette256, (UINT)768);
      if (im_hdr->dvc_mod == 0x0411)
         {
         for (i=0; i<768; i++)
            palette256[i] >>= 2;
         }

   /* move to the beginning of the pel data */
   lseek(srcfile, image_off, SEEK_SET);         /* first to img entry */
   lseek(srcfile, img_header_siz, SEEK_CUR);    /* then pass the header */


   //*********************************************************
   // Setup - video, coordinates, etc
   //*********************************************************
   if ( !cp_loaded ( ) ) {
     printf ( "API not yet active, install API and try again\n" );
     return ( 0 );
     }

   _setvideomode (_VRES16COLOR);

   _getvideoconfig ( &vc );

   // now use BIOS to get the border right, Microsoft screws it up
   setBIOSbordercolor ( );

   cp_init ( );

   // show a video file
   cpvid_Reset ( );
   cpvid_Freeze ( TRUE );

   cpvid_TranspColor (0, 0);
   cpvid_Transparency (TRUE);

   /**************************/
   /* Convert the image data */
   /**************************/
   if (!ReadLoops ())
      printf ("Error in AVC file type\n");


   /* free remaining allocated memory */
   free((char *)im_hdr);
   free((char *)map_hdr);

   cpvid_Freeze ( FALSE );
   getch ();

   cpvid_Transparency(FALSE);

   _clearscreen ( _GCLEARSCREEN );

   cp_exit ( );

   return (TRUE);
   }


/********************************************************************
 * Function : ReadLoops                         Author : S.Swix
 *
 * Description :
 *      Reads one line at a time of a TRANSITIONALLY compressed
 * AVC 640x480 image.  Each line is decompressed into the varaible
 * array VGALine.  This VGA data is then sent to the MMotion Video
 * Adapter using the cpvid_Show256Pic () function call.  This
 * performs the conversion from RGB to YUV, and places the comparable
 * YUV pels in the video buffer.
 *
 * Input Parms : None
 * Return Code :
 *    TRUE if all went OK
 *    FALSE if error - NOT TRANSITIONALLY COMPRESSED
 ********************************************************************/
int ReadLoops ()
   {
   int row;
   int AVCTableLen;

   /****************************************/
   /* Perform the conversion, line by line */
   /****************************************/
   switch(im_hdr->chd.cmp_typ)
      {
       case 0:   /* No compression */
         printf("UNCOMPRESSED\n");

         for (row=0; row < 480; row++)
            {
            read( srcfile, VGALine, 640 );

            cpvid_Show256Pic ((BYTE)0x0A, palette256, VGALine,
                              640L, (USHORT)640,
                              (USHORT)0, (USHORT)row,
                              (USHORT)640, (USHORT)1);
            }
         break;

       case 1:   /* Run-Length compression */
         printf("RUN-LENGTH COMPRESSION\n");

         for (row=0; row < 480; row++)
            {
            ReadRunLengthLine ();

            cpvid_Show256Pic ((BYTE)0x0A, palette256, VGALine,
                              640L, (USHORT)640,
                              (USHORT)0, (USHORT)row,
                              (USHORT)640, (USHORT)1);
            }
         break;

       case 3:   /* transitional compression */
         printf("TRANSITIONAL COMPRESSION\n");
         read (srcfile, (char *)&AVCTableLen, 2);

         /* Read in each row in the transition table */
         for (row = 0;  row < (AVCTableLen / 16);  row++)
            {
            read    (srcfile, &(AVCTableRow[0]), 16);
            memmove (&(AVCTable[AVCTableRow[0]][0]), AVCTableRow, 16);
            }

         for (row=0; row < 480; row++)
            {
            ReadTransLine ();

            cpvid_Show256Pic ((BYTE)0x0A, palette256, VGALine,
                              640L, (USHORT)640,
                              (USHORT)0, (USHORT)row,
                              (USHORT)640, (USHORT)1);
            }
         break;

      default:
         return (FALSE);
      }

   return (TRUE);
   }


/********************************************************************
 * Function : ReadTransLine
 *
 * Description :
 *      Decompresses one line from the AVC file.  Since the transition
 * sequence is line independent, global variables for the current col,
 * data byte and new-byte status are used to maintain integrity between
 * calls to this function.
 *
 * Input Parms : None
 * Return Code : None
 ********************************************************************/
void ReadTransLine ()
   {
   BYTE *ReadBuf;        /* buffer to hold data from file */
   long FilePos;         /* Beginning file pos              */
   int  BufIndex;        /* Pos in Read buffer */
   int  PelCount;        /* Number of PELS we've decoded */


   FilePos = lseek (srcfile, 0L, SEEK_CUR);
   ReadBuf = (BYTE *)calloc (1500, 1);
   read (srcfile, ReadBuf, 1200);
   BufIndex = 0;

   PelCount = 0;

   /****************************************************/
   /* ONLY load 640 bytes - ONE 640x480 256 color line */
   /****************************************************/
   while (PelCount < 640)
      {
      if (!TableColumn)
         {
         /**************************************/
         /* Grab both the ROW and COLUMN values.
            This is a START sequence entry */
         /**************************************/
         if (bNewByte) {
           /* Get one byte for the row */
           DataByte = ReadBuf[BufIndex];
           BufIndex++;
           TableColumn = (BYTE)(ReadBuf[BufIndex] >> 4);
           bNewByte = FALSE;
           }

         /***********************************************************/
         /* We are somewhere in the middle of a buffer, and need both
            the ROW and COLUMN values.
         /***********************************************************/
         else {
           /* Build one byte for the row */
           DataByte = (BYTE)((ReadBuf[BufIndex] << 4) |
                             (ReadBuf[BufIndex + 1] >> 4));
           BufIndex++;
           TableColumn = (BYTE)(ReadBuf[BufIndex] & 0x0F);
           BufIndex++;
           bNewByte = TRUE;
           }


         /* Put the data byte in the one-line array */
         VGALine[PelCount++] = DataByte;
         }

      /**************************************/
      /* Now work from the table until a NULL
         (not in row) or end of VGA line */
      /**************************************/
      while (TableColumn && (PelCount < 640))
         {
         DataByte = AVCTable[DataByte][TableColumn];

         /* Put the data byte in the one-line array */
         VGALine[PelCount++] = DataByte;

         if (bNewByte) {
            TableColumn = (BYTE)(ReadBuf[BufIndex] >> 4);
            bNewByte = FALSE;
            }
         else {
            TableColumn = (BYTE)(ReadBuf[BufIndex] & 0x0F);
            BufIndex++;
            bNewByte = TRUE;
            }
         }
      }

   free (ReadBuf);

   /* reset the pointer to the just after the last byte read */
   lseek (srcfile, FilePos + BufIndex, SEEK_SET);
   }

/********************************************************************
 * Function : ReadRunLengthLine
 *
 * Description :
 *      Decompresses one line from the AVC file using Run-Length
 * compression.  See AVCFILES.CNC in \AVAAPI directory of AVC
 * for information about run-length encoding format.
 *
 * Input Parms : None
 * Return Code : None
 ********************************************************************/
void ReadRunLengthLine ()
   {
   BYTE *ReadBuf;        /* buffer to hold data from file */
   long FilePos;         /* Beginning file pos */
   int  BufIndex;        /* Pos in Read buffer */
   int  PelCount;        /* Number of PELS we've decoded */
   int  RunCount;        /* Run-Length count */


   FilePos = lseek (srcfile, 0L, SEEK_CUR);
   ReadBuf = (BYTE *)malloc (650);
   read (srcfile, ReadBuf, 650);
   BufIndex = 0;
   PelCount = 0;

   /****************************************************/
   /* ONLY load 640 bytes - ONE 640x480 256 color line */
   /****************************************************/
   while (PelCount < 640)
      {
        RunCount  = (int)ReadBuf[BufIndex++];       /* Lo-order */
        RunCount |= (int)ReadBuf[BufIndex++] << 8;  /* Hi-order */

        if (RunCount & 0x8000)              /* Hi bit on...We have a run */
           {
            RunCount ^= 0x8000;             /* XOR the hi bit off */
            memset (&VGALine[PelCount], ReadBuf[BufIndex++], RunCount);
           }
        else                                /* It is not a run */
           {
            memcpy( &VGALine[PelCount], &ReadBuf[BufIndex], RunCount );
            BufIndex += RunCount;
           }
        PelCount += RunCount;
      }

   free (ReadBuf);

   /* reset the pointer to just after the last byte read */
   lseek (srcfile, FilePos + BufIndex, SEEK_SET);
   }
