/******************************** AVC.H ***********************************
INCLUDE FILE FOR AVC FILES */

typedef  unsigned char   UCHR;
typedef  unsigned long   ULNG;

typedef struct {           /* FILE DIRECTORY HEADER                        */
  char  sig [8];             /* Signature = "+A+V+C+" (null terminator)      */
  UINT  dvers;               /* Version/Mod Level                            */
  UINT  typ;                 /* File type: !IM = 0x0200  !FN = 0x0400        */
  ULNG  siz;                 /* Total file size (bytes)                      */
  ULNG  spc;                 /* Total free space in file (bytes), normally 0 */
  int   dir;                 /* Nbr of directory entries: !IM = 20  !FN = 15 */
  int   dactiv;              /* Active directory entries (non-placeholders)  */
  ULNG  dserl;               /* Reserved = 00                                */
  ULNG  dvdf;                /* Reserved = 00                                */
  UCHR  gen;                 /* Reserved = 00                                */
  UCHR  kpt;                 /* Reserved = 00                                */
  UINT  dcopy;               /* Copyright flags ... bit 0 (lsb)=copyrighted  */
  char  dname [64];          /* File name (max 25 chars; null term; pad null)*/
  char  dnote [32];          /* Copyright notice (null terminator; pad null) */
  long  dtime;               /* Time stamp (seconds since 1-1-70)            */
  char  dres [24];           /* Reserved = 00                                */
  } DIRHEADER;


typedef struct {             /* DIRECTORY ENTRY                              */
  UINT  typ;                 /* Object type (0x0000 = placeholder)           */
  UINT  sub;                 /* Object sub-type (see specific object)        */
  UINT  obstat;              /* Reserved = 00                                */
  UINT  sfx;                 /* Size of object header (= header struct size) */
  ULNG  svr;                 /* Size of data section in memory               */
  ULNG  siz;                 /* Object size (hdr + data, compressed in file) */
  ULNG  obserl;              /* Reserved = 00                                */
  ULNG  off;                 /* Offset of object in file (0 = dir header)    */
  ULNG  oboff;               /* Reserved = 00                                */
  int   obesch;              /* Reserved = 00                                */
  int   obesct;              /* Reserved = 00                                */
  } DIRENTRY;

typedef struct   {           /* COMMON HEADER FIELDS                         */
  char  obj_id [8];          /* Visual ID (7 chars plus NULL)                */
  int   obj_ver;             /* Version number (see actual file for value)   */
  int   cmp_typ;             /* Compression (0 = uncompressed  1 = run-lng)  */
  char  rsv0000 [4];         /* Reserved - 00                                */
  } COMHEADER;

typedef struct {             /* IMAGE HEADER -- OBJECT 0200                  */
  COMHEADER chd;             /* Common header fields                         */
  int   dvc_mod;             /* Device mode (0x0012  0x0013  0x0411)         */
  int   pln_pel;             /* Image width: pels (320 or 640 if full)       */
  int   pln_row;             /* Image height: scan lines (200 or 480 if full)*/
  int   pln_byt;             /* Image width: bytes (PLN_PEL if mode 13, 411) */
  int   pel_bit;             /* Number of bits in a pel (4 or 8)             */
  char  clrcod;              /* Color coding  (0 = mapped image)             */
  char  g_step;              /* Gun step (1 for modes 12 or 13; 4 for 411)   */
  int   red_max;             /* Max red gun value  ) Same for all ...        */
  int   grn_max;             /* Max green gun value)   63 if mode 12 or 13   */
  int   blu_max;             /* Max blue gun value )  252 if mode 411        */
  char  pelcode;             /* Pel format: 0x00 mode 13, 411; 0x04 mode 12  */
  char  pln_r3;              /* Reserved = 00                                */
  int   xaspect;             /* Aspect ratio: X ) 24 x 24 for mode 12 or 411 */
  int   yaspect;             /* Aspect ratio: Y ) 12 x 10 for mode 13        */
  int   wnd_pof;             /* Reserved = 00                                */
  int   wnd_rof;             /* Reserved = 00                                */
  int   wnd_pel;             /* Same as PLN_PEL                              */
  int   wnd_row;             /* Same as PLN_ROW                              */
  char  pln_res[18];         /* Reserved = 00                                */
  int   cur_row;             /* Reserved = 00                                */
  int   cur_byt;             /* Reserved = 00                                */
  int   cur_pel;             /* Reserved = 00                                */
  int   cur_msk;             /* Reserved = 00                                */
  int   cur_clr;             /* Reserved = 00                                */
  char  *map_ads;            /* Reserved = 00                                */
  } IMAGEHEADER;


typedef struct   {           /* MAP HEADER -- OBJECT 0201                    */
  COMHEADER chd;             /* Common header fields                         */
  int   mapclr;              /* Number of colors in map (16 or 256)          */
  UCHR  coding;              /* Color coding (1 = RGB)                       */
  UCHR  gstep;               /* Gun step (1 for modes 12 or 13; 4 for 411)   */
  int   redmax;              /* Max red gun value  ) Same for all ...        */
  int   grnmax;              /* Max green gun value)   63 if mode 12 or 13   */
  int   blumax;              /* Max blue gun value )  252 if mode 411        */
  int   rsvbas;              /* Start of reserved colors (1)                 */
  int   rsvcnt;              /* Number of reserved colors (0)                */
  int   huebas;              /* Start of hues/intensities (1)                */
  int   huecnt;              /* Number of hues (number of art colors)        */
  int   ntys;                /* Number of intensities (1 or more)            */
  int   cusbas;              /* Start of custom color index                  */
  int   cuscnt;              /* Number of custom colors                      */
  int   bakbas;              /* Start of background colors (1)               */
  int   bakcnt;              /* Number of background colors (0)              */
  int   curone;              /* Index of map entry closest to white (not 0)  */
  int   curtwo;              /* Index of map entry closest to black (not 0)  */
  int   clear;               /* Transparent color (0)                        */
  int   border;              /* Border pel value (0)                         */
  UINT  drclr;               /* Draw color pel value (any map index)         */
  int   drnty;               /* Draw color intensities (1 to 10; normally 1) */
  int   hxntys;              /* HUECNT * NTYS                                */
  char  cusfix;              /* Map type (0 = custom  1 = fixed  2 = std)    */
  char  rsv3[21];            /* Reserved = 00                                */
  } MAPHEADER;


typedef struct  {            /* ENTRY IN MAP -- OBJECT 0201 DATA             */
  UCHR  red;                 /* Red gun value                                */
  UCHR  grn;                 /* Green gun value                              */
  UCHR  blu;                 /* Blue gun value                               */
  } MAPENTRY;
