;Copyright IBM Corporation 1989, 1992
;
pointer      equ 5       ;pointer device destination
pointer_resp equ 4       ;pointer device response
keymode      equ 2       ;pointer device response is in keymode
xy_outside   equ 1       ;pointer device response in XY, select was outside
                         ; calibrated area
vdp          equ 6       ;VDP destination
match        equ 9       ;VDP match frame response number
key1         equ 1       ;key number one, wakeup_var return code
key2         equ 2       ;key number two, wakeup_var return code
match_found  equ 3       ;matchframe, wakeup_var return code
outside_select equ 4     ;pointer device select was outside of a valid key
resp_invalid   equ 5     ;invalid response received, wakeup_var return code

WAKEUP_TEXT   SEGMENT  BYTE PUBLIC 'CODE'
WAKEUP_TEXT   ENDS

ASSUME   CS: WAKEUP_TEXT

EXTRN         _wakeup_var:WORD
EXTRN         _match_ret:WORD

; The underscore in front of WAKE_UP_ROUTINE is the way C identifies the label.
; It must be there in order to link properly.

WAKEUP_TEXT   SEGMENT
     public   _WAKE_UP_ROUTINE
_WAKE_UP_ROUTINE proc far
;
;  To assemble this program, type "MASM AVICWAKE"
;  This routine sets global variable wakeup_var according to certain responses
; received.  If a VDP matchframe response is received, the variable is set to
; a certain value, causing the main program to exit the VDP and halt.  Variable
; match_ret is also set when the matchframe is received so VDP responses are
; not received out of order.
;  If the response received is a pointer device response in key1, the variable
; is set to a value causing the main program to enable VDP audio channel one,
; speak some ROM words and sound a beep.
;  If the response received is a pointer device response in key2, the variable
; is set to a value causing the main program to enable VDP audio channel two,
; speak ROM words and sound a beep.
;  If a pointer device response is received, indicating a select occured
; outside the calibrated area, the variable is set, causing the main program to
; read and discard the response.  This also occurs if the response is from a
; select inside the calibrated area but not in a key.  Other responses do not
; set the variable, unless they are known to be in error.
;  If an invalid response is receive, the variable is set, causing the main
; program to halt.
;
        push ax                    ;save registers used
        push bx
        push es
        push di
        push bp
        mov  bp,sp
        mov di,[bp+14]             ;get offset of command buffer
        mov es,[bp+16]             ;get segment of command buffer
        mov ah,es:[di+3]           ;get destination of buffer
        cmp ah,vdp
        jne not_vdp                ;if destination is not VDP, see if it is
                                   ; a pointer device response.
        mov ah,es:[di+4]           ;get command from buffer
        cmp ah,match
        jne quit                   ;if VDP command is not MATCH, quit.

; VDP Match frame response has been received.
        mov bh,match_found
        mov ax,SEG _match_ret
        mov es,ax                  ;ES = segment of match variable
        mov di,OFFSET _match_ret   ;DI = offset of match variable
        mov es:[di+0],bh           ;Write value to the match variable
        jmp write_wakeup_var

; Check to see if response is a pointer device response in key 1 or key 2.
not_vdp:
        mov ah,es:[di+3]           ;read destination of response
        cmp ah,pointer             ;if response is not a pointer device
                                   ; response and not a VDP response,
        jne quit                   ; then quit

        mov ah,es:[di+4]           ;read command of response
        cmp ah,pointer_resp
        jne quit                   ;if its not a pointer device response, quit

; A pointer device response was found.

        mov ah,es:[di+0]           ;read the length of the response
        cmp ah,8                   ;length of a key response is 8.  If length
        jne check_XY_resp          ;is not 8, check to see if the response is
                                   ; in XY (outside of calibrated area) or in
                                   ; Row,Column (outside of a key)
        mov ah,es:[di+6]           ;read second data byte
        cmp ah,keymode             ;data byte must = 2 (response in key mode)
        jne invalid_resp
        mov bh,es:[di+7]           ;read key, write key value to wakeup_var
        jmp write_wakeup_var

check_XY_resp:
        cmp ah,9                   ;if the select was outside of the calibrated
                                   ;area, length of response will be 9 bytes
        jne check_RC_resp          ;check to see if response is in Row,Column
                                   ; (select outside of key)
        mov ah,es:[di+6]           ;read second data byte
        cmp ah,xy_outside          ;see if data byte = 1 (outside calib. area)
        jne invalid_resp
        mov bh,outside_select       ;write value to wakeup_var
        jmp write_wakeup_var

check_RC_resp:
        cmp ah,10                  ;if select was outside of a key and the
                                   ; response is in Row,Column, the length of
                                   ; response is 10 bytes
        jne invalid_resp           ;length is not 10, invalid response
        mov ah,es:[di+6]           ;read second data byte
        cmp ah,keymode             ;this should be a keymode response
        jne invalid_resp
        mov ah,es:[di+7]           ;read third response byte.  It should be
                                   ;zero, indicating response is in R,C
        cmp ah,0
        jne invalid_resp
        mov bh,outside_select       ;write value to wakeup_var
        jmp write_wakeup_var

invalid_resp:
        mov bh,resp_invalid

write_wakeup_var:
        mov ax,SEG _wakeup_var
        mov es,ax                  ;ES = segment of wakeup variable
        mov di,OFFSET _wakeup_var  ;DI = offset of wakeup variable
        mov es:[di+0],bh           ;Write value to the wakeup variable

quit:   pop bp                     ;Restore registers
        pop di
        pop es
        pop bx
        pop ax
        ret
_WAKE_UP_ROUTINE endp
WAKEUP_TEXT   ENDS
        END
