
//*************************************************************************
// Copyright 1989, 1992 IBM Corporation
/**************************************************************************
*
*  MODULE:  MCALLS.H
*
*  DESCRIPTION:  M-Control Program 'C' function call header
*
*  CHANGES:
*   01.01  09-09-91  Definition of new function cpaud_Wait()
*
**************************************************************************/

/*  Standard Typedefs and Defines                                        */

typedef void           VOID;
typedef VOID far      *PVOID;
typedef unsigned char  BYTE;
typedef signed char    SCHAR;
typedef BYTE far      *PBYTE;
typedef unsigned int   UINT;
typedef unsigned int   BOOL;
typedef UINT far      *PUINT;
typedef long           LONG;
typedef unsigned long  ULONG;
typedef ULONG far     *PULONG;

/*#define PVOID void far * */

typedef int            SHORT;
typedef unsigned short USHORT;

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#define HFILE SHORT
#endif


/*****************************************************************************
*   M-Control Program Receive Buffer Definition
*****************************************************************************/

typedef struct _CP_RCVBUF {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          data[247];
} CP_RCVBUF;


typedef CP_RCVBUF far *CP_FRCVBUF;


/*****************************************************************************
*  General Comm Structures
*****************************************************************************/
typedef struct _CP_CMD {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
} CP_CMD;

typedef struct _CP_CMD_N_DATA1 {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          cmd_byte1;
} CP_CMD_N_DATA1;


typedef struct _CP_LEVELS {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  BYTE          default_flag;
  char          level;
} CP_LEVELS;

//************************
/*  Defines for cp_snd()                                                     */
//************************
#define NO_DATA  sizeof (CP_CMD)
#define ONE_DATA sizeof (CP_CMD_N_DATA1)
#define ID1      1

/*****************************************************************************
*  Video Comm Structures
*****************************************************************************/

typedef struct _VID_IMAGE_PARMS {
  BYTE          ErrData;
  BYTE          bits_per_pixel;
  BYTE          storage_format;
  USHORT        VGAHlen;
  USHORT        VGAVlen;
  USHORT        ShownBufHlen;
  USHORT        ShownBufVlen;
  BYTE          LBorder;
  BYTE          RBorder;
  BYTE          TBorder;
  BYTE          BBorder;
  USHORT        TotalBufHlen;
  USHORT        TotalBufVlen;
  USHORT        TotalSourceHlen;
  USHORT        TotalSourceVlen;
} VID_IMAGE_PARMS;

typedef struct _VIDEO_STATE {
  BYTE          control1;
  BYTE          control2;
  BYTE          V1Config;
  BYTE          V1Audio;
  BYTE          V2Config;
  BYTE          V2Audio;
  BYTE          V3Config;
  BYTE          V3Audio;
  BYTE          Scan;
  BYTE          Algorithm;
  BYTE          Freeze;
  BYTE          TranspCntl;
  BYTE          TranspColor;
  BYTE          TranspNull;
  BYTE          Brightness;
  BYTE          Contrast;
  BYTE          Color;
  char          Tint;
  BYTE          VideoLevel;
  BYTE          Sharpness;
  USHORT        SourceHorg;
  USHORT        SourceVorg;
  USHORT        SourceHlen;
  USHORT        SourceVlen;
  USHORT        DestHorg;
  USHORT        DestVorg;
  USHORT        DestHlen;
  USHORT        DestVlen;
} VIDEO_STATE;
typedef VIDEO_STATE far *PVIDEO_STATE;

typedef struct _READ_VID_STATUS {
  BYTE          error;
  VIDEO_STATE   state_data;
} READ_VIDEO_STATUS;
typedef READ_VIDEO_STATUS far *PREAD_VIDEO_STATUS;

typedef VID_IMAGE_PARMS far *PVID_IMAGE_PARMS;


/*****************************************************************************
*  Audio Comm Structures
*****************************************************************************/

typedef struct _AUDIO_STATE {
  BYTE          control1;
  BYTE          control2;
  BYTE          control3;
  BYTE          volume;
  char          balance;
  char          bass;
  char          treble;
  BYTE          fade;
  BYTE          record_volume;
  BYTE          playback_volume;
} AUDIO_STATE;
typedef AUDIO_STATE far *PAUDIO_STATE;

typedef struct _READ_AUD_STATUS {
  BYTE          error;
  AUDIO_STATE   state_data;
} READ_AUDIO_STATUS;
typedef READ_AUDIO_STATUS far *PREAD_AUDIO_STATUS;


typedef struct _AUDIO_SEGMENTS {
  ULONG         offset;
  ULONG         for_cp_use;
} AUDIO_SEGMENTS;


typedef struct _AUDIO_MONITOR {
  ULONG         total_time;
  ULONG         sample_time;
  BYTE          peak_percent;
  USHORT          saturations;
} AUDIO_MONITOR;

typedef struct _RESTORE_AUD_STATE {
  USHORT        count;
  BYTE          id;
  BYTE          dest;
  BYTE          cmd;
  AUDIO_STATE   state_data;
} RESTORE_AUD_STATE;


typedef struct _AUDIO_MULTI_PLAY {
  BYTE          ahand;
  BYTE          sub;
  ULONG         begin;
  ULONG         end;
} AUDIO_MULTI_PLAY;

typedef struct _AUDIO_HEADER {
  BYTE   name[8];
  BYTE   reserved[4];
  ULONG  time;
  ULONG  length;
} AUDIO_HEADER;

/*****************************************************************************
*   M-Control Program Destinations
*****************************************************************************/

#define ERROR_DEST      0    /* Error Device                                 */
#define SYSTEM_DEST     1    /* System Control                               */
#define VIDEO_DEST      2    /* Video Control                                */
#define POINTER_DEST    5    /* Pointer Device                               */
#define VDP1_DEST       6    /* Videodisc Player 1                           */
#define VDP2_DEST       7    /* Videodisc Player 2                           */
#define TIME_DEST       8    /* Time of Day                                  */
#define AUDIO_DEST      9    /* Audio                                        */

/*****************************************************************************
*  System Control Command Numbers
*****************************************************************************/

#define STOP_RESET_CMD          0x00
#define START_CMD               0x01
#define WAKE_UP_CMD             0x03
#define SOFT_STOP_CMD           0x04
#define SOFT_START_CMD          0x05
#define RD_SYS_CONFIG_CMD       0x06

/*****************************************************************************
*  Video Control Command Numbers
*****************************************************************************/

#define VID_RESET_CMD           0x00
#define VID_CONTROL_CMD         0x01
#define VID_READ_STATUS_CMD     0x02
#define VID_TRANS_COLOR_CMD     0x03
#define VID_RESTORE_STATUS_CMD  0x04
#define VID_READ_IMG_PARMS_CMD  0x05
#define VID_CONFIG_INPUTS_CMD   0x06
#define VID_SET_COORDINATES_CMD 0x07
#define VID_VIDEO_PROTECT_CMD   0x08
#define VID_GRAPHIC_PROTECT_CMD 0x09
#define VID_OPTIMIZE_CMD        0x0A
#define VID_BRIGHTNESS_CMD      0x0B
#define VID_CONTRAST_CMD        0x0C
#define VID_COLOR_CMD           0x0D
#define VID_TINT_CMD            0x0E
#define VID_FADE_CMD            0x0F
#define VID_SHARPNESS_CMD       0x10
#define VID_FREEZE_CMD          0x11
#define VID_CAPTURE_CMD         0x12
#define VID_RESTORE_CMD         0x13
#define VID_COPY_CMD            0x14
#define VID_CLEAR_CMD           0x15
#define VID_SHOW_256PIC_CMD     0x16
#define VID_MULTI_VID_CMD       0x17

#define VID_FADE_ID             0x82

/*****************************************************************************/
// Audio Command Numbers
/*****************************************************************************/
#define AUD_RESET_CMD            0x00
#define AUD_CONTROL_CMD          0x01
#define AUD_READ_STATUS_CMD      0x02
#define AUD_RESTORE_STATUS_CMD   0x03
#define AUD_CONFIG_INPUTS_CMD    0x04
#define AUD_MAIN_VOLUME_CMD      0x05
#define AUD_BALANCE_CMD          0x06
#define AUD_BASS_CMD             0x07
#define AUD_TREBLE_CMD           0x08
#define AUD_FADE_CMD             0x09
#define AUD_INIT_BUFFER_CMD      0x0A
#define AUD_ACTIVATE_BUFFER_CMD  0x0B
#define AUD_OPEN_REC_HANDLE_CMD  0x0C
#define AUD_OPEN_PLAY_HANDLE_CMD 0x0D
#define AUD_CLOSE_HANDLE_CMD     0x0E
#define AUD_HANDLE_STATUS_CMD    0x0F
#define AUD_RECORD_VOLUME_CMD    0x10
#define AUD_RECORD_CMD           0x11
#define AUD_MONITOR_DIGI_LVL_CMD 0x12
#define AUD_DIGI_PLAY_VOLUME_CMD 0x13
#define AUD_PLAY_CMD             0x14
#define AUD_PLAY_MULTIPLE_CMD    0x15
#define AUD_STOP_CMD             0x16
#define AUD_REPORT_CHANGE_CMD    0x17

#define AUD_PLAY_ID              0x82
#define AUD_STOP_ID              0x99

/*****************************************************************************
*  Videodisc Player Command Numbers
*****************************************************************************/

#define VDP_RESET_CMD            0x00
#define VDP_INIT_CMD             0x01
#define VDP_STILL_FRAME_CMD      0x02
#define VDP_PLAY_FROM_TO_CMD     0x03
#define VDP_PLAY_CMD             0x04
#define VDP_FRAME_STEP_CMD       0x05
#define VDP_DISPLAY_INFO_CMD     0x06
#define VDP_AUDIO_CMD            0x07
#define VDP_PASS_THRU_CMD        0x08
#define VDP_MATCH_FRAME_CMD      0x09
#define VDP_READ_FRAME_CMD       0x0A
#define VDP_MANUAL_CMD           0x0B
#define VDP_RANGE_EXC_CMD        0X0C
#define VDP_MATCH_STATUS_CMD     0X0D
#define VDP_REPEAT_CMD           0X0E
#define VDP_SET_SPEED_CMD        0X0F
#define VDP_RD_DISC_ID_CMD       0X10
#define VDP_RD_DISC_STORAGE_CMD  0X11
#define VDP_EXIT_DISC_CMD        0X12
#define VDP_PLAY_AUDIO_CMD       0X13
#define VDP_REPEAT_AUDIO_CMD     0x14
#define VDP_DEFINE_AUDIO_CMD     0x15
#define VDP_STATUS_CMD           0x16

/*****************************************************************************
*  Misc Functions
*****************************************************************************/
BOOL        far cp_loaded    (VOID);
CP_FRCVBUF  far cp_init      (VOID);
PBYTE       far cp_exit      (VOID);
VOID        far cp_purge_que (VOID);

USHORT      far cp_snd       (PVOID dbuff);
SHORT       far cp_rcv       (BYTE dest);

/*****************************************************************************
*  Video Control Functions
*****************************************************************************/
PREAD_VIDEO_STATUS  far cpvid_ReadStatus     (BOOL extended_flag );
PVID_IMAGE_PARMS    far cpvid_ReadImageParms (VOID);

CP_FRCVBUF  far cpvid_Reset              (VOID);
CP_FRCVBUF  far cpvid_Control            (BYTE Control);
CP_FRCVBUF  far cpvid_Source             (BYTE Source);
CP_FRCVBUF  far cpvid_VideoOnly          (BOOL VidOnly);
CP_FRCVBUF  far cpvid_Transparency       (BOOL TranspOn);
CP_FRCVBUF  far cpvid_BlackWhite         (BOOL BlackWhiteON);
CP_FRCVBUF  far cpvid_TranspColor        (BYTE All256,  BYTE ColorNum);
CP_FRCVBUF  far cpvid_RestoreStatus      (PVIDEO_STATE StatusData);
CP_FRCVBUF  cpvid_ConfigInputs           (BYTE V1Type, BYTE V1Audio,
                                          BYTE V2Type, BYTE V2Audio,
                                          BYTE V3Type, BYTE V3Audio);
CP_FRCVBUF  far cpvid_SetCoordinates     (SHORT  SourceHorg, SHORT  SourceVorg,
                                          USHORT SourceHlen, USHORT SourceVlen,
                                          SHORT  DestHorg,   SHORT  DestVorg,
                                          USHORT DestHlen,   USHORT DestVlen);
CP_FRCVBUF  far cpvid_SetFullscreenCoord (VOID);
CP_FRCVBUF  far cpvid_VideoProtect       (BOOL VidProtectFlag,
                                          SHORT  DestHorg,  SHORT DestVorg,
                                          USHORT DestHlen,  USHORT DestVlen);
CP_FRCVBUF  far cpvid_VideoProtectAll    (BOOL VidProtectFlag);
CP_FRCVBUF  far cpvid_GraphicProtect     (BOOL GrafProtectFlag,
                                          SHORT  DestHorg,  SHORT  DestVorg,
                                          USHORT DestHlen,  USHORT DestVlen);
CP_FRCVBUF  far cpvid_GraphicProtectAll  (BOOL GrafProtectFlag);
CP_FRCVBUF  far cpvid_OptimizeVid        (BYTE ScanMethod,  BYTE Algorithm);
CP_FRCVBUF  far cpvid_Brightness         (BOOL MakeDefault,  BYTE Level);
CP_FRCVBUF  far cpvid_DefBrightness      (VOID);
CP_FRCVBUF  far cpvid_Contrast           (BOOL MakeDefault,  BYTE Level);
CP_FRCVBUF  far cpvid_DefContrast        (VOID);
CP_FRCVBUF  far cpvid_Color              (BOOL MakeDefault,  BYTE Level);
CP_FRCVBUF  far cpvid_DefColor           (VOID);
CP_FRCVBUF  far cpvid_Tint               (BOOL MakeDefault,  char level);
CP_FRCVBUF  far cpvid_DefTint            (VOID);
void        far cpvid_Fade               (BYTE Level,  BYTE TimeInTenths);
void        far cpvid_DefFade            (VOID);
CP_FRCVBUF  far cpvid_Sharpness          (BOOL MakeDefault,  BYTE Level);
CP_FRCVBUF  far cpvid_DefSharpness       (VOID);
CP_FRCVBUF  far cpvid_Freeze             (BOOL FreezeOn);
CP_FRCVBUF  far cpvid_Capture            (BYTE Control,
                                          USHORT SourceHorg, USHORT SourceVorg,
                                          USHORT SourceHlen, USHORT SourceVlen,
                                          PBYTE Buffer, ULONG Bufferlen,
                                          HFILE Filehandle, ULONG Fileoffset);
CP_FRCVBUF  far cpvid_CaptureBuffer      (BYTE Control,
                                          USHORT SourceHorg, USHORT SourceVorg,
                                          USHORT SourceHlen, USHORT SourceVlen,
                                          PBYTE Buffer, ULONG Bufferlen);
CP_FRCVBUF  far cpvid_CaptureFile        (PBYTE filename);
CP_FRCVBUF  far cpvid_Restore            (BYTE  control,
                                          SHORT   DestHorg,  SHORT   DestVorg,
                                          USHORT  DestHlen,  USHORT  DestVlen,
                                          PBYTE   Vbuffer,   ULONG Vbufferlen,
                                          SHORT   DataHorg,  SHORT   DataVorg,
                                          SHORT Filehandle, ULONG Fileoffset);
CP_FRCVBUF  far cpvid_RestoreBuffer      (BYTE    control,
                                          SHORT   DestHorg,  SHORT DestVorg,
                                          USHORT  DestHlen,  USHORT DestVlen,
                                          PBYTE   Vbuffer,   ULONG Vbufferlen,
                                          SHORT   DataHorg,  SHORT DataVorg);
CP_FRCVBUF  far cpvid_RestoreFile        (PBYTE Filename);
CP_FRCVBUF  far cpvid_Copy               (BYTE control,
                                          SHORT  SourceHorg,  SHORT  SourceVorg,
                                          USHORT SourceHlen,  USHORT SourceVlen,
                                          SHORT  DestHorg,    SHORT  DestVorg);
CP_FRCVBUF  far cpvid_Clear              (BYTE control,
                                          SHORT  DestHorg,  SHORT  DestVorg,
                                          USHORT DestHlen,  USHORT DestVlen,
                                          BYTE Red_1, BYTE Green_1,  BYTE Blue_1,
                                          BYTE Red_2, BYTE Green_2,  BYTE Blue_2);
CP_FRCVBUF  far cpvid_ClearAllBlack      (VOID);
CP_FRCVBUF  far cpvid_Show256Pic         (BYTE  control,
                                          PBYTE color_table, PBYTE pic_ptr,
                                          ULONG TotalByteLen, USHORT  line_len,
                                          SHORT   DestHorg,  SHORT   DestVorg,
                                          USHORT  DestHlen,  USHORT  DestVlen);
CP_FRCVBUF  far cpvid_MultiVid           (BYTE entries,  PVOID video_table);

/*****************************************************************************
*  Audio Functions
*****************************************************************************/

/* Line Audio current state information    */
PREAD_AUDIO_STATUS far cpaud_ReadStatus     (VOID);
CP_FRCVBUF         far cpaud_RestoreStatus  (PAUDIO_STATE StateData);

/* Line Audio Reset                        */
CP_FRCVBUF  far cpaud_Reset           (VOID);


/* Line Audio Configuration switches       */
CP_FRCVBUF  far cpaud_Follow          (BOOL FollowOn);
CP_FRCVBUF  far cpaud_Mute            (BYTE MuteVal);
CP_FRCVBUF  far cpaud_Inputs          (BYTE Inputs);
CP_FRCVBUF  far cpaud_Outputs         (BYTE Outputs);
CP_FRCVBUF  far cpaud_ConfigInputs    (BOOL L1On, BYTE L1Level,
                                       BOOL R1On, BYTE R1Level,
                                       BOOL L2On, BYTE L2Level,
                                       BOOL R2On, BYTE R2Level);

/* Line Audio levels                       */
CP_FRCVBUF  far cpaud_Volume          (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_DefVolume       (VOID);
CP_FRCVBUF  far cpaud_Balance         (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_DefBalance      (VOID);
CP_FRCVBUF  far cpaud_Bass            (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_DefBass         (VOID);
CP_FRCVBUF  far cpaud_Treble          (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_DefTreble       (VOID);
void        far cpaud_Fade            (BYTE Level, BYTE Time_in_Tenths);
void        far cpaud_DefFade         (VOID);
CP_FRCVBUF  far cpaud_PlayVolume      (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_DefPlayVolume   (VOID);

/* Digital Audio buffer functions          */
CP_FRCVBUF  far cpaud_BufferInit      (PVOID Buffer, USHORT BufferLen);
CP_FRCVBUF  far cpaud_BufferActivate  (PVOID Buffer);
CP_FRCVBUF  far cpaud_BufferClose     (VOID);

/* Digital Audio Handle functions          */
CP_FRCVBUF  far cpaud_OpenRecord      (char *Filename, ULONG Offset, PBYTE Ahand);
BYTE        far cpaud_MakeSegmentList (char *Filename, AUDIO_SEGMENTS **SegmentArray);
CP_FRCVBUF  far cpaud_OpenPlay        (char *Filename, PBYTE Ahand,
                                       AUDIO_SEGMENTS **SegmentArray, BYTE *NumSegments);
CP_FRCVBUF  far cpaud_Close           (BYTE Ahand, AUDIO_SEGMENTS **SegmentArray);
CP_FRCVBUF  far cpaud_AhandleStatus   (VOID);

/* Digital Audio Record functions          */
CP_FRCVBUF  far cpaud_RecordVolume    (BOOL MakeDefault, BYTE Level);
CP_FRCVBUF  far cpaud_Record          (BYTE Ahand, BYTE Control);
CP_FRCVBUF  far cpaud_RecordTimed     (BYTE Ahand, BYTE Control,
                                       ULONG Milliseconds);
CP_FRCVBUF  far cpaud_RecordSource    (BYTE Control);

CP_FRCVBUF  far cpaud_Monitor         (AUDIO_MONITOR *MonitorBuff);

/* Digital Audio Play functions            */
CP_FRCVBUF  far cpaud_Play            (BYTE Control, BYTE Ahand, BYTE Segment,
                                       ULONG Start, ULONG Stop);
CP_FRCVBUF  far cpaud_PlaySegment     (BYTE Control, BYTE Ahand, BYTE Segment);
CP_FRCVBUF  far cpaud_PlayCurrent     (BYTE Control, BYTE Ahand);
CP_FRCVBUF  far cpaud_PlayMulti       (BYTE Control, BYTE Entries, BYTE Ahand,
                                       PVOID PlayArray);

/* Digital Audio Control function          */
CP_FRCVBUF  far cpaud_Stop            (VOID);

VOID        cpaud_ReportChange        (VOID);

BOOL        far cpaud_Wait            ( BYTE dest, BYTE cmd );        /*01.01*/


/*****************************************************************************
*  Conversion Functions
*****************************************************************************/
typedef struct _RGBSTRUCT {
   BYTE         Red;
   BYTE         Green;
   BYTE         Blue;
} RGBStruct;

VOID        cp_YUVtoRGB               (UINT YUVWord1, UINT YUVWord2, UINT YUVWord3,
                                       RGBStruct *RGB1, RGBStruct *RGB2,
                                       RGBStruct *RGB3, RGBStruct *RGB4,
                                       float Sat);
VOID        ConvertYUVtoRGB           (BYTE Y,
                                       SCHAR U, SCHAR V,
                                       RGBStruct *RGB, float SATURATION);
