//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : MULTIVID.C
//
// Description :  Example program to demonstrate multiple video windows on
//                the screen at once.  If different sources are used they
//                must be synchronized.
//
//      To run:
//              1. Compile
//              2. Type "MULTIVID"
//              3. Press any key to exit
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile MULTIVID.C with IBM C/2 ver. 1.1 using the following parms
//
//         cl /AL /G2s /Zp multivid.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//**********************************************************************

/****************************/
/* Standard C include files */
/****************************/
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <graph.h>

/**********************************/
/* M-Control Program include file */
/**********************************/
#include "mcalls.h"


#define NUM_OF_WINDOWS  2     // Number of video windows on screen at once.

// Structure defines
typedef struct {              // Structure for multi-video windows
   BYTE     source;           //  1       source
   BYTE     cycle_count;      //  2       cycle count
   SHORT    sourceHorg;       //  3 - 4   source horizontal origin
   SHORT    sourceVorg;       //  5 - 6   source vertical origin
   SHORT    sourceHlen;       //  7 - 8   source horizontal length
   SHORT    sourceVlen;       //  9 - 10  source vertical length
   SHORT    destHorg;         // 11 - 12  destination horizontal origin
   SHORT    destVorg;         // 13 - 14  destination vertical origin
   SHORT    destHlen;         // 15 - 16  destination horizontal length
   SHORT    destVlen;         // 17 - 18  destination vertical length
} VIDEO_WIN;

// Main program
int main ( void )
{
   VIDEO_WIN      video_array[NUM_OF_WINDOWS];

   // Setup video window 1
   video_array[0].source      =  1;
   video_array[0].cycle_count =  1;
   video_array[0].sourceHorg  =  0;
   video_array[0].sourceVorg  =  0;
   video_array[0].sourceHlen  =  640;
   video_array[0].sourceVlen  =  480;
   video_array[0].destHorg    =  328;
   video_array[0].destVorg    =  0;
   video_array[0].destHlen    =  312;
   video_array[0].destVlen    =  234;

   // Setup video window 2
   video_array[1].source      =  1;
   video_array[1].cycle_count =  1;
   video_array[1].sourceHorg  =  0;
   video_array[1].sourceVorg  =  0;
   video_array[1].sourceHlen  =  640;
   video_array[1].sourceVlen  =  480;
   video_array[1].destHorg    =  328;
   video_array[1].destVorg    =  246;
   video_array[1].destHlen    =  312;
   video_array[1].destVlen    =  234;


   // Set video mode to 640 x 480 x 16 colors (Mode 0x12)
   _setvideomode ( _VRES16COLOR );

   // Check if we can start API
   if ( !cp_loaded ( ) ) {
     printf ( "MMEDIA and MMOTION must be loaded to run this program." );
     return ( 3 );
   }

   // Start M-Control Program
   cp_init ( );

   // Perform initial setup
   cpvid_Reset ( );
   cpvid_Freeze ( TRUE );
   cpvid_ClearAllBlack ( );
   cpvid_TranspColor ( FALSE, 0 );
   cpvid_Transparency ( TRUE );
   cpaud_Mute ( FALSE );

   // Setup multivideo windows
   printf ("\n");
   printf ("%s %i \n", "video_array[0].source=     ",video_array[0].source);
   printf ("%s %i \n", "video_array[0].cycle_count=",video_array[0].cycle_count);
   printf ("%s %i \n", "video_array[0].sourceHorg= ",video_array[0].sourceHorg);
   printf ("%s %i \n", "video_array[0].sourceVorg= ",video_array[0].sourceVorg);
   printf ("%s %i \n", "video_array[0].sourceHlen= ",video_array[0].sourceHlen);
   printf ("%s %i \n", "video_array[0].sourceVlen= ",video_array[0].sourceVlen);
   printf ("%s %i \n", "video_array[0].destHorg=   ",video_array[0].destHorg);
   printf ("%s %i \n", "video_array[0].destVorg=   ",video_array[0].destVorg);
   printf ("%s %i \n", "video_array[0].destHlen=   ",video_array[0].destHlen);
   printf ("%s %i \n", "video_array[0].destVlen=   ",video_array[0].destVlen);
   printf ("\n\n\n\n\n");

   printf ("%s %i \n", "video_array[1].source=     ",video_array[1].source);
   printf ("%s %i \n", "video_array[1].cycle_count=",video_array[1].cycle_count);
   printf ("%s %i \n", "video_array[1].sourceHorg= ",video_array[1].sourceHorg);
   printf ("%s %i \n", "video_array[1].sourceVorg= ",video_array[1].sourceVorg);
   printf ("%s %i \n", "video_array[1].sourceHlen= ",video_array[1].sourceHlen);
   printf ("%s %i \n", "video_array[1].sourceVlen= ",video_array[1].sourceVlen);
   printf ("%s %i \n", "video_array[1].destHorg=   ",video_array[1].destHorg);
   printf ("%s %i \n", "video_array[1].destVorg=   ",video_array[1].destVorg);
   printf ("%s %i \n", "video_array[1].destHlen=   ",video_array[1].destHlen);
   printf ("%s %i \n", "video_array[1].destVlen=   ",video_array[1].destVlen);
   cpvid_MultiVid ( NUM_OF_WINDOWS, video_array );
   cpvid_Freeze ( FALSE );

   // Wait for user to hit a key
   getch ( );

   // Reset to avoid lock after exit.
   cpvid_Reset ( );

   cpaud_Mute ( TRUE );
   cp_exit ( );

   _setvideomode ( _DEFAULTMODE );
   return (0);
}
