//**********************************************************************
// Copyright IBM Corporation 1989, 1992
//**********************************************************************
// File : SHOWVID.C
//
// Description
//    Test cases/demos for MCALLS function lib.
//
//    Exercises a variety of M-Control Program functions
// and capabilities.
//
//    Select the operations to be tested/demoed by UNCOMMENTING
// the appropriate #define statements below under the headings
// 'Miscellaneous Actions' and 'File Actions'.
//
//    Note: IF NO command line parameters are provided, the
//          miscellaneous actions are performed.
//
//          IF ANY command line parms are entered, they are
//          ASSUMED to be file names, and the file actions
//          will be performed.  Only ONE file name can be
//          entered on the command line, so it is advisable to
//          test the file capture and file restore separately.
//          Otherwise the image captured will overwrite your
//          previously captured image.
//
// Compilation Instructions:
//      1) First, compile MCALLS.C into MCALLS.OBJ.
//
//      2) Compile SHOWVID.C with IBM C/2 version 1.1 using the following parms
//
//              cl /AL /G2s /Zp showvid.c mcalls.obj /link graphics
//
//         If you created combined libraries with graphics you will not
//         need to link graphics at this stage.
//
//         Do NOT select/use ANSI compatibility for your compiler.
//***********************************************************

#include <graph.h>
#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "mcalls.h"

//****************************
//***** Miscellaneous Actions
//****************************
// #define clear_video
#define multiwindows
#define centerwindow
// #define vga_reference
// #define graphics_protect
// #define video_protect
// #define stripes
// #define video_levels
// #define video_fade
// #define color256

//****************************
//***** File Actions
//****************************
// #define file_restore
#define file_capture


#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

extern BOOL in_video_fade;

//***********************************************************
// SPECIAL FUNCTIONS
//***********************************************************
void setBIOSbordercolor (void)
  {
  union REGS myregs;

  // just set the border color to  0
  myregs.x.ax = 0x1001;
  myregs.x.bx = 0x0000;

  int86 ( 0x10, &myregs, &myregs );
  }


static char blocktext[] = "";


void pink_stripe ( SHORT col )
  {
  SHORT row;

  _settextcolor ( 13 );

  for ( row=1; row<=25; row++ ) {
    _settextposition ( row, col );
    _outtext ( blocktext );
    }

  _settextcolor ( 7 );
  }



//***********************************************************
// Main Program
//***********************************************************
main ( int argc, char *argv[] )
{
  struct videoconfig vc;
  SHORT    xdiv;
  SHORT    ydiv;
  SHORT    col;
  SHORT    pscol;
  PBYTE    ColorTable;
  PBYTE    PicPtr;


  //*********************************************************
  // Setup - video, coordinates, etc
  //*********************************************************
  if ( !cp_loaded ( ) ) {
    printf ( "API not yet active, install API and try again" );
    return ( 0 );
    }

  _setvideomode ( _DEFAULTMODE );

  _getvideoconfig ( &vc );

  // now use BIOS to get the border right, Error in Microsoft code
  setBIOSbordercolor ( );

  if ( vc.numxpixels == 320 )
    xdiv = 2;
  else
    xdiv = 1;

  if ( vc.numypixels == 200 )
    ydiv = 2;
  else
    ydiv = 1;

  // check for text modes
  if ( ( vc.mode < 4 ) || ( vc.mode == 7 ) )
    xdiv = 0;

  if ( vc.mode < 2 )
    col = 16;
  else
    col = 31;

  cp_init ( );

  _settextposition ( 10, 10);
  _outtext ("Ensure your video source is ready.  Press any key to continue.");
  getch ( );
  _settextposition ( 10, 10);
  _outtext ("                                                              ");

  cpaud_Mute ( 0 );

  //*********************************************************
  // NO PARAMETERS - Video manipulation only
  //*********************************************************
  if ( argc == 1 )
    {
    // with no parms show the full screen
    cpvid_Reset ( );
    cpvid_TranspColor (0, 0);
    cpvid_Transparency (TRUE);
    cpvid_Freeze ( FALSE );

    getch ( );

    //************************************************************
    //************************************************************
#ifdef clear_video
    cpvid_Freeze ( TRUE );
    cpvid_Clear ( 0x0a, 240, 160, 160, 100, 100, 0, 100, 100, 0, 100);
    getch ();
    cpvid_Freeze ( FALSE );
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef multiwindows
    cpvid_SetCoordinates ( 32, 32, 656, 413, 40, 10, 160, 100 );
    cpvid_SetCoordinates ( 32, 32, 656, 413, 140, 260, 160, 100 );
    cpvid_SetCoordinates ( 32, 32, 656, 413, 440, 100, 160, 100 );
    cpvid_SetCoordinates ( 32, 32, 656, 413, 540, 360, 160, 100 );
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef centerwindow
    cpvid_SetCoordinates ( 32, 32, 656, 413, 240, 160, 160, 100 );
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef vga_reference
    if ( xdiv ) {
      _rectangle ( _GBORDER, 240/xdiv, 160/ydiv, 400/xdiv, 260/ydiv );
      _setcolor ( 0 );
      _rectangle ( _GFILLINTERIOR, 220/xdiv, 140/ydiv, 260/xdiv, 180/ydiv );
      _rectangle ( _GFILLINTERIOR, 380/xdiv, 240/ydiv, 420/xdiv, 280/ydiv );
      }
    else {
      _settextposition ( 11, col ); // (1 based)
      _outtext ( blocktext );
      }
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef graphics_protect
    cpvid_GraphicProtect ( 0x10, 240, 160, 160, 100 );
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef video_protect
    cpvid_VideoProtect ( 0x02, 240, 160, 160, 100 );
    cpvid_SetFullscreenCoord ( );
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef stripes
    for ( pscol=1; pscol<30; pscol+= 2 ) {
      pink_stripe ( pscol );
      }
    getch ();
    cpvid_TranspColor (0, 13);
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef video_levels
    cpvid_Brightness (FALSE, 0);
    getch ();
    cpvid_Brightness (FALSE, 50);
    getch ();
    cpvid_Brightness (FALSE, 75);
    getch ();

    cpvid_Contrast (FALSE, 0);
    getch ();
    cpvid_Contrast (FALSE, 100);
    getch ();
    cpvid_Contrast (FALSE, 50);
    getch ();

    cpvid_Color (FALSE, 0);
    getch ();
    cpvid_Color (FALSE, 100);
    getch ();
    cpvid_Color (FALSE, 50);
    getch ();

    cpvid_Tint (FALSE, 99);
    getch ();
    cpvid_Tint (FALSE, -99);
    getch ();
    cpvid_Tint (FALSE, 0);
    getch ();
#endif

#ifdef video_fade
    cpvid_Fade ( 0, 30 );
    while ( in_video_fade ) {}

    cpvid_Fade ( 100, 30 );
    while ( in_video_fade ) {}
    getch ( );
#endif

    //************************************************************
    //************************************************************
#ifdef color256
    // Display a 256 color image (solid tone in memory)
    PicPtr = (PBYTE)calloc ((USHORT)64000, 1);
    ColorTable = (PBYTE)calloc (256, 3);
    if (PicPtr && ColorTable)
       {
       /* Make color 0 a GRAY tone */
       ColorTable[0] = 50;
       ColorTable[1] = 25;
       ColorTable[2] = 25;

       /* Turn on video protect */
       cpvid_Show256Pic ((BYTE)0x0a, ColorTable, PicPtr, 64000L, (USHORT)320,
                         (USHORT)0, (USHORT)0, (USHORT)320, (USHORT)200);
       }

    if (ColorTable) free (ColorTable);
    if (PicPtr) free (PicPtr);

//    cpvid_Copy (0x20, 0, 0, 320, 200, 319, 199);

    cpvid_Freeze ( FALSE );
    getch ();
#endif
    }

  //*********************************************************
  // FILE NAME PARAMETER - Capture/restore video
  //*********************************************************
  else if ( argc == 2 )
    {
    // show a video file
    cpvid_Reset ( );
    cpvid_Freeze ( TRUE );
    cpvid_ClearAllBlack ( );
    cpvid_VideoOnly (TRUE);

    //************************************************************
    //************************************************************
#ifdef file_restore
    cpvid_RestoreFile ( argv[1] );
    getch ();
#endif


    //************************************************************
    //************************************************************
#ifdef file_capture
    // capture the whole screen to a video file
    cpvid_TranspColor (0, 0);
    cpvid_Transparency (TRUE);
    cpvid_Freeze ( FALSE );
    getch ( );
    cpvid_CaptureFile ( argv[1] );
#endif

    }
  else {
    printf ( "USAGE:  showvid mypic.vid" );
    }

  cpvid_VideoOnly (FALSE);
  cpvid_GraphicProtectAll (TRUE);
  _clearscreen ( _GCLEARSCREEN );

  cpaud_Mute ( 1 );

  cp_exit ( );

  return ( 0 );
}


