//***********************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// Program: ATTRIBUT.C
// Version: 1.0
//
// Purpose: Window Procs for Video and Audio Attributes
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_NLS

#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EXTERN  extern
#include "BROWDEFS.H"        // This Program's Definitions


//***********************************************************************
// Func: VideoAttributesProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: This is the window procedure for the Video Attributes Window.
//
//***********************************************************************

MRESULT EXPENTRY VideoAttributesProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
 {
  static char       temp [STRBUFLEN],
                    ctlstr [STRBUFLEN];
  static SHORT      bright, contrast, color, tint;   // static to retain values
  static PMVWNQRY   pmvwntempqry;                    // static to retain values
  SHORT             ctl_id,
                    event;

   switch ( msg )
      {
      //=================================================================
      case WM_INITDLG:    // dialog box init

         WinPostMsg( hwnd, WM_USER + 1, NULL, NULL );
            // Get control string from resource

         WinLoadString (hab, NULL, ID_FORMATPERCENT, STRBUFLEN, ctlstr);

            // Query current values so they can be restored on a Cancel

         pmvwntempqry = NULL;
         if ( MVWNCreateQry( &pmvwntempqry, MVWN_QRY_STATUS ) )
            {
            WinSendMsg (hwndVideoWindow, mvwnQueryMsg, NULL, (MPARAM)pmvwntempqry);

            bright   = pmvwntempqry->mvidst.usBrightness;
            contrast = pmvwntempqry->mvidst.usContrast;
            color    = pmvwntempqry->mvidst.usColor;
            tint     = pmvwntempqry->mvidst.sTint + 100;

            WinSendDlgItemMsg (hwnd, MVWN_CMD_BRIGHTNESS, SBM_SETSCROLLBAR,
                               MPFROMSHORT (bright), MPFROM2SHORT (0, 100));

            sprintf (temp, ctlstr, bright);
            WinSetDlgItemText (hwnd, ID_BRIGHTNESSTEXT, temp);
            WinSendDlgItemMsg (hwnd,MVWN_CMD_CONTRAST,   SBM_SETSCROLLBAR,
                               MPFROMSHORT (contrast), MPFROM2SHORT (0, 100));

            sprintf (temp, ctlstr, contrast);
            WinSetDlgItemText (hwnd, ID_CONTRASTTEXT, temp);
            WinSendDlgItemMsg (hwnd, MVWN_CMD_COLOR, SBM_SETSCROLLBAR,
                              MPFROMSHORT (color), MPFROM2SHORT (0,100));

            sprintf (temp, ctlstr, color);
            WinSetDlgItemText (hwnd, ID_COLORTEXT, temp);
            WinSendDlgItemMsg (hwnd, MVWN_CMD_TINT, SBM_SETSCROLLBAR,
                               MPFROMSHORT (tint), MPFROM2SHORT(0,200));

            sprintf (temp, ctlstr, tint-100);
            WinSetDlgItemText (hwnd, ID_TINTTEXT,temp);
            }

         break;

      case WM_ACTIVATE:
         if  ( mp1 )
              HandleBecomingActive();
         break;

      case WM_USER + 1:
         SetBoxPosition( hwndVideoAttributes, TOP );
         return( ( MRESULT ) 1 );

      //=================================================================
      case WM_HSCROLL:

            // One of the four scroll bars has been touched

         ctl_id = SHORT1FROMMP( mp1 );   // get scroll bar ID
         switch ( ctl_id )               // get current value
            {
            //-----------------------------------------------------------
            case MVWN_CMD_BRIGHTNESS:
               switch (event = SHORT2FROMMP(mp2))   // adjust new value
                  {
                  case SB_LINELEFT:
                     if (bright) bright--;
                     break;

                  case SB_LINERIGHT:
                     if (bright < 100) bright++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     bright = (event == SB_PAGEUP) ? bright - 10
                                                   : bright + 10;
                     if (bright < 0)
                        bright = 0;
                     else if (bright > 100)
                        bright = 100;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     bright = SHORT1FROMMP( mp2 );
                     break;
                  }

               WinSendDlgItemMsg( hwnd, MVWN_CMD_BRIGHTNESS, SBM_SETPOS,
                                  MPFROMSHORT( bright ), NULL );

               sprintf( temp, ctlstr, bright );
               WinSetDlgItemText( hwnd, ID_BRIGHTNESSTEXT, temp );
               MVWNCommand (MVWN_CMD_BRIGHTNESS, MPFROM2SHORT (bright, 0) ,NULL);
               break;

            //-----------------------------------------------------------
            case MVWN_CMD_CONTRAST:
               switch ( event = SHORT2FROMMP( mp2 ) )   // adjust new value
                  {
                  case SB_LINELEFT:
                     if  ( contrast )  contrast--;
                     break;

                  case SB_LINERIGHT:
                     if  ( contrast < 100 )  contrast++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     contrast = ( event == SB_PAGEUP ) ? contrast - 10
                                                       : contrast + 10;
                     if ( contrast < 0 )
                         contrast = 0;
                     else if ( contrast > 100 )
                         contrast = 100;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     contrast = SHORT1FROMMP( mp2 );
                     break;
                  }

               WinSendDlgItemMsg( hwnd, MVWN_CMD_CONTRAST, SBM_SETPOS,
                                  MPFROMSHORT( contrast ), NULL );

               sprintf( temp, ctlstr, contrast );
               WinSetDlgItemText( hwnd, ID_CONTRASTTEXT, temp );
               MVWNCommand( MVWN_CMD_CONTRAST, MPFROM2SHORT( contrast, 0 ) ,NULL );
               break;

            //-----------------------------------------------------------
            case MVWN_CMD_COLOR:
               switch ( event = SHORT2FROMMP( mp2 ) )   // adjust new value
                  {
                  case SB_LINELEFT:
                     if ( color )  color--;
                     break;

                  case SB_LINERIGHT:
                     if (color < 100) color++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     color = ( event == SB_PAGEUP ) ? color - 10 : color + 10;
                     if ( color < 0 )
                         color = 0;
                     else if ( color > 100 )
                         color = 100;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     color = SHORT1FROMMP( mp2 );
                     break;
                  }

               WinSendDlgItemMsg( hwnd, MVWN_CMD_COLOR, SBM_SETPOS,
                                  MPFROMSHORT( color ), NULL );

               sprintf( temp, ctlstr, color );
               WinSetDlgItemText( hwnd, ID_COLORTEXT, temp );
               MVWNCommand( MVWN_CMD_COLOR, MPFROM2SHORT( color, 0 ) ,NULL );
               break;

            //-----------------------------------------------------------
            case MVWN_CMD_TINT:
               switch ( event = SHORT2FROMMP( mp2 ) )   // adjust new value
                  {
                  case SB_LINELEFT:
                     if ( tint )  tint--;
                     break;

                  case SB_LINERIGHT:
                     if ( tint < 200 )  tint++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     tint = ( event == SB_PAGEUP ) ? tint - 10 : tint + 10;
                     if ( tint < 0 )
                         tint = 0;
                     else if ( tint > 200 )
                         tint = 200;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     tint = SHORT1FROMMP( mp2 );
                     break;
                  }

               WinSendDlgItemMsg( hwnd, MVWN_CMD_TINT, SBM_SETPOS,
                                  MPFROMSHORT( tint ), NULL );

               sprintf( temp, ctlstr, tint-100 );
               WinSetDlgItemText( hwnd, ID_TINTTEXT, temp );
               MVWNCommand (MVWN_CMD_TINT, MPFROM2SHORT( tint - 100, 0 ) ,NULL );
               break;
            }
         break;

      //=================================================================
      case WM_COMMAND:
         switch ( COMMANDMSG (&msg)->cmd )
            {
            case ID_CANCEL:
               MVWNCommand( MVWN_CMD_BRIGHTNESS, MPFROM2SHORT (pmvwntempqry->mvidst.usBrightness, 0), NULL );
               MVWNCommand( MVWN_CMD_CONTRAST,  MPFROM2SHORT (pmvwntempqry->mvidst.usContrast, 0), NULL );
               MVWNCommand( MVWN_CMD_COLOR, MPFROM2SHORT (pmvwntempqry->mvidst.usColor, 0), NULL );
               MVWNCommand( MVWN_CMD_TINT, MPFROM2SHORT (pmvwntempqry->mvidst.sTint, 0), NULL );

               // Fall through and dismiss the dialog

            case ID_SETATTRIBUTES:
               if ( pmvwntempqry )
                   MVWNDestroyQry( pmvwntempqry );

               WinDismissDlg( hwnd, TRUE );
               hwndVideoAttributes = NULL;
               WinSetWindowPos( hwndFrame, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER );
               break;
            }
         break;

      //=================================================================
      default:
         break;
      }

   return( WinDefDlgProc( hwnd, msg, mp1, mp2 ) );
   }


//***********************************************************************
// Func: AudioAttributesProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: This is the window procedure for the Audio attributes window.
//
//***********************************************************************

MRESULT EXPENTRY AudioAttributesProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   static char     temp[STRBUFLEN],
                   ctlstr[STRBUFLEN];
   static SHORT    volume, balance, bass, treble;  // static to retain values
   static PMVWNQRY pmvwntempqry;                   // static to retain values
   static BOOL     fMuted;
   SHORT           ctl_id,
                   event;

   switch ( msg )
      {
      //================================================================
      case WM_INITDLG:    // dialog box init
         WinPostMsg( hwnd, WM_USER + 1, NULL, NULL );

            // Get control string from resource

         WinLoadString (hab, NULL, ID_FORMATPERCENT, STRBUFLEN, ctlstr);

            // Query current values so they can be restored on a Cancel

         pmvwntempqry = NULL;
         if ( MVWNCreateQry( &pmvwntempqry, MVWN_QRY_STATUS ) )
            {
            WinSendMsg (hwndVideoWindow, mvwnQueryMsg, NULL, (MPARAM)pmvwntempqry);

            volume   = pmvwntempqry->maudst.usVolume;
            balance  = pmvwntempqry->maudst.sBalance + 100;
            bass     = pmvwntempqry->maudst.sBass + 100;
            treble   = pmvwntempqry->maudst.sTreble + 100;
            fMuted   = pmvwntempqry->maudst.fMuted;

            WinSendDlgItemMsg (hwnd, MVWN_CMD_VOLUME, SBM_SETSCROLLBAR,
                               MPFROMSHORT (volume), MPFROM2SHORT (0, 100));

            sprintf (temp, ctlstr, volume);
            WinSetDlgItemText (hwnd, ID_VOLUMETEXT, temp);
            WinSendDlgItemMsg (hwnd,MVWN_CMD_BALANCE,   SBM_SETSCROLLBAR,
                               MPFROMSHORT (balance), MPFROM2SHORT (0, 200));

            sprintf (temp, ctlstr, balance-100);
            WinSetDlgItemText (hwnd, ID_BALANCETEXT, temp);
            WinSendDlgItemMsg (hwnd, MVWN_CMD_BASS, SBM_SETSCROLLBAR,
                               MPFROMSHORT (bass), MPFROM2SHORT (0,200));

            sprintf (temp, ctlstr, bass-100);
            WinSetDlgItemText (hwnd, ID_BASSTEXT, temp);
            WinSendDlgItemMsg (hwnd, MVWN_CMD_TREBLE, SBM_SETSCROLLBAR,
                               MPFROMSHORT (treble), MPFROM2SHORT(0,200));

            sprintf (temp, ctlstr, treble-100);
            WinSetDlgItemText (hwnd, ID_TREBLETEXT,temp);
            WinSendDlgItemMsg (hwnd, MVWN_CMD_MUTE, BM_SETCHECK,
                               MPFROMSHORT (fMuted),NULL);
            }
         break;

      case WM_ACTIVATE:
         if  ( mp1 )
              HandleBecomingActive();
         break;

      case WM_USER + 1:
         SetBoxPosition( hwndAudioAttributes, BOTTOM );
         return( ( MRESULT ) 1 );

      //================================================================
      case WM_HSCROLL:

         // One of the four scroll bars has been touched

         ctl_id = SHORT1FROMMP (mp1);  // get scroll bar ID
         switch (ctl_id)               // get current value
            {
            //----------------------------------------------------------
            case MVWN_CMD_VOLUME:
               switch (event = SHORT2FROMMP(mp2))   // adjust new value
                  {
                  case SB_LINELEFT:
                     if (volume) volume --;
                     break;

                  case SB_LINERIGHT:
                     if (volume < 100) volume++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     volume = (event == SB_PAGEUP) ? volume - 10 : volume + 10;
                     if (volume < 0)
                        volume = 0;
                     else if (volume > 100)
                        volume = 100;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     volume = SHORT1FROMMP (mp2);
                     break;
                  }

               WinSendDlgItemMsg (hwnd, MVWN_CMD_VOLUME, SBM_SETPOS,
                                  MPFROMSHORT (volume), NULL);

               sprintf (temp, ctlstr, volume);
               WinSetDlgItemText (hwnd, ID_VOLUMETEXT, temp);
               MVWNCommand (MVWN_CMD_VOLUME, MPFROM2SHORT (volume, 0) ,NULL);
               break;

            //----------------------------------------------------------
            case MVWN_CMD_BALANCE:
               switch (event = SHORT2FROMMP(mp2))   // adjust new value
                  {
                  case SB_LINELEFT:
                     if (balance) balance--;
                     break;

                  case SB_LINERIGHT:
                     if (balance < 200) balance++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     balance = (event == SB_PAGEUP) ? balance - 10 : balance + 10;
                     if (balance < 0)
                        balance = 0;
                     else if (balance > 200)
                        balance = 200;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     balance = SHORT1FROMMP (mp2);
                     break;
                  }

               WinSendDlgItemMsg (hwnd, MVWN_CMD_BALANCE, SBM_SETPOS,
                                  MPFROMSHORT (balance), NULL);

               sprintf (temp, ctlstr, balance - 100);
               WinSetDlgItemText (hwnd, ID_BALANCETEXT, temp);
               MVWNCommand (MVWN_CMD_BALANCE, MPFROM2SHORT (balance - 100, 0) ,NULL);
               break;


            //----------------------------------------------------------
            case MVWN_CMD_BASS:
               switch (event = SHORT2FROMMP(mp2))   // adjust new value
                  {
                  case SB_LINELEFT:
                     if (bass) bass--;
                     break;

                  case SB_LINERIGHT:
                     if (bass < 200) bass++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     bass = (event == SB_PAGEUP) ? bass - 10 : bass + 10;
                     if (bass < 0)
                        bass = 0;
                     else if (bass > 200)
                        bass = 200;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     bass = SHORT1FROMMP (mp2);
                     break;
                  }

               WinSendDlgItemMsg (hwnd, MVWN_CMD_BASS, SBM_SETPOS,
                                  MPFROMSHORT (bass), NULL);

               sprintf (temp, ctlstr, bass - 100);
               WinSetDlgItemText (hwnd, ID_BASSTEXT, temp);
               MVWNCommand (MVWN_CMD_BASS, MPFROM2SHORT (bass - 100, 0) ,NULL);
               break;

            //----------------------------------------------------------
            case MVWN_CMD_TREBLE:
               switch (event = SHORT2FROMMP(mp2))   // adjust new value
                  {
                  case SB_LINELEFT:
                     if (treble) treble--;
                     break;

                  case SB_LINERIGHT:
                     if (treble < 200) treble++;
                     break;

                  case SB_PAGEUP:
                  case SB_PAGEDOWN:
                     treble = (event == SB_PAGEUP) ? treble - 10 : treble + 10;
                     if (treble < 0)
                        treble = 0;
                     else if (treble > 200)
                        treble = 200;
                     break;

                  case SB_SLIDERTRACK:
                  case SB_SLIDERPOSITION:
                     treble = SHORT1FROMMP (mp2);
                     break;
                  }

               WinSendDlgItemMsg (hwnd, MVWN_CMD_TREBLE, SBM_SETPOS,
                                  MPFROMSHORT (treble), NULL);

               sprintf (temp, ctlstr, treble - 100);
               WinSetDlgItemText (hwnd, ID_TREBLETEXT, temp);
               MVWNCommand (MVWN_CMD_TREBLE, MPFROM2SHORT (treble - 100, 0) ,NULL);
               break;
            }
         break;

      //=================================================================
      case WM_CONTROL:
         if ( fMuted ) fMuted = FALSE;
            else  fMuted = TRUE;
         WinSendDlgItemMsg (hwnd, MVWN_CMD_MUTE, BM_SETCHECK,
                            MPFROMSHORT( fMuted ), NULL );
         MVWNCommand( MVWN_CMD_MUTE, ( MPARAM ) fMuted , NULL );
         break;

      //=================================================================
      case WM_COMMAND:
         switch ( COMMANDMSG (&msg)->cmd )
            {
            case ID_CANCEL:
               MVWNCommand( MVWN_CMD_VOLUME, MPFROM2SHORT (pmvwntempqry->maudst.usVolume, 0) ,NULL);
               MVWNCommand( MVWN_CMD_BALANCE,  MPFROM2SHORT (pmvwntempqry->maudst.sBalance, 0) ,NULL);
               MVWNCommand( MVWN_CMD_BASS, MPFROM2SHORT (pmvwntempqry->maudst.sBass, 0) ,NULL);
               MVWNCommand( MVWN_CMD_TREBLE, MPFROM2SHORT (pmvwntempqry->maudst.sTreble, 0) ,NULL);
               MVWNCommand( MVWN_CMD_MUTE, MPFROM2SHORT (pmvwntempqry->maudst.fMuted, 0) ,NULL);

               // Fall through and dismiss the dialog

            case ID_SETATTRIBUTES:
               if ( pmvwntempqry )
                  MVWNDestroyQry( pmvwntempqry );

               WinDismissDlg( hwnd, TRUE );
               hwndAudioAttributes = NULL;
               WinSetWindowPos( hwndFrame, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER );
               break;
            }
         break;

      //=================================================================
      default:
         break;
      }

   return( WinDefDlgProc( hwnd, msg, mp1, mp2 ) );
   }

//***********************************************************************
// END OF FILE
//***********************************************************************
