//***********************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// Module:  BITMAP.C
// Version: 1.00
//
// Purpose:  Supply bitmap->clipboard facility to the Browser program.
//
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#define  INCL_GPI

#include <os2.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.

static HBITMAP CopyOfBitMap( HBITMAP );


//***********************************************************************
// Func: BitMapToClipboard
//
// Parm: SHORT    sStep;
//
// Desc: Performs 2 functions, as requested by Browser.  These are:
//
//       BMClp_START: Send a message to MVWN to capture the current bitmap.
//                    This must be performed before the bitmap can be
//                    sent to the clipboard.
//       BMClp_DONE : If the START was successful, and Browser received an
//                    ACK, BROWSER sends the DONE message to have the
//                    bitmap sent up to the Clipboard.  First, a copy of
//                    the current bitmap must be made (by CopyOfBitMap).
//
// Retn: TRUE:  Success
//       FALSE: Fail
//
//***********************************************************************

BOOL cdecl BitMapToClipboard( SHORT sStep )
{
 static PMVWNCMD pmvwncmd;
 static HBITMAP  hbmMVWN, hbmClip;
 BOOL            fOK = FALSE;

 switch (sStep)
    {
    case BMClp_START:  // Send a message to MVWN to capture the current bitmap.
       hbmMVWN = NULL;
       if ( MVWNCreateCmd( &pmvwncmd, MVWN_CMD_CAPTUREBITMAP ) )
          {
          pmvwncmd->data.mpCmdParam1 = MPFROMP( &hbmMVWN );
          fOK = WinPostMsg( hwndVideoWindow,
                            mvwnCommandMsg,
                            hwndMVWNResponse,
                            pmvwncmd);
          }
       break;

    case BMClp_DONE:  // Send a copy of the bitmap to the Clipboard
       if ( hbmMVWN )
          {
          // First, get a copy.  If this does NOT work DON'T send to Clipboard
          if ( hbmClip = CopyOfBitMap( hbmMVWN ) )
             {
             // Open the Clipboard, Empty it, Send the bitmap, close Clipboard.
             if ( WinOpenClipbrd( hab ) )
                {
                WinEmptyClipbrd( hab );
                fOK = WinSetClipbrdData( hab,
                                         ( ULONG ) hbmClip,
                                         CF_BITMAP,
                                         CFI_HANDLE );
                WinCloseClipbrd( hab );
                }
             }

          if ( MVWNCreateCmd( &pmvwncmd, MVWN_CMD_RELEASEBITMAP ) )
             {
              pmvwncmd->data.mpCmdParam1 = MPFROMP( &hbmMVWN );
              WinPostMsg( hwndVideoWindow,
                          mvwnCommandMsg,
                          hwndMVWNResponse,
                          pmvwncmd );
              }
          }
       break;
    }

 return( fOK );
 }


//***********************************************************************
// Func: CopyOfBitMap
//
// Parm: SHORT    sStep;
//
// Desc: Copies the current bitmap held in hbmSrc into the new (to be
//       sent to the clipboard) bitmap hbmDest.
//
// Retn: TRUE:  Success
//       FALSE: Fail
//
//***********************************************************************

static HBITMAP CopyOfBitMap( HBITMAP hbmSrc )
{
 HDC               hdcSrc, hdcDest;        // Temp Dev Contexts
 HPS               hpsSrc, hpsDest;        // Temp Presentation Spaces
 SIZEL             sizl;
 POINTL            bmarray[3];
 HBITMAP           hbmDest;                // New copy of Bit Map
 BITMAPINFOHEADER  bmp;                    // Bit Map Header structure

 hdcSrc  = DevOpenDC( hab, OD_MEMORY, "*", 0L, ( PDEVOPENDATA ) NULL, NULL );
 hdcDest = DevOpenDC( hab, OD_MEMORY, "*", 0L, ( PDEVOPENDATA ) NULL, NULL );

 // create both PS with size 0,0
 sizl.cx = sizl.cy = 0;

 hpsSrc = GpiCreatePS( hab, hdcSrc, &sizl,
                       PU_PELS | GPIF_DEFAULT | GPIT_MICRO | GPIA_ASSOC );

 hpsDest = GpiCreatePS( hab, hdcDest, &sizl,
                        PU_PELS | GPIF_DEFAULT | GPIT_MICRO | GPIA_ASSOC );

 // Query the size of the Source bitmap, and make the Dest bitmap the same
 GpiQueryBitmapParameters( hbmSrc, &bmp );
 hbmDest = GpiCreateBitmap( hpsDest,
                            ( PBITMAPINFOHEADER ) &bmp,
                            0L,
                            ( PBYTE ) NULL,
                            ( PBITMAPINFO ) NULL );

 // if the creation of the Dest bitmap was successful, perform the copy
 if ( hbmDest )
    {
    // Copy the ENTIRE bitmap
    GpiSetBitmap( hpsSrc, hbmSrc );
    GpiSetBitmap( hpsDest, hbmDest );

    bmarray[0].x = bmarray[0].y = 0;
    bmarray[1].x = bmp.cx;
    bmarray[1].y = bmp.cy;
    bmarray[2]   = bmarray[0];

    GpiBitBlt( hpsDest, hpsSrc, 3L, bmarray, ROP_SRCCOPY, BBO_IGNORE );
    }

 // Destroy all the temporary PS's & DC's
 GpiDestroyPS( hpsSrc );
 GpiDestroyPS( hpsDest );
 DevCloseDC( hdcSrc );
 DevCloseDC( hdcDest );

 // Return the Dest bitmap, since this will be sent to the Clipboard
 return( hbmDest );
 }

//***********************************************************************
// END OF FILE
//***********************************************************************
