/*****************************************************************************/
/*  MODULE NAME: BROWHELP.C                                                  */
/*                                                                           */
/*  DESCRIPTION: Help for Browser                                            */
/*****************************************************************************/

#define INCL_WIN
#define INCL_DOS
#define INCL_WINHELP                       /* Include IPF Header File       */
#include <OS2.H>                           /* PM Header File                */

#include "STRING.H"

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.
#include "BROWHELP.H"                    // Help Resource Symbolic Identifiers

static PFNWP pfnHelp;


                /***************************************/
                /* Required IPF Structure Declarations */
                /***************************************/

static HELPINIT hmiHelpData;                /* Help Initialization Structure */
static char HelpFileName[64] = "BROWSER.HLP";


/*****************************************************************************/
/* FUNCTION NAME: HelpInit()                                                 */
/*                                                                           */
/* DESCRIPTION: Set-Up for Browser Help                                      */
/*****************************************************************************/

VOID HelpInit()
{

                 /********************************/
                 /* IPF Initialization Structure */
                 /********************************/

   hmiHelpData.cb = sizeof( HELPINIT );       /* Size of Init. Structure     */
   hmiHelpData.ulReturnCode = NULL;           /* Store HM RetCode from Init. */
   hmiHelpData.pszTutorialName = NULL;        /* No Tutorial Program         */
                                              /* Help Table is Defined in    */
                                              /*      the RC file.           */
   hmiHelpData.phtHelpTable = ( PVOID ) ( 0xFFFF0000 | MAIN_HELPTABLE );
   hmiHelpData.hmodAccelActionBarModule = NULL;  /* Action Bar Not Tailored  */
   hmiHelpData.idAccelTable = NULL;
   hmiHelpData.idActionBar = NULL;
                                                 /* Help Window Title        */
   hmiHelpData.pszHelpWindowTitle = "Video Browser Help";
   hmiHelpData.hmodHelpTableModule = NULL;    /* Help Table in Not in a DLL  */
   hmiHelpData.usShowPanelId = NULL;          /* Help Panel ID Not Displayed */
                                              /* Library with Help Panels    */
   hmiHelpData.pszHelpLibraryName = HelpFileName;

   /********************************************/
   /* Create Instance of IPF ...               */
   /* Pass Anchor Block handle and address of  */
   /* IPF initialization structure, and check  */
   /* that creation was successful.            */
   /********************************************/

   hwndHelpInstance = WinCreateHelpInstance( hab, &hmiHelpData );
   if  ( !hwndHelpInstance )
        ShowMessage( HWND_DESKTOP, ID_HELPNOTAVAILABLE, ID_HELPCREATERROR );
   else
      { if  ( hmiHelpData.ulReturnCode )
           { ShowMessage( HWND_DESKTOP, ID_HELPTERMINATED, ID_HELPCREATERROR );
             WinDestroyHelpInstance( hwndHelpInstance );
             }
        }

  pfnHelp = WinSubclassWindow( hwndClient, MyHelpProc );


   /********************************************/
   /* Associate Instance of IPF.               */
   /* Pass handle to Help Window and handle to */
   /* frame window with which Help is to be    */
   /* associated.                              */
   /********************************************/

   if  ( hwndHelpInstance )
        WinAssociateHelpInstance( hwndHelpInstance, hwndFrame );

   return;
   }


/*****************************************************************************/
/* FUNCTION NAME: MyHelpProc()                                               */
/*                                                                           */
/* DESCRIPTION: Help Procedure                                               */
/*****************************************************************************/

MRESULT EXPENTRY MyHelpProc( HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2 )
{
  USHORT command;                            /* WM_COMMAND command value     */

  switch ( msg )
       {
         case WM_COMMAND:
           command = SHORT1FROMMP( mp1 );    /* Extract the command value    */
           switch ( command )
            {
                   /************************************************/
                   /* DISPLAY HELP_FOR_HELP                        */
                   /************************************************/

              case IDM_HELP_FOR_HELP:
                   if  ( hwndHelpInstance )
                        WinSendMsg( hwndHelpInstance, HM_DISPLAY_HELP,
                                    MPFROMSHORT( IDM_HELP_FOR_HELP ),
                                    MPFROMSHORT( HM_RESOURCEID ) );
                    break;

              default:
                    break;
              }
           break;

              /**************************************************/
              /*     QUERY_KEYS_HELP                            */
              /* If the user requests KEYS HELP from the help   */
              /* pull-down, IPF sends the HM_QUERY_             */
              /* KEYS_HELP message to the application, which    */
              /* should return the panel id of the keys_help    */
              /* panel, or a 0 to tell IPF to do nothing.       */
              /**************************************************/

         case HM_QUERY_KEYS_HELP:
              return( ( MRESULT ) KEYS_HELP_PANEL_ID );
              break;

              /**************************************************/
              /*     HM_ERROR                                   */
              /* If an error occurs using IPF, an HM_ERROR msg  */
              /* will be sent to the application.  At this point*/
              /* you can display a message box and destroy the  */
              /* help instance.  This code can be replaced to   */
              /* perform the error processing which you desire. */
              /**************************************************/

         case HM_ERROR:
              if  ( ( hwndHelpInstance && ( ULONG ) mp1 ) == HMERR_NO_MEMORY )
                 { ShowMessage( HWND_DESKTOP, ID_HELPTERMINATED, ID_HELPERROR );
                   WinDestroyHelpInstance( hwndHelpInstance );
                   }
              else
                   ShowMessage( HWND_DESKTOP, ID_HELPF1, ID_HELPERROR );
              break;

         default:
              break;
         }
 return( ( * pfnHelp ) ( hwnd, msg, mp1, mp2 ) );
 }
