//***********************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// Program: BROWSER
// Version: 1.0
//
// Purpose: Simple videodisc browser utility demonstrating programming
//          concepts of the Multimedia Toolkit for OS/2 PM
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_NLS

#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EXTERN
#include "BROWDEFS.H"                  // This Program's Definitions

//***********************************************************************
// Local Functions
//***********************************************************************

static  BOOL  InitBrowser( int argc, char *argv [] );
static  VOID  TerminateBrowser( VOID );

static  VOID  DisplayNoMMInit( VOID );
static  VOID  InitMyProgram( VOID );
static  VOID  CheckMenuItem( USHORT );
static  VOID  CreateVideodiscControls( VOID );
static  VOID  DisableAllFileItems( VOID );
static  VOID  DisableAllPlayerOptions( VOID );
static  VOID  DisableMenuItem( USHORT );
static  VOID  DoFullScreenVideo( VOID );
static  VOID  EnableAllPlayerOptions( VOID );
static  VOID  EnableMenuItem( USHORT);
static  VOID  HandleFileMenuInit( VOID );
static  VOID  HandleVideodiscMenuInit( VOID );
static  VOID  HandleVideoWindowMenuInit( VOID );
static  VOID  ProcessMinMaxFrame( MPARAM );
static  VOID  ResizeVideoControls( VOID );
static  VOID  SetIdealCoord( VOID );
static  VOID  SetSourceCoordinates( VOID );
static  VOID  UnCheckMenuItem( USHORT );


//***********************************************************************
// Local Variables
//***********************************************************************

static HPOINTER hptrVid[3]    = { NULL, NULL, NULL };  // System Pointers
static HPOINTER hptrActive[3] = { NULL, NULL, NULL };
static HPOINTER hptrBusy[3]   = { NULL, NULL, NULL };

static SWP   OldPosition;         // Old Position ( When in Full Screen Video )
static RECTL Rectangle;           // Frame Window Coordinates: Ideal Size Wind.
static SHORT IdealFrameHeight;    //                                 Height
static SHORT IdealFrameWidth;     //                                 Width
static SHORT FullScreenVideo = 0; // In Full Screen Video ? ( = 1, Yes )
static SHORT BitMapOn = 0;        // Bit Map Overlay in Use Flag
static PFNWP pfnVideoProc;        // Ptr to Original Video winproc
static PFNWP pfnFrameProc;        // Ptr to original Frame window winproc
static USHORT CurrentPlayer = 0;  // Current Videodisc Player ( 0, 1, or 2 )
static BOOL   fScaling = TRUE;
static BOOL Minimized = FALSE;    // Is Frame Minimized ?

static USHORT SaveSourceHorg[4];  // Default Video Source Coordinates
static USHORT SaveSourceVorg[4];
static USHORT SaveSourceHlen[4];
static USHORT SaveSourceVlen[4];


//***********************************************************************
// Func: main()
//
// Parm: SHORT    argc;
//       CHAR    *argv[];
//
// Desc: This is the main function that begins the BROWSER program.
//***********************************************************************

VOID cdecl main( int argc, char *argv [] )
{
  char szError[MAX_LEN_LOADSTRING];

  hab = WinInitialize( NULL );         // Initialize for PM
  hmq = WinCreateMsgQueue( hab, 30 );  // Create a message queue
                                       // NOTE: additional queue elements
                                       //       because we have 3 MVPCs

  if ( InitBrowser( argc, argv ) )     // Initialize Browser
     {
     while ( WinGetMsg( hab, ( PQMSG ) &qmsg, ( HWND ) NULL, 0, 0 ) )
            WinDispatchMsg( hab, ( PQMSG ) &qmsg );

     TerminateBrowser( ) ;
     }
  else if ( WinLoadString ( hab,
                            NULL,
                            ID_NOMMINIT,
                            MAX_LEN_LOADSTRING,
                            szError ) )
        {
        WinMessageBox ( HWND_DESKTOP,
                        WinQueryActiveWindow ( HWND_DESKTOP, FALSE ),
                        szError,
                        ( PSZ )NULL,
                        1,
                        MB_ICONHAND | MB_SYSTEMMODAL | MB_CANCEL ) ;
        }

  WinDestroyMsgQueue ( hmq ) ;
  WinTerminate ( hab ) ;
  }


//***********************************************************************
// Func: ClientWndProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: This is the main window procedure that coordinates the
//       Video Window and player controls, and responds to menu
//       item selections.
//
//***********************************************************************

MRESULT EXPENTRY ClientWndProc( HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2 )
{
 static CHAR szWindowText[80];
 static CHAR szBuffer[30];
 SWP         swp;
 SEL         selKeyData;
 LONG        lDataLength;
 LONG        Height, Width;
 SHORT       player,
             xpos,
             ypos;
 USHORT      usDlgReturnCode,
             usMenuCommand;
 PSWP        pswp;

 switch ( msg )
    {
    case WM_CREATE:

         hwndPopUpMenu = WinLoadMenu( hwnd, NULL, ID_POPUPMENU );
         WinSetWindowPos( hwndPopUpMenu, NULL, 0, 0, 0, 0, SWP_SIZE );
         break;

    case WM_SHOW :

         if ( hwndVideoAttributes )
             WinShowWindow( hwndVideoAttributes, ( BOOL )SHORT1FROMMP( mp1 ) );

         if ( hwndAudioAttributes )
             WinShowWindow( hwndAudioAttributes, ( BOOL )SHORT1FROMMP( mp1 ) );

         if ( hwndRestoreImage )
             WinShowWindow( hwndRestoreImage, ( BOOL )SHORT1FROMMP( mp1 ) );

         break;

    case WM_ACTIVATE:

         if  ( mp1 )
              HandleBecomingActive();
         else
              MVWNRefreshVideo( hwndVideoWindow );
         break;

    case WM_SIZE:

       WinQueryWindowPos( hwnd, &swp );       // Get Client Position

       if  ( hwndVideoAttributes )
          { WinPostMsg( hwndVideoAttributes, WM_USER + 1, NULL, NULL );
            WinShowWindow( hwndVideoAttributes, !Minimized );
            }

       if  ( hwndAudioAttributes )
          { WinPostMsg( hwndAudioAttributes, WM_USER + 1, NULL, NULL );
            WinShowWindow( hwndAudioAttributes, !Minimized );
            }

       if  ( hwndRestoreImage )
          { WinPostMsg( hwndRestoreImage, WM_USER + 1, NULL, NULL );
            WinShowWindow( hwndRestoreImage, !Minimized );
            }

// Resize Videodisc control and video window to share client area

       if  ( Minimized == TRUE )
          { WinSetWindowPos( hwndVideoWindow, HWND_TOP,
                             0, 0, swp.cx, swp.cy,
                             SWP_SIZE | SWP_MOVE | SWP_ZORDER | SWP_SHOW );
            }
       else
          { if  ( ( hwndMVPC ) && ( WinIsWindowVisible ( hwndMVPC ) ) )
               { WinSetWindowPos( hwndMVPC, HWND_TOP, 0, 0,
                                  swp.cx, scrollbarheight,
                                  SWP_SIZE | SWP_MOVE | SWP_ZORDER );

                 WinSetWindowPos( hwndVideoWindow, HWND_TOP,
                                  0, scrollbarheight, swp.cx,
                                  swp.cy - scrollbarheight,
                                  SWP_SIZE | SWP_MOVE | SWP_ZORDER );
                 }
            else
               { WinSetWindowPos( hwndVideoWindow, HWND_TOP,
                                  0, 0, swp.cx, swp.cy,
                                  SWP_SIZE | SWP_MOVE | SWP_ZORDER );
                 }
            }
       break;

     case WM_MINMAXFRAME:
     case WM_ADJUSTWINDOWPOS:
       pswp = ( PSWP ) PVOIDFROMMP( mp1 );
       if  ( pswp->fs & SWP_MINIMIZE )
            Minimized = TRUE;
       else
            Minimized = FALSE;
       break;

    //=================================================================

    case WM_BUTTON1DBLCLK:                      // Exit Full Screen Mode
       if  ( FullScreenVideo )
          {
          if ( fScaling )
             MVWNCommand( MVWN_CMD_SCALE, MPFROM2SHORT (TRUE, 0), NULL);

          if ( hwndMVPC )
             WinShowWindow( hwndMVPC, TRUE );

          WinSetWindowPos( hwndFrame, HWND_TOP, OldPosition.x,
                           OldPosition.y, OldPosition.cx,
                           OldPosition.cy, SWP_SIZE | SWP_MOVE | SWP_ZORDER );

          FullScreenVideo = 0 ;
          SetSourceCoordinates();
          }
       else
          DoFullScreenVideo( ) ;

       break;

    case WM_BUTTON2UP:                          // Display Pop-Up Menu
       xpos = SHORT1FROMMP( mp1 );
       ypos = SHORT2FROMMP( mp1 );
       WinSetWindowPos( hwndPopUpMenu, NULL, xpos, ypos,
                        0, 0, SWP_MOVE | SWP_SHOW );
       WinPostMsg( hwndPopUpMenu, MM_STARTMENUMODE,
                   MPFROM2SHORT( TRUE, TRUE ), NULL );
       break;

    case WM_INITMENU:                           // Initializating PopUp Menu

       hwndMenu = ( HWND ) HWNDFROMMP( mp2 );
       switch ( SHORT1FROMMP( mp1 ) )
          {
          case ID_FILE:
             HandleFileMenuInit( ) ;
             break;

          case ID_MVPC:
             HandleVideodiscMenuInit( ) ;
             break;

          case ID_MVWN:
             HandleVideoWindowMenuInit( ) ;
             break;
          }
       break;

    //=================================================================
    case WM_COMMAND:
       usMenuCommand = COMMANDMSG (&msg)->cmd;
       switch ( usMenuCommand )
          {
          //-----------------------------------------------------------
          case MI_SCROLLBAR:
               if  ( hwndMVPC )
                  { if  ( WinIsWindowVisible( hwndMVPC ) )
                        {
                         WinQueryWindowPos( hwndClient, &swp );
                         WinShowWindow( hwndMVPC, FALSE );
                         WinSetWindowPos( hwndVideoWindow, HWND_TOP,  // Set Video Window Position
                                          0, 0, swp.cx, swp.cy,
                                          SWP_SIZE | SWP_MOVE | SWP_ZORDER );
                         }
                    else
                       { WinQueryWindowPos( hwndClient, &swp );
                         WinSetWindowPos( hwndVideoWindow, HWND_TOP,  // Set Video Window Position
                                          0, scrollbarheight, swp.cx,
                                          swp.cy - scrollbarheight,
                                          SWP_SIZE | SWP_MOVE | SWP_ZORDER );
                         WinSetWindowPos( hwndMVPC, HWND_TOP, 0, 0,
                                          swp.cx, scrollbarheight,
                                          SWP_SIZE | SWP_MOVE | SWP_SHOW | SWP_ZORDER );
                         }
                    }
               break;

          case MI_VIDEOIMAGE:
               if ( !hwndRestoreImage )         // IF Window Not Already Active
                   hwndRestoreImage = WinLoadDlg( HWND_DESKTOP, hwnd,
                                                  ImageDlgProc, NULL,
                                                  DLG_LOADIMAGE, NULL );
               break;

          case MI_RANGEDIT:
               EditRange( hwnd, szProfileName );
               break;

          case MI_RANGESAVE:

                // User wants to save the current range
                //    so query from videodisc control what it is,
                //    then provide dialog box
                //    to get the keyword to save this range under

             hini = PrfOpenProfile( hab, szProfileName );
             if ( hini )
                {
                WinDlgBox( HWND_DESKTOP, hwnd, SaveRangeDlgProc,
                           NULL, DLG_SAVEFRAMERANGE, NULL );
                WinSetFocus( HWND_DESKTOP, hwndFrame );
                PrfCloseProfile( szProfileName );
                }
             break;

          //-----------------------------------------------------------
          case MI_NEWPROFILE:        /* allow user to enter new Profile file */
             usDlgReturnCode = WinDlgBox( HWND_DESKTOP, hwnd, ProfileDlgProc,
                                          NULL, DLG_NEWPROFILE, NULL );
             if  ( usDlgReturnCode == ID_OPENPROFILE )
                { WinLoadString( hab, NULL, ID_WINDOWTEXT,
                                 sizeof( szWindowText ), szWindowText );
                  if  ( !(hini = PrfOpenProfile( hab, szProfileName ) ) )
                     { ShowMessage( HWND_DESKTOP, ID_PROFILECREATIONERROR,
                                    ID_FILEERROR );
                       szProfileName[0] = 0x0;
                       }
                  else
                     { PrfCloseProfile( hini );
                       strcat( szWindowText, " -- " );
                       strcat( szWindowText, szShortProfileName );
                       }
                  WinSetWindowText( hwndFrame, szWindowText );
                  }
             WinSetFocus( HWND_DESKTOP, hwndFrame );
             break;

          //-----------------------------------------------------------
          case MI_OPENPROFILE:                   /* Open an existing profile */
             usDlgReturnCode = WinDlgBox( HWND_DESKTOP, hwnd,
                                          OpenProfileDlgProc,
                                          NULL, DLG_OPENPROFILE, NULL );
             if  ( usDlgReturnCode == ID_OPENPROFILE )
                { WinLoadString( hab, NULL, ID_WINDOWTEXT,
                                 sizeof( szWindowText ), szWindowText );
                  if  ( !( hini = PrfOpenProfile( hab, szProfileName ) ) )
                     { ShowMessage( HWND_DESKTOP, ID_PROFILEOPENERROR,
                                    ID_FILEERROR );
                       szProfileName[0] = 0x0;
                       }
                  else
                     { PrfCloseProfile( hini );
                       strcat( szWindowText, " -- ");
                       strcat( szWindowText, szShortProfileName );
                       }
                  WinSetWindowText( hwndFrame, szWindowText );
                  }
             WinSetFocus( HWND_DESKTOP, hwndFrame );
             break;

          //-----------------------------------------------------------
          case MI_RANGELOAD:   // load a frame range into the vdisc control
             if ( hini = PrfOpenProfile( hab, szProfileName ) )
                {
                if ( !DosAllocSeg( 1000, &selKeyData, 0 ) )
                   {
                   pKeyStart = pKeyName = ( PCHAR ) ( MAKEP( selKeyData, 0 ) );
                   lDataLength = 1000L;
                   if ( PrfQueryProfileData( hini, "Browser", NULL, pKeyName,
                                             ( PLONG ) &lDataLength ) )
                      {
                       WinDlgBox( HWND_DESKTOP, hwnd, LoadRangeDlgProc,
                                  NULL, DLG_LOADFRAMERANGE, NULL );
                       WinSetFocus( HWND_DESKTOP, hwndFrame );
                       }

                   DosFreeSeg( SELECTOROF( pKeyStart ) );
                   }

                PrfCloseProfile( hini );
                }
             break;

          //-----------------------------------------------------------
          case MI_FRAMERANGE:  // dialog box to set vdisc control frame range
             WinDlgBox( HWND_DESKTOP, hwnd, FrameRangeDlgProc,
                        NULL, DLG_SETFRAMERANGE, NULL );
             WinSetFocus( HWND_DESKTOP, hwndFrame );
             break;

          //-----------------------------------------------------------
                               // Set videodisc information display
          case MI_INFODISPLAY:
                               // NOTE: Query was done by Vdisc Menu Init
             if  ( pmvpcqry->fInfoDisplay )
                  MVPCCommand( MVPC_CMD_INFODISPLAY, NULL, NULL, NULL );
             else
                  MVPCCommand( MVPC_CMD_INFODISPLAY,
                               MPFROM2SHORT( 1, 0 ), NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_TIE:          // tie players and video sources together
             if ( fTiePlayersToSources )
                 fTiePlayersToSources = FALSE;
             else
                 fTiePlayersToSources = TRUE;
             break;

          //-----------------------------------------------------------
          case MI_QUERYFRAME:   /* query and display current frame */
             WinSendMsg( hwndMVPC, mvpcQueryMsg, NULL, pmvpcqry );
             sprintf( szBuffer, "Current frame = %lu ", pmvpcqry->lCurrentFrame );
             WinMessageBox( HWND_DESKTOP, hwnd, szBuffer, "", 0, MB_OK );
             break;

          //-----------------------------------------------------------
          case MI_QUERYRANGE:   /* query and display current frame */
             WinSendMsg( hwndMVPC, mvpcQueryMsg, NULL, pmvpcqry );
             sprintf( szBuffer, "Frame Range = %lu - %lu ", pmvpcqry->lLowerBound, pmvpcqry->lUpperBound);
             WinMessageBox( HWND_DESKTOP, hwnd, szBuffer, "", 0, MB_OK );
             break;

          //-----------------------------------------------------------
          case MI_SCALING_ON:   /* Video scaling on/off */
          case MI_SCALING_OFF:
             fScaling = (usMenuCommand == MI_SCALING_ON) ? TRUE : FALSE;
             MVWNCommand( MVWN_CMD_SCALE,
                          (usMenuCommand == MI_SCALING_ON) ? MPFROM2SHORT( TRUE, 0 )
                                                           : NULL,
                          NULL );
             break;

          //-----------------------------------------------------------
          case MI_SPEEDFAST:                              // Video Disc Speed
             MVPCCommand( MVPC_CMD_SPEED,
                          MPFROM2SHORT( 300, 0 ), NULL, NULL );
             break;

          case MI_SPEEDNORMAL:
             MVPCCommand( MVPC_CMD_SPEED,
                          MPFROM2SHORT( 100, 0 ), NULL, NULL );
             break;

          case MI_SPEEDSLOW:
             MVPCCommand( MVPC_CMD_SPEED,
                          MPFROM2SHORT( 50, 0 ), NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_AUDIO1:        /* videodisc audio */
          case MI_AUDIO2:
          case MI_BOTH:
             MVPCCommand( MVPC_CMD_AUDIO,
                          MPFROM2SHORT (usMenuCommand - MI_AUDIO1 + 1, 0),
                          NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_NOAUDIO:
             MVPCCommand( MVPC_CMD_AUDIO, NULL, NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_LOAD:          /* initialize videodisc player */
             MVPCCommand( MVPC_CMD_INIT, NULL, NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_UNLOAD:        /* unload videodisc player */
             MVPCCommand( MVPC_CMD_UNLOAD, NULL, NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_BLACKWHITE:     /* black white or color video mode */
          case MI_COLOR:
             MVWNCommand( MVWN_CMD_BLACKWHITE,
                          (usMenuCommand == MI_BLACKWHITE) ? MPFROM2SHORT (TRUE, 0)
                                                           : NULL,
                          NULL );
             break;

          //-----------------------------------------------------------
          case MI_FREEZE:
             MVWNCommand( MVWN_CMD_FREEZE, ( MPARAM ) TRUE, NULL );
             break;

          //-----------------------------------------------------------
          case MI_UNFREEZE:
             MVWNCommand( MVWN_CMD_FREEZE, ( MPARAM ) FALSE, NULL );
             break;

          //-----------------------------------------------------------
          case MI_INACTIVEBITMAP:
             MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                          MPFROM2SHORT( MVWN_INACTIVE_MONOBITMAP, 0 ),
                          NULL );
             break;

          //-----------------------------------------------------------
          case MI_INACTIVEIMAGE:
             MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                          MPFROM2SHORT( MVWN_INACTIVE_VIDEOIMAGE, 0 ),
                          NULL );
             break;

          //-----------------------------------------------------------
          case MI_INACTIVECOLOR:
             MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                          MPFROM2SHORT( MVWN_INACTIVE_COLORFILL, 0 ),
                          MPFROMLONG( CLR_BLACK ) );
             break;

          //-----------------------------------------------------------
          case MI_VS1:         /* Select video source */
          case MI_VS2:         /* Freezing video before setting new source */
          case MI_VS3:
             usVideoSource = usMenuCommand - MI_VS1 + 1;
             if  ( fTiePlayersToSources )
                  CurrentPlayer = usVideoSource - 1;
             MVWNCommand( MVWN_CMD_VIDEOSOURCE,
                          MPFROM2SHORT( usVideoSource, 0 ),  NULL );
             ResizeVideoControls();
             break;

          //-----------------------------------------------------------
          case MI_PROGRESSIVE:                       // Set Video Scan Modes
             MVWNCommand( MVWN_CMD_SCANMODE,
                          MPFROM2SHORT( MVWN_SCANA_DEFAULT,
                                        MVWN_SCANM_PROGRESSIVE ),
                          NULL );
             break;

          case MI_DOUBLING:
             MVWNCommand( MVWN_CMD_SCANMODE,
                          MPFROM2SHORT( MVWN_SCANA_DEFAULT,
                                        MVWN_SCANM_LINEDOUBLING ),
                          NULL );
             break;

          case MI_SINGLEFIELDSCAN:
             MVWNCommand( MVWN_CMD_SCANMODE,
                          MPFROM2SHORT( MVWN_SCANA_DEFAULT,
                                        MVWN_SCANM_SINGLEFIELD ),
                          NULL );
             break;

          //-----------------------------------------------------------
          case MI_BITMAP:
             BitMapToClipboard( BMClp_START );
             break;

          //-----------------------------------------------------------
          case MI_IDEALSIZE:
             if  ( fScaling )
                  MVWNCommand( MVWN_CMD_SCALE, MPFROM2SHORT( TRUE, 0 ), NULL );
             if  ( hwndMVPC )
                  WinShowWindow( hwndMVPC, TRUE );
             SetIdealCoord();
             break;

          //-----------------------------------------------------------
          case MI_4THSCREEN:
             FullScreenVideo = 0 ;
             SetSourceCoordinates();
             if ( fScaling )
                 MVWNCommand( MVWN_CMD_SCALE, MPFROM2SHORT (TRUE, 0), NULL);
             if  ( hwndMVPC )
                  WinShowWindow( hwndMVPC, TRUE );
             Height = WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN ) / 2L;
             Width  = WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN ) / 2L;
             WinSetWindowPos( hwndFrame, ( HWND ) NULL, 0, 0,
                              ( SHORT ) Width, ( SHORT ) Height,
                              SWP_SIZE | SWP_SHOW | SWP_RESTORE );
             break;

          //-----------------------------------------------------------
          case MI_FULLSCREEN:
             DoFullScreenVideo();
             break;

          //-----------------------------------------------------------
          case MI_PLAYER1:  /* select current videodisc player */
          case MI_PLAYER2:
          case MI_PLAYER3:
             CurrentPlayer = usMenuCommand - MI_PLAYER1;

             // Switch Players IF This is Different from Current Player

             if  ( hwndMVPC != hwndMVPCs[CurrentPlayer] )
                { if  ( fTiePlayersToSources )        // Switch Video Sources ?
                     { usVideoSource = CurrentPlayer + 1;
                       MVWNCommand( MVWN_CMD_VIDEOSOURCE,
                                    MPFROM2SHORT( usVideoSource, 0 ), NULL );
                       }
                  ResizeVideoControls();
                  }
             break;

          //-----------------------------------------------------------
          case MI_SETATTRIBUTES:
             if ( !hwndVideoAttributes ) // IF control window not already active
                 hwndVideoAttributes = WinLoadDlg( HWND_DESKTOP,
                                                   hwnd,
                                                   VideoAttributesProc,
                                                   NULL,
                                                   DLG_VIDEO,
                                                   NULL );
             else
                 SetBoxPosition( hwndVideoAttributes, TOP );
             break;

          //-----------------------------------------------------------
          case MI_SETAUDIO:
             if ( !hwndAudioAttributes ) // IF control window not already active
                 hwndAudioAttributes = WinLoadDlg( HWND_DESKTOP,
                                                   hwnd,
                                                   AudioAttributesProc,
                                                   NULL,
                                                   DLG_AUDIO,
                                                   NULL );
             else
                 SetBoxPosition( hwndAudioAttributes, BOTTOM );
             break;


          //-----------------------------------------------------------
          case MI_SHOWLOG:
             MVPCCommand( VSB_CMD_SHOWLOG,
                          MPFROM2SHORT( TRUE, 0 ), NULL, NULL );
             break;

          //-----------------------------------------------------------
          case MI_HIDELOG:
             MVPCCommand( VSB_CMD_SHOWLOG,
                          MPFROM2SHORT( FALSE, 0 ), NULL, NULL );
             break;
          }
       break;

    //=================================================================
    case WM_CLOSE:
       //**************************************************************
       //  CLOSE MESSAGE:
       //    Do:
       //      Explicit destroy of Videodisc controls(s) so they can
       //      do necessary clean up processing.
       //**************************************************************

       for ( player = 0; player != MAX_PLAYERS; player++ )
            if  ( hwndMVPCs [player] )
                 WinDestroyWindow( hwndMVPCs [player] );
       break;

    //=================================================================
    default:
       break;
    }

 return( WinDefWindowProc( hwnd, msg, mp1, mp2 ) );
 }


//**************************************************************************
// Func: HandleVideoWindowMenuInit
//
// Desc: This function initializes the VideoWindow menu.
//
//*****************************************************************************

static VOID HandleVideoWindowMenuInit ()
{
 int    i;
 RECTL  rcl;

  if ( hwndVideoWindow )
     {
     WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
     if ( pmvwnqry->usReturnCode == MVWN_ERR_NOERROR )
       {
       CheckMenuItem( (pmvwnqry->mvidst.fBlackWhite) ? MI_BLACKWHITE:MI_COLOR );
       UnCheckMenuItem( (pmvwnqry->mvidst.fBlackWhite) ? MI_COLOR:MI_BLACKWHITE );

       CheckMenuItem( (pmvwnqry->fScale) ? MI_SCALING_ON:MI_SCALING_OFF );
       UnCheckMenuItem( (pmvwnqry->fScale) ? MI_SCALING_OFF:MI_SCALING_ON );

       CheckMenuItem( ( pmvwnqry->mvidst.fFreeze) ? MI_FREEZE:MI_UNFREEZE );
       UnCheckMenuItem( ( pmvwnqry->mvidst.fFreeze) ? MI_UNFREEZE:MI_FREEZE );

       UnCheckMenuItem( MI_PROGRESSIVE );
       UnCheckMenuItem( MI_DOUBLING );
       UnCheckMenuItem( MI_SINGLEFIELDSCAN );

       switch ( pmvwnqry->mvidst.usScanMethod )
          {
          case MVWN_SCANM_PROGRESSIVE:
             CheckMenuItem( MI_PROGRESSIVE );
             break;

          case MVWN_SCANM_LINEDOUBLING:
             CheckMenuItem( MI_DOUBLING );
             break;

          case MVWN_SCANM_SINGLEFIELD:
             CheckMenuItem( MI_SINGLEFIELDSCAN );
             break;
          }

       UnCheckMenuItem( MI_INACTIVEBITMAP );
       UnCheckMenuItem( MI_INACTIVEIMAGE );
       UnCheckMenuItem( MI_INACTIVECOLOR );

       switch ( pmvwnqry->fsInactiveStyle )
         {
          case MVWN_INACTIVE_MONOBITMAP:
             CheckMenuItem( MI_INACTIVEBITMAP );
             break;

          case MVWN_INACTIVE_VIDEOIMAGE:
             CheckMenuItem( MI_INACTIVEIMAGE );
             break;

          case MVWN_INACTIVE_COLORFILL:
             CheckMenuItem( MI_INACTIVECOLOR );
             break;
          }
       }
    }
  for ( i = 1; i <= MAX_VIDEOSOURCES; i++ )
     {
     if ( MVWNVideoSourceType( i ) )   // if source exists
        {
        if ( i == usVideoSource )
           CheckMenuItem( i + MI_VS1 - 1 );
        else
           UnCheckMenuItem( i + MI_VS1 - 1 );
        }
     else                               // if source does NOT exist disable
        DisableMenuItem( i + MI_VS1 - 1 );
     }

  WinQueryWindowRect( hwndFrame, &rcl );           // Get Frame Window Coord.

  UnCheckMenuItem( MI_IDEALSIZE );
  if ( rcl.xRight == IdealFrameWidth )            // IF in Ideal Coordinates
    { if  ( hwndMVPC )
         { if  ( rcl.yTop == IdealFrameHeight )
                CheckMenuItem( MI_IDEALSIZE );
           }
      else
         { if  ( rcl.yTop == ( IdealFrameHeight - scrollbarheight ) )
                CheckMenuItem( MI_IDEALSIZE );
           }
      }

  if ( ( rcl.xRight == 320 ) &&                    // IF 1/4 Screen
       ( rcl.yTop == 240 ) )
     CheckMenuItem( MI_4THSCREEN );
  else
     UnCheckMenuItem( MI_4THSCREEN );

  if ( FullScreenVideo )                           // IF Full Screen Video
     CheckMenuItem( MI_FULLSCREEN );
  else
     UnCheckMenuItem( MI_FULLSCREEN );

  return;
  }

//**************************************************************************
// Func: HandleFileMenuInit
//
// Desc: This function initializes the File menu.
//       IF no Videodisc Control, then disable all file items.
//       Otherwise, enable/disable items based on whether a profile is open.
//*****************************************************************************

static VOID HandleFileMenuInit( )
{
 if ( ! hwndMVPC )
    DisableAllFileItems( ) ;
 else
    {
    EnableMenuItem( MI_NEWPROFILE );
    EnableMenuItem( MI_OPENPROFILE );

    if ( !szProfileName[0] )
       {
       DisableMenuItem( MI_RANGELOAD );
       DisableMenuItem( MI_RANGESAVE );
       DisableMenuItem( MI_RANGEDIT );
       }
    else
       {
       EnableMenuItem( MI_RANGELOAD );
       EnableMenuItem( MI_RANGESAVE );
       EnableMenuItem( MI_RANGEDIT );
       }
    }
 return;
 }


//**************************************************************************
// Func: HandleVideodiscMenuInit
//
// Desc: This function updates the selections on the videodisc control
//       menu.  If there is a videodisc control, it does a query to update
//       the fields.  If no controls, greys out the menu selections.
//
//*****************************************************************************

static VOID HandleVideodiscMenuInit()
{
 CHAR   szTempBuff[BUFFSIZE];
 SHORT  player, i;

 if ( hwndMVPC )
    {
    EnableAllPlayerOptions();

    WinLoadString( hab, NULL, ( fTiePlayersToSources ) ? ID_UNTIE : ID_TIE,
                   sizeof( szTempBuff ), szTempBuff );
    WinSendMsg( hwndMenu,
                MM_SETITEMTEXT,
                MPFROMSHORT( ( SHORT ) MI_TIE ),
                MPFROMP( ( PSZ ) szTempBuff ) );
    WinSendMsg( hwndPopUpMenu,
                MM_SETITEMTEXT,
                MPFROMSHORT( ( SHORT ) MI_TIE ),
                MPFROMP( ( PSZ ) szTempBuff ) );

    for ( player = 0; player != MAX_PLAYERS; player++ )
       {
       if ( !hwndMVPCs[player] )
          DisableMenuItem( player + MI_PLAYER1 );
       else
          {
          if ( hwndMVPC == hwndMVPCs[player] )
             {
             CheckMenuItem( player + MI_PLAYER1 );

// Get player status to check/uncheck various menu items

             pmvpcqry->usQuery = MVPC_QRY_STATUS;
             WinSendMsg( hwndMVPC, mvpcQueryMsg, NULL, pmvpcqry );

             if ( pmvpcqry->usReturnCode == MVPC_ERR_NOERROR )
                {
                for  ( i = 0; i <= BOTHCHANNELS; i++ )
                   {
                   if ( pmvpcqry->usAudio == i )   // Then check the menu item
                       CheckMenuItem( ( i ) ? i + MI_AUDIO1 - 1 : MI_NOAUDIO );
                   else                            // Uncheck the item
                       UnCheckMenuItem( ( i ) ? i + MI_AUDIO1 -1 : MI_NOAUDIO);
                   }

                WinLoadString( hab, NULL, ( pmvpcqry->fInfoDisplay ) ?
                               ID_FRAMEOFF : ID_FRAMEON,
                               sizeof( szTempBuff ), szTempBuff );

                WinSendMsg( hwndMenu, MM_SETITEMTEXT,
                            MPFROMSHORT( MI_INFODISPLAY ),
                            MPFROMP( ( PSZ ) szTempBuff ) );

                WinSendMsg( hwndPopUpMenu, MM_SETITEMTEXT,
                            MPFROMSHORT( MI_INFODISPLAY ),
                            MPFROMP( ( PSZ ) szTempBuff ) );

                UnCheckMenuItem( MI_SPEEDSLOW );
                UnCheckMenuItem( MI_SPEEDNORMAL );
                UnCheckMenuItem( MI_SPEEDFAST );

                switch ( pmvpcqry->usSpeed )
                   {
                   case 50:
                      CheckMenuItem( MI_SPEEDSLOW );
                      break;

                   case 100:
                      CheckMenuItem( MI_SPEEDNORMAL );
                      break;

                   case 300:
                      CheckMenuItem( MI_SPEEDFAST );
                      break;
                   }
                }
             }
          else
             UnCheckMenuItem( player + MI_PLAYER1 );
          }
       }
    }
 else                           // IF no videodisc control, disable all options
    DisableAllPlayerOptions( ) ;

 if ( ! FullScreenVideo )                  // Fix Up Hide/Show Scrollbar Option
    {
    WinLoadString( hab, NULL, ID_HIDE,
                   sizeof( szTempBuff ), szTempBuff );
    DisableMenuItem( MI_SCROLLBAR );
    }
 else
    {
    i = ( WinIsWindowVisible( hwndMVPC ) )  ?  ID_HIDE : ID_SHOW ;
    WinLoadString( hab, NULL, i, sizeof( szTempBuff ), szTempBuff );
    EnableMenuItem( MI_SCROLLBAR );
    }

 WinSendMsg( hwndPopUpMenu, MM_SETITEMTEXT,
             MPFROMSHORT( MI_SCROLLBAR ),
             MPFROMP( ( PSZ ) szTempBuff ) );

 return;
 }

//***********************************************************************
// Func: SetIdealCoord()
//
// Desc: Set Coordinates for Ideal Video Window Size
//***********************************************************************

static VOID SetIdealCoord( VOID )
{
 SWP  swp;

 FullScreenVideo = 0 ;
 SetSourceCoordinates();
 if  ( hwndMVPC )                                 // IF We Have Scroll Bars ...
    { WinSetWindowPos( hwndFrame,  ( HWND ) NULL, // Set Ideal FrameWindow Size
                       ( SHORT ) Rectangle.xLeft,
                       ( SHORT ) Rectangle.yBottom,
                       IdealFrameWidth,  IdealFrameHeight,
                       SWP_SIZE | SWP_SHOW | SWP_RESTORE );

      WinQueryWindowPos( hwndClient, &swp );       // Query Client Window Size
      WinSetWindowPos( hwndMVPC,  HWND_TOP, 0, 0,  // Set Scroll Bar Position
                       swp.cx, scrollbarheight,
                       SWP_SIZE | SWP_MOVE | SWP_ZORDER | SWP_SHOW );
      WinSetWindowPos( hwndVideoWindow, HWND_TOP,  // Set Video Window Position
                       0, scrollbarheight, swp.cx,
                       swp.cy - scrollbarheight,
                       SWP_SIZE | SWP_MOVE | SWP_ZORDER );
      }
 else                                             // ELSE No Scroll Bars ...
    { WinSetWindowPos( hwndFrame, ( HWND ) NULL,  // Set Ideal FrameWindow Size
                       ( SHORT ) Rectangle.xLeft,
                       ( SHORT ) Rectangle.yBottom,
                       IdealFrameWidth,
                       IdealFrameHeight - scrollbarheight,
                       SWP_SIZE | SWP_SHOW | SWP_RESTORE );
      WinQueryWindowPos( hwndClient, &swp );       // Query Client Window Size
      WinSetWindowPos( hwndVideoWindow, HWND_TOP,  // Set Video Window Position
                       0, 0, swp.cx, swp.cy,
                       SWP_SIZE | SWP_MOVE | SWP_ZORDER );
      }
 return;
 }

//***********************************************************************
// Func: ResizeVideoControls()
//
// Desc: ReSize VideoDisc Controls When Player or Video Source Changes
//***********************************************************************

static VOID ResizeVideoControls()
{
 SWP  swp;

 if  ( SaveSourceHlen[ usVideoSource ] == 0 )     // IF Source Defaults Not Set
    { WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
      SaveSourceHorg[ usVideoSource ] = pmvwnqry->usSourceHorg;
      SaveSourceVorg[ usVideoSource ] = pmvwnqry->usSourceVorg;
      SaveSourceHlen[ usVideoSource ] = pmvwnqry->usSourceHlen;
      SaveSourceVlen[ usVideoSource ] = pmvwnqry->usSourceVlen;
      }
 SetSourceCoordinates();

 WinQueryWindowPos( hwndFrame, &swp );               // Get Frame Size
 if  ( hwndMVPC )
      WinShowWindow( hwndMVPC, FALSE );
 else
      swp.cy += scrollbarheight;                     // Adjust for Ideal Check

 hwndMVPC = hwndMVPCs[CurrentPlayer];

 if  ( ( swp.cx == IdealFrameWidth ) &&              // IF in Ideal Coordinates
       ( swp.cy == IdealFrameHeight ) )              //    Keep it That Way !
      SetIdealCoord();
 else
    { WinQueryWindowPos( hwndClient, &swp );
      if  ( ( hwndMVPC ) &&
            ( !FullScreenVideo ) )
         { WinSetWindowPos( hwndMVPC, HWND_TOP, 0, 0,
                            swp.cx, scrollbarheight,
                            SWP_SIZE | SWP_MOVE | SWP_SHOW | SWP_ZORDER );
           WinSetWindowPos( hwndVideoWindow, HWND_TOP,
                            0, scrollbarheight, swp.cx,
                            swp.cy - scrollbarheight,
                            SWP_SIZE | SWP_MOVE | SWP_ZORDER );
           }
      else
           WinSetWindowPos( hwndVideoWindow, HWND_TOP,
                            0, 0, swp.cx, swp.cy,
                            SWP_SIZE | SWP_MOVE | SWP_ZORDER );
      }
 return;
 }


//***********************************************************************
// Func: DoFullScreenVideo()
//
// Desc: Snap to Full Screen Video ( No Scroll Bar )
//***********************************************************************

static VOID DoFullScreenVideo()
{
   RECTL Rect;

   FullScreenVideo = 1;
   SetSourceCoordinates();

   if ( fScaling )
      MVWNCommand( MVWN_CMD_SCALE, MPFROM2SHORT ( FALSE, 0 ), NULL );

   WinQueryWindowPos( hwndFrame, &OldPosition );

   if ( hwndMVPC )
      WinShowWindow( hwndMVPC, FALSE );

   Rect.xLeft   = 0L;
   Rect.yBottom = 0L;
   Rect.xRight = WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN ) - 1L;
   Rect.yTop   = WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN ) - 1L;
   WinCalcFrameRect( hwndFrame, &Rect, FALSE );
   WinSetWindowPos( hwndFrame, HWND_TOP,
                   ( SHORT )( Rect.xLeft ),
                   ( SHORT )( Rect.yBottom ),
                   ( SHORT )( Rect.xRight - Rect.xLeft + 1L ),
                   ( SHORT )( Rect.yTop - Rect.yBottom + 1L ),
                   SWP_SIZE | SWP_MOVE | SWP_ZORDER );


   return;
   }


//***********************************************************************
// Func: SetSourceCoordinates()
//
// Desc: Video Window generic command function.  All commands
//***********************************************************************

static VOID SetSourceCoordinates()
{
 PMVWNCMD  pmvwncmd;
 USHORT    SourceType;
 BOOL      Pal = FALSE;

 if  ( hwndVideoWindow == NULL )
      return;

 if  ( MVWNCreateCmd( &pmvwncmd, MVWN_CMD_SOURCEDIM ) )
    { pmvwncmd->data.mpCmdParam1 = ( MPARAM ) SaveSourceHorg[ usVideoSource ];
      pmvwncmd->data.mpCmdParam3 = ( MPARAM ) SaveSourceHlen[ usVideoSource ];

      SourceType = MVWNVideoSourceType( usVideoSource );
      if  ( ( SourceType == MVWN_VST_PAL ) ||
            ( SourceType == MVWN_VST_SPAL ) )
           Pal = TRUE;

      if  ( ( FullScreenVideo )  &&  ( Pal == FALSE ) )
         { pmvwncmd->data.mpCmdParam2 = ( MPARAM ) 1;
           pmvwncmd->data.mpCmdParam4 = ( MPARAM ) 480;
           }
      else
         { pmvwncmd->data.mpCmdParam2 = ( MPARAM ) SaveSourceVorg[ usVideoSource ];
           pmvwncmd->data.mpCmdParam4 = ( MPARAM ) SaveSourceVlen[ usVideoSource ];
           }

      WinSendMsg( hwndVideoWindow, mvwnCommandMsg, NULL, pmvwncmd );
      MVWNDestroyCmd( pmvwncmd, hwndVideoWindow, MVWN_ACK );
      }

 return;
 }


//***********************************************************************
// Func: CreateVideodiscControls
//
// Desc: Creates the videodisc player controls.
//
//***********************************************************************

static VOID CreateVideodiscControls( VOID )
{
 LONG     lPlayerBits[MAX_PLAYERS];
 CHAR     szPlayerName[9];
 SHORT    player;
 PMVPCCTL pmvpcctl;

 MVPCCreateCtl( &pmvpcctl );                     // create a MVPC create struct

 pmvpcctl->hwndAdviseActive = hwndMVPCAdvise;    // initialize it
 pmvpcctl->hwndAdviseSeek   = hwndMVPCAdvise;
 pmvpcctl->hwndAdvisePlayerError = hwndMVPCAdvise;

 lPlayerBits [0] = WS_VDP1;
 lPlayerBits [1] = WS_VDP2;
 lPlayerBits [2] = WS_VDP3;

 for ( player = MAX_PLAYERS - 1; player >= 0; player-- )
    {
    if ( MVPCPlayerExists( player+1, szPlayerName ) )
       {
       hwndMVPCs[player] = WinCreateWindow(
                                  hwndClient,       // Create on client
                                  WC_MVPC,
                                  NULL,             // No window text
                                  lPlayerBits [player],
                                  0, 0,             // at bottom of window
                                  0, 0,             // Position set upon return
                                  hwndClient,
                                  HWND_TOP,
                                  0,
                                  pmvpcctl,         // MVPC create struct
                                  NULL );


       hwndMVPC = hwndMVPCs[player];

          // This is a special VSB user interface command that
          //    could be implemented by an custom user interface.
          //    VSB_CMD_SETPOINTERS sets an active and busy pointer,
          //    that will be used by the VSB DLL to set the pointer
          //    to reflect the current state (available or busy) of the MVPC.

       hptrActive[player] = WinLoadPointer( HWND_DESKTOP,
                                            ( HMODULE ) NULL,
                                            ID_ACTIVE1 + ( player * 10 ) );

       hptrBusy[player] = WinLoadPointer( HWND_DESKTOP,
                                          ( HMODULE ) NULL,
                                          ID_BUSY1 + ( player * 10 ) );

       MVPCCommand( VSB_CMD_SETPOINTERS,
                    hptrActive[player],
                    hptrBusy[player],  NULL );

       MVPCCommand( MVPC_CMD_INIT, NULL, NULL, NULL );    // INIT/LOAD command

       ClearTryCount( hwndMVPC );       // Clear Player Recovery Retry Attempts
       }
    }

 hwndMVPC = hwndMVPCs[0];
 MVPCDestroyCtl( pmvpcctl );
 }


//**************************************************************************
// Func: EnableMenuItem
//
// Desc: Enables a menu item
//
//*****************************************************************************

static VOID EnableMenuItem ( USHORT item )
{
 WinSendMsg( hwndMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_DISABLED, 0 ) );
 WinSendMsg( hwndPopUpMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_DISABLED, 0 ) );
 return;
 }


//**************************************************************************
// Func: DisableMenuItem
//
// Desc: Disables a menu option
//
//*****************************************************************************

static VOID DisableMenuItem ( USHORT item )
{
 WinSendMsg( hwndMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_DISABLED, MIA_DISABLED ) );
 WinSendMsg( hwndPopUpMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_DISABLED, MIA_DISABLED ) );
 return;
 }


//**************************************************************************
// Func: CheckMenuItem
//
// Desc: Places a check mark next to a menu item
//
//*****************************************************************************

static VOID CheckMenuItem ( USHORT item )
{
 WinSendMsg( hwndMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_CHECKED, MIA_CHECKED ) );
 WinSendMsg( hwndPopUpMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_CHECKED, MIA_CHECKED ) );
 return;
 }


//**************************************************************************
// Func: UnCheckMenuItem
//
// Desc: Clears any check mark next to a menu item
//
//*****************************************************************************

static VOID UnCheckMenuItem( USHORT item )
{
 WinSendMsg( hwndMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_CHECKED, FALSE ) );
 WinSendMsg( hwndPopUpMenu,
             MM_SETITEMATTR,
             MPFROM2SHORT( item, TRUE ),
             MPFROM2SHORT( MIA_CHECKED, FALSE ) );
 return;
 }


//**************************************************************************
// Func: EnableAllPlayerOptions
//
// Desc: Enables all options on the videodisc control menu
//
//*****************************************************************************

static VOID EnableAllPlayerOptions()
{
 SHORT i;

 for  ( i = FIRST_VCONTROLITEM; i <= LAST_VCONTROLITEM; i++ )
       EnableMenuItem( i );
 return;
 }


//**************************************************************************
// Func: DisableAllPlayerOptions
//
// Desc: Disables all options on the videodisc control menu
//
//*****************************************************************************

static VOID DisableAllPlayerOptions()
{
 SHORT i;
 BOOL HavePlayer = FALSE;

 for ( i = FIRST_VCONTROLITEM; i <= LAST_VCONTROLITEM; i++ )
    {
    if  ( ( i != MI_PLAYER1 )         // Always leave these items available.
       && ( i != MI_PLAYER2 )
       && ( i != MI_PLAYER3 )
       && ( i != MI_PLAYER  )
       && ( i != MI_TIE  ) )
       { UnCheckMenuItem( i );
         DisableMenuItem( i );
         }
    }

 for   ( i = 0; i <= 2; i++ )        // UnCheck Players Which Don't Exist
     {  if  ( hwndMVPCs[i] == NULL )
           { UnCheckMenuItem( MI_PLAYER1 + i );
             DisableMenuItem( MI_PLAYER1 + i );
             }
        else
             HavePlayer = TRUE;
        }

 if  ( HavePlayer == FALSE )         // IF There are No Players
   { UnCheckMenuItem( MI_TIE );      //    Disable Player Option and
     DisableMenuItem( MI_TIE );      //    Tie Players Option
     UnCheckMenuItem( MI_PLAYER );
     DisableMenuItem( MI_PLAYER );
     }

 return;
 }


//**************************************************************************
// Func: DisableAllFileItems
//
// Desc: Disables all options on the File menu
//
//*****************************************************************************

static VOID DisableAllFileItems()
{
 SHORT i;

 for  ( i = FIRST_FILEITEM;  i <= LAST_FILEITEM;  i++ )
       DisableMenuItem( i );
 return;
 }


//**************************************************************************
// Func: ProcessMinMaxFrame
//
// Desc: If we are being minimized, Then hide the videodisc control.
//       Otherwise, Show the videodisc control.
//*****************************************************************************

static VOID ProcessMinMaxFrame( MPARAM mp1 )
{
 PSWP   pswp;
 USHORT fs;

 if  ( hwndMVPC )
    { pswp = PVOIDFROMMP( mp1 );
      fs = pswp->fs;
      if  ( fs & SWP_MINIMIZE )
           WinShowWindow( hwndMVPC, FALSE );
      if  ( ( fs & SWP_RESTORE ) || ( fs & SWP_MAXIMIZE ) )
           WinShowWindow( hwndMVPC, TRUE );
      }
 return;
 }




//***********************************************************************
// Func: InitBrowser
//
// Desc: Creates the Video Window and the Videodisc Player controls,
//       and performs other initializations.
//
//***********************************************************************

static BOOL InitBrowser( int argc, char *argv [] )
{
 char     szUIDLLName[MAX_LEN_UIDLLNAME];

 ULONG    fcdata =   FCF_MENU
                   | FCF_TASKLIST
                   | FCF_TITLEBAR
                   | FCF_ACCELTABLE
                   | FCF_SIZEBORDER
                   | FCF_SHELLPOSITION
                   | FCF_MINMAX
                   | FCF_SYSMENU
                   | FCF_DBE_APPSTAT ;

 CHAR     szTempBuffer[50];             // Temporary WinLoadString buffer
 HPOINTER hptrWait;
 PMVWNCTL pmvwnctl;

       // Initialize Global Variables

 hwndRestoreImage     = NULL;
 hwndVideoAttributes  = NULL;
 hwndAudioAttributes  = NULL;
 hwndMVPC             = NULL;
 hwndMVPCs[0]         = NULL;
 hwndMVPCs[1]         = NULL;
 hwndMVPCs[2]         = NULL;
 fTiePlayersToSources = TRUE;
 usVideoSource        = 1;              // Current Video Source = 1
 fromFrame            = 1L;             // Set Current Frame Range
 toFrame              = 54000L;


//***********************************************************************
   // If Command Line User Inerface DLL, Use it
   // Else Use default in resource file (VSB)
//***********************************************************************

   if ( ( argc >= 2 ) && ( argv[ 1 ][ 0 ] != OPTION_DELIMITER ) )
      {
      strcpy ( szUIDLLName, argv[ 1 ] ) ;
      }
   else
      {
      WinLoadString (hab, NULL, ID_DEFAULTDLL,
                     MAX_LEN_UIDLLNAME, szUIDLLName);
      }

 //------------------------------------------------------------------
 // MVWNInitialize:
 //
 //    Here we pass the message atoms that we desire the MVWN to use to
 //    notify us of ACK, RESPONSE, and ADVISE messages.
 //    We also receive the message atoms that MVWN wishes us to use to
 //    communicate COMMANDS and QUERIES to it with.
 //------------------------------------------------------------------

 //---------------------------------------------------------------
 // MVPCInitialize:
 //
 //    Here we pass the message atoms that we desire the MVPC to
 //    use to notify us of ACK, RESPONSE, and ADVISE messages.
 //
 //    We also receive the message atoms that MVPC wishes us to
 //    use to communicate COMMANDS and QUERIES to it with.
 //
 //---------------------------------------------------------------

 if ( MVWNInitialize ( hab, &mvwnCommandMsg,
                            &mvwnQueryMsg,
                            MVWN_ACK,
                            MVWN_RESPONSE,
                            MVWN_ADVISE ) &&
      MVPCInitialize ( hab, &mvpcCommandMsg,
                            &mvpcQueryMsg,
                            MVPC_ACK,
                            MVPC_RESPONSE,
                            MVPC_ADVISE,
                            szUIDLLName ) )
    {

// Set the system supplied wait pointer

    hptrWait = WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE ) ;
    WinSetPointer( HWND_DESKTOP, hptrWait ) ;

// Get height of the videodisc user interface.
// For our purposes, we'll just the height of a horizontal scrollbar


    scrollbarheight = ( SHORT )WinQuerySysValue( HWND_DESKTOP, SV_CYHSCROLL );

// Create frame/client

    WinLoadString (hab, NULL, ID_CLIENTCLASS, sizeof szTempBuffer, szTempBuffer);
    WinRegisterClass( hab, szTempBuffer, ClientWndProc, 0L, 0 );

    hwndFrame = WinCreateStdWindow( HWND_DESKTOP,
                                    0L,             // Invisible till we're done
                                    &fcdata,
                                    szTempBuffer,
                                    NULL,            // Title bar text
                                    WS_CLIPSIBLINGS | WS_VISIBLE, // prevent overpainting
                                    NULL,            // Resources in EXE
                                    ID_MAINWND,      // ID of resources
                                    &hwndClient );   // Handle

// Now create the video window, passing in the CtlData field the
//    address of this control field

    MVWNCreateCtl( &pmvwnctl );

// Now check which of the three possible video sources are defined,
//    and load active pointers for those that exist, and set the
//    MVWNCTL pointer fields accordingly

    if ( MVWNVideoSourceType( 1 ) )
      {
      hptrVid[ 0 ] = WinLoadPointer( HWND_DESKTOP,
                                     ( HMODULE )NULL,
                                     ID_VIDEO1 );
      pmvwnctl->hptrSrc1 = hptrVid[ 0 ] ;
      }

    if ( MVWNVideoSourceType( 2 ) )
      {
      hptrVid[ 1 ] = WinLoadPointer( HWND_DESKTOP,
                                     ( HMODULE )NULL,
                                     ID_VIDEO2 ) ;
      pmvwnctl->hptrSrc2 = hptrVid[ 1 ] ;
      }

    if ( MVWNVideoSourceType( 3 ) )
      {
      hptrVid[ 2 ] = WinLoadPointer( HWND_DESKTOP,
                                     ( HMODULE )NULL,
                                     ID_VIDEO3 ) ;
      pmvwnctl->hptrSrc3 = hptrVid[ 2 ] ;
      }

// Load String, Register Class, Create Video Window Response Window

    WinLoadString( hab,
                   NULL,
                   ID_MVWNRESPONSECLASS,
                   sizeof( szTempBuffer ),
                   szTempBuffer ) ;

    WinRegisterClass( hab, szTempBuffer, MVWNResponseWndProc, 0L, 0 );
    hwndMVWNResponse = WinCreateWindow( hwndFrame,
                                        szTempBuffer,
                                        NULL,
                                        0L,
                                        0, 0, 0, 0,
                                        hwndFrame,
                                        HWND_TOP,
                                        0, NULL, NULL );

    pmvwnctl->fUserActivate = FALSE;          // I'll do all specific activations
    pmvwnctl->hwndAdviseActive = hwndMVWNResponse;     // Set up Advise window

// Get client rectangle in order to size video window
//     which will be created as a child of the client window

    hwndVideoWindow = WinCreateWindow( hwndClient,     // Create on client
                                       WC_MVWN,        // Video window class
                                       NULL,           // No window text
                                       WS_VISIBLE,
                                       0, 0, 0, 0,     // At bottom of window
                                       hwndClient,
                                       HWND_TOP,
                                       0,
                                       pmvwnctl,       // CtlData
                                       NULL );

       // Free the MVWN control

    MVWNDestroyCtl( pmvwnctl );

    WinLoadString( hab,
                   NULL,
                   ID_MVPCRESPONSECLASS,
                   sizeof( szTempBuffer ),
                   szTempBuffer );

    WinRegisterClass( hab, szTempBuffer, MVPCResponseWndProc, 0L, 0 );
    hwndMVPCResponse = WinCreateWindow( hwndFrame,
                                        szTempBuffer,
                                        NULL,
                                        0L,
                                        0, 0, 0, 0,
                                        hwndFrame,
                                        HWND_TOP,
                                        0,
                                        NULL,
                                        NULL );

// LoadString, RegisterClass, Create videodisc control advise window

    WinLoadString( hab,
                   NULL,
                   ID_MVPCADVISECLASS,
                   sizeof( szTempBuffer ),
                   szTempBuffer );

    WinRegisterClass( hab, szTempBuffer, MVPCAdviseWndProc, 0L, 0 );
    hwndMVPCAdvise = WinCreateWindow( hwndFrame,
                                      szTempBuffer,
                                      NULL,
                                      0L,
                                      0, 0, 0, 0,
                                      hwndFrame,
                                      HWND_TOP,
                                      0,
                                      NULL,
                                      NULL );


// Create a permanent videodisc control query structure

    MVPCCreateQry( &pmvpcqry, MVPC_QRY_STATUS ) ;

// Create a permanent video window control query structure

    MVWNCreateQry( &pmvwnqry, MVWN_QRY_STATUS ) ;

// Save Video Source Coordinates to Be Restored When Exiting Full Screen

   SaveSourceHlen[2] = 0;    // Coordinates for Sources 2 & 3 are Uninitialized
   SaveSourceHlen[3] = 0;

   WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
   SaveSourceHorg[1] = pmvwnqry->usSourceHorg;
   SaveSourceVorg[1] = pmvwnqry->usSourceVorg;
   SaveSourceHlen[1] = pmvwnqry->usSourceHlen;
   SaveSourceVlen[1] = pmvwnqry->usSourceVlen;

// Create the Videodisc controls

    CreateVideodiscControls();

// If we created and assigned an active videodisc player, send
//    the control an init command (MVPC_CMD_INIT) to load and
//    initialize the player

    if ( hwndMVPC )             // hwndMVPC tracks the active videodisc control
        WinShowWindow( hwndMVPC, TRUE );

    // Calculate Ideal Frame Size

    WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
    Rectangle.xLeft   = 0L;                      //  Calculate Frame Window for
    Rectangle.yBottom = 0L;                      //  Desired Video Window Size
                                                 //  Use Half of Actual Height
    Rectangle.yTop = ( pmvwnqry->usSourceVlen / 2 ) - 1 +
                     ( LONG ) scrollbarheight ;
                                                 // Use 4:3 Ratio for Width
    Rectangle.xRight  = ( ( pmvwnqry->usSourceVlen / 2 ) * 4 ) / 3;

    WinCalcFrameRect( hwndFrame, &Rectangle, FALSE ) ;
                                                 // Rectangle Now Has Ideal
                                                 // Frame Window Coordinates
    IdealFrameHeight = ( SHORT ) ( Rectangle.yTop - Rectangle.yBottom + 1L ) ;
    IdealFrameWidth  = ( SHORT ) ( Rectangle.xRight - Rectangle.xLeft + 1L ) ;

// Set the Title Bar Text

    WinLoadString( hab,
                   NULL,
                   ID_WINDOWTEXT,
                   sizeof( szTempBuffer ),
                   szTempBuffer ) ;

    WinSetWindowText( hwndFrame, szTempBuffer ) ;
    SetIdealCoord();

// Change the pointer back to the normal arrow pointer

   WinSetPointer( HWND_DESKTOP,
                  WinQuerySysPointer( HWND_DESKTOP, SPTR_ARROW, FALSE ) );
   WinSetFocus( HWND_DESKTOP, hwndFrame );
   HelpInit();                  // Initialize Help Instance

   return( TRUE ) ;
   }

 return( FALSE ) ;
 }


//***********************************************************************
// Func: TerminateBrowser
//
// Desc: Destroys the Video Window and the Videodisc Player controls,
//       and performs other Termination Functions.
//
//***********************************************************************

static VOID  TerminateBrowser( VOID )
{
   SHORT i ;

   WinDestroyWindow( hwndVideoWindow ) ;
   WinDestroyWindow( hwndFrame ) ;

   if ( hwndHelpInstance )
       WinDestroyHelpInstance ( hwndHelpInstance ) ;  // Destroy Help Instance

   MVPCDestroyQry ( pmvpcqry ) ;
   MVWNDestroyQry ( pmvwnqry ) ;

   for ( i = 0 ; i < 3 ; i++ )                         // Destroy System Pointers
      {
      if ( hptrActive[ i ] )
         WinDestroyPointer( hptrActive[ i ] ) ;

      if ( hptrBusy[ i ] )
         WinDestroyPointer( hptrBusy[ i ] ) ;

      if ( hptrVid[ i ] )
         WinDestroyPointer( hptrVid[ i ] ) ;
      }
   MVWNTerminate ( hab ) ;
   MVPCTerminate ( hab ) ;

}


//**************************************************************************
// Func: HandleBecomingActive
//
// Desc: When Browser is re-gaining the focus, this function is called to
//       ensure that the video window and the videodisc control are both
//       active (i.e., hold the token) as well.
//
//*****************************************************************************

VOID HandleBecomingActive()
{
 SHORT player;
 HWND  hwndTemp;

 if  ( hwndMVPC )
    { WinSendMsg( hwndMVPC, mvpcQueryMsg, NULL, pmvpcqry );
      if  ( pmvpcqry->fActive )                  // IF MVPC Active
         { WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
           if  ( !pmvwnqry->fActive )
                MVWNCommand( MVWN_CMD_ACTIVATE, ( MPARAM ) TRUE, NULL );
           }
      else
           MVPCCommand( MVPC_CMD_ACTIVATE, ( MPARAM ) TRUE, NULL, NULL );
      }
 else                                            // ELSE No Active ScrollBar
    { WinSendMsg( hwndVideoWindow, mvwnQueryMsg, NULL, pmvwnqry );
      if  ( !pmvwnqry->fActive )
           MVWNCommand( MVWN_CMD_ACTIVATE, ( MPARAM ) TRUE, NULL );
      }

 hwndTemp = hwndMVPC;
 for  ( player = 0;  player != MAX_PLAYERS;  player++ )
     { if  ( hwndMVPCs[player] &&
             ( hwndMVPCs[player] != hwndTemp ) )
          { hwndMVPC = hwndMVPCs[player];
            WinSendMsg( hwndMVPC, mvpcQueryMsg, NULL, pmvpcqry );
            if  ( !pmvpcqry->fActive )
                 MVPCCommand( MVPC_CMD_ACTIVATE, ( MPARAM ) TRUE, NULL, NULL );
            }
       }
 hwndMVPC = hwndTemp;

 return;
 }


//***********************************************************************
// END OF FILE
//***********************************************************************
