//***********************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// Module:  COMMAND.C
// Version: 1.00
//
// Purpose: Simple videodisc browser utility demonstrating programming
//          concepts of the Multimedia Toolkit for OS/2 PM
//
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#include <os2.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.


//***********************************************************************
// Func: MVPCCommand
//
// Parm: USHORT   usCommand;
//       MPARAM   mp1;
//       MPARAM   mp2;
//       MPARAM   mp3;
//
// Desc: Videodisc control object generic command function.
//       All commands to the control get funneled through here.
//
// NOTE: The following global variables must be initialized before
//       calling this function:
//
//          hwndMVPC    mvpcCommandMsg    hwndMVPCResponse
//
// Retn: TRUE:  Success
//       FALSE: Fail
//
//***********************************************************************

BOOL MVPCCommand( USHORT usCommand, MPARAM mp1, MPARAM mp2, MPARAM mp3 )
 {
 static PMVPCCMD pmvpccmd;
 BOOL            fOK;

 if  ( hwndMVPC == NULL )
      return( FALSE );

 if  ( fOK = MVPCCreateCmd( &pmvpccmd, usCommand ) )    // If created command
    { // Set up command parms
      pmvpccmd->data.mpCmdParam1 = mp1;
      pmvpccmd->data.mpCmdParam2 = mp2;
      pmvpccmd->data.mpCmdParam3 = mp3;

      // Then post to videodisc control
      WinPostMsg( hwndMVPC, mvpcCommandMsg, hwndMVPCResponse, pmvpccmd );
      fOK = TRUE;
      }

 return( fOK );
 }


//***********************************************************************
// Func: MVWNCommand
//
// Parm: USHORT   usCommand;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: Video Window generic command function.  All commands
//       to the video window get funneled through here.
//
// NOTE: The following global variables must be initialized before
//       calling this function:
//
//          hwndVideoWindow   mvwnCommandMsg    hwndMVWNResponse
//
// Retn: TRUE:  Success
//       FALSE: Fail
//
//***********************************************************************

BOOL MVWNCommand( USHORT usCommand, MPARAM mp1, MPARAM mp2 )
{
 static PMVWNCMD  pmvwncmd;
 BOOL             fOK = FALSE;

 if  ( hwndVideoWindow == NULL )
      return( FALSE );

 if ( fOK = MVWNCreateCmd( &pmvwncmd, usCommand ) )   // If created command
    {
    // Set up command parms
    pmvwncmd->data.mpCmdParam1 = mp1;
    pmvwncmd->data.mpCmdParam2 = mp2;

    WinSendMsg( hwndVideoWindow, mvwnCommandMsg, hwndMVWNResponse, pmvwncmd );
    }

 return( fOK );
 }

//***********************************************************************
// END OF FILE
//***********************************************************************
