//***********************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// Program: IMAGE.C
// Version: 1.0
//
// Purpose: Handle Video Image Processing
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_DOSERRORS

#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


typedef struct {
    char          id[6];       // set to "YUV12C" always
    char      future[2];       // 0, 0 : Reserved for future use
    unsigned short xorg;       // x origin of captured video buffer
    unsigned short yorg;       // y origin of captured video buffer
    unsigned short xlen;       // number of pels in captured line
    unsigned short ylen;       // number of lines captured
    } MMOTION_VIDEO_HEADER ;

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.

static  SHORT   RestoreVideoImage( PCHAR );
static  SHORT   SaveVideoImage( PCHAR );

#define SEGMENTLEN  0x10000L
#define HALFSEGMENT 0x8000

//***********************************************************************
// Things Needed by the Show All Thread
//***********************************************************************

static  SHORT  ImageCount;               // Total Number of Images
static  HWND   hwndSHOWALL;
static  HWND   hwndSHOW;


//***********************************************************************
// Func: SaveVideoImage()
//
// Parm: PCHAR  Disk File Name
//
// Desc: Save the Current Video Image to a Disk File
//
// Returns: =1 if Entry Needs to be Added to the List
//***********************************************************************

static SHORT SaveVideoImage( FileName )
PCHAR FileName;
{
 static MVWNVIDIMAGE Image;
 HFILE    FileHandle;
 USHORT   ActionTaken;
 ULONG    ByteCount ;
 ULONG    FileSize = 0L;
 ULONG    Reserved = 0L;
 USHORT   rc = FALSE ;
 USHORT   BytesWritten;
 USHORT   RequestCount ;
 PCHAR    Buffer;
 CHAR     TextBuff[80];
 PMVWNCMD pmvwncmd;
 SEL      Selector;
 USHORT   Offset ;
 USHORT   ShiftCount;

 Image.pImageBuffer = 0L;
 Image.cImage = 0L;

 if ( MVWNCreateCmd( &pmvwncmd, MVWN_CMD_CAPTUREIMAGE ) )
    {
    pmvwncmd->data.mpCmdParam1 = MPFROMP( &Image );
    pmvwncmd->data.mpCmdParam2 = ( MPARAM )NULL;
    WinSendMsg( hwndVideoWindow, mvwnCommandMsg, NULL, pmvwncmd );
    if ( pmvwncmd->command.usReturnCode == MVWN_ERR_NOERROR )
      {
      switch( rc = DosOpen2( FileName,
                             &FileHandle,
                             &ActionTaken,
                             0L,
                             FILE_NORMAL,
                             FILE_OPEN | FILE_CREATE,
                             ( ULONG ) ( OPEN_ACCESS_WRITEONLY |
                                         OPEN_FLAGS_FAIL_ON_ERROR |
                                         OPEN_SHARE_DENYWRITE ),
                             ( PEAOP ) 0L,
                             0L ) )
         {
         case NO_ERROR :
            if  ( ActionTaken == FILE_EXISTED )
               { if  ( WinMessageBox( HWND_DESKTOP, hwndRestoreImage,
                                      "Do you wish to overwrite this file ?",
                                      "File Already Exists", 0,
                                      MB_ICONQUESTION |
                                      MB_YESNO |
                                      MB_APPLMODAL ) == MBID_NO )
                    { DosClose( FileHandle ) ;
                      rc = FALSE ;
                      break ;
                      }
                 }

            switch ( rc = DosNewSize( FileHandle, Image.cImage ) )
               {
               case NO_ERROR :

                  DosGetHugeShift( &ShiftCount );

                  Selector = SELECTOROF( Image.pImageBuffer );
                  Offset = 0 ;

                  for ( ByteCount = Image.cImage ; ByteCount > 0L ; )
                     {
                     Buffer = MAKEP( Selector, Offset );
                     RequestCount =
                           ( USHORT ) min( ( ULONG ) HALFSEGMENT, ByteCount ) ;

                     rc = DosWrite( FileHandle,
                                    Buffer,
                                    RequestCount,
                                    &BytesWritten );

                     if ( ! rc && ( BytesWritten == RequestCount ) )
                        {
                        if  ( ( Offset + BytesWritten ) < Offset )
                           { Selector  = SELECTOROF( Buffer );
                             Selector += ( 1 << ShiftCount );
                             }
                        ByteCount -= BytesWritten ;
                        Offset += BytesWritten ;
                        rc = TRUE ;
                        }
                     else
                        {
                        sprintf( TextBuff,
                           "File Name = %s\n Return Code = %d", FileName, rc );
                        WinMessageBox( HWND_DESKTOP, hwndFrame, TextBuff,
                                  "ERROR: Cannot Write File", 0, MB_ICONHAND );
                        rc = FALSE ;
                        break ;
                        }
                     }
                  DosClose( FileHandle ) ;

                  if  ( rc == FALSE )
                     { DosDelete( FileName, NULL ) ;
                       return( FALSE ) ;
                       }

                  break ;

               case ERROR_DISK_FULL :
                  WinMessageBox( HWND_DESKTOP, hwndFrame,
                                 "Cannot save image. Insufficient disk space",
                                 "Save Image Error", 0,MB_ERROR | MB_CANCEL);
                  DosClose( FileHandle ) ;
                  if  ( ActionTaken != FILE_EXISTED )
                       DosDelete( FileName, NULL ) ;
                  rc = FALSE ;
                  break ;
               }
            break ;

         default :
            sprintf( TextBuff,
                     "File Name = %s\n Return Code = %d", FileName, rc );
            WinMessageBox( HWND_DESKTOP, hwndFrame, TextBuff,
                           "ERROR: Cannot Open File", 0, MB_ICONHAND );
            rc = FALSE ;
            break ;

         }

      MVWNDestroyCmd( pmvwncmd, hwndVideoWindow, mvwnCommandMsg );
      }
    else
      {
      WinMessageBox( HWND_DESKTOP, hwndFrame,
                     "Could not capture image. Possible insufficient memory.",
                     "Save Image Error", 0,
                     MB_ERROR | MB_CANCEL);
      rc = FALSE ;
      MVWNDestroyCmd( pmvwncmd, hwndVideoWindow, mvwnCommandMsg );
      }
    }

 return( rc ) ;
 }



//***********************************************************************
// Func: RestoreVideoImage()
//
// Parm: PCHAR  Disk File Name
//
// Desc: Restore a Video Image From A Disk File
//***********************************************************************

static SHORT RestoreVideoImage( FileName )
PCHAR FileName;
{
 PMVWNVIDIMAGE  Image;
 MMOTION_VIDEO_HEADER mvh ;
 HFILE   FileHandle;
 USHORT  ActionTaken;
 ULONG   FileSize = 0L;
 ULONG   Reserved = 0L;
 USHORT  rc;
 USHORT  ReturnCode = FALSE ;
 USHORT  BytesRead;
 USHORT  RequestCount ;
 PCHAR   Buffer;
 SEL     Selector;
 USHORT  ShiftCount;
 ULONG   ByteCount ;
 CHAR    TextBuff[40];
 USHORT  Offset ;
 ULONG   FilePtr ;

 switch ( rc = DosOpen2( FileName,
                         &FileHandle,
                         &ActionTaken,
                         FileSize,
                         FILE_NORMAL,
                         FILE_OPEN,
                         ( ULONG )( OPEN_ACCESS_READONLY | OPEN_SHARE_DENYWRITE ),
                         ( PEAOP ) 0L, Reserved ) )
   {
   case NO_ERROR :

      Image = ( PMVWNVIDIMAGE ) malloc( sizeof( MVWNVIDIMAGE ) );
      if  ( Image == NULL )
           return( FALSE ) ;
      if ( ! ( rc = DosRead( FileHandle, &mvh,
                             sizeof( MMOTION_VIDEO_HEADER ), &BytesRead ) ) )
         {
         if  ( BytesRead != sizeof( MMOTION_VIDEO_HEADER ) )
            { DosClose( FileHandle ) ;
              return( FALSE ) ;
              }

         ReturnCode = TRUE ;

         if ( ! strncmp( mvh.id, "YUV12C", sizeof( mvh.id ) ) )
            {
            Image->cImage = sizeof( MMOTION_VIDEO_HEADER ) +
                       ( LONG ) ( mvh.xlen >> 2 ) * ( LONG ) ( mvh.ylen * 6 ) ;

            if ( ! DosAllocHuge( ( USHORT )( Image->cImage >> 16 ),
                          ( USHORT ) Image->cImage,
                          &Selector,
                          0,
                          SEG_NONSHARED ) )
               {
               Image->pImageBuffer = MAKEP( Selector, 0 );
               DosGetHugeShift( &ShiftCount );

               DosChgFilePtr( FileHandle, 0L, FILE_BEGIN, &FilePtr ) ;

               for ( ByteCount = Image->cImage, Offset = 0 ; ByteCount > 0L ; )
                  {
                  Buffer = MAKEP( Selector, Offset );
                  RequestCount =
                           ( USHORT )min( ( ULONG )HALFSEGMENT, ByteCount ) ;

                  rc = DosRead( FileHandle,
                                Buffer,
                                RequestCount,
                                &BytesRead ) ;

                  if  ( ! rc && ( BytesRead == RequestCount ) )
                     { if  ( ( Offset + BytesRead ) < Offset )
                          { Selector  = SELECTOROF( Buffer );
                            Selector += ( 1 << ShiftCount );
                            }
                       ByteCount -= BytesRead ;
                       Offset += BytesRead ;
                       }
                  else
                     { ReturnCode = FALSE ;
                       break ;
                       }
                  }

               if  ( ReturnCode )
                  { Image->xLen = mvh.xlen ;
                    Image->yLen = mvh.ylen ;
                    MVWNCommand( MVWN_CMD_RESTOREIMAGE,
                                 ( MPARAM ) Image, NULL );
                    }
               }
            else
               {
               WinMessageBox( HWND_DESKTOP, hwndFrame,
                              "Cannot allocate memory for image buffer.",
                              "ERROR: DosAllocHuge() Failed", 0,
                              MB_ICONHAND | MB_APPLMODAL );
               ReturnCode = FALSE ;
               }
            }
         }

      DosClose( FileHandle ) ;

      break ;

   default :
      strcpy( TextBuff, "FileName = " );
      strcat( TextBuff, FileName );
      WinMessageBox( HWND_DESKTOP, hwndFrame, TextBuff,
                     "ERROR: Cannot Open File", 0, MB_ICONHAND );
      ReturnCode = FALSE ;
      break ;

   }

 return( ReturnCode ) ;
 }


//***********************************************************************
// Func: ImageDlgProc()
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: Dialog procedure that allows the user to Select and Display
//       a Video Image from a Disk File
//***********************************************************************

MRESULT EXPENTRY ImageDlgProc( HWND hwnd,  USHORT msg,
                               MPARAM mp1, MPARAM mp2 )
{
 USHORT      usItem;
 SHORT       Frozen;
 SHORT       i;
 SHORT       Answer;
 FILEFINDBUF findbuf;
 HDIR        hdir = 0xFFFF;
 USHORT      usSearchCount = 1;
 CHAR        ImageName[13];
 CHAR        TempBuffer[128];
 MPARAM      rc;

 switch ( msg )
     {
      case WM_INITDLG:
         WinPostMsg( hwnd, WM_USER + 1, NULL, NULL );
         WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX, LM_SELECTITEM,
                            MPFROMSHORT( 0 ), MPFROMSHORT( TRUE ) );
         WinSetFocus( HWND_DESKTOP, hwnd );
         hwndSHOWALL = WinWindowFromID( hwnd, ID_SHOWALL );
         hwndSHOW    = WinWindowFromID( hwnd, DID_OK );
         return( ( MRESULT ) 1 );
         break;

      case WM_ACTIVATE:
         if  ( mp1 )
              HandleBecomingActive();
         break;

      case WM_USER + 1:
         WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX, LM_DELETEALL, NULL, NULL );
         SetBoxPosition( hwndRestoreImage, TOP );

         if  ( !DosFindFirst2( "*.VID", &hdir, FILE_NORMAL,
                               ( PVOID ) &findbuf, sizeof( findbuf ),
                               &usSearchCount, FIL_STANDARD, 0L ) )
            { do
               { i = 1;
                 while  ( findbuf.achName[i] != '.' )        // Strip Extension
                         i++;
                 findbuf.achName[i] = 0;
                 WinSendDlgItemMsg( hwnd, 256, LM_INSERTITEM,
                                    MPFROM2SHORT( LIT_SORTASCENDING, 0 ),
                                    MPFROMP( findbuf.achName ) );
                 }  while ( !DosFindNext( hdir, &findbuf,
                                         sizeof (findbuf),
                                         &usSearchCount ) );
              }

         WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX, LM_SELECTITEM,
                            MPFROMSHORT( 0 ), MPFROMSHORT( TRUE ) );

         return( ( MRESULT ) 1 );
         break;

      case WM_CONTROL:
         switch ( SHORT1FROMMP( mp1 ) )
            {
            case ID_LOADLISTBOX:
               switch ( SHORT2FROMMP( mp1 ) )
                  {
                  case LN_ENTER:
                     usItem = (USHORT) WinSendDlgItemMsg( hwnd, 256,
                                                          LM_QUERYSELECTION,
                                                          MPFROM2SHORT( 0, 0 ),
                                                          ( MPARAM ) NULL );
                     if  ( usItem != LIT_NONE )
                        { WinSendDlgItemMsg( hwnd, 256, LM_QUERYITEMTEXT,
                                             MPFROM2SHORT( usItem, 9 ),
                                             MPFROMP( ImageName ) );
                          strcat( ImageName, ".VID" );
                          MVWNCommand( MVWN_CMD_FREEZE,        // Freeze It
                                       ( MPARAM ) TRUE, NULL );
                          RestoreVideoImage( ImageName );
                          }
                     return( 0L );
                  }
               break;
            }
         break;

      case WM_USER + 2:                             // Show 1 Image of SHOW ALL
         rc = WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX, LM_QUERYITEMTEXT,
                                 MPFROM2SHORT( ( SHORT ) mp1, 9 ),
                                 MPFROMP( ImageName ) );
         if  ( rc )               // IF Get the List Name Was Successful
            { strcat( ImageName, ".VID" );
              RestoreVideoImage( ImageName );
                                  // IF There Are More Images Post for Next One
              if  ( ( SHORT ) mp1  <  ( ( SHORT ) mp2 - 1 ) )
                   WinPostMsg( hwnd, WM_USER + 2,
                               ( MPARAM ) ( ( SHORT ) mp1 + 1 ), mp2 );
              else
                 { WinEnableWindow( hwndSHOWALL, TRUE );
                   WinEnableWindow( hwndSHOW, TRUE );
                   }
              }
          else
            { WinEnableWindow( hwndSHOWALL, TRUE );
              WinEnableWindow( hwndSHOW, TRUE );
              }
         return( 0 );

      case WM_COMMAND:
         switch ( COMMANDMSG( &msg )->cmd )
               { case ID_SHOWALL:
                      ImageCount = ( USHORT ) WinSendDlgItemMsg( hwnd,
                                                             ID_LOADLISTBOX,
                                                             LM_QUERYITEMCOUNT,
                                                             ( MPARAM ) NULL,
                                                             ( MPARAM ) NULL );
                      if  ( ImageCount )
                         { WinEnableWindow( hwndSHOWALL, FALSE );
                           WinEnableWindow( hwndSHOW, FALSE );
                           MVWNCommand( MVWN_CMD_FREEZE,
                                        ( MPARAM ) TRUE, NULL );
                           WinPostMsg( hwnd, WM_USER + 2, NULL,
                                       ( MPARAM ) ImageCount );
                           }
                      return( 0L );

                 case DID_OK:       // ID_SHOW
                      WinEnableWindow( hwndSHOWALL, FALSE );
                      WinEnableWindow( hwndSHOW, FALSE );

                      usItem = (USHORT) WinSendDlgItemMsg( hwnd,
                                                           ID_LOADLISTBOX,
                                                           LM_QUERYSELECTION,
                                                           MPFROM2SHORT( 0, 0 ),
                                                           ( MPARAM ) NULL );
                      if  ( usItem != LIT_NONE )
                         { WinSendDlgItemMsg( hwnd, 256, LM_QUERYITEMTEXT,
                                              MPFROM2SHORT( usItem, 9 ),
                                              MPFROMP( ImageName ) );
                           strcat( ImageName, ".VID" );
                           MVWNCommand( MVWN_CMD_FREEZE,    //    Freeze It
                                        ( MPARAM ) TRUE,
                                        NULL );
                           RestoreVideoImage( ImageName );
                           }

                      WinEnableWindow( hwndSHOWALL, TRUE );
                      WinEnableWindow( hwndSHOW, TRUE );
                      return( 0L );

                 case ID_SAVE:
                      WinSendMsg( hwndVideoWindow,   // Get Frozen/Thawed State
                                  mvwnQueryMsg,
                                  NULL, pmvwnqry );
                      Frozen = pmvwnqry->mvidst.fFreeze;
                      if  ( !Frozen )                       // IF Not Frozen
                           MVWNCommand( MVWN_CMD_FREEZE,    //    Freeze It
                                        ( MPARAM ) TRUE, NULL );
                      WinDlgBox( HWND_DESKTOP, hwnd,
                                 SaveImageDlgProc,
                                 NULL, DLG_SAVEIMAGE, NULL );
                      if  ( !Frozen )                       // IF Wasn't Frozen
                           MVWNCommand( MVWN_CMD_FREEZE,    //    Thaw It Now
                                        ( MPARAM ) FALSE, NULL );
                      return( 0L );

                 case ID_DELETE:
                      usItem = (USHORT) WinSendDlgItemMsg( hwnd, 256,
                                                           LM_QUERYSELECTION,
                                                           MPFROM2SHORT( 0, 0 ),
                                                           ( MPARAM ) NULL );
                      if  ( usItem == LIT_NONE )
                           return( 0L );
                      WinSendDlgItemMsg( hwnd, 256, LM_QUERYITEMTEXT,
                                         MPFROM2SHORT( usItem, 9 ),
                                         MPFROMP( ImageName ) );
                      sprintf( TempBuffer, " %s ", ImageName );
                      Answer = WinMessageBox( HWND_DESKTOP, hwnd, TempBuffer,
                                              "Delete this entry ?", 0,
                                   MB_ICONQUESTION | MB_YESNO | MB_APPLMODAL );
                      if  ( Answer == MBID_NO )
                           return( 0L );

                      WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX, LM_DELETEITEM,
                                         MPFROMSHORT( usItem ), ( MPARAM ) 0L );
                      ImageCount = ( USHORT ) WinSendDlgItemMsg( hwnd,
                                                             ID_LOADLISTBOX,
                                                             LM_QUERYITEMCOUNT,
                                                             ( MPARAM ) NULL,
                                                             ( MPARAM ) NULL );
                      if  ( ImageCount )
                         { if  ( usItem > 0 )
                                usItem -= 1;
                           WinSendDlgItemMsg( hwnd, ID_LOADLISTBOX,
                                              LM_SELECTITEM,
                                              MPFROMSHORT( usItem ),
                                              MPFROMSHORT( TRUE ) );
                           }
                      strcat( ImageName, ".VID" );
                      DosDelete( ImageName, 0L );
                      return( 0L );

                 case ID_CANCEL:
                      WinDismissDlg( hwnd, 0 );
                      hwndRestoreImage = NULL;
                      break;
                 }
         break;
      }

 return( WinDefDlgProc( hwnd, msg, mp1, mp2 ) );
 }


//***********************************************************************
// Func: SetBoxPosition()
//
// Desc: Place Window at top ( or bottom ) right corner
//       of main window, but keep it on the screen.
//***********************************************************************

VOID  SetBoxPosition( Window, TopOrBottom )
HWND  Window;
SHORT TopOrBottom;
{
 SHORT  xpos;                                     // New X Position
 SHORT  ypos;                                     // New Y Position
 LONG   lcxFullScreen;                            // Full Screen Width
 LONG   lcyFullScreen;                            // Full Screen Height
 SWP    swpNew;                                   // This Window's Size
 SWP    swp;                                      // Frame Window Coordinates

 WinQueryWindowPos( hwndFrame, &swp );            // Get Frame Window Position
 if  ( swp.fs & SWP_MINIMIZE )                    // If Frame is Minimized
      return;

 WinQueryWindowPos( Window, &swpNew );            // Get This Window's Size

 lcxFullScreen = WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN );
 lcyFullScreen = WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN );

 xpos = swp.x + swp.cx;
 if  ( ( xpos + swpNew.cx ) > (SHORT) lcxFullScreen )
      xpos = ( SHORT ) lcxFullScreen - swpNew.cx;

 if  ( TopOrBottom == TOP )                       // Top or Bottom of Frame ?
      ypos = swp.y + swp.cy - swpNew.cy ;
 else
      ypos = swp.y;

 if  ( ( ypos + swpNew.cy )  >  ( SHORT ) lcyFullScreen )
      ypos = ( SHORT ) lcyFullScreen - swpNew.cy;

 if  ( ypos < 0 )                                 // Don't Go Too Low
      ypos = 0;

 if  ( ( swpNew.x != xpos )  ||
       ( swpNew.y != ypos ) )
      WinSetWindowPos( Window, HWND_TOP, xpos, ypos, 0, 0,
                       SWP_MOVE | SWP_ACTIVATE | SWP_SHOW );
 else
      WinShowWindow( Window, TRUE );
 return;
 }


//***********************************************************************
// Func: SaveImageDlgProc()
//
// Parms: HWND     hwnd
//        USHORT   msg
//        MPARAM   mp1
//        MPARAM   mp2
//
// Desc: Dialog procedure for Saving Video Images to Disk
//***********************************************************************

MRESULT EXPENTRY SaveImageDlgProc( HWND hwnd,  USHORT msg,
                                   MPARAM mp1, MPARAM mp2 )
{
 char    FileName[13];
 char    ImageName[9];
 HWND    Owner;
 MRESULT Found;

 switch ( msg )
       { case WM_INITDLG:
            WinSendDlgItemMsg( hwnd, ID_KEYWORD, EM_SETTEXTLIMIT,
                               MPFROM2SHORT( 8, 0 ), NULL );
            break;

         case WM_COMMAND:
            switch ( COMMANDMSG( &msg )->cmd )
                  { case ID_SAVEKEYWORD:
                         WinQueryDlgItemText( hwnd, ID_KEYWORD, 9, ImageName );
                         if  ( ( ImageName[0] == 0 ) ||
                               ( ImageName[0] == ' ' ) )
                            { ShowMessage( hwnd, ID_BLANKNAME, ID_IMAGENAME );
                              return( 0L );
                              }

                         strcpy( FileName, ImageName );
                         strcat( FileName, ".VID" );
                         if ( !SaveVideoImage( FileName ) )
                             break;
                         Owner = WinQueryWindow( hwnd, QW_OWNER, FALSE );
                         Found = WinSendDlgItemMsg( Owner, ID_LOADLISTBOX,
                                                    LM_SEARCHSTRING,
                                                    MPFROM2SHORT( 0, LIT_FIRST ),
                                                    MPFROMP( ImageName ) );
                         if  ( ( SHORT ) Found == LIT_NONE )
                              WinSendDlgItemMsg( Owner, ID_LOADLISTBOX,
                                                 LM_INSERTITEM,
                                                 MPFROM2SHORT( LIT_SORTASCENDING, 0 ),
                                                 MPFROMP( ImageName ) );
                         WinDismissDlg( hwnd, TRUE );
                         break;

                    case ID_CANCEL:
                         WinDismissDlg( hwnd, FALSE );
                         break;
                    }
            break;
         }

 return( WinDefDlgProc( hwnd, msg, mp1, mp2 ) );
 }
