//***********************************************************************
// (C) Copyright IBM Corporation 1989, 1992
//
// Module:  RESPONSE.C
// Version: 1.00
//
// Purpose:  Includes the window procedures that handle
//           feedback ( acknowledges, responses, advises ) from
//           the video window and the videodisc control
//
//***********************************************************************

//***********************************************************************
// Defines and Includes
//***********************************************************************

#define  INCL_WIN
#define  INCL_DOSPROCESS

#include <os2.h>
#include <stdlib.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.


//***********************************************************************
// Func: MVPCResponseWndProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: This is the response wndproc for the videodisc control. We have
//       requested that all acknowledges and responses from the control be
//       sent here.  Basically, we'll just look for player errors.
//
//***********************************************************************

MRESULT EXPENTRY MVPCResponseWndProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
 USHORT   usResponse,
          usCommand,
          usReturnCode;
 PMVPCCMD pmvpccmd;

 switch ( msg )
    {
    case MVPC_ACK:
       pmvpccmd     = (PMVPCCMD) mp2;               // Point to parameter block
       usReturnCode = pmvpccmd->command.usReturnCode;   // Get return code
       usResponse   = pmvpccmd->command.usResponse;     // Get response code
       if  ( usReturnCode != MVPC_ERR_NOERROR )     // IF there WAS an error
          {                                         //  Display error condition
            if  ( usReturnCode != MVPC_ERR_RETURNED )
                 ShowMessage( hwndFrame, usReturnCode, ID_ACKERROR );
            }

       if  ( usResponse == MRESP_NORESPONSEPENDING )  // IF No Response Pending
            MVPCDestroyCmd( pmvpccmd, ( HWND ) mp1, msg );  // Destroy Command
       return FALSE;
       break;

    case MVPC_RESPONSE:
       pmvpccmd     = (PMVPCCMD) mp2;               // point to parameter block
       usCommand    = pmvpccmd->command.usCommand;  // get original command
       usReturnCode = pmvpccmd->command.usReturnCode;  // get return code
       switch ( usCommand )
          {
          case MVPC_CMD_INIT:                       // IF INIT went well
             switch ( usReturnCode )
                {
                case MVPC_ERR_NOERROR:
                   if ( ( HWND ) mp1 == hwndMVPC )
                      {
                      MVPCCommand( MVPC_CMD_SETFRAMERANGE,
                                   MPFROMLONG( fromFrame ),
                                   MPFROMLONG( toFrame ),
                                   NULL );
                      }
                   break;

                case MVPC_ERR_VDPERROR:    // You could put error recovery here
                   break;
                }
             break;

          default:
             break;
          }

       MVPCDestroyCmd( pmvpccmd, ( HWND ) mp1, msg );  // Destroy this command
       return FALSE;
       break;
    }

 return( WinDefWindowProc( hwnd, msg, mp1, mp2 ) );
 }


//***********************************************************************
// Func: MVPCAdviseWndProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: We have told the videodisc control to send all advises (for example
//       seeks and activations) here.  On Seeks, we'll freeze the video
//       video on MVPC_ADV_SEEKBEGIN, unfreeze on MVPC_ADV_SEEKEND.
//       This will give us 'seamless' video.
//
//***********************************************************************

MRESULT EXPENTRY MVPCAdviseWndProc( HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2 )
{
 PMVPCADV pmvpcadv;
 HWND     hwndTemp;
 USHORT   usAdvise, usError;

 if ( msg == MVPC_ADVISE )                 // IF this is control advise message
    {
    pmvpcadv = ( PMVPCADV ) mp2;           // THEN set up addressing
    usAdvise = pmvpcadv->usAdvise;         // and get the advise condition
    switch ( usAdvise )
       {
       case MVPC_ADV_REQUESTINGTOKEN:      // Control is requesting the focus
          break;

       case MVPC_ADV_LOSINGTOKEN:          // Control is losing the focus
          if ( ( HWND ) mp1 == hwndMVPC )
              MVWNCommand( MVWN_CMD_ACTIVATE, MPFROM2SHORT( FALSE, 0 ), NULL );
          break;

       case MVPC_ADV_TOKENCAPTURED:        // Control is gaining the focus
          ClearTryCount (HWNDFROMMP (mp1));
          if ( ( HWND ) mp1 == hwndMVPC )
              {
              MVWNCommand( MVWN_CMD_ACTIVATE, MPFROM2SHORT( TRUE, 0 ), NULL );
              }
          break;

       case MVPC_ADV_TOKENNOTCAPTURED:     // Try Again
          if (GetTryCount (HWNDFROMMP (mp1)) < MAX_RETRIES)
            {
            hwndTemp = hwndMVPC;
            hwndMVPC = ( HWND ) mp1;
            MVPCCommand( MVPC_CMD_ACTIVATE, ( MPARAM ) TRUE, NULL, NULL );
            hwndMVPC = hwndTemp;
            IncTryCount (HWNDFROMMP (mp1));
            }
          else
            {
            ClearTryCount (HWNDFROMMP (mp1));
            }
          break;

       case MVPC_ADV_SEEKBEGIN:
            // We'll make an assumption here that IF the players and sources
            // are tied together, THEN the video window is displaying the
            // current player source and we will do the freeze/unfreeze
            // for seeks.
          if ( ( HWND ) mp1 == hwndMVPC )
             {
             if ( fTiePlayersToSources )
                {
                WinSendMsg( hwndVideoWindow,
                            mvwnQueryMsg,
                            ( MPARAM ) NULL,
                            pmvwnqry );

                if ( pmvwnqry->fActive )
                    MVWNCommand( MVWN_CMD_FREEZE, MPFROM2SHORT( TRUE, 0 ), NULL );
                }
             }
          break;

       case MVPC_ADV_SEEKEND:
       case MVPC_ADV_SEEKFAILED:
          if ( ( HWND ) mp1 == hwndMVPC )
             {
             if ( fTiePlayersToSources )
                {
                WinSendMsg( hwndVideoWindow,
                            mvwnQueryMsg,
                            ( MPARAM ) NULL,
                            pmvwnqry );

                if ( pmvwnqry->fActive )
                   MVWNCommand( MVWN_CMD_FREEZE,
                                 MPFROM2SHORT( FALSE, 0 ), NULL );
                }
             }
          break;

       case MVPC_ADV_PLAYERERROR:
          usError = SHORT1FROMMP( pmvpcadv->mpAdvParam1 );
          ShowMessage( hwndFrame, usError, ID_ADVISERROR );
          break;

       case VSB_ADV_USER:
          switch ( SHORT1FROMMP( pmvpcadv->mpAdvParam1 ) )
             {
             case VSB_ADV_SLIDERTRACK:
                // here's where you'd put any vsb slidertrack stuff
                break;
             }
          break;
       }

    MVPCDestroyAdv( pmvpcadv );
    return FALSE;
    }

 return( WinDefWindowProc( hwnd, msg, mp1, mp2 ) );
 }


//***********************************************************************
// Func: MVWNResponseWndProc
//
// Parm: HWND     hwnd;
//       USHORT   msg;
//       MPARAM   mp1;
//       MPARAM   mp2;
//
// Desc: This is the response wndproc for the video window. We have
//       requested that all acknowledges and responses from the control be
//       sent here.
//
//***********************************************************************

MRESULT EXPENTRY MVWNResponseWndProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
 USHORT   usAdvise;
 PMVWNCMD pmvwncmd;
 PMVWNADV pmvwnadv;

 switch ( msg )
    {
    case MVWN_ACK:
       pmvwncmd = ( PMVWNCMD ) mp2;
       if  ( pmvwncmd->command.usCommand == MVWN_CMD_CAPTUREBITMAP )
            BitMapToClipboard( BMClp_DONE );

       if  ( pmvwncmd->command.usCommand == MVWN_CMD_RESTOREIMAGE )
           { if  ( pmvwncmd->command.usReturnCode == MVWN_ERR_INVALIDSIZE )
                  ShowMessage( hwndFrame, ID_SIZEMATCH, ID_SIZERROR );
             MVWNCommand( MVWN_CMD_RELEASEIMAGE,
                          pmvwncmd->data.mpCmdParam1, NULL );
             free( pmvwncmd->data.mpCmdParam1 );
             }

       MVWNDestroyCmd( pmvwncmd, ( HWND ) mp1, msg );
       return FALSE;
       break;

    case MVWN_ADVISE:
       pmvwnadv = ( PMVWNADV ) mp2;
       usAdvise = pmvwnadv->usAdvise;
       switch ( usAdvise )
          {
          case MVWN_ADV_LOSINGTOKEN:      /* VideoWindow is losing the focus */
             break;
          }

       MVWNDestroyAdv( pmvwnadv );
       return FALSE;
       break;
    }

 return( WinDefWindowProc( hwnd, msg, mp1, mp2 ) );
 }


//***********************************************************************
// Func: ClearTryCount
//
// Desc:
//
// Retn: Void
//
//***********************************************************************

VOID ClearTryCount (HWND hwnd)
   {
   USHORT   i;

   for (i = 0; i < MAX_PLAYERS; i++)      // i = 0 to 2
      {
      if (hwnd == hwndMVPCs[i])
         break;
      }

   if (i == MAX_PLAYERS)
      return;

   ausTryCount[i] = 0;
   return;
   }


//***********************************************************************
// Func: GetTryCount
//
// Desc:
//
// Retn: usTryCount;       // Number of retries attempted
//
//***********************************************************************

USHORT GetTryCount (HWND hwnd)
   {
   USHORT   i;

   for (i = 0; i < MAX_PLAYERS; i++)      // i = 0 to 2
      {
      if (hwnd == hwndMVPCs[i])
         break;
      }

   if (i == MAX_PLAYERS)
      return (MAX_RETRIES);

   return (ausTryCount[i]);
   }


//***********************************************************************
// Func: IncTryCount
//
// Desc:
//
// Retn: Void
//
//***********************************************************************

VOID IncTryCount (HWND hwnd)
   {
   USHORT   i;

   for (i = 0; i < MAX_PLAYERS; i++)      // i = 0 to 2
      {
      if (hwnd == hwndMVPCs[i])
         break;
      }

   if (i == MAX_PLAYERS)
      return;

   ausTryCount[i]++;
   return;
   }


//***********************************************************************
// END OF FILE
//***********************************************************************
