
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBAdv.c          (VSB Advise Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that process advises.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>
#include <stdlib.h>     // min ()

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: ProcessADVISE                                Author: Steve Henson
//
// Parm: HWND     hwnd;       --
//       MPARAM   mp1;        --
//       MPARAM   mp2;        --
//
// Desc: Processes ADVISES from the MVPC
//
// Retn: Returns through WinDefWindowProc ()
//
//*****************************************************************************

MRESULT ProcessADVISE (HWND hwnd, MPARAM mp1, MPARAM mp2)
   {
   PVSBDATA    pvsbdata;
   PLOGINFO    pLogInfo;
   USHORT      posSlider;
   LONG        lRange;
   SHORT       sSBRange;
   PUIADVISE   puiadvise = (PUIADVISE) mp2;

   // Address this instance's data area
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);
   pLogInfo = pvsbdata->pLogInfo;

   switch (SHORT1FROMMP (mp1))         // switch on the specific advise
      {
      //-----------------------------------------------------------------------
      case MVPC_ADV_AUDIO:
         if (pLogInfo && pLogInfo->lf.fsAudio)
            LogMessage (hwnd, IDS_LM_AUDIO, (LONG) puiadvise->usAudio);
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_BUSY:
         if (pvsbdata->fLoaded)
            {
            WinEnableWindow (pvsbdata->hwndScrollBar,
                             (puiadvise->fBusy) ? FALSE : TRUE);
            }

         if (pLogInfo && pLogInfo->lf.fsBusy)
            {
            if (puiadvise->fBusy)
               {
               pLogInfo->lBusyCount++;
               LogMessage (hwnd, IDS_LM_BUSY, pLogInfo->lBusyCount);
               }
            else
               {
               if (pLogInfo && pLogInfo->lBusyCount)
                  pLogInfo->lBusyCount--;

               LogMessage (hwnd, IDS_LM_READY, pLogInfo->lBusyCount);
               }
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_FRAMERANGE:        // frame range change advise
         // Remember new bounds & current frame
         pvsbdata->lFromFrame    = puiadvise->lLowerBound;
         pvsbdata->lToFrame      = puiadvise->lUpperBound;

         lRange = pvsbdata->lToFrame - pvsbdata->lFromFrame + 1L; // (54000 - 1 + 1 == 54000)
         sSBRange = (SHORT) min ((LONG)usMaxSBRange, lRange);

         // Set the width of the scroll bar thumb
         WinSendMsg (pvsbdata->hwndScrollBar,
                     SBM_SETTHUMBSIZE,
                     MPFROM2SHORT (1, sSBRange),
                     (MPARAM) NULL);

         if (pLogInfo && pLogInfo->lf.fsFrameRange)
            {
            LogMessage (hwnd, IDS_LM_FRAMERANGE,
                        pvsbdata->lFromFrame,
                        pvsbdata->lToFrame,
                        pvsbdata->lCurrentFrame);
            }

         // FALL THROUGH

      //-----------------------------------------------------------------------
      case MVPC_ADV_FRAMECHANGE:     // frame change advise
         // remember new frame
         pvsbdata->lCurrentFrame = puiadvise->lCurrentFrame;

         // and update scrollbar accordingly
         posSlider = PosSlider (pvsbdata->lFromFrame, pvsbdata->lToFrame, pvsbdata->lCurrentFrame);
         if (posSlider != pvsbdata->posSlider)
            {
            pvsbdata->posSlider = posSlider;
            WinSendMsg (pvsbdata->hwndScrollBar,
                        SBM_SETPOS,
                        MPFROMSHORT (posSlider),
                        NULL);
            }

         if (pLogInfo && pLogInfo->lf.fsFrameChange)
            LogFrame (hwnd, pvsbdata->lCurrentFrame);
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_INFODISPLAY:
         if (pLogInfo && pLogInfo->lf.fsInfoDisplay)
            {
            if (puiadvise->usInfoDisplay)
               LogMessage (hwnd, IDS_LM_INFOON);
            else
               LogMessage (hwnd, IDS_LM_INFOOFF);
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_LOADED:
         if (pvsbdata->fLoaded != puiadvise->fLoaded)
            {
            // Save the loaded flag
            pvsbdata->fLoaded = puiadvise->fLoaded;

            // If we are not already busy, then disable/enable the window
            if (!pLogInfo->lBusyCount)
               WinEnableWindow (pvsbdata->hwndScrollBar, pvsbdata->fLoaded);
            }

         if (pLogInfo && pLogInfo->lf.fsLoaded)
            {
            if (puiadvise->fLoaded)
               LogMessage (hwnd, IDS_LM_PLAYERLOADED);
            else
               LogMessage (hwnd, IDS_LM_PLAYERUNLOADED);
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_MOTION:
         if (pLogInfo && pLogInfo->lf.fsMotion)
            {
            if (!puiadvise->usPlayDirection)
               LogMessage (hwnd, IDS_LM_PLAYSTOPPED);
            else
               LogMessage (hwnd, (puiadvise->usPlayDirection == MVDP_PLAYFORWARD)
                                 ? IDS_LM_PLAYFORWARD
                                 : IDS_LM_PLAYBACKWARD);
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_PLAYERERROR:
         //-------------------------------------------------------------------
         // Rudimentary Error Recovery:
         //    If we get an invalid frame, then reposition (seek) to the
         //    last reported frame before the seek.  This depends on us
         //    not receiving an advise for the invalid frame number until
         //    AFTER a successful seek.
         //-------------------------------------------------------------------
         if (puiadvise->usError == MVPC_ERR_INVALIDFRAME)
            CmdSeek (pvsbdata->hwndOwner, hwnd, pvsbdata->lCurrentFrame);

         if (pLogInfo && pLogInfo->lf.fsPlayerError)
            {
            if (puiadvise->usError == MVPC_ERR_INVALIDFRAME)
               LogMessage (hwnd, IDS_LM_INVALIDFRAME);
            else
               LogMessage (hwnd, IDS_LM_PLAYERERROR);
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_SEEK:
         if (pLogInfo && pLogInfo->lf.fsSeek)
            {
            switch (puiadvise->usSeek)
               {
               case MVPC_ADV_SEEKBEGIN:
                  LogMessage (hwnd, IDS_LM_SEEKBEGIN);
                  break;

               case MVPC_ADV_SEEKEND:
                  LogMessage (hwnd, IDS_LM_SEEKEND);
                  break;

               case MVPC_ADV_SEEKFAILED:
                  LogMessage (hwnd, IDS_LM_SEEKFAILED);
                  break;
               }
            }
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_SPEED:
         if (pLogInfo && pLogInfo->lf.fsSpeed)
            LogMessage (hwnd, IDS_LM_SPEED, (LONG)puiadvise->usSpeed);
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_STEPINCREMENT:
         if (pLogInfo && pLogInfo->lf.fsStepIncrement)
            LogMessage (hwnd, IDS_LM_STEPINCREMENT, (LONG)puiadvise->usStepIncrement);
         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_TOKENCHANGE:
         if (pLogInfo && pLogInfo->lf.fsTokenChange)
            {
            switch (puiadvise->usFocus)
               {
               case MVPC_ADV_LOSINGTOKEN:
                  LogMessage (hwnd, IDS_LM_FOCUSLOST);
                  break;

               case MVPC_ADV_REQUESTINGTOKEN:
                  LogMessage (hwnd, IDS_LM_FOCUSREQUESTED);
                  break;

               case MVPC_ADV_TOKENCAPTURED:
                  LogMessage (hwnd, IDS_LM_FOCUSCAPTURED);
                  break;

               case MVPC_ADV_TOKENNOTCAPTURED:
                  LogMessage (hwnd, IDS_LM_FOCUSNOTCAPTURED);
                  break;
               }
            }
         break;
      }
   }


//**************************************************************************<f>
// Func: AdviseUserInteraction                        Author: Steve Henson
//
// Desc: This function passes an advise back to the application window
//       set with a VSB_CMD_ADVISEUSER command.
//
// Retn: VOID
//
//*****************************************************************************

VOID AdviseUserInteraction (PVSBDATA pvsbdata, MPARAM mpAdvParam1,
                                               MPARAM mpAdvParam2,
                                               MPARAM mpAdvParam3)
   {
   PMVPCADV pmvpcadv;

   if (MVPCCreateAdv (&pmvpcadv, VSB_ADV_USER, pvsbdata->usPlayer, pvsbdata->hwndAdviseUser))
      {
      pmvpcadv->mpAdvParam1 = mpAdvParam1;
      pmvpcadv->mpAdvParam2 = mpAdvParam2;
      pmvpcadv->mpAdvParam3 = mpAdvParam3;

      WinPostMsg (pvsbdata->hwndAdviseUser,
                  msgMVPC_ADVISE,
                  pvsbdata->hwndOwner,
                  MPFROMP (pmvpcadv));
      }
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
