
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBFunc.c         (VSB Misc. Functions)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains miscellaneous functions used by the
//          Videodisc Scroll Bar (VSB) DLL.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>
#include <string.h>     // memset ()

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: CreateVSBDATA                                Author: Steve Henson
//
// Parm: none
//
// Desc: Allocates and sets to NULL the VSBDATA structure for this
//       User Interface
//
// Retn: NULL  = An error occurred while trying to create the structure
//       !NULL = The Ptr to the VSBDATA structure
//
//*****************************************************************************

PVSBDATA CreateVSBDATA ()
   {
   SEL      selvsbdata;                   // VSBData Selector
   PVSBDATA pvsbdata = (PVSBDATA) NULL;   // VSBData Pointer (returned)

   if (!DosAllocSeg (sizeof (VSBDATA), &selvsbdata, SEG_NONSHARED))
      {
      pvsbdata = (PVSBDATA)(MAKEP (selvsbdata, 0));   // Convert sel to ptr
      memset (pvsbdata, 0, sizeof (VSBDATA));         // Clear the VSBDATA structure

      pvsbdata->cbData = (USHORT) sizeof (VSBDATA);   // Set the size field
      }

   return (pvsbdata);
   }


//**************************************************************************<f>
// Func: PosSlider                                    Author: Steve Henson
//
// Parm: LONG  lLow;       -- The lower boundary of the frame range
//       LONG  lHigh;      -- The upper boundary of the frame range
//       LONG  lCurrent;   -- The current frame number
//
// Desc: Unfortunately, Scroll bars in OS/2 1.2 are only allowed a range
//       from 1 to 32767 (signed int).  Since we can easily have more
//       than that in a frame range for videodisc, we have to map our
//       frame range into a signed int.  This function performs that
//       mapping.
//
// Retn: The scroll bar position (from 1 to 32767) for the current frame.
//
//*****************************************************************************

USHORT PosSlider (LONG lLow, LONG lHigh, LONG lCurrent)
   {
   LONG   lOffset;      // The offset of the current frame from the beginning
   LONG   lRange;       // The range of frames
   USHORT posSlider;    // The position (returned)

   //--------------------------------------------------------------------------
   // Here's how it works:
   //    the FIRST and LAST positions on the scroll bar are saved for the
   //    Beginning and End of the range.  All other values are mapped to
   //    some intermediate position.  This allows us to step backward from
   //    frame #2 to #1, likewise from 53999 to 54000.
   //--------------------------------------------------------------------------

   if (lCurrent == lLow)
      posSlider = 1;
   else if (lCurrent == lHigh)
      posSlider = usMaxSBRange;
   else
      {
      lOffset = lCurrent - lLow;
      if (lOffset < 0)
         lOffset = 0;      // Or else we would set a negative position

      lRange = lHigh - lLow;
      if (lRange == 0)
         lRange = 1;       // Or else we would get a DIVIDE BY ZERO ERROR!

      //-----------------------------------------------------------------------
      // Note: we multiply with a ULONG before dividing to minimize
      //       round-off error.
      //-----------------------------------------------------------------------
      posSlider = (SHORT)((((ULONG)lOffset) * usMaxSBRange) / lRange) + 1;

      //-----------------------------------------------------------------------
      // Error caused by the following round-off should not exceed 2 frames
      //    on each end over a maximum frame range of 108,000 frames (CLV).
      //    Given an SB_RANGE of 32767 (& 108,000 frames):
      //       Position 2 will have 5 frames (frames 2, 3, 4, 5, & 6).
      //       Position 32766 will have 5 frames.
      //       All others will have 3 frames (frames 7, 8, & 9).
      //-----------------------------------------------------------------------
      if (posSlider == 1)
         posSlider++;
      else if (posSlider == usMaxSBRange)
         posSlider--;
      }

   return (posSlider);
   }


//**************************************************************************<f>
// Func: SetMaxSBRange
//
// Parm: VOID
//
// Desc: This function calculates the biggest possible horizontal scroll bar
//       thumb tracking area in pels and sets the SBRange to that maximum.
//
// Retn: VOID
//
//*****************************************************************************

VOID SetMaxSBRange ()
   {
   LONG  cxScreen;
   LONG  cxHScrollArrow;

   cxScreen       = WinQuerySysValue (HWND_DESKTOP, SV_CXSCREEN);
   cxHScrollArrow = WinQuerySysValue (HWND_DESKTOP, SV_CXHSCROLLARROW);

   usMaxSBRange = (USHORT)(cxScreen - (2 * cxHScrollArrow));

   return;
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
