
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBInit.c         (VSB Initialize/Terminate)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that initialize and terminate
//          the Videodisc Scroll Bar (VSB) object.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>
#include <string.h>     // strcpy ()

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: DLLLoad                                      Author: Daryl Warinner
//
// Desc: This function is called when the DLL is first loaded.
//
//       This function does NOT have to be exported.  It does
//       have to be declared as "int far pascal".
//
// Retn: NON-ZERO number means this function was successful.
//
//*****************************************************************************

SHORT EXPENTRY DLLLoad (VOID)
   {

   //--------------------------------------------------------------------------
   // This function currently does nothing.  It's here if you need it.
   //
   // This function must return a non-zero number if it was successful.
   //--------------------------------------------------------------------------
   return (1);
   }


//**************************************************************************<f>
// Func: VSBInitialize     (@1 function of DLL)       Author: Steve Henson
//
// Desc: This function initializes the User Interface.  It is automatically
//       called by MVPC in MVPCInitialize.
//
//       This function must be exported in the @1 position. ie:
//             EXPORTS
//                   VSBInitialize  @1
//
//*****************************************************************************

BOOL EXPENTRY VSBInitialize (HAB hab, USHORT msgCmd,
                                      USHORT msgAck,
                                      USHORT msgResp,
                                      USHORT msgQuery,
                                      USHORT msgAdvise,
                                      PSZ    pszUIClassName)
   {
   CHAR        szVSBClass [MAX_LEN_VSBCLASS];
   BOOL        fReturnCode;
   CLASSINFO   classinfo;

   // Load the name of our User Interface class and copy it
   WinLoadString (hab, hmodDLL, IDS_VSB, sizeof (szVSBClass), szVSBClass);
   strcpy (pszUIClassName, szVSBClass);

   // If the class is already registered, return TRUE
   if (WinQueryClassInfo (hab, szVSBClass, &classinfo))
      return (TRUE);

   // This is the first time.  Save the message values.
   msgMVPC_COMMAND = msgCmd;
   msgMVPC_ACK     = msgAck;
   msgMVPC_RESP    = msgResp;
   msgMVPC_QUERY   = msgQuery;
   msgMVPC_ADVISE  = msgAdvise;

   // Otherwise, return the result from WinRegisterClass
   fReturnCode = WinRegisterClass (hab,         // Anchor block handle
                    szVSBClass,                 // Class name
                    (PFNWP)VSBWndProc,          // Window procedure
                    CS_SIZEREDRAW,              // Class style flags
                    VSB_WWSIZE);                // to store pointer

   LogInitialize (hab);                // Initialize the log support

   SetMaxSBRange ();                   // Set the max scroll bar range

   return (fReturnCode);
   }


//**************************************************************************<f>
// Func: VSBTerminate      (@2 function of DLL)       Author: Daryl Warinner
//
// Parm: HAB      hab;     -- The process anchor block handle
//
// Desc: This function perform exit processing for the UI.
//       None is required for this control.  But it is provided
//       for your convenience.
//
// Retn: VOID
//
//*****************************************************************************

VOID EXPENTRY VSBTerminate (HAB hab)
   {
   LogTerminate (hab);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
