
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBLog.h
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This include file contains the DEFINEs used internally by the
//          Log facility of the Videodisc Scroll Bar (VSB) User Interface (UI).
//          This file is imbedded by VSB2.H.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES
//*****************************************************************************

#define MAX_LOGMESSAGES    250

//--- LENGTHS -----------------------------------------------------------------

#define MAX_LEN_LOGMESSAGE 100
#define MAX_LEN_LINESTRING 6
#define MAX_LEN_LOGMASK    15
#define MAX_LEN_LOGENTRY   120
#define MAX_LEN_FRAME      9

#define MAX_LEN_LOGTITLE   60
#define MAX_LEN_MARK       60

//--- TITLES ------------------------------------------------------------------

#define IDS_LOG_TITLE            0x0800
#define IDS_LOG_PLAYER1          0x0801
#define IDS_LOG_PLAYER2          0x0802
#define IDS_LOG_PLAYER3          0x0803

#define IDS_LOG_MSGMASK          0x0810

//--- MESSAGE STRING IDs ------------------------------------------------------

#define IDS_LM_TOPOFLOG          0x0900

#define IDS_LM_BUSY              0x0910
#define IDS_LM_READY             0x0911

#define IDS_LM_FOCUSLOST         0x0920
#define IDS_LM_FOCUSREQUESTED    0x0921
#define IDS_LM_FOCUSCAPTURED     0x0922
#define IDS_LM_FOCUSNOTCAPTURED  0x0923

#define IDS_LM_INVALIDFRAME      0x0930
#define IDS_LM_PLAYERERROR       0x0931

#define IDS_LM_PLAYERLOADED      0x0940
#define IDS_LM_PLAYERUNLOADED    0x0941

#define IDS_LM_SEEKBEGIN         0x0950
#define IDS_LM_SEEKEND           0x0951
#define IDS_LM_SEEKFAILED        0x0952

#define IDS_LM_PLAYSTOPPED       0x0960
#define IDS_LM_PLAYFORWARD       0x0961
#define IDS_LM_PLAYBACKWARD      0x0962

#define IDS_LM_AUDIO             0x0970
#define IDS_LM_FRAMERANGE        0x0971
#define IDS_LM_INFOOFF           0x0972
#define IDS_LM_INFOON            0x0973
#define IDS_LM_SPEED             0x0974
#define IDS_LM_STEPINCREMENT     0x0975
#define IDS_LM_FRAMECHANGE       0x0976

//--- DIALOG BOX & CONTROL IDs ------------------------------------------------

#define IDD_VSBLOG               0x0100   // Dialog ID
#define IDD_VSBLOGMARK           0x0200   // Dialog ID

#define IDC_LOGBOX               0x0101   // Control ID
#define IDC_OK                   0x0102   // Control ID
#define IDC_MARK                 0x0103   // Control ID
#define IDC_CLEAR                0x0104   // Control ID
#define IDC_FRAME                0x0105   // Control ID

#define IDC_EDITBOX              0x0201   // Control ID
#define IDC_MARKADD              0x0202   // Control ID
#define IDC_MARKCANCEL           0x0203   // Control ID


//**************************************************************************<f>
// TYPEDEFS
//*****************************************************************************

//-----------------------------------------------------------------------------
typedef struct _LOGFLAGS
   {
   unsigned fsAudio:1;
   unsigned fsBusy:1;
   unsigned fsFrameChange:1;
   unsigned fsFrameRange:1;
   unsigned fsInfoDisplay:1;
   unsigned fsLoaded:1;
   unsigned fsMotion:1;
   unsigned fsPlayerError:1;
   unsigned fsSeek:1;
   unsigned fsSpeed:1;
   unsigned fsStepIncrement:1;
   unsigned fsTokenChange:1;
   } LOGFLAGS;

//-----------------------------------------------------------------------------
typedef struct _LOGINFO          // Log Information
   {
   HWND     hwndVSBLOG;          // Hwnd to the VSBLOG Dlg Box
   HWND     hwndLOGBOX;          // LB hwnd to post debug statements
   LONG     lBusyCount;          // Semaphore count for busy status
   BOOL     fLoaded;             // Loaded Flag
   LOGFLAGS lf;                  // Flags for what to log
   } LOGINFO;

typedef LOGINFO FAR *PLOGINFO;


//-----------------------------------------------------------------------------
typedef struct _LOGCREATEPARAMS
   {
   USHORT   cbData;        // Suggested for future compliance with SAA
   USHORT   usType;        // Suggested for future compliance with SAA
   HAB      hab;
   HWND     hwndVSB;
   USHORT   usPlayer;
   } LOGCREATEPARAMS;

typedef LOGCREATEPARAMS FAR *PLOGCREATEPARAMS;


//*****************************************************************************
// FUNCTION PROTOTYPES
//*****************************************************************************

//--- VSBLog.c   --------------------------------------------------------------
VOID        LogLock (BOOL fLock);
VOID        LogInitialize (HAB hab);
VOID        LogTerminate  (HAB hab);
VOID        LogCreate  (HWND hwndVSB);
VOID        LogDestroy (HWND hwndVSB);
VOID        LogShow (HWND hwndVSB, BOOL fShow);
VOID        LogFrame (HWND hwndVSB, LONG lFrame);
VOID cdecl  LogMessage (HWND hwndVSB, USHORT usLogMsgID, ...);

MRESULT EXPENTRY  VSBLogDlgProc         (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY  VSBLogMarkDlgProc     (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2);


//*****************************************************************************
// END OF FILE
//*****************************************************************************
