
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBQry.c          (VSB Query Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module include functions that provide support for
//          the Query operations of this User Interface.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>
#include <string.h>     // memset ()

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: VSBCreateQry                             Author: Steve Henson
//
// Parm: PMVPCQRY FAR  *ppmvpcqry   -- Ptr to the address to hold the MVPCQRY
//                                     pointer. (ptr to ptr to MVPCQRY)
//       USHORT         usQuery     -- The MVPC_QRY_* value for the query
//
// Desc: This function will allocate and initialize a MVPCQRY structure.
//
// Retn: TRUE  (1) = The MVPCQRY structure was allocated and initialized.
//       FALSE (0) = Failed in allocation or initialization.
//
//*****************************************************************************

BOOL EXPENTRY VSBCreateQry (PMVPCQRY FAR *ppmvpcqry, USHORT usQuery)
   {
   SEL      selmvpcqry;       // Selector for the allocated MVPCQRY structure
   PMVPCQRY pmvpcqry;         // Ptr to the MVPCQRY structure

   if (!DosAllocSeg (sizeof (VSBQRY), &selmvpcqry, SEG_NONSHARED))
      {
      pmvpcqry = (PMVPCQRY) MAKEP (selmvpcqry, 0);    // Make into a pointer
      memset (pmvpcqry, 0, sizeof (VSBQRY));          // Clear the structure

      pmvpcqry->cbData  = sizeof (VSBQRY);            // Required
      pmvpcqry->usQuery = usQuery;                    // Required

      *ppmvpcqry = pmvpcqry;  // The contents of ppmvpcqry gets the address of
                              //    the MVPCQRY structure.
      return TRUE;
      }

   *ppmvpcqry = (PMVPCQRY) NULL;
   return FALSE;
   }


//**************************************************************************<f>
// Func: VSBDestroyQry                            Author: Steve Henson
//
// Parm: PMVPCQRY pmvpcqry -- Ptr to the MVPCQRY structure.
//
// Desc: This function will free the MVPCQRY structure created by a previous
//       call to CreateQry.  Since all Query calls are SENDMSGs, there
//       is no further processing that is necessary to be done.
//
// Retn: VOID
//
//*****************************************************************************

VOID EXPENTRY VSBDestroyQry (PMVPCQRY pmvpcqry)
   {

   DosFreeSeg (SELECTOROF (pmvpcqry));
   }


//**************************************************************************<f>
// Func: ProcessQUERY                                  Author: Steve Henson
//
// Desc: Processes QUERYS from the MVPC
//
// Retn: Returns MRESULT 0
//
//*****************************************************************************
MRESULT ProcessQUERY (HWND hwnd, MPARAM mp2)
   {
   PVSBDATA pvsbdata;
   PVSBQRY  pvsbqry;

   // address this instance's data area
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   // Convert mp2 to the MVPCCMD ptr:
   pvsbqry = (PVSBQRY) PVOIDFROMMP (mp2);

   switch (pvsbqry->usQuery)
      {
      case VSB_QRY_STATUS:
         pvsbqry->hptrActive   = pvsbdata->hptrActive;
         pvsbqry->hptrBusy     = pvsbdata->hptrBusy;
         pvsbqry->usReturnCode = VSB_OKQRY;
         break;

      default:
         pvsbqry->usReturnCode = VSB_INVALIDQRY;
         return ((MRESULT)TRUE);
         break;
      }

   return ((MRESULT)FALSE);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
