
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBWM.c           (VSB Window Message Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions for processing many of the
//          window messages received by the Videodisc Scroll Bar (VSB).
//
//          Other window message functions can be found in VSBADV.C,
//          VSBCMD.C, and VSBQRY.C.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: ProcessCREATE                                Author: Steve Henson
//
// Desc: This function creates and initializes the Videodisc Scroll Bar.
//       Each VSB contains a window word data structure that contains
//       pertinent state information about itself.  This VSB creates a
//       "real" scroll bar for its visible representation and manipulates
//       it as required.
//
//*****************************************************************************

MRESULT ProcessCREATE (HWND hwnd, MPARAM mp2)
   {
   PCREATESTRUCT  pCreateStruct;    // Pointer to the CREATESTRUCT structure
   SHORT          xWidth;           // Width of ScrollBar interface
   SHORT          yHeigth;          // Heigth of ScrollBar interface
   HWND           hwndScrollBar;    // the ScrollBar handle
   PVSBDATA       pvsbdata;         // pointer to user interface instance data

   pCreateStruct = (PCREATESTRUCT) PVOIDFROMMP (mp2);
   xWidth  = pCreateStruct->cx;           // Get the Width of the SB
   yHeigth = pCreateStruct->cy;           // Get the Heigth of the SB

   if (pvsbdata = CreateVSBDATA ())       // create an instance data structure
      {
      WinSetWindowPtr (hwnd, VSB_WWOFF_PVSB, pvsbdata); // remember this ptr

      // Create the actual user interface controls (i.e., 1 scrollbar)
      hwndScrollBar = WinCreateWindow (hwnd,    // Parent window handle
                        WC_SCROLLBAR,           // Window class name
                        NULL,                   // Window text
                        WS_VISIBLE | SBS_HORZ   // Window style
                           | WS_DISABLED,       //   (not loaded yet)
                        0,0,                    // x, y,
                        xWidth,                 // cx,
                        yHeigth,                // cy,
                        hwnd,                   // Owner is also parent
                        HWND_TOP,               // on top
                        0,                      // Window ID
                        NULL,                   // No CTLDATA
                        NULL);                  // Reserved

      if (hwndScrollBar)
         {
         // Remember handle of physical scrollbar
         pvsbdata->hwndScrollBar = hwndScrollBar;

         // Subclass the Scroll Bar
         pvsbdata->pfnSBWndProc = WinSubclassWindow (pvsbdata->hwndScrollBar,
                                                     VSBSubclassWndProc);

         // Get and remember handle of the MVPC
         // this is who gets our commands and queries
         pvsbdata->hwndOwner = WinQueryWindow (hwnd, QW_OWNER, FALSE);

         // Create a permanent query structure
         MVPCCreateQry (&pvsbdata->pmvpcqry, MVPC_QRY_STATUS);

         // Remember what player we are associated with (for the title)
         pvsbdata->usPlayer = (USHORT) ((pCreateStruct->flStyle & WS_PLAYERMASK) + 1);

         pvsbdata->lFromFrame    = 1L;             // initialize frame range
         pvsbdata->lToFrame      = 54000L;
         pvsbdata->lCurrentFrame = 1L;
         pvsbdata->posSlider     = 1;

         WinSendMsg (hwndScrollBar,                // set scrollbar values
                     SBM_SETSCROLLBAR,
                     MPFROMSHORT  (1),
                     MPFROM2SHORT (1, usMaxSBRange));

         // Create the Log Dialog box (modeless)
         LogCreate (hwnd);                         // Hidden until shown

         return ((MRESULT) FALSE);   // return all OK
         }
      }

   return ((MRESULT)TRUE);       // return unsuccessful
   }


//**************************************************************************<f>
// Func: ProcessDESTROY                               Author: Steve Henson
//
// Desc: This function frees the VSBDATA window word structure and
//       destroys the query structure.
//
// Retn: Returns through WinDefWindowProc ()
//
//*****************************************************************************

MRESULT ProcessDESTROY (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   PVSBDATA pvsbdata;         // Pointer to the VSBDATA structure

   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   WinSubclassWindow (pvsbdata->hwndScrollBar, pvsbdata->pfnSBWndProc);
   pvsbdata->pfnSBWndProc = (PFNWP) NULL;

   MVPCDestroyQry (pvsbdata->pmvpcqry);

   // Destroy the Log Dialog box (modeless)
   LogDestroy (hwnd);

   DosFreeSeg (SELECTOROF (pvsbdata));                   // and release it

   return (WinDefWindowProc (hwnd, msg, mp1, mp2));
   }


//**************************************************************************<f>
// Func: ProcessSIZE                                  Author: Steve Henson
//
// Desc: This function sets the size of the "real" scroll bar.
//
// Retn: Returns through WinDefWindowProc ()
//       (MRESULT)0 = means we processed this message
//
//*****************************************************************************

MRESULT ProcessSIZE (HWND hwnd, MPARAM mp2)
   {
   PVSBDATA pvsbdata;
   SHORT    cy, cx;

   cx = SHORT1FROMMP (mp2);
   cy = SHORT2FROMMP (mp2);

   // Address this instance's data area in order to scrollbar handle
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   // Translate the WM_SIZE message into a WinSetWindowPos () call:
   WinSetWindowPos (pvsbdata->hwndScrollBar,
                    HWND_TOP,
                    0,0,
                    cx,
                    cy,
                    SWP_SIZE);

   return ((MRESULT)0);
   }


//**************************************************************************<f>
// Func: ProcessCONTROLPOINTER                        Author: Steve Henson
//
// Parm: HWND     hwnd;    -- The MVPC scroll bar window handle
//       MPARAM   mp2;     -- handle to mouse pointer
//
// Desc: This function process the WM_CONTROLPOINTER message for the MVPC
//       scrollbar. In processing this message, I set the handle to the mouse
//       pointer to that specified by the application if so requested, otherwise
//       return through handle given to this function in MP2.
//
// Retn: Returns through WinDefWindowProc ()
//
//***************************************************************************** */

MRESULT ProcessCONTROLPOINTER (HWND hwnd, MPARAM mp2)
   {
   HPOINTER hptrPointer;            // Handle to the pointer to use
   PVSBDATA pvsbdata;

   // address this instance's data area
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   if (pvsbdata->fInSliderTrack)
      return ((MRESULT) mp2);

   if (WinIsWindowEnabled (pvsbdata->hwndScrollBar))
      hptrPointer = pvsbdata->hptrActive;       // Enabled
   else
      hptrPointer = pvsbdata->hptrBusy;         // Disabled

   return ((MRESULT) hptrPointer ? hptrPointer : mp2);
   }


//**************************************************************************<f>
// Func: ProcessHSCROLL                                Author: Steve Henson
//
// Desc: processes the user interactions with the scrollbar
//
//       Action            If Stopped        If Moving
//
//       LINELEFT          Step Backward     Stop
//       LINERIGHT         Step Forward      Stop
//       PAGELEFT          Play Backward     Play Backward
//       PAGERIGHT         Play Forward      Play Forward
//       SLIDERPOSITION    Still @ position  Resume play from position
//
// Retn: Returns through WinDefWindowProc ()
//       (MRESULT)0 = means we processed this message
//
//*****************************************************************************

MRESULT ProcessHSCROLL (HWND hwnd, MPARAM mp2)
   {
   PVSBDATA pvsbdata;
   LONG     lFrame;
   SHORT    rcPlaying;
   HWND     hwndMVPCOwner;
   USHORT   usCmd;
   USHORT   posSlider;

   // Address the instance data
   pvsbdata = WinQueryWindowPtr (hwnd, VSB_WWOFF_PVSB);

   // Get handle to MVPC owner
   hwndMVPCOwner = pvsbdata->hwndOwner;

   // Determine player state
   WinSendMsg (hwndMVPCOwner, msgMVPC_QUERY, NULL, pvsbdata->pmvpcqry);

   rcPlaying = pvsbdata->pmvpcqry->usPlayStatus;

   switch (usCmd = SHORT2FROMMP (mp2))
      {
      //-----------------------------------------------------------------------
      case SB_SLIDERTRACK:
         pvsbdata->fInSliderTrack = TRUE;
         if (pvsbdata->hwndAdviseUser)
            {
            posSlider = SHORT1FROMMP(mp2);
            lFrame = MVPCUserRangeToFrame (hwndMVPCOwner, 1, usMaxSBRange, posSlider);
            AdviseUserInteraction (pvsbdata,
                                   (MPARAM) VSB_ADV_SLIDERTRACK,
                                   MPFROMLONG (lFrame),
                                   (MPARAM) NULL);
            }
         break;

      //-----------------------------------------------------------------------
      case SB_SLIDERPOSITION:   // user has repositioned slider
         pvsbdata->fInSliderTrack = FALSE;
         posSlider = SHORT1FROMMP(mp2);

         //-------------------------------------------------------------------
         // If we let the Slider up at the same location then we don't
         //    need to do anything.  Otherwise, we need to preload the
         //    scroll bar position since we can't set the scroll bar
         //    slider while the scroll bar is disabled.
         //-------------------------------------------------------------------
         if (posSlider != pvsbdata->posSlider)
            {
            pvsbdata->posSlider = posSlider;

            lFrame = MVPCUserRangeToFrame (hwndMVPCOwner, 1, usMaxSBRange, posSlider);
            switch (rcPlaying)
               {
               case MVDP_NOTPLAYING:
                  if (pvsbdata->hwndAdviseUser)
                     AdviseUserInteraction (pvsbdata,
                                            (MPARAM) MVDP_STILL,
                                            MPFROMLONG (lFrame),
                                            (MPARAM) NULL);

                  CmdSeek (hwndMVPCOwner, hwnd, lFrame);
                  break;

               case MVDP_PLAYFORWARD:
                  if (pvsbdata->hwndAdviseUser)
                     AdviseUserInteraction (pvsbdata,
                                            (MPARAM) MVDP_PLAYFT,
                                            MPFROMLONG (lFrame),
                                            MPFROMLONG (pvsbdata->lToFrame));

                  CmdPlayFromTo (hwndMVPCOwner, hwnd,
                                 lFrame, pvsbdata->lToFrame, TRUE);
                  break;

               case MVDP_PLAYREVERSE:
                  if (pvsbdata->hwndAdviseUser)
                     AdviseUserInteraction (pvsbdata,
                                            (MPARAM) MVDP_PLAYFT,
                                            MPFROMLONG (lFrame),
                                            MPFROMLONG (pvsbdata->lFromFrame));

                  CmdPlayFromTo (hwndMVPCOwner, hwnd,
                                 lFrame, pvsbdata->lFromFrame, TRUE);
                  break;
               }
            }
         break;

      //-----------------------------------------------------------------------
      case SB_PAGELEFT:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
            case MVDP_PLAYFORWARD:
               if (pvsbdata->hwndAdviseUser)
                  AdviseUserInteraction (pvsbdata,
                                         (MPARAM) MVDP_PLAYFT,
                                         MPFROMLONG (-1L),
                                         MPFROMLONG (pvsbdata->lFromFrame));

               CmdPlayFromTo (hwndMVPCOwner, hwnd,
                              -1L, pvsbdata->lFromFrame, TRUE);
               break;

            case MVDP_PLAYREVERSE:     // already playing backwards
               if (!pvsbdata->pmvpcqry->fActive)  // do nothing unless not active
                  CmdGetToken (hwndMVPCOwner, hwnd);
               break;
            }
         break;

      //-----------------------------------------------------------------------
      case SB_PAGERIGHT:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
            case MVDP_PLAYREVERSE:
               if (pvsbdata->hwndAdviseUser)
                  AdviseUserInteraction (pvsbdata,
                                         (MPARAM) MVDP_PLAYFT,
                                         MPFROMLONG (-1L),
                                         MPFROMLONG (pvsbdata->lToFrame));

               CmdPlayFromTo (hwndMVPCOwner, hwnd,
                              -1L, pvsbdata->lToFrame, TRUE);
               break;

            case MVDP_PLAYFORWARD:  // already playing forwards
               if (!pvsbdata->pmvpcqry->fActive)  // do nothing unless not active
                  CmdGetToken (hwndMVPCOwner, hwnd);
               break;
            }
         break;

      //-----------------------------------------------------------------------
      case SB_LINELEFT:
      case SB_LINERIGHT:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
               if (pvsbdata->hwndAdviseUser)
                  AdviseUserInteraction (pvsbdata,
                                         (MPARAM) ((usCmd == SB_LINELEFT) ? MVDP_STEPB
                                                                          : MVDP_STEPF),
                                         NULL,
                                         NULL);

               CmdStep (hwndMVPCOwner, hwnd,
                        (usCmd == SB_LINELEFT) ? BACKWARD : FORWARD);
               break;

            case MVDP_PLAYFORWARD:
            case MVDP_PLAYREVERSE:
               if (pvsbdata->hwndAdviseUser)
                  AdviseUserInteraction (pvsbdata,
                                         (MPARAM) MVDP_STOP,
                                         NULL,
                                         NULL);

               CmdStop (hwndMVPCOwner, hwnd);
               break;
            }
         break;
      }

   return ((MRESULT) 0);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
