
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBWP.c           (VSB Window Procedure)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains the main window procedure for the
//          Videodisc Scroll Bar (VSB) DLL.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#define INCL_WIN

#include <os2.h>

#include "mvpc.h"       // Include for the MVPC object
#include "vsb.h"        // Public  includes
#include "vsb2.h"       // Private includes


//**************************************************************************<f>
// Func: VSBWndProc                                   Author: Steve Henson
//
// Parm: HWND     hwnd;    -- The user interface window handle
//       USHORT   msg;     -- The message
//       MPARAM   mp1;     -- message specific
//       MPARAM   mp2;     -- message specific
//
// Desc: The window procedure for this user interface.  Handles messages
//       from the interface controls.
//
//*****************************************************************************

MRESULT EXPENTRY VSBWndProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {

   switch (msg)
      {
      case WM_CONTROLPOINTER:
         return (ProcessCONTROLPOINTER (hwnd, mp2));
         break;

      case WM_HSCROLL:
         return (ProcessHSCROLL (hwnd, mp2));
         break;

      case WM_CREATE:
         return (ProcessCREATE (hwnd, mp2));
         break;

      case WM_SIZE:
         return (ProcessSIZE (hwnd, mp2));
         break;

      case WM_QUERYDLGCODE:
         return ((MRESULT) DLGC_SCROLLBAR);
         break;

      case WM_DESTROY:
         return (ProcessDESTROY (hwnd, msg, mp1, mp2));
         break;

      default:
         if (msg == msgMVPC_ADVISE)
            {
            return (ProcessADVISE (hwnd, mp1, mp2));
            }
         else if (msg == msgMVPC_ACK)
            {
            return (ProcessACK (hwnd, msg, mp1, mp2));
            }
         else if (msg == msgMVPC_RESP)
            {
            return (ProcessRESP (hwnd, msg, mp1, mp2));
            }
         else if (msg == msgMVPC_COMMAND)
            {
            return (ProcessCOMMAND (hwnd, mp2));
            }
         else if (msg == msgMVPC_QUERY)
            {
            return (ProcessQUERY (hwnd, mp2));
            }
         else
            return WinDefWindowProc (hwnd, msg, mp1, mp2);
      }

   return ((MRESULT)0);
   }


//**************************************************************************<f>
// Func: VSBSubclassWndProc                           Author: Daryl Warinner
//
// Parm: HWND     hwnd;    -- The user interface window handle
//       USHORT   msg;     -- The message
//       MPARAM   mp1;     -- message specific
//       MPARAM   mp2;     -- message specific
//
// Desc: This window procedure subclasses the "real" scroll bar and traps
//       out right button double clicks to activate (show) the Log Window.
//
//*****************************************************************************

MRESULT EXPENTRY VSBSubclassWndProc (HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
   {
   PVSBDATA    pvsbdata;

   pvsbdata = WinQueryWindowPtr (WinQueryWindow (hwnd, QW_OWNER, FALSE),
                                 VSB_WWOFF_PVSB);

   switch (msg)
      {
      case WM_BUTTON1UP:
         //-------------------------------------------------------------------
         // Rudimentary Player Recovery:
         //
         //    If we get a button 1 hit while the scroll bar is disabled and
         //    the reason for disabling was that the player is unloaded,
         //
         //    Then we will send an init command to the player.  Nothing Fancy.
         //
         //-------------------------------------------------------------------
         if ((!WinIsWindowEnabled (hwnd))
            && !pvsbdata->fLoaded)
            {
            CmdInit (pvsbdata->hwndOwner, hwnd);
            }
         break;

      case WM_BUTTON2DBLCLK:
         LogShow (WinQueryWindow (hwnd, QW_OWNER, FALSE), TRUE);
         return ((MRESULT) 0);

      case WM_ENABLE:
         if (SHORT1FROMMP (mp1))          // If enabling
            {
            // If we are enabling, then refresh the scroll bar position
            //    in case it changed while we were disabled.
            WinSendMsg (pvsbdata->hwndScrollBar, SBM_SETPOS,
                        MPFROMSHORT (pvsbdata->posSlider), NULL);
            }
         break;
      }

   return ((pvsbdata->pfnSBWndProc)(hwnd, msg, mp1, mp2));
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
