/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  ATTRIBUT.C                                                       */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: Window Procs for Video and Audio Attributes                      */
/*          Also Contains About Dialog Proc                                  */
/*****************************************************************************/

/*****************************************************************************/
/* Defines and Includes                                                      */
/*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EXTERN  extern
#include "BROWDEFS.H"                 // This Program's Definitions

static  HWND  hwndBright;             // Handle of Brightness Scroll Bar Window
static  HWND  hwndContrast;           // Handle of Contrast Scroll Bar Window
static  HWND  hwndColor;              // Handle of Color Scroll Bar Window
static  HWND  hwndTint;               // Handle of Tint Scroll Bar Window
static  HWND  hwndSharpness;          // Handle of Sharpness Scroll Bar Window

static  HWND  hwndVolume;             // Handle of Volume Scroll Bar Window
static  HWND  hwndBalance;            // Handle of Balance Scroll Bar Window
static  HWND  hwndBass;               // Handle of Bass Scroll Bar Window
static  HWND  hwndTreble;             // Handle of Treble Scroll Bar Window

static SHORT   volume, balance, bass, treble;             // Audio Values
static BOOL    fMuted;
static MVWNQRY AudioQuery;

static SHORT   bright, contrast, color, tint, sharpness;  // Video Values
static MVWNQRY VideoQuery;

static VOID AttrCommand( WORD, LONG );

/*****************************************************************************/
/* Func: VideoAttributesProc                                                 */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: This is the window procedure for the Video Attributes Window.       */
/*****************************************************************************/

WORD FAR PASCAL VideoAttributesProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 HWND  hwndScrollBar;
 SHORT NewValue;
 char  temp[STRBUFLEN];

 switch ( Message )
    {
    case WM_INITDIALOG:    // dialog box init

       // Query current values so they can be restored on a Cancel

       UpdateVideoStatus( ( LPMVWNQRY ) &VideoQuery );

       bright    = VideoQuery.mvidst.wBrightness;
       contrast  = VideoQuery.mvidst.wContrast;
       color     = VideoQuery.mvidst.wColor;
       tint      = VideoQuery.mvidst.sTint;
       sharpness = VideoQuery.mvidst.wSharpness;

       hwndBright = GetDlgItem( hwnd, ID_BRIGHTNESS_SB );
       SetScrollRange( hwndBright, SB_CTL, 0, 100, TRUE );
       SetScrollPos( hwndBright, SB_CTL, bright, TRUE );
       wsprintf( temp, ctlstr, bright );
       SetDlgItemText( hwnd, ID_BRIGHTNESS, temp );

       hwndContrast = GetDlgItem( hwnd, ID_CONTRAST_SB );
       SetScrollRange( hwndContrast, SB_CTL, 0, 100, TRUE );
       SetScrollPos( hwndContrast, SB_CTL, contrast, TRUE );
       wsprintf( temp, ctlstr, contrast );
       SetDlgItemText( hwnd, ID_CONTRAST, temp );

       hwndColor = GetDlgItem( hwnd, ID_COLOR_SB );
       SetScrollRange( hwndColor, SB_CTL, 0, 100, TRUE );
       SetScrollPos( hwndColor, SB_CTL, color, TRUE );
       wsprintf( temp, ctlstr, color );
       SetDlgItemText( hwnd, ID_COLOR, temp );

       hwndTint = GetDlgItem( hwnd, ID_TINT_SB );
       SetScrollRange( hwndTint, SB_CTL, -100, 100, TRUE );
       SetScrollPos( hwndTint, SB_CTL, tint, TRUE );
       wsprintf( temp, ctlstr, tint );
       SetDlgItemText( hwnd, ID_TINT, temp );

       hwndSharpness = GetDlgItem( hwnd, ID_SHARP_SB );
       SetScrollRange( hwndSharpness, SB_CTL, 0, 100, TRUE );
       SetScrollPos( hwndSharpness, SB_CTL, sharpness, TRUE );
       wsprintf( temp, ctlstr, sharpness );
       SetDlgItemText( hwnd, ID_SHARP, temp );

       PostMessage( hwnd, WM_USER+1, 0, 0L );
       break;

    case WM_ACTIVATE:
       if  ( wParam  &&  hwndVideoWindow )      // If Being Activated
          { if  ( !FullScreenVideo )
                 SetWindowPos( hwndFrame, hwnd, 0, 0, 0, 0,
                               SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE );
            BecomingActive();
            }
       break;

    case WM_USER+1:
       SetBoxPosition( hwndVideoAttributes, TOP );
       ShowWindow( hwndVideoAttributes, SHOW_OPENWINDOW );
       UpdateWindow( hwndVideoAttributes );
       break;

    case WM_HSCROLL:
       hwndScrollBar = HIWORD( lParam );
       if  ( hwndScrollBar == hwndBright )
          {  switch ( wParam )               // adjust new value
                {
                case SB_TOP:
                   bright = 100;
                   break;

                case SB_BOTTOM:
                   bright = 0;
                   break;

                case SB_LINEDOWN:
                   bright++;
                   break;

                case SB_LINEUP:
                   bright--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   bright = ( wParam == SB_PAGEUP ) ? bright - 10
                                                    : bright + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= 0 ) &&
                         ( NewValue <= 100 ) )
                        bright = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

             if  ( bright < 0 )
                  bright = 0;
             else if ( bright > 100 )
                  bright = 100;
             if  ( wParam != SB_THUMBTRACK )
                  SetScrollPos( hwndBright, SB_CTL, bright, TRUE );
             wsprintf( temp, ctlstr, bright );
             SetDlgItemText( hwnd, ID_BRIGHTNESS, temp );
             AttrCommand( MVWN_CMD_BRIGHTNESS, ( LONG ) bright );
             }

       if  ( hwndScrollBar == hwndContrast )
          {  switch ( wParam )                // adjust new value
                {
                case SB_TOP:
                   contrast = 100;
                   break;

                case SB_BOTTOM:
                   contrast = 0;
                   break;

                case SB_LINEDOWN:
                   contrast++;
                   break;

                case SB_LINEUP:
                   contrast--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   contrast = ( wParam == SB_PAGEUP ) ? contrast - 10
                                                      : contrast + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= 0 ) &&
                         ( NewValue <= 100 ) )
                        contrast = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

             if  ( contrast < 0 )
                  contrast = 0;
              else if ( contrast > 100 )
                  contrast = 100;
             if  ( wParam != SB_THUMBTRACK )
                  SetScrollPos( hwndContrast, SB_CTL, contrast, TRUE );
             wsprintf( temp, ctlstr, contrast );
             SetDlgItemText( hwnd, ID_CONTRAST, temp );
             AttrCommand( MVWN_CMD_CONTRAST, ( LONG ) contrast );
             }

       if  ( hwndScrollBar == hwndColor )
          {  switch ( wParam )   // adjust new value
                {
                case SB_TOP:
                   color = 100;
                   break;

                case SB_BOTTOM:
                   color = 0;
                   break;

                case SB_LINEDOWN:
                   color++;
                   break;

                case SB_LINEUP:
                   color--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   color = ( wParam == SB_PAGEUP ) ? color - 10 : color + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= 0 ) &&
                         ( NewValue <= 100 ) )
                        color = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

             if  ( color < 0 )
                  color = 0;
             else if ( color > 100 )
                  color = 100;
             if  ( wParam != SB_THUMBTRACK )
                  SetScrollPos( hwndColor, SB_CTL, color, TRUE );
             wsprintf( temp, ctlstr, color );
             SetDlgItemText( hwnd, ID_COLOR, temp );
             AttrCommand( MVWN_CMD_COLOR, ( LONG ) color );
             }

       if  ( hwndScrollBar == hwndTint )
          { switch ( wParam )   // adjust new value
                {
                case SB_TOP:
                   tint = 100;
                   break;

                case SB_BOTTOM:
                   tint = -100;
                   break;

                case SB_LINEDOWN:
                   tint++;
                   break;

                case SB_LINEUP:
                   tint--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   tint = ( wParam == SB_PAGEUP ) ? tint - 10 : tint + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= -100 ) &&
                         ( NewValue <= 100 ) )
                        tint = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

            if ( tint < -100 )
                tint = -100;
            else if ( tint > 100 )
                tint = 100;
            if  ( wParam != SB_THUMBTRACK )
                 SetScrollPos( hwndTint, SB_CTL, tint, TRUE );
            wsprintf( temp, ctlstr, tint );
            SetDlgItemText( hwnd, ID_TINT, temp );
            AttrCommand( MVWN_CMD_TINT, ( LONG ) tint );
            }

       if  ( hwndScrollBar == hwndSharpness )
          {  switch ( wParam )                // adjust new value
                {
                case SB_TOP:
                   sharpness = 100;
                   break;

                case SB_BOTTOM:
                   sharpness = 0;
                   break;

                case SB_LINEDOWN:
                   sharpness++;
                   break;

                case SB_LINEUP:
                   sharpness--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   sharpness = ( wParam == SB_PAGEUP ) ? sharpness - 10
                                                       : sharpness + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= 0 ) &&
                         ( NewValue <= 100 ) )
                        sharpness = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

             if  ( sharpness < 0 )
                  sharpness = 0;
              else if ( sharpness > 100 )
                  sharpness = 100;
             if  ( wParam != SB_THUMBTRACK )
                  SetScrollPos( hwndSharpness, SB_CTL, sharpness, TRUE );
             wsprintf( temp, ctlstr, sharpness );
             SetDlgItemText( hwnd, ID_SHARP, temp );
             AttrCommand( MVWN_CMD_SHARPNESS, ( LONG ) sharpness );
             }
       break;

    case WM_COMMAND:
       switch ( wParam )
          {
          case IDCANCEL:
             AttrCommand( MVWN_CMD_BRIGHTNESS,
                          ( LONG ) VideoQuery.mvidst.wBrightness );
             AttrCommand( MVWN_CMD_CONTRAST,
                          ( LONG ) VideoQuery.mvidst.wContrast );
             AttrCommand( MVWN_CMD_COLOR,
                          ( LONG ) VideoQuery.mvidst.wColor );
             AttrCommand( MVWN_CMD_TINT,
                          ( LONG ) VideoQuery.mvidst.sTint );
             AttrCommand( MVWN_CMD_SHARPNESS,
                          ( LONG ) VideoQuery.mvidst.wSharpness );

             // Fall through and dismiss the dialog

          case IDOK:
             hwndVideoAttributes = NULL;
             DestroyWindow( hwnd );
             if  ( FullScreenVideo )
                  SetFocus( hwndVideoWindow );
             else
                  SetFocus( hwndFrame );
             return( TRUE );
          }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: AudioAttributesProc                                                 */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: This is the window procedure for the Audio attributes window.       */
/*****************************************************************************/

WORD FAR PASCAL AudioAttributesProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 char  temp[STRBUFLEN];
 HWND  hwndScrollBar;
 SHORT NewValue;

 switch ( Message )
     {
    case WM_INITDIALOG:    // dialog box init

       // Query current values so they can be restored on a Cancel

       UpdateVideoStatus( ( LPMVWNQRY ) &AudioQuery );

       volume  = AudioQuery.maudst.wVolume;
       balance = AudioQuery.maudst.sBalance;
       bass    = AudioQuery.maudst.sBass;
       treble  = AudioQuery.maudst.sTreble;
       fMuted  = AudioQuery.maudst.bMuted;

       hwndVolume = GetDlgItem( hwnd, ID_VOLUME_SB );
       SetScrollRange( hwndVolume, SB_CTL, 0, 100, TRUE );
       SetScrollPos( hwndVolume, SB_CTL, volume, TRUE );
       wsprintf( temp, ctlstr, volume );
       SetDlgItemText( hwnd, ID_VOLUME, temp );

       hwndBalance = GetDlgItem( hwnd, ID_BALANCE_SB );
       SetScrollRange( hwndBalance, SB_CTL, -100, 100, TRUE );
       SetScrollPos( hwndBalance, SB_CTL, balance, TRUE );
       wsprintf( temp, ctlstr, balance );
       SetDlgItemText( hwnd, ID_BALANCE, temp);

       hwndBass = GetDlgItem( hwnd, ID_BASS_SB );
       SetScrollRange( hwndBass, SB_CTL, -100, 100, TRUE );
       SetScrollPos( hwndBass, SB_CTL, bass, TRUE );
       wsprintf( temp, ctlstr, bass );
       SetDlgItemText( hwnd, ID_BASS, temp );

       hwndTreble = GetDlgItem( hwnd, ID_TREBLE_SB );
       SetScrollRange( hwndTreble, SB_CTL, -100, 100, TRUE );
       SetScrollPos( hwndTreble, SB_CTL, treble, TRUE );
       wsprintf( temp, ctlstr, treble );
       SetDlgItemText( hwnd, ID_TREBLE, temp );

       CheckDlgButton( hwnd, ID_MUTE, fMuted );
       PostMessage( hwnd, WM_USER+1, 0, 0L );
       break;

    case WM_ACTIVATE:
       if  ( wParam  &&  hwndVideoWindow )      // If Being Activated
          { if  ( !FullScreenVideo )
                 SetWindowPos( hwndFrame, hwnd, 0, 0, 0, 0,
                               SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE );
            BecomingActive();
            }
       break;

    case WM_USER+1:
       SetBoxPosition( hwndAudioAttributes, BOTTOM );
       ShowWindow( hwndAudioAttributes, SHOW_OPENWINDOW );
       UpdateWindow( hwndAudioAttributes );
       break;

    case WM_HSCROLL:
       hwndScrollBar = HIWORD( lParam );

       if  ( hwndScrollBar == hwndVolume )
          { switch ( wParam )             // adjust new value
                {
                case SB_TOP:
                   volume = 100;
                   break;

                case SB_BOTTOM:
                   volume = 0;
                   break;

                case SB_LINEDOWN:
                   volume++;
                   break;

                case SB_LINEUP:
                   volume --;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   volume = ( wParam == SB_PAGEUP ) ? volume - 10
                                                    : volume + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= 0 ) &&
                         ( NewValue <= 100 ) )
                        volume = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

            if  ( volume < 0 )
                 volume = 0;
            else if ( volume > 100 )
                 volume = 100;
            if  ( wParam != SB_THUMBTRACK )
                 SetScrollPos( hwndVolume, SB_CTL, volume, TRUE );
            wsprintf( temp, ctlstr, volume );
            SetDlgItemText( hwnd, ID_VOLUME, temp );
            AttrCommand( MVWN_CMD_VOLUME, ( LONG ) volume );
            }

       if  ( hwndScrollBar == hwndBalance )
          { switch ( wParam )   // adjust new value
                {
                case SB_TOP:
                   balance = 100;
                   break;

                case SB_BOTTOM:
                   balance = -100;
                   break;

                case SB_LINEDOWN:
                   balance++;
                   break;

                case SB_LINEUP:
                   balance--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   balance = ( wParam == SB_PAGEUP ) ? balance - 10
                                                     : balance + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= -100 ) &&
                         ( NewValue <= 100 ) )
                        balance = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

            if  ( balance < -100 )
                 balance = -100;
            else if ( balance > 100 )
                 balance = 100;
            if  ( wParam != SB_THUMBTRACK )
                 SetScrollPos( hwndBalance, SB_CTL, balance, TRUE );
            wsprintf( temp, ctlstr, balance );
            SetDlgItemText( hwnd, ID_BALANCE, temp );
            AttrCommand( MVWN_CMD_BALANCE, ( LONG ) balance );
            }

       if  ( hwndScrollBar == hwndBass )
          { switch ( wParam )   // adjust new value
                {
                case SB_TOP:
                   bass = 100;
                   break;

                case SB_BOTTOM:
                   bass = -100;
                   break;

                case SB_LINEDOWN:
                   bass++;
                   break;

                case SB_LINEUP:
                   bass--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   bass = ( wParam == SB_PAGEUP ) ? bass - 10 : bass + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= -100 ) &&
                         ( NewValue <= 100 ) )
                        bass = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

            if  ( bass < -100 )
                 bass = -100;
            else if ( bass > 100 )
                 bass = 100;
            if  ( wParam != SB_THUMBTRACK )
                 SetScrollPos( hwndBass, SB_CTL, bass, TRUE );
            wsprintf( temp, ctlstr, bass );
            SetDlgItemText( hwnd, ID_BASS, temp );
            AttrCommand( MVWN_CMD_BASS, ( LONG ) bass );
            }

       if  ( hwndScrollBar == hwndTreble )
          { switch ( wParam )   // adjust new value
                {
                case SB_TOP:
                   treble = 100;
                   break;

                case SB_BOTTOM:
                   treble = -100;
                   break;

                case SB_LINEDOWN:
                   treble++;
                   break;

                case SB_LINEUP:
                   treble--;
                   break;

                case SB_PAGEUP:
                case SB_PAGEDOWN:
                   treble = ( wParam == SB_PAGEUP ) ? treble - 10
                                                    : treble + 10;
                   break;

                case SB_THUMBTRACK:
                case SB_THUMBPOSITION:
                   NewValue = LOWORD( lParam );
                   if  ( ( NewValue >= -100 ) &&
                         ( NewValue <= 100 ) )
                        treble = NewValue;
                   break;

                case SB_ENDSCROLL:
                   break;
                }

            if  ( treble > 100 )
                 treble = 100;
            if  ( treble < -100 )
                 treble = -100;
            if  ( wParam != SB_THUMBTRACK )
                 SetScrollPos( hwndTreble, SB_CTL, treble, TRUE );
            wsprintf( temp, ctlstr, treble );
            SetDlgItemText( hwnd, ID_TREBLE, temp );
            AttrCommand( MVWN_CMD_TREBLE, ( LONG ) treble );
            }
       return( TRUE );
       break;

    case WM_COMMAND:
       switch ( wParam )
          {
          case IDCANCEL:
             AttrCommand( MVWN_CMD_VOLUME,
                          ( LONG ) AudioQuery.maudst.wVolume );
             AttrCommand( MVWN_CMD_BALANCE,
                          ( LONG ) AudioQuery.maudst.sBalance );
             AttrCommand( MVWN_CMD_BASS,
                          ( LONG ) AudioQuery.maudst.sBass );
             AttrCommand( MVWN_CMD_TREBLE,
                          ( LONG ) AudioQuery.maudst.sTreble );
             AttrCommand( MVWN_CMD_MUTE,
                          ( LONG ) AudioQuery.maudst.bMuted );

             // Fall through and dismiss the dialog

          case IDOK:
             hwndAudioAttributes = NULL;
             DestroyWindow( hwnd );
             if  ( FullScreenVideo )
                  SetFocus( hwndVideoWindow );
             else
                  SetFocus( hwndFrame );
             return( TRUE );

          case ID_MUTE:
             if  ( fMuted )
                  fMuted = FALSE;
             else
                  fMuted = TRUE;
             CheckDlgButton( hwnd, ID_MUTE, fMuted );
             AttrCommand( MVWN_CMD_MUTE, ( LONG ) fMuted );
             break;
          }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* FUNCTION: AboutDlgProc()                                                  */
/*                                                                           */
/* PARM: HWND   hwnd          - window handle                                */
/*       WORD   Message       - the message                                  */
/*       WORD   wParam        - word length data for the message             */
/*       DWORD  lParam        - long length data for the message             */
/*                                                                           */
/* DESCRIPTION: Display "About Browser" Info Dialog                          */
/*****************************************************************************/

WORD FAR PASCAL AboutDlgProc( hDlg, Message, wParam, lParam )
HWND hDlg;                                /* Window Handle of the Dialog Box */
WORD Message;                             /* Type of Message                 */
WORD wParam;                              /* Message-Specific Information    */
LONG lParam;
{
 switch ( Message )
       { case WM_INITDIALOG:               /* Message: Initialize Dialog Box */
              return( TRUE );

         case WM_COMMAND:                  /* Message: Received a Command    */
              if ( wParam == IDOK  ||      /* "OK" Box Selected?             */
                   wParam == IDCANCEL )    /* System Menu Close Command ?    */
                { EndDialog( hDlg, TRUE ); /* Exit the Dialog Box            */
                  return( TRUE );
                  }
              break;
         }
 return( FALSE );                          /* Didn't Process a Message       */
 }


/*****************************************************************************/
/* Func: AttrCommand                                                         */
/*                                                                           */
/* Parm: USHORT   usCommand;                                                 */
/*       MPARAM   mp1;                                                       */
/*                                                                           */
/* Desc: Video Window generic command function.  All commands in attributes  */
/*       to the video window get funneled through here.                      */
/*       NOTE: We do SendMessage() with no response so the scroll bars don't */
/*             overload Windows with unneccessary messages.                  */
/*****************************************************************************/

static VOID AttrCommand( Command, mp1 )
WORD Command;
LONG mp1;
{
 LPMVWNCMD  pmvwncmd;
 HANDLE CommandHandle;

 if  ( IsWindow( hwndVideoWindow ) == TRUE )
    { if  ( CommandHandle = MVWNCreateCmd( Command ) )   // IF Created Command
         { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
           if  ( pmvwncmd )
              { pmvwncmd->data.dwCmdParam1 = mp1;        // Set up Command Parm
                GlobalUnlock( CommandHandle );
                SendMessage( hwndVideoWindow, mvwnCommandMsg,
                             NULL, ( LONG ) CommandHandle );
                MVWNDestroyCmd( CommandHandle, hwndVideoWindow, MVWN_ACK );
                }
           }
      }
 return;
 }
