/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  BROWSER.C                                                        */
/* VERSION: M-Control Program / 2     Windows 3.0 ( TM ) Version             */
/*                                                                           */
/* PURPOSE: Videodisc Browser Utility demonstrating programming              */
/*          concepts of the Multimedia Toolkit for Windows                   */
/*****************************************************************************/

/*****************************************************************************/
/* Defines and Includes                                                      */
/*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <direct.h>

#define  EXTERN
#include "BROWDEFS.H"        // This Program's Definitions
#include "BROWHELP.H"


/*****************************************************************************/
/* Local Functions                                                           */
/*****************************************************************************/

static  VOID   CheckBrowserMenuItem( WORD );
static  VOID   DisableAllFileItems( VOID );
static  VOID   DisableAllPlayerOptions( VOID );
static  VOID   DisableMenuItem( WORD );
static  WORD   GetHelpId( WORD );
static  VOID   EnableAllPlayerOptions( VOID );
static  VOID   EnableBrowserMenuItem( WORD);
static  BOOL   PASCAL InitApplication( HANDLE );
static  BOOL   PASCAL InitInstance( HANDLE );
static  VOID   InitMyProgram( PSZ );
static  VOID   UnCheckBrowserMenuItem( WORD );


/*****************************************************************************/
/* Func: WinMain                                                             */
/*                                                                           */
/* Parm: HANDLE hInstance     - this task instance                           */
/*       HANDLE hPrevInstance - the last task instance                       */
/*       LPSTR  lpszCmdLine   - the command line params                      */
/*       int    nCmdShow      - the initial window state                     */
/*                                                                           */
/* Desc: This is the main function that begins the BROWSER program.          */
/*****************************************************************************/

int PASCAL WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR  lpszCmdLine;
int    nCmdShow;
{
 MSG  msg;
 WORD i;
 HCURSOR OriginalCursor;

 HourGlass = LoadCursor( NULL, IDC_WAIT );  // Change the Pointer to Hour Glass
 OriginalCursor = SetCursor( HourGlass );

 if  ( !hPrevInstance )                     // Is this is the First instance ?
    { if  ( !InitApplication( hInstance ) )
           return( FALSE );
      }

 if  ( !InitInstance( hInstance ) )         // Initial This Instance
      return( FALSE );

 SetCursor( OriginalCursor );              // Change Pointer Back to Arrow Ptr
 ShowWindow( hwndFrame, nCmdShow );         // Display the Application Window
 UpdateWindow( hwndFrame );

 while ( GetMessage( &msg, NULL, NULL, NULL ) )
      { /** Only Translate the Message If it is Not an Accelerator Message **/
        if  ( !TranslateAccelerator( hwndFrame, AccTableHandle, &msg ) )
           { if  ( hwndVideoAttributes  &&
                   IsDialogMessage( hwndVideoAttributes, &msg ) )
                  continue;
             if  ( hwndAudioAttributes  &&
                   IsDialogMessage( hwndAudioAttributes, &msg ) )
                  continue;
             if  ( hwndRestoreImage  &&
                   IsDialogMessage( hwndRestoreImage, &msg ) )
                  continue;
             TranslateMessage( &msg );
             DispatchMessage( &msg );
             }
        }

 /***********************************************/
 /* Application Terminated, Clean Up Loose Ends */
 /***********************************************/

 MVPCTerminate();                               // Terminate MVPCWIN.DLL
 MVWNTerminate();                               // Terminate MVWNWIN.DLL

 for  ( i = 0; i < 3; i++ )                     // Destroy System Cursors
     { if  ( hptrActive[i] )
            DestroyCursor( hptrActive[i] );
       if  ( hptrBusy[i] )
            DestroyCursor( hptrBusy[i] );
       if  ( hptrVid[i] )
            DestroyCursor( hptrVid[i] );
       }

 if  ( HelpCursor )                             // Destroy Help Cursor
      DestroyCursor( HelpCursor );

 if  ( HourGlass )
      DestroyCursor( HourGlass );               // Don't Need Hour Glass Cursor Now

 return( msg.wParam );
 }


/*****************************************************************************/
/* Func: InitApplication                                                     */
/*                                                                           */
/* Parm: HANDLE hInstance     - this task instance                           */
/*                                                                           */
/* Desc: Create the main window class                                        */
/*****************************************************************************/

static BOOL PASCAL InitApplication( hInstance )
HANDLE hInstance;
{
  WNDCLASS  wc;
  CHAR ClassName[50];

  /**********************************/
  /* Register the Main Window Class */
  /**********************************/

  wc.style         = CS_DBLCLKS;
  wc.lpfnWndProc   = MainWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon( hInstance, "BrowserMinIcon" );
  wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
  wc.hbrBackground = GetStockObject( WHITE_BRUSH );
  wc.lpszMenuName  = "BrowserMenu";
  wc.lpszClassName = "BrowserWClass";

  if  ( !RegisterClass( &wc ) )
       return( FALSE );


 /*****************************************************/
 /* Register the Player Control Response Window Class */
 /*****************************************************/

  LoadString( hInstance, ID_MVPCRESPONSECLASS,
              ClassName, sizeof( ClassName ) );

  wc.style         = NULL;
  wc.lpfnWndProc   = MVPCResponseWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = NULL;
  wc.hbrBackground = GetStockObject( WHITE_BRUSH );
  wc.lpszMenuName  = ( LPSTR ) NULL;
  wc.lpszClassName = ClassName;

 if  ( !RegisterClass( &wc ) )
      return( FALSE );

 /***************************************************/
 /* Register the Player Control Advise Window Class */
 /***************************************************/

  LoadString( hInstance, ID_MVPCADVISECLASS,
              ClassName, sizeof( ClassName ) );

  wc.style         = NULL;
  wc.lpfnWndProc   = MVPCAdviseWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = NULL;
  wc.hbrBackground = GetStockObject( WHITE_BRUSH );
  wc.lpszMenuName  = ( LPSTR ) NULL;
  wc.lpszClassName = ClassName;

  if  ( !RegisterClass( &wc ) )
       return(FALSE);

 /****************************************************/
 /* Register the Video Window Response Window Class  */
 /****************************************************/

  LoadString( hInstance, ID_MVWNRESPONSECLASS,
              ClassName, sizeof( ClassName ) );

  wc.style         = NULL;
  wc.lpfnWndProc   = MVWNResponseWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = NULL;
  wc.hCursor       = NULL;
  wc.hbrBackground = GetStockObject( WHITE_BRUSH );
  wc.lpszMenuName  = ( LPSTR ) NULL;
  wc.lpszClassName = ClassName;

  if  ( !RegisterClass( &wc ) )
       return(FALSE);

  return( TRUE );
  }


/*****************************************************************************/
/* Func: InitInstance                                                        */
/*                                                                           */
/* Parm: HANDLE hInstance     - this task instance                           */
/*                                                                           */
/* Desc: Init info for this instance                                         */
/*****************************************************************************/

static BOOL PASCAL InitInstance( hInstance )
HANDLE hInstance;
{
 CHAR     szTempBuffer[80];
 RECT     Rect;
 WORD     rc;
 WORD     i;

 hInst = hInstance;                         // Save This Instance Handle

 if  ( SetMessageQueue( 32 ) == 0 )         // Try to Get a Few Extra Msgs
    { if  ( SetMessageQueue( 16 ) == 0 )    // Since Browser is so "Busy"
         { if  ( SetMessageQueue( 8 ) == 0 )
                return( FALSE );
           }
      }

 // Load the Accelerator Table

 AccTableHandle = LoadAccelerators( hInst, "BrowserAcc" );

 // Create the Main Application Window

 hwndFrame = CreateWindow( "BrowserWClass",
                           "Video Browser",
                           WS_OVERLAPPEDWINDOW |
                           WS_CLIPCHILDREN,
                           CW_USEDEFAULT,
                           CW_USEDEFAULT,
                           CW_USEDEFAULT,
                           CW_USEDEFAULT,
                           NULL,
                           NULL,
                           hInstance,
                           NULL );

 if  ( !hwndFrame )
      return( FALSE );

 /*******************************/
 /* Initialize Global Variables */
 /*******************************/

 FullScreenVideo      = 0;
 hwndRestoreImage     = NULL;
 hwndVideoAttributes  = NULL;
 hwndAudioAttributes  = NULL;
 hwndMVPC             = NULL;
 hwndDeskTop          = GetDesktopWindow();
 fTiePlayersToSources = TRUE;
 usVideoSource        = 1;                       // Current Video Source = 1
 fromFrame            = 1L;                      // Set Current Frame Range
 toFrame              = 54000L;
 MaxFrame             = 54000L;
 scrollbarheight      = GetSystemMetrics( SM_CYHSCROLL );
 HelpPending          = FALSE;
 CurrentPlayer        = 0;

 for   ( i = 0; i < 3; i++ )
      { hptrVid[i]    = NULL;
        hptrActive[i] = NULL;
        hptrBusy[i]   = NULL;
        hwndMVPCs[i]  = NULL;
        Loaded[i]     = TRUE;
        }

 getcwd( CurrentDirectory, sizeof( CurrentDirectory ) - 1 );
 if  ( CurrentDirectory[ strlen( CurrentDirectory ) - 1 ]  !=  '\\' )
      strcat( CurrentDirectory, "\\" );

 /*********************************************/
 /* Get Global String Data From Resource File */
 /*********************************************/
                                           // Scroll Bar Value Control String
 LoadString( hInst, ID_FORMATPERCENT, ctlstr, STRBUFLEN );
                                           // Application Name ( for Profiles )
 LoadString( hInst, ID_APPLICATIONNAME, szAppName, BUFFSIZE );
                                           // Help File Name
 LoadString( hInst, ID_HELPFILENAME, HelpFileName, sizeof( HelpFileName ) );
                                           // Title Bar Text
 LoadString( hInst, ID_WINDOWTEXT, szTempBuffer, sizeof( szTempBuffer ) );
 SetWindowText( hwndFrame, szTempBuffer );

 /*************************/
 /* Create Video Controls */
 /*************************/

 rc = CreateVideoControls();               // Don't Continue Without a Video
 if  ( rc )                                //       Window
      CreatePlayerControls();

 /**********************************/
 /* Set Coordinates for Ideal Size */
 /**********************************/

 UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
 Rect.left   = 0;
 Rect.top    = 0;
                                               // Use 1/2 the Video Height
                                               // Assume Scroll Bar Exists
 Rect.bottom = ( VideoStatus.wSourceVlen / 2 ) + scrollbarheight - 1;
                                               // Use a 4:3 Ratio for the Width
 Rect.right  = ( ( VideoStatus.wSourceVlen / 2 ) * 4 ) / 3;

 AdjustWindowRect( &Rect, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, TRUE );
 IdealFrameHeight = Rect.bottom - Rect.top;
 IdealFrameWidth  = Rect.right - Rect.left;
 SetIdealCoord();

 HelpCursor = LoadCursor( hInst, "HelpCursor" );   // Load Help Cursor
 SetFocus( hwndFrame );                            // Set Focus to Main Window
 return( rc );
 }


/*****************************************************************************/
/* Func: MainWndProc                                                         */
/*                                                                           */
/* Parm: HWND   hWnd          - window handle                                */
/*       WORD   wMessage      - the message                                  */
/*       WORD   wParam        - word length data for the message             */
/*       DWORD  lParam        - long length data for the message             */
/*                                                                           */
/* Desc: handle window messages                                              */
/*****************************************************************************/

LONG  FAR PASCAL MainWndProc( hWnd, wMessage, wParam,lParam )
HWND  hWnd;
WORD  wMessage;
WORD  wParam;
LONG  lParam;
{
 static CHAR szWindowText[80];
 WORD  HelpContextId;
 WORD  MenuCommand;
 CHAR  szBuffer[BUFFSIZE + 6];
 CHAR  TempBuffer[BUFFSIZE];
 WORD  Width;
 WORD  Height;
 RECT  Rect;
 WORD  DlgReturnCode;
 WORD  player;
 FARPROC lpProcTemp;                     // Temp Dialog Procedure Instance Addr
 RECT  VideoRect;

  switch ( wMessage )
      {
       case WM_KEYDOWN:
          if  ( wParam == VK_F1 )
             { // If Shift-F1, turn help mode on and set help cursor
               if  ( GetKeyState( VK_SHIFT ) < 0 )
                  { HelpPending = TRUE;
                    SetCursor( HelpCursor );
                    return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
                    }
               else
                 { // If F1 without shift, then call up help main index topic
                   WinHelp( hWnd, HelpFileName, HELP_INDEX, 0L );
                   }
               }
          else
             { if ( wParam == VK_ESCAPE && HelpPending )
                 { // Escape during help mode: turn help mode off
                   HelpPending = FALSE;
                   SetCursor( ( HCURSOR ) GetClassWord( hWnd, GCW_HCURSOR ) );
                   }
               }
          break;

       case WM_SETCURSOR:
          /* In help mode it is necessary to reset the cursor in response  */
          /* to every WM_SETCURSOR message. Otherwise, by default, Windows */
          /* will reset the cursor to that of the window class.            */
          if  ( HelpPending )
             { SetCursor( HelpCursor );
               break;
               }
          return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );

       case WM_INITMENU:
          if  ( HelpPending )
               SetCursor( HelpCursor );
          return( TRUE );

       case WM_ENTERIDLE:
          if  ( ( wParam == MSGF_MENU ) && ( GetKeyState( VK_F1 ) & 0x8000 ) )
             { HelpPending = TRUE;
               PostMessage( hWnd, WM_KEYDOWN, VK_RETURN, 0L );
               }
          break;

       case WM_LBUTTONDOWN:
          if ( HelpPending )
            { HelpPending = FALSE;
              WinHelp( hWnd, HelpFileName, HELP_CONTEXT,
                       ( DWORD ) HLP_INDEX );
              break;
              }

          return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );

       case WM_NCLBUTTONDOWN:
          /* If we are in help mode ( Shift-F1 ), then display */
          /*    context-sensitive help for non-client area.    */
          if  ( HelpPending )
             { HelpContextId = GetHelpId( wParam );
               if  ( HelpContextId == 0 )
                    return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
               HelpPending = FALSE;
               WinHelp( hWnd, HelpFileName, HELP_CONTEXT, HelpContextId );
               break;
               }

          return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );

       case WM_CREATE:
          hwndPopUpMenu = LoadMenu( hInst, "PopUpMenu" );
          break;

       case WM_ACTIVATE:
          if  ( hwndVideoWindow )
             { if  ( wParam )                      // IF Being Activated
                  { if  ( FullScreenVideo )
                         DestroyFullScreenVideo(); // Destroy Full Screen Video
                    else
                         BecomingActive();
                    }
               else                                // ELSE DeActivating
                  { //  We Resize Here so Z-Order Doesn't Get Confused
                    //  If Activating Applications From the Task Manager
                    //  or With Alt-Esc
                    GetWindowRect( hwndVideoWindow, &VideoRect );
                    SetVideoWindowPos( hwndVideoWindow, 0, 0,
                                       VideoRect.right - VideoRect.left,
                                       VideoRect.bottom - VideoRect.top );
                    }
               }
          break;

       case WM_ERASEBKGND:                         // Eliminate BkGnd Flashes
          return( FALSE );

       case WM_INITMENUPOPUP:                      // Initializating PopUp Menu
          hwndMenu = ( HWND ) wParam;
          switch ( LOWORD( lParam ) )
                { case 0:
                       HandleFileMenuInit();
                       break;
                  case 1:
                       HandleVideodiscMenuInit();
                       break;
                  case 2:
                       HandleVideoWindowMenuInit();
                       break;
                  }
          break;

       case WM_COMMAND:
          if  ( HelpPending )
             { HelpContextId = GetHelpId( wParam );
               if  ( !HelpContextId )
                    return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
               HelpPending = FALSE;
               WinHelp( hWnd, HelpFileName, HELP_CONTEXT,
                        ( DWORD ) HelpContextId );
               break;
               }

          MenuCommand = wParam;
          switch ( wParam )
              {
                case MI_SCROLLBAR:
                     if  ( hwndMVPC )
                        { if  ( IsWindowVisible( hwndMVPC ) )
                               ShowWindow( hwndMVPC, SW_HIDE );
                          else
                             { ShowWindow( hwndMVPC, SW_SHOW );
                               BringWindowToTop( hwndMVPC );
                               UpdateWindow( hwndMVPC );
                               }
                          }
                     break;

                case MI_VIDEOIMAGE:
                     if ( !hwndRestoreImage )
                       { lpRestoreImage = MakeProcInstance( ImageDlgProc, hInst );
                         hwndRestoreImage = CreateDialog( hInst, "Image",
                                                          hwndVideoWindow,
                                                          lpRestoreImage );
                         }
                     break;

                case MI_RANGEDIT:
                     EditRange( hWnd, szProfileName );
                     break;

                case MI_RANGESAVE:
                     lpProcTemp = MakeProcInstance( SaveRangeDlgProc, hInst );
                     DialogBox( hInst, "SAVERANGE", hwndVideoWindow, lpProcTemp );
                     FreeProcInstance( lpProcTemp );
                     break;

                case MI_NEWPROFILE:                 /* New Profile File      */
                     lpProcTemp = MakeProcInstance( ProfileDlgProc, hInst );
                     DlgReturnCode = DialogBox( hInst, "NEWPROFILE",
                                                hwndVideoWindow, lpProcTemp );
                     FreeProcInstance( lpProcTemp );
                     if  ( DlgReturnCode == IDOK )
                        { LoadString( hInst, ID_WINDOWTEXT,
                                      szWindowText, sizeof( szWindowText ) );
                          strcat( szWindowText, " -- " );
                          strcat( szWindowText, szShortProfileName );
                          SetWindowText( hWnd, szWindowText );
                          }
                     break;

                case MI_OPENPROFILE:                /* Open Existing Profile */
                     lpProcTemp = MakeProcInstance( OpenProfileDlgProc, hInst );
                     DlgReturnCode = DialogBox( hInst, "OPENPROFILE",
                                                hwndVideoWindow, lpProcTemp );
                     FreeProcInstance( lpProcTemp );
                     if  ( DlgReturnCode == IDOK )
                        { LoadString( hInst, ID_WINDOWTEXT,
                                      szWindowText, sizeof( szWindowText ) );
                          strcat( szWindowText, " -- ");
                          strcat( szWindowText, szShortProfileName );
                          SetWindowText( hWnd, szWindowText );
                          }
                     break;

                case MI_RANGELOAD:     // Load Frame Range to the vdisc control
                     lpProcTemp = MakeProcInstance( LoadRangeDlgProc, hInst );
                     DialogBox( hInst, "LOADRANGE", hwndVideoWindow, lpProcTemp );
                     FreeProcInstance( lpProcTemp );
                     break;

                case MI_FRAMERANGE:                 /* Set New Frame Range   */
                     lpProcTemp = MakeProcInstance( FrameRangeDlgProc, hInst );
                     DialogBox( hInst, "SETRANGE", hwndVideoWindow, lpProcTemp );
                     FreeProcInstance( lpProcTemp );
                     break;

                                           // Set Videodisc Information Display
                case MI_INFODISPLAY:
                                     // NOTE: Query was done by Vdisc Menu Init
                   if  ( PlayerStatus.bInfoDisplay )
                        MVPCCommand( MVPC_CMD_INFODISPLAY, NULL, NULL, NULL );
                   else
                        MVPCCommand( MVPC_CMD_INFODISPLAY,
                                     1L, NULL, NULL );
                   break;

                case MI_TIE:          // Tie Players and Video Sources together
                   if ( fTiePlayersToSources )
                       fTiePlayersToSources = FALSE;
                   else
                       fTiePlayersToSources = TRUE;
                   break;

                case MI_QUERYFRAME:       /* Query and Display Current Frame */
                   UpdatePlayerStatus();
                   LoadString( hInst, ID_QUERYFRAME, TempBuffer, BUFFSIZE );
                   wsprintf( szBuffer, TempBuffer, PlayerStatus.lCurrentFrame );
                   MessageBox( hwndVideoWindow, szBuffer, "", MB_OK );
                   break;

                case MI_QUERYRANGE:   /* Query & Display Current Frame Range */
                   UpdatePlayerStatus();
                   LoadString( hInst, ID_QUERYRANGE, TempBuffer, BUFFSIZE );
                   wsprintf( szBuffer, TempBuffer, PlayerStatus.lLowerBound,
                             PlayerStatus.lUpperBound );
                   MessageBox( hwndVideoWindow, szBuffer, "", MB_OK );
                   break;

                case MI_SCALING_ON:                 /* Video Scaling On/Off  */
                case MI_SCALING_OFF:
                   MVWNCommand( MVWN_CMD_SCALE,
                                ( MenuCommand == MI_SCALING_ON ) ? ( LONG ) TRUE
                                                                 : NULL,
                                NULL );
                   break;

                case MI_SPEEDFAST:                  // Video Disc Speed
                   MVPCCommand( MVPC_CMD_SPEED, 300L, NULL, NULL );
                   break;

                case MI_SPEEDNORMAL:
                   MVPCCommand( MVPC_CMD_SPEED, 100L, NULL, NULL );
                   break;

                case MI_SPEEDSLOW:
                   MVPCCommand( MVPC_CMD_SPEED, 50L, NULL, NULL );
                   break;

                case MI_AUDIO1:                     // Videodisc audio
                case MI_AUDIO2:
                case MI_BOTH:
                   MVPCCommand( MVPC_CMD_AUDIO,
                                ( LONG ) ( MenuCommand - MI_AUDIO1 + 1 ),
                                NULL, NULL );
                   break;

                //-------------------------------------------------------------
                // FDW: New support P8000
                //-------------------------------------------------------------

                case MI_AUDIO3:                     // FDW: New support P8000
                case MI_AUDIO4:                     // FDW: New support P8000
                case MI_DIG_STEREO:                 // FDW: New support P8000
                   MVPCCommand( MVPC_CMD_AUDIO,
                                ( LONG ) ( MenuCommand - MI_AUDIO3 + 5 ),
                                NULL, NULL );
                   break;

                case MI_NOAUDIO:
                   MVPCCommand( MVPC_CMD_AUDIO, NULL, NULL, NULL );
                   break;

                case MI_LOAD:                       // Load Videodisc Player
                   MVPCCommand( MVPC_CMD_INIT, NULL, NULL, NULL );
                   break;

                case MI_UNLOAD:                   // Unload Videodisc Player
                   MVPCCommand( MVPC_CMD_UNLOAD, NULL, NULL, NULL );
                   break;

                case MI_BLACKWHITE:         // Black/White or Color Video Mode
                case MI_COLOR:
                   MVWNCommand( MVWN_CMD_BLACKWHITE,
                                (MenuCommand == MI_BLACKWHITE) ? ( LONG ) TRUE
                                                                 : NULL,
                                NULL );
                   break;

                case MI_FREEZE:
                   MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) TRUE, NULL );
                   break;

                case MI_UNFREEZE:
                   MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) FALSE, NULL );
                   break;

                case MI_INACTIVEBITMAP:
                   MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                                ( LONG ) MVWN_INACTIVE_MONOBITMAP, NULL );
                   break;

                case MI_INACTIVEIMAGE:
                   MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                                ( LONG ) MVWN_INACTIVE_VIDEOIMAGE, NULL );
                   break;

                case MI_INACTIVECOLOR:
                   MVWNCommand( MVWN_CMD_SETINACTIVESTYLE,
                                ( LONG ) MVWN_INACTIVE_COLORFILL,
                                INACTIVECOLOR );
                   break;

                case MI_VS1:                        // Select video source
                case MI_VS2:       // Freezing video before setting new source
                case MI_VS3:
                   usVideoSource = MenuCommand - MI_VS1 + 1;
                   if  ( fTiePlayersToSources )
                        CurrentPlayer = usVideoSource - 1;
                   MVWNCommand( MVWN_CMD_VIDEOSOURCE,
                                ( LONG ) usVideoSource, NULL );
                   ResizeVideoControls();
                   break;

                case MI_PROGRESSIVE:                // Set Video Scan Modes
                   MVWNCommand( MVWN_CMD_SCANMODE,
                                MAKELONG( MVWN_SCANA_DEFAULT,
                                          MVWN_SCANM_PROGRESSIVE ), NULL );
                   break;

                case MI_DOUBLING:
                   MVWNCommand( MVWN_CMD_SCANMODE,
                                MAKELONG( MVWN_SCANA_DEFAULT,
                                          MVWN_SCANM_LINEDOUBLING ), NULL );
                   break;

                case MI_SINGLEFIELDSCAN:
                   MVWNCommand( MVWN_CMD_SCANMODE,
                                MAKELONG( MVWN_SCANA_DEFAULT,
                                          MVWN_SCANM_SINGLEFIELD ), NULL );
                   break;

                case MI_BITMAP:
                   BitMapToClipboard();
                   break;

                case MI_IDEALSIZE:
                   DestroyFullScreenVideo();
                   SetIdealCoord();
                   break;

                case MI_4THSCREEN:
                   DestroyFullScreenVideo();
                   Width  = GetSystemMetrics( SM_CXSCREEN ) / 2;
                   Height = GetSystemMetrics( SM_CYSCREEN ) / 2;
                   GetWindowRect( hWnd, &Rect );
                   if  ( Rect.top < 0 )
                        Rect.top = 0;
                   if  ( Rect.left < 0 )
                        Rect.left = 0;
                   MoveWindow( hWnd, Rect.left, Rect.top,
                               Width, Height, TRUE );
                   break;

                case MI_FULLSCREEN:
                   DoFullScreenVideo();
                   break;

                case MI_PLAYER1:             // Select Current Videodisc Player
                case MI_PLAYER2:
                   CurrentPlayer = MenuCommand - MI_PLAYER1;

                   // Switch Players IF This is Different from Current Player

                   if  ( hwndMVPC != hwndMVPCs[CurrentPlayer] )
                      { if  ( fTiePlayersToSources )    // Switch Video Sources ?
                           { usVideoSource = CurrentPlayer + 1;
                             MVWNCommand( MVWN_CMD_VIDEOSOURCE,
                                          ( LONG ) usVideoSource, NULL );
                             }
                        ResizeVideoControls();
                        }
                   break;

                case MI_SETATTRIBUTES:
                 if ( !hwndVideoAttributes )
                   { lpVideoAttributes = MakeProcInstance( VideoAttributesProc,
                                                           hInst );
                     hwndVideoAttributes = CreateDialog( hInst, "Video",
                                                         hwndVideoWindow,
                                                         lpVideoAttributes );
                     }
                 else
                     SetBoxPosition( hwndVideoAttributes, TOP );
                   break;

                case MI_SETAUDIO:
                 if ( !hwndAudioAttributes )
                   { lpAudioAttributes = MakeProcInstance( AudioAttributesProc,
                                                           hInst );
                     hwndAudioAttributes = CreateDialog( hInst, "Audio",
                                                         hwndVideoWindow,
                                                         lpAudioAttributes );
                     }
                 else
                     SetBoxPosition( hwndAudioAttributes, BOTTM );
                 break;

                case MI_HELPINDEX:
                 WinHelp( hWnd, HelpFileName, HELP_INDEX, 0L );
                 break;

                case MI_HELPFORHELP:
                 WinHelp( hWnd, HelpFileName, HELP_HELPONHELP, 0L );
                 break;

                case MI_HELPKEYBOARD:
                 WinHelp( hWnd, HelpFileName, HELP_CONTEXT, HLP_KEYS );
                 break;

                case MI_ABOUT:
                 lpProcTemp = MakeProcInstance( AboutDlgProc, hInst );
                 DialogBox( hInst, "AboutBrowser", hwndVideoWindow, lpProcTemp );
                 FreeProcInstance( lpProcTemp );
                 break;
                }
          break;

       case WM_MOVE:
          if  ( IsIconic( hwndFrame ) )
               SetVideoWindowPos( hwndVideoWindow, 0, 0,
                                  LOWORD( lParam ), HIWORD( lParam ) );
          else
               SizeVideo();
          break;

       case WM_SIZE:
          if  ( wParam == SIZEICONIC )
               SetVideoWindowPos( hwndVideoWindow, 0, 0,
                                  LOWORD( lParam ), HIWORD( lParam ) );
          else
               SizeVideo();
          break;

       case WM_CLOSE:
          if  ( hWnd == hwndFrame )
             { for ( player = 0; player < MAX_PLAYERS; player++ )
                  { if  ( hwndMVPCs[player] )
                       { if  ( IsWindow( hwndMVPCs[player] ) )
                              DestroyWindow( hwndMVPCs[player] );
                         }
                    }
               if  ( IsWindow( hwndVideoWindow ) )
                    DestroyWindow( hwndVideoWindow );
               }
          return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );

       case WM_DESTROY:
          if  ( hWnd == hwndFrame )
             { DestroyMenu( hwndPopUpMenu );             // Destroy Pop-Up Menu
               WinHelp( hWnd, HelpFileName, HELP_QUIT, 0L );
               }
          PostQuitMessage( 0 );
          break;

       default:
          return( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
       }

  return( NULL );
  }


/*****************************************************************************/
/* Func: EnableBrowserMenuItem                                               */
/*                                                                           */
/* Desc: Enables a menu item                                                 */
/*                                                                           */
/*****************************************************************************/

static VOID EnableBrowserMenuItem ( WORD item )
{
 EnableMenuItem( hwndMenu, item, MF_ENABLED );
 EnableMenuItem( hwndPopUpMenu, item, MF_ENABLED );
 return;
 }


/*****************************************************************************/
/* Func: DisableBrowserMenuItem                                              */
/*                                                                           */
/* Desc: Disables a menu option                                              */
/*                                                                           */
/*****************************************************************************/

static VOID DisableBrowserMenuItem ( WORD item )
{
 EnableMenuItem( hwndMenu, item, MF_DISABLED | MF_GRAYED );
 EnableMenuItem( hwndPopUpMenu, item, MF_DISABLED | MF_GRAYED );
 return;
 }


/*****************************************************************************/
/* Func: CheckBrowserMenuItem                                                */
/*                                                                           */
/* Desc: Places a check mark next to a menu item                             */
/*****************************************************************************/

static VOID CheckBrowserMenuItem ( WORD item )
{
 CheckMenuItem( hwndMenu, item, MF_CHECKED );
 CheckMenuItem( hwndPopUpMenu, item, MF_CHECKED );
 return;
 }


/*****************************************************************************/
/* Func: UnCheckBrowserMenuItem                                              */
/*                                                                           */
/* Desc: Clears any check mark next to a menu item                           */
/*****************************************************************************/

static VOID UnCheckBrowserMenuItem( WORD item )
{
 CheckMenuItem( hwndMenu, item, MF_UNCHECKED );
 CheckMenuItem( hwndPopUpMenu, item, MF_UNCHECKED );
 return;
 }


/*****************************************************************************/
/* Func: EnableAllPlayerOptions                                              */
/*                                                                           */
/* Desc: Enables all options on the videodisc control menu                   */
/*                                                                           */
/*****************************************************************************/

static VOID EnableAllPlayerOptions()
{
 WORD  i;

 for  ( i = FIRST_VCONTROLITEM; i <= LAST_VCONTROLITEM; i++ )
       EnableBrowserMenuItem( i );

 for  ( i = 0; i < MAX_PLAYERS; i++ )
     { if  ( hwndMVPCs[i] == NULL )
            DisableBrowserMenuItem( i );
       }

 return;
 }


/*****************************************************************************/
/* Func: DisableAllPlayerOptions                                             */
/*                                                                           */
/* Desc: Disables all options on the videodisc control menu                  */
/*                                                                           */
/*****************************************************************************/

static VOID DisableAllPlayerOptions()
{
 WORD  i;

 for  ( i = FIRST_VCONTROLITEM; i <= LAST_VCONTROLITEM; i++ )
     { UnCheckBrowserMenuItem( i );
       DisableBrowserMenuItem( i );
       }

 for  ( i = 0; i < MAX_PLAYERS; i++ )
     { if  ( hwndMVPCs[i] )
          { EnableBrowserMenuItem( MI_TIE );
            EnableBrowserMenuItem( MI_PLAYER1 + i );
            }
       }

 return;
 }


/*****************************************************************************/
/* Func: DisableAllFileItems                                                 */
/*                                                                           */
/* Desc: Disables all options on the File menu                               */
/*                                                                           */
/*****************************************************************************/

static VOID DisableAllFileItems()
{
 WORD  i;

 for  ( i = FIRST_FILEITEM;  i <= LAST_FILEITEM;  i++ )
       DisableBrowserMenuItem( i );
 return;
 }


/*****************************************************************************/
/* Func: HandleVideoWindowMenuInit                                           */
/*                                                                           */
/* Desc: This function initializes the VideoWindow menu.                     */
/*                                                                           */
/*****************************************************************************/

VOID HandleVideoWindowMenuInit()
{
 WORD   i;
 BOOL   fOK;
 RECT   Rect;
 WORD   Height;
 WORD   Width;

  fOK = FALSE;
  if  ( hwndVideoWindow )
     { UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
       if  ( VideoStatus.wReturnCode == MVWN_ERR_NOERROR )
            fOK = TRUE;
       }

  if ( fOK )
     {
     CheckBrowserMenuItem( ( VideoStatus.mvidst.bBlackWhite ) ? MI_BLACKWHITE : MI_COLOR );
     UnCheckBrowserMenuItem( ( VideoStatus.mvidst.bBlackWhite ) ? MI_COLOR : MI_BLACKWHITE );

     CheckBrowserMenuItem( ( VideoStatus.bScale ) ? MI_SCALING_ON : MI_SCALING_OFF );
     UnCheckBrowserMenuItem( ( VideoStatus.bScale ) ? MI_SCALING_OFF : MI_SCALING_ON );

     CheckBrowserMenuItem( ( VideoStatus.mvidst.bFreeze ) ? MI_FREEZE : MI_UNFREEZE );
     UnCheckBrowserMenuItem( ( VideoStatus.mvidst.bFreeze ) ? MI_UNFREEZE : MI_FREEZE );

     UnCheckBrowserMenuItem( MI_PROGRESSIVE );
     UnCheckBrowserMenuItem( MI_DOUBLING );
     UnCheckBrowserMenuItem( MI_SINGLEFIELDSCAN );
     switch ( VideoStatus.mvidst.wScanMethod )
           { case MVWN_SCANM_PROGRESSIVE:
             case MVWN_SCANM_DEFAULT:
                  CheckBrowserMenuItem( MI_PROGRESSIVE );
                  break;
             case MVWN_SCANM_LINEDOUBLING:
                  CheckBrowserMenuItem( MI_DOUBLING );
                  break;
             case MVWN_SCANM_SINGLEFIELD:
                  CheckBrowserMenuItem( MI_SINGLEFIELDSCAN );
                  break;
             }

     UnCheckBrowserMenuItem( MI_INACTIVEBITMAP );
     UnCheckBrowserMenuItem( MI_INACTIVEIMAGE );
     UnCheckBrowserMenuItem( MI_INACTIVECOLOR );

     switch ( VideoStatus.wInactiveStyle )
        {
        case MVWN_INACTIVE_MONOBITMAP:
             CheckBrowserMenuItem( MI_INACTIVEBITMAP );
             break;

        case MVWN_INACTIVE_VIDEOIMAGE:
             CheckBrowserMenuItem( MI_INACTIVEIMAGE );
             break;

        case MVWN_INACTIVE_COLORFILL:
             CheckBrowserMenuItem( MI_INACTIVECOLOR );
             break;
        }
     }

  for  ( i = 1; i <= MAX_VIDEOSOURCES; i++ )
      {
        if  ( MVWNVideoSourceType( i ) )         // IF source exists
           {
             if  ( i == usVideoSource )
                  CheckBrowserMenuItem( i + MI_VS1 - 1 );
             else
                  UnCheckBrowserMenuItem( i + MI_VS1 - 1 );
             }
        else                                // IF source does NOT exist disable
             DisableBrowserMenuItem( i + MI_VS1 - 1 );
        }

  UnCheckBrowserMenuItem( MI_IDEALSIZE );
  UnCheckBrowserMenuItem( MI_4THSCREEN );
  UnCheckBrowserMenuItem( MI_FULLSCREEN );
  GetWindowRect( hwndFrame, &Rect );
  Height = Rect.bottom - Rect.top;
  Width  = Rect.right - Rect.left;
  if  ( FullScreenVideo )                            // IF Full Screen Video
       CheckBrowserMenuItem( MI_FULLSCREEN );
  else if  ( Width == IdealFrameWidth )              // IF in Ideal Coordinates
          { if  ( hwndMVPC )
               { if   ( Height == IdealFrameHeight )
                       CheckBrowserMenuItem( MI_IDEALSIZE );
                 }
            else
               { if   ( Height == ( IdealFrameHeight - scrollbarheight ) )
                       CheckBrowserMenuItem( MI_IDEALSIZE );
                 }
            }
       else if  ( ( Width == 320 ) &&                // IF 1/4 Screen
                  ( Height== 240 ) )
                 CheckBrowserMenuItem( MI_4THSCREEN );
  return;
  }


/*****************************************************************************/
/* Func: HandleFileMenuInit                                                  */
/*                                                                           */
/* Desc: This function initializes the File menu.                            */
/*       IF no Videodisc Control, then disable all file items.               */
/*       Otherwise, enable/disable items based on whether a profile is open. */
/*****************************************************************************/

VOID HandleFileMenuInit()
{
 if  ( !hwndMVPC )
      DisableAllFileItems();
 else
    { EnableBrowserMenuItem( MI_NEWPROFILE );
      EnableBrowserMenuItem( MI_OPENPROFILE );
      if  ( !szProfileName[0] )
         { DisableBrowserMenuItem( MI_RANGELOAD );
           DisableBrowserMenuItem( MI_RANGESAVE );
           DisableBrowserMenuItem( MI_RANGEDIT );
           }
      else
         { EnableBrowserMenuItem( MI_RANGELOAD );
           EnableBrowserMenuItem( MI_RANGESAVE );
           EnableBrowserMenuItem( MI_RANGEDIT );
           }
      }
 return;
 }


/*****************************************************************************/
/* Func: HandleVideodiscMenuInit                                             */
/*                                                                           */
/* Desc: This function updates the selections on the videodisc control       */
/*       menu.  If there is a videodisc control, it does a query to update   */
/*       the fields.  If no controls, greys out the menu selections.         */
/*                                                                           */
/*****************************************************************************/

VOID HandleVideodiscMenuInit()
{
 CHAR   szTempBuff[BUFFSIZE];
 WORD   player, i;
 BOOL   fOK = FALSE;

 fOK = FALSE;
 if ( hwndMVPC )
    {
    EnableAllPlayerOptions();

    LoadString( hInst, ( fTiePlayersToSources ) ? ID_UNTIE : ID_TIE,
                szTempBuff, sizeof( szTempBuff ) );
    ModifyMenu( hwndMenu, MI_TIE, MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                MI_TIE, szTempBuff );
    ModifyMenu( hwndPopUpMenu, MI_TIE, MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                MI_TIE, szTempBuff );

        // Get player status to check/uncheck various menu items

    UpdatePlayerStatus();
    if  ( PlayerStatus.wReturnCode == MVPC_ERR_NOERROR )
         fOK = TRUE;

    for ( player = 0; player != MAX_PLAYERS; player++ )
       {
       if ( !hwndMVPCs[player] )
          DisableBrowserMenuItem( player + MI_PLAYER1 );
       else
          {
          if ( hwndMVPC == hwndMVPCs[player] )
             {
             CheckBrowserMenuItem( player + MI_PLAYER1 );
             if ( fOK )                // IF we got a good query on this player
                {
                // Digital Audio Channels support:
                // Note: The Windows MVPC API has been enhanced to support
                //       selection of Digital Audio channels 3 and 4.
                //       The capabilities of the player and driver determine
                //       whether this is supported or not.
                //
                //       wAudio (binary)
                //       0000 0XXX
                //              Channel 1 (or 3) OFF (0) or ON (1)
                //              Channel 2 (or 4) OFF (0) or ON (1)
                //              0 Analog:  selects channels 1 & 2
                //                  1 Digital: selects channels 3 & 4
                //
                //      Note: 4 (Digital with no channels on) is NO AUDIO
                //
                UnCheckBrowserMenuItem (MI_NOAUDIO);
                UnCheckBrowserMenuItem (MI_AUDIO1);
                UnCheckBrowserMenuItem (MI_AUDIO2);
                UnCheckBrowserMenuItem (MI_BOTH);
                UnCheckBrowserMenuItem (MI_AUDIO3);
                UnCheckBrowserMenuItem (MI_AUDIO4);
                UnCheckBrowserMenuItem (MI_DIG_STEREO);

                i = PlayerStatus.wAudio & 0x07; // AND off lower 3 bits

                if (i & 0x03)                   // If an audio channel(s) is ON
                   {
                   if (i & 0x04)                // If Digital Audio
                      CheckBrowserMenuItem (i + MI_AUDIO3 - 5);
                   else                         // Else Analog Audio
                      CheckBrowserMenuItem (i + MI_AUDIO1 - 1);
                   }
                else
                   CheckBrowserMenuItem (MI_NOAUDIO);

                LoadString( hInst, ( PlayerStatus.bInfoDisplay ) ?
                            ID_FRAMEOFF : ID_FRAMEON,
                            szTempBuff, sizeof( szTempBuff ) );

                ModifyMenu( hwndMenu, MI_INFODISPLAY,
                            MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                            MI_INFODISPLAY, szTempBuff );
                ModifyMenu( hwndPopUpMenu, MI_INFODISPLAY,
                            MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                            MI_INFODISPLAY, szTempBuff );

                UnCheckBrowserMenuItem( MI_SPEEDSLOW );
                UnCheckBrowserMenuItem( MI_SPEEDNORMAL );
                UnCheckBrowserMenuItem( MI_SPEEDFAST );
                switch ( PlayerStatus.wSpeed )
                      { case 50:
                             CheckBrowserMenuItem( MI_SPEEDSLOW );
                             break;
                        case 100:
                             CheckBrowserMenuItem( MI_SPEEDNORMAL );
                             break;
                        case 300:
                             CheckBrowserMenuItem( MI_SPEEDFAST );
                             break;
                        }
                }
             }
          else
             UnCheckBrowserMenuItem( player + MI_PLAYER1 );
          }
       }
    }
 else                           // IF no videodisc control, disable all options
    DisableAllPlayerOptions();

 if  ( !FullScreenVideo )                  // Fix Up Hide/Show Scrollbar Option
    { LoadString( hInst, ID_HIDE,
                  szTempBuff, sizeof( szTempBuff ) );
      ModifyMenu( hwndPopUpMenu, MI_SCROLLBAR,
                  MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                  MI_SCROLLBAR, szTempBuff );
      DisableBrowserMenuItem( MI_SCROLLBAR );
      }
 else
    { i = ( IsWindowVisible( hwndMVPC ) )  ?  ID_HIDE : ID_SHOW ;
      LoadString( hInst, i, szTempBuff, sizeof( szTempBuff ) );
      ModifyMenu( hwndPopUpMenu, MI_SCROLLBAR,
                  MF_ENABLED | MF_BYCOMMAND | MF_STRING,
                  MI_SCROLLBAR, szTempBuff );
      EnableBrowserMenuItem( MI_SCROLLBAR );
      }

 return;
 }


/*****************************************************************************/
/* FUNCTION: GetHelpId()                                                     */
/*                                                                           */
/* DESCRIPTION: Convert Menu ID Into Help ID                                 */
/*                                                                           */
/* PARMS: Parm #1  WORD  Menu ID                                             */
/*                                                                           */
/* RETURNS:  WORD  Help ID                                                   */
/*****************************************************************************/

static WORD GetHelpId( MenuId )
WORD MenuId;
{
 WORD HelpId;

 switch  ( MenuId )
        { case MI_AUDIOCHANNELS:
          case MI_AUDIO1:
          case MI_AUDIO2:
          case MI_NOAUDIO:
          case MI_BOTH:
          case MI_AUDIO3:
          case MI_AUDIO4:
          case MI_DIG_STEREO:
               HelpId = HLP_AUDIO;
               break;
          case MI_SPEED:
          case MI_SPEEDFAST:
          case MI_SPEEDNORMAL:
          case MI_SPEEDSLOW:
               HelpId = HLP_SPEED;
               break;
          case MI_PLAYER:
          case MI_PLAYER1:
          case MI_PLAYER2:
               HelpId = HLP_PLAYER;
               break;
          case MI_VIDEOSOURCE:
          case MI_VS1:
          case MI_VS2:
          case MI_VS3:
               HelpId = HLP_VIDEOSOURCE;
               break;
          case MI_INACTIVESTYLE:
          case MI_INACTIVEBITMAP:
          case MI_INACTIVEIMAGE:
          case MI_INACTIVECOLOR:
               HelpId = HLP_INACTIVE;
               break;
          case MI_SCANMODE:
          case MI_PROGRESSIVE:
          case MI_DOUBLING:
          case MI_SINGLEFIELDSCAN:
               HelpId = HLP_SCAN;
               break;
          case MI_WINDOWSIZE:
          case MI_FULLSCREEN:
          case MI_IDEALSIZE:
          case MI_4THSCREEN:
               HelpId = HLP_SCREENSIZE;
               break;
          case MI_QUERY:
          case MI_QUERYFRAME:
          case MI_QUERYRANGE:
               HelpId = HLP_QUERY;
               break;
          case MI_LOAD:
          case MI_UNLOAD:
               HelpId = HLP_LOAD;
               break;
          case MI_SCALING_ON:
          case MI_SCALING_OFF:
               HelpId = HLP_SCALE;
               break;
          case MI_FREEZE:
          case MI_UNFREEZE:
               HelpId = HLP_FREEZE;
               break;
          default:
               HelpId = ( MenuId == MI_NEWPROFILE    )  ?  HLP_NEWPROFILE :
                        ( MenuId == MI_OPENPROFILE   )  ?  HLP_OPENPROFILE :
                        ( MenuId == MI_RANGELOAD     )  ?  HLP_LOADFRAMERANGE :
                        ( MenuId == MI_RANGESAVE     )  ?  HLP_SAVEFRAMERANGE :
                        ( MenuId == MI_RANGEDIT      )  ?  HLP_EDITRANGE :
                        ( MenuId == MI_VIDEOIMAGE    )  ?  HLP_VIDEOIMAGE :
                        ( MenuId == MI_FRAMERANGE    )  ?  HLP_SETFRAMERANGE :
                        ( MenuId == MI_INFODISPLAY   )  ?  HLP_INFO :
                        ( MenuId == MI_TIE           )  ?  HLP_TIE :
                        ( MenuId == MI_SCROLLBAR     )  ?  HLP_SHOWHIDESBAR :
                        ( MenuId == MI_SETATTRIBUTES )  ?  HLP_SETVIDEO :
                        ( MenuId == MI_SETAUDIO      )  ?  HLP_SETAUDIO :
                        ( MenuId == MI_EFFECTS       )  ?  HLP_EFFECTS :
                        ( MenuId == MI_BLACKWHITE    )  ?  HLP_BLACKWHITE :
                        ( MenuId == MI_COLOR         )  ?  HLP_COLOR :
                        ( MenuId == MI_BITMAP        )  ?  HLP_BITMAP :
                                                           0;
               break;
          }
 return( HelpId );
 }
