/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  COMMAND.C                                                        */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: Simple videodisc browser utility demonstrating programming       */
/*          concepts of the Multimedia Toolkit for Windows                   */
/*****************************************************************************/

//****************************************************************************/
// Defines and Includes                                                      */
//****************************************************************************/

#include <WINDOWS.H>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.


/*****************************************************************************/
/* Func: MVPCCommand                                                         */
/*                                                                           */
/* Parm: USHORT   usCommand;                                                 */
/*       MPARAM   mp1;                                                       */
/*       MPARAM   mp2;                                                       */
/*       MPARAM   mp3;                                                       */
/*                                                                           */
/* Desc: Videodisc control object generic command function.                  */
/*       All commands to the control get funneled through here.              */
/*                                                                           */
/* NOTE: The following global variables must be initialized before           */
/*       calling this function:                                              */
/*                                                                           */
/*          hwndMVPC    mvpcCommandMsg    hwndMVPCResponse                   */
/*                                                                           */
/* Retn: TRUE:  Success                                                      */
/*       FALSE: Fail                                                         */
/*****************************************************************************/

BOOL MVPCCommand( Command, mp1, mp2, mp3 )
WORD Command;
LONG mp1;
LONG mp2;
LONG mp3;
{
 HANDLE CommandHandle;
 LPMVPCCMD pmvpccmd;
 BOOL   fOK = FALSE;

 if  ( hwndMVPC == NULL )
      return( FALSE );

 if  ( IsWindow( hwndMVPC ) == FALSE )        // Avoid Problems During Shutdown
      return( FALSE );

 if  ( CommandHandle = MVPCCreateCmd( Command ) )       // IF Created Command
    { pmvpccmd = ( LPMVPCCMD ) GlobalLock( CommandHandle );
      if  ( pmvpccmd )
         { pmvpccmd->data.dwCmdParam1 = mp1;            // Set up Command Parms
           pmvpccmd->data.dwCmdParam2 = mp2;
           pmvpccmd->data.dwCmdParam3 = mp3;
           GlobalUnlock( CommandHandle );
           PostMessage( hwndMVPC, mvpcCommandMsg,       // Post to VDP Control
                        hwndMVPCResponse, ( LONG ) CommandHandle );
           fOK = TRUE;
           }
      }
 return( fOK );
 }


/*****************************************************************************/
/* Func: MVWNCommand                                                         */
/*                                                                           */
/* Parm: USHORT   usCommand;                                                 */
/*       MPARAM   mp1;                                                       */
/*       MPARAM   mp2;                                                       */
/*                                                                           */
/* Desc: Video Window generic command function.  All commands                */
/*       to the video window get funneled through here.                      */
/*                                                                           */
/* NOTE: The following global variables must be initialized before           */
/*       calling this function:                                              */
/*                                                                           */
/*          hwndVideoWindow   mvwnCommandMsg    hwndMVWNResponse             */
/*                                                                           */
/* Retn: TRUE:  Success                                                      */
/*       FALSE: Fail                                                         */
/*****************************************************************************/

BOOL MVWNCommand( Command, mp1, mp2 )
WORD Command;
LONG mp1;
LONG mp2;
{
 LPMVWNCMD  pmvwncmd;
 BOOL   fOK = FALSE;
 HANDLE CommandHandle;

 if  ( hwndVideoWindow == NULL )
      return( FALSE );

 if  ( IsWindow( hwndVideoWindow ) == FALSE ) // Avoid Problems During Shutdown
      return( FALSE );

 if  ( CommandHandle = MVWNCreateCmd( Command ) )       // IF Created Command
    { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
      if  ( pmvwncmd )
         { pmvwncmd->data.dwCmdParam1 = mp1;            // Set up Command Parms
           pmvwncmd->data.dwCmdParam2 = mp2;
           GlobalUnlock( CommandHandle );

           if  ( ( Command == MVWN_CMD_FREEZE )  ||
                 ( Command == MVWN_CMD_RELEASEIMAGE ) ||
                 ( Command == MVWN_CMD_RESTOREIMAGE ) ||
                 ( Command == MVWN_CMD_VIDEOSOURCE )  ||
                 ( Command == MVWN_CMD_ACTIVATE ) )
                SendMessage( hwndVideoWindow, mvwnCommandMsg,
                             hwndMVWNResponse, ( LONG ) CommandHandle );
           else
                PostMessage( hwndVideoWindow, mvwnCommandMsg,
                             hwndMVWNResponse, ( LONG ) CommandHandle );
           fOK = TRUE;
           }
      }
 return( fOK );
 }
