/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  IMAGE.C                                                          */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: Handle Video Image and Bitmap Processing                         */
/*****************************************************************************/

/*****************************************************************************/
/* Defines and Includes                                                      */
/*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <direct.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.

static  VOID  RestoreVideoImage( PCHAR );
static  WORD  SaveVideoImage( PCHAR );

static  HWND  hwndListBox;               // File Names List Box Window Handle

#define HALFSEGMENT ( unsigned int ) 0x8000


/*****************************************************************************/
/* Func: SaveVideoImage()                                                    */
/*                                                                           */
/* Parm: PCHAR  Disk File Name                                               */
/*                                                                           */
/* Desc: Save the Current Video Image to a Disk File                         */
/*                                                                           */
/* Returns: =1 if Entry Needs to be Added to the List                        */
/*****************************************************************************/

static WORD SaveVideoImage( FileName )
PCHAR FileName;
{
 MVWNVIDIMAGE Image;                             // Toolkit Image Structure
 int    FileHandle;                              // Image File Handle
 WORD   rc;                                      // Temporary Return Code
 DWORD  ImageLength;                             // Captured Image's Length
 CHAR   TextBuff[80];                            // Temporary Text Buffer
 WORD   ReturnCode = 1;                          // This Function's Return Code
 WORD   BytesWritten;                            // Bytes Written ( Returned )
 HANDLE CommandHandle;                           // MVWN Toolkit Command Handle
 char huge *Buffer;                              // Image Buffer Pointer
 LPMVWNCMD pmvwncmd;                             // Toolkit MVWN Command Struct
 OFSTRUCT  FileInfo;                             // Parm Needed by OpenFile()
 CHAR   String1[STRBUFLEN];                      // Error Message String 1
 CHAR   String2[STRBUFLEN];                      // Error Message String 2
 BOOL   UnLock = TRUE;
 CHAR   FullPathName[80];

 Image.hImageBuffer = NULL;
 Image.dwImageSize = 0L;

 /************************************************/
 /* Use the Toolkit to Capture the Current Image */
 /************************************************/

 if ( CommandHandle = MVWNCreateCmd( MVWN_CMD_CAPTUREIMAGE ) )
    { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
      if  ( pmvwncmd )
         { pmvwncmd->data.dwCmdParam1 = ( LONG ) ( LPSTR ) &Image;
           pmvwncmd->data.dwCmdParam2 = NULL;
           GlobalUnlock( CommandHandle );
           SendMessage( hwndVideoWindow, mvwnCommandMsg, 0, ( LONG ) CommandHandle );
           }
      MVWNDestroyCmd( CommandHandle, hwndVideoWindow, MVWN_ACK );
      }

 if  ( Image.hImageBuffer == NULL )              // IF the Capture Image Failed
    { LoadString( hInst, ID_NOTSAVED, String1, STRBUFLEN );
      wsprintf( TextBuff, String1, ( LPSTR ) FileName, rc );
      LoadString( hInst, ID_NOMEMORY, String2, STRBUFLEN );
      MessageBeep( 0 );
      MessageBox( hwndRestoreImage, TextBuff, String2, MB_ICONHAND );
      return( 0 );
      }

 /***********************/
 /* Open the Image File */
 /***********************/

 getcwd( FullPathName, 64 );                  // Path the Name so OpenFile()
 if  ( FullPathName[ strlen( FullPathName ) - 1 ]  !=  '\\' )
      strcat( FullPathName, "\\" );
 strcat( FullPathName, FileName );            // Won't Search Other Directories
 FileHandle = OpenFile( ( LPSTR ) FullPathName,
                        ( LPOFSTRUCT ) &FileInfo, OF_EXIST );

 if  ( FileHandle != -1 )
    { LoadString( hInst, ID_OVERWRITE,  String1, STRBUFLEN );
      LoadString( hInst, ID_FILEEXISTS, String2, STRBUFLEN );
      MessageBeep( 0 );
      rc = MessageBox( hwndRestoreImage, String1, String2,
                       MB_ICONQUESTION | MB_YESNO | MB_APPLMODAL );
      if  ( rc == IDNO )
           return( 0 );
      }

 FileHandle = _lcreat( ( LPSTR ) FullPathName, 0 );

 if  ( FileHandle == -1 )
    { LoadString( hInst, ID_FILERC, String1, STRBUFLEN );
      wsprintf( TextBuff, String1, ( LPSTR ) FileName, rc );
      LoadString( hInst, ID_CANTOPEN, String2, STRBUFLEN );
      MessageBeep( 0 );
      MessageBox( hwndRestoreImage, TextBuff, String2, MB_ICONHAND );
      return( 0 );
      }

 ImageLength = Image.dwImageSize;
 Buffer = GlobalLock( Image.hImageBuffer );
 if  ( Buffer == NULL )                          // IF Can't Lock Memory
    { ReturnCode  = 0;                           //    Set Error Code &
      ImageLength = 0L;                          //    No Image Available
      LoadString( hInst, ID_ALLOCERROR, String1, STRBUFLEN );
      LoadString( hInst, ID_LOCKERROR,  String2, STRBUFLEN );
      MessageBeep( 0 );
      MessageBox( hwndRestoreImage, String1,
                  String2, MB_ICONHAND | MB_APPLMODAL );
      UnLock = FALSE;
      }

 /****************************************/
 /* Write the Captured Image to the File */
 /****************************************/

 while ( ImageLength > 0L )
      { if  ( ImageLength >= HALFSEGMENT )
           { BytesWritten = _lwrite( FileHandle, ( LPSTR ) Buffer,
                                     HALFSEGMENT );
             if  ( BytesWritten != HALFSEGMENT )
                { MessageBeep( 0 );
                  LoadString( hInst, ID_NOTSAVED, String1, STRBUFLEN );
                  LoadString( hInst, ID_ERRORWRITING, String2, STRBUFLEN );
                  MessageBeep( 0 );
                  MessageBox( hwndRestoreImage, String1, String2,
                              MB_ICONHAND | MB_APPLMODAL );
                  ReturnCode = 0;
                  break;
                  }
             }
        else
           { BytesWritten = _lwrite( FileHandle, ( LPSTR ) Buffer,
                                     ( WORD ) ImageLength );
             if  ( BytesWritten != ( WORD ) ImageLength )
                { MessageBeep( 0 );
                  LoadString( hInst, ID_NOTSAVED, String1, STRBUFLEN );
                  LoadString( hInst, ID_ERRORWRITING, String2, STRBUFLEN );
                  MessageBeep( 0 );
                  MessageBox( hwndRestoreImage, String1, String2,
                              MB_ICONHAND | MB_APPLMODAL );
                  ReturnCode = 0;
                  }
             break;
             }
        ImageLength -= ( LONG ) HALFSEGMENT;
        Buffer += HALFSEGMENT;
        }

 if  ( UnLock )                                    // IF GlobalLock() Succeeded
      GlobalUnlock( Image.hImageBuffer );
 MVWNCommand( MVWN_CMD_RELEASEIMAGE, ( LONG ) ( LPSTR ) &Image, NULL );
 _lclose( FileHandle );

 if  ( ReturnCode == 0 )                           // IF a Disk Write Failed
      OpenFile( ( LPSTR ) FullPathName, ( LPOFSTRUCT ) &FileInfo, OF_DELETE );

 return( ReturnCode );
 }


/*****************************************************************************/
/* Func: RestoreVideoImage()                                                 */
/*                                                                           */
/* Parm: PCHAR  Disk File Name                                               */
/*                                                                           */
/* Desc: Restore a Video Image From A Disk File                              */
/*****************************************************************************/

static VOID RestoreVideoImage( FileName )
PCHAR FileName;
{
 LPMVWNVIDIMAGE ImageStructPtr;                  // Toolkit Image Structure Ptr
 HANDLE StructHandle;                            // Image Structure Handle
 int    FileHandle;                              // Image File Handle
 WORD   BytesRead = 1;                           // Bytes Read ( Returned )
 CHAR   String1[STRBUFLEN];                      // Error Message String 1
 CHAR   String2[STRBUFLEN];                      // Error Message String 2
 DWORD  ImageLength;                             // Maximum Image Size
 CHAR   huge *ImageBuffer;                       // Image Buffer Pointer
 CHAR   FAR  *ImageStart;                        // Local Ptr to Image Start
 HANDLE ImageHandle;                             // Image Buffer Handle

 /***********************/
 /* Open the Image File */
 /***********************/

 FileHandle = _lopen( ( LPSTR ) FileName, OF_READ );
 if  ( FileHandle == -1 )                        // IF Can't Open Image File
      return;                                    //    Exit Function

 /****************************************/
 /* Allocate the Toolkit Image Structure */
 /****************************************/

 StructHandle = GlobalAlloc( LMEM_FIXED | LMEM_ZEROINIT,
                             sizeof( MVWNVIDIMAGE ) );
 if  ( StructHandle == NULL )                    // IF Struct Allocation Failed
    { _lclose( FileHandle );                     //    Close the Image File
      return;                                    //    Exit Function
      }

 ImageStructPtr = ( LPMVWNVIDIMAGE ) GlobalLock( StructHandle );
 if  ( ImageStructPtr == NULL )                  // IF Can't Lock Memory
    { GlobalFree( StructHandle );                //    Release Structure Memory
      _lclose( FileHandle );                     //    Close the Image File
      return;                                    //    Exit Function
      }

 ImageLength = 460816L;
 ImageStructPtr->dwImageSize = ImageLength;

 /*****************************/
 /* Allocate the Image Buffer */
 /*****************************/

 ImageHandle = GlobalAlloc( GMEM_ZEROINIT | GMEM_MOVEABLE, ImageLength );
 if  ( ImageHandle == NULL )                     // IF Buffer Allocation Failed
    { LoadString( hInst, ID_ALLOCERROR, String1, STRBUFLEN );
      LoadString( hInst, ID_GLOBALLERR, String2, STRBUFLEN );
      MessageBeep( 0 );
      MessageBox( hwndRestoreImage, String1,     //    Display Error Message
                  String2, MB_ICONHAND | MB_APPLMODAL );
      GlobalUnlock( StructHandle );              //    Release Image Structure
      GlobalFree( StructHandle );                //    Free Image Structure
      _lclose( FileHandle );                     //    Close the Image File
      return;                                    //    Exit Function
      }

 ImageBuffer = GlobalLock( ImageHandle );
 if  ( ImageBuffer == NULL )                     // IF Can't Lock Memory
    { LoadString( hInst, ID_ALLOCERROR, String1, STRBUFLEN );
      LoadString( hInst, ID_LOCKERROR, String2, STRBUFLEN );
      MessageBeep( 0 );
      MessageBox( hwndRestoreImage, String1,     //    Display Error Message
                  String2, MB_ICONHAND | MB_APPLMODAL );
      GlobalUnlock( StructHandle );              //    Release Image Structure
      GlobalFree( StructHandle );                //    Free Image Structure
      GlobalFree( ImageHandle );                 //    Free Image Buffer
      _lclose( FileHandle );                     //    Close the Image File
      return;                                    //    Exit Function
      }

 ImageStart = ( LPSTR ) ImageBuffer;
 ImageStructPtr->hImageBuffer = ImageHandle;

 /****************************/
 /* Read the Image From Disk */
 /****************************/

 while ( TRUE )
      { BytesRead = _lread( FileHandle, ( LPSTR ) ImageBuffer, HALFSEGMENT );
        if  ( BytesRead < ( unsigned int ) HALFSEGMENT )
             break;
        ImageBuffer += HALFSEGMENT;
        }

 ImageStructPtr->sLenX = * ( ( int far * ) &ImageStart[12] );
 ImageStructPtr->sLenY = * ( ( int far * ) &ImageStart[14] );

 /*****************************************/
 /* Now Let the Toolkit Display the Image */
 /*****************************************/

 _lclose( FileHandle );                        // Close the Image File
 GlobalUnlock( ImageHandle );                  // Release the Image Buffer

 MVWNCommand( MVWN_CMD_RESTOREIMAGE,           // Let Toolkit Display the Image
              ( LONG ) ( LPSTR ) ImageStructPtr,
              NULL );
 MVWNCommand( MVWN_CMD_RELEASEIMAGE,           // Let Toolkit Free Image Buffer
              ( LONG ) ( LPSTR ) ImageStructPtr,
              NULL );
 GlobalUnlock( StructHandle );                 // Release the Image Structure
 GlobalFree( StructHandle );                   // Free the Image Structure
 return;
 }


/*****************************************************************************/
/* Func: ImageDlgProc()                                                      */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: Dialog procedure that allows the user to Select and Display         */
/*       a Video Image from a Disk File                                      */
/*****************************************************************************/

WORD  FAR PASCAL ImageDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 WORD  Item;
 WORD  ImageCount;
 CHAR  ImageName[13];
 WORD  i;
 WORD  Answer;
 FARPROC lpProcSave;                      // Save Image Procedure Instance Addr
 BOOL  Frozen;
 CHAR  String1[STRBUFLEN];

   switch ( Message )
       {
        case WM_INITDIALOG:
           hwndListBox = GetDlgItem( hwnd, ID_LISTBOX );
           SetFocus( hwnd );
           PostMessage( hwnd, WM_USER + 1, 0, 0L );
           break;

        case WM_ACTIVATE:
           if  ( wParam  &&  hwndVideoWindow )      // If Being Activated
              { if  ( !FullScreenVideo )
                     SetWindowPos( hwndFrame, hwnd, 0, 0, 0, 0,
                                   SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE );
                BecomingActive();
                }
           break;

        case WM_USER+1:
           SetBoxPosition( hwndRestoreImage, TOP );
           ShowWindow( hwndRestoreImage, SHOW_OPENWINDOW );
           UpdateWindow( hwndRestoreImage );
           DlgDirList( hwnd, "*.VID", ID_LISTBOX, 0, 0 );
           SendMessage( hwndListBox, LB_SETCURSEL, 0, 0L );
           return( TRUE );

        case WM_COMMAND:
           switch ( wParam )
                 { case ID_LISTBOX:
                        if  ( HIWORD( lParam ) == LBN_DBLCLK )
                           { DlgDirSelect( hwnd, ImageName, ID_LISTBOX );
                             if  ( ImageName[0] )
                                { MVWNCommand( MVWN_CMD_FREEZE,
                                              ( LONG ) TRUE, NULL );
                                  RestoreVideoImage( ImageName );
                                  }
                             }
                        break;

                   case ID_SHOWALL:
                        ImageCount = ( WORD ) SendMessage( hwndListBox,
                                                          LB_GETCOUNT, 0, 0L );
                        if  ( ImageCount )
                           { MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) TRUE, NULL );
                             for  ( i = 0; i < ImageCount; i++ )
                                 { SendMessage( hwndListBox, LB_GETTEXT, i,
                                                ( LONG ) ( LPSTR ) ImageName );
                                   RestoreVideoImage( ImageName );
                                   }
                             }
                        break;

                   case ID_LOAD:
                        DlgDirSelect( hwnd, ImageName, ID_LISTBOX );
                        if  ( ImageName[0] )
                           { MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) TRUE, NULL );
                             RestoreVideoImage( ImageName );
                             }
                        break;

                   case ID_SAVE:
                        UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
                        Frozen = VideoStatus.mvidst.bFreeze;
                        if  ( !Frozen )                     // IF Is Not Frozen
                             MVWNCommand( MVWN_CMD_FREEZE,  //    Freeze It
                                          ( LONG ) TRUE, NULL );

                        lpProcSave = MakeProcInstance( SaveImageDlgProc, hInst );
                        DialogBox( hInst, "ImageName", hwnd, lpProcSave );
                        FreeProcInstance( lpProcSave );

                        if  ( !Frozen )                     // IF Wasn't Frozen
                             MVWNCommand( MVWN_CMD_FREEZE,  //    Thaw It Now
                                          ( LONG ) FALSE, NULL );
                        break;

                   case ID_DELETE:
                        Item = ( WORD ) SendMessage( hwndListBox,
                                                     LB_GETCURSEL, 0, 0L );
                        if  ( Item == LB_ERR )
                             return( 0L );

                        SendMessage( hwndListBox, LB_GETTEXT,
                                     Item, ( LONG ) ( LPSTR ) ImageName );
                        if  ( ImageName[0] == 0 )
                             return( 0L );

                        LoadString( hInst, ID_DELETENTRY, String1, STRBUFLEN );
                        MessageBeep( 0 );
                        Answer = MessageBox( hwndRestoreImage,
                                             ImageName, String1,
                                             MB_ICONQUESTION |
                                             MB_YESNO | MB_APPLMODAL );
                        if  ( Answer == IDNO )
                             return( 0L );

                        SendMessage( hwndListBox, LB_DELETESTRING, Item, 0L );
                        SendMessage( hwndListBox, LB_SETCURSEL, Item, 0L );
                        unlink( ImageName );
                        break;

                   case IDCANCEL:
                        hwndRestoreImage = NULL;
                        DestroyWindow( hwnd );
                        SetFocus( hwndVideoWindow );
                        return( TRUE );
                   }
           break;
        }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: SetBoxPosition()                                                    */
/*                                                                           */
/* Desc: Place Window at top ( or bottom ) right corner                      */
/*       of main window, but keep it on the screen.                          */
/*****************************************************************************/

VOID  SetBoxPosition( Window, TopOrBottom )
HWND  Window;
SHORT TopOrBottom;
{
 SHORT  xpos;                                     // New X Position
 SHORT  ypos;                                     // New Y Position
 SHORT  FullScreenWidth;                          // Full Screen Width
 SHORT  FullScreenHeight;                         // Full Screen Height
 RECT   FrameRect;                                // Frame Window Coordinates
 RECT   Rect;                                     // This Window's Coordinates
 SHORT  Height;                                   // This Window's Height
 SHORT  Width;                                    // This Window's Width

 GetWindowRect( hwndFrame, &FrameRect );          // Get Frame Window Coord.

 GetWindowRect( Window, &Rect );                  // Get This Window's Coord.
 Height = Rect.bottom - Rect.top;
 Width  = Rect.right - Rect.left;

 FullScreenWidth  = GetSystemMetrics( SM_CXSCREEN );
 FullScreenHeight = GetSystemMetrics( SM_CYSCREEN );

 xpos = FrameRect.right;
 if  ( ( xpos + Width ) > FullScreenWidth )
      xpos = FullScreenWidth - Width;

 if  ( TopOrBottom == TOP )                       // Top or Bottom of Frame ?
    { if  ( FullScreenVideo )
         { ypos = 0;
           xpos = FullScreenWidth - Width;
           }
      else
           ypos = FrameRect.top;
      }
 else
    { if  ( FullScreenVideo )
         { ypos = FullScreenHeight - Height - scrollbarheight;
           xpos = FullScreenWidth - Width;
           }
      else
           ypos = FrameRect.bottom - Height;
      }

 if  ( ( ypos + Height )  >  FullScreenHeight )
      ypos = FullScreenHeight - Height;

 if  ( ypos < 0 )                                 // Don't Go Too Low
      ypos = 0;

 if  ( ( Rect.left != xpos )  ||
       ( Rect.top != ypos ) )
      MoveWindow( Window, xpos, ypos, Width, Height, TRUE );

 ShowWindow( Window, SW_SHOW );
 BringWindowToTop( Window );
 UpdateWindow( Window );

 return;
 }


/*****************************************************************************/
/* Func: SaveImageDlgProc()                                                  */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: Dialog procedure for Saving Video Images to Disk                    */
/*****************************************************************************/

WORD  FAR PASCAL SaveImageDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 char  ImageName[14];
 WORD  Found;
 WORD  rc;

   switch ( Message )
         { case WM_INITDIALOG:
              SendDlgItemMessage( hwnd, ID_KEYWORD, EM_LIMITTEXT, 8, 0L );
              SetFocus( GetDlgItem( hwnd, ID_KEYWORD ) );
              break;

           case WM_ACTIVATE:
              if  ( wParam  &&  hwndVideoWindow )      // If Being Activated
                   BecomingActive();
              break;

           case WM_COMMAND:
              switch ( wParam )
                    { case IDOK:
                           GetDlgItemText( hwnd, ID_KEYWORD, ImageName, 9 );
                           if  ( ( ImageName[0] == 0 ) ||
                                 ( ImageName[0] == ' ' ) )
                              { ShowMessage( hwnd, ID_BLANKNAME, ID_IMAGENAME );
                                return( FALSE );
                                }
                           strcat( ImageName, ".VID" );
                           rc = SaveVideoImage( ImageName );
                           Found = ( WORD ) SendMessage( hwndListBox,
                                                         LB_FINDSTRING, -1,
                                                ( LONG ) ( LPSTR ) ImageName );
                           if  ( ( Found == LB_ERR ) &&
                                 ( rc == 1 ) )
                                SendMessage( hwndListBox, LB_ADDSTRING,
                                            -1, ( LONG ) ( LPSTR ) ImageName );
                           EndDialog( hwnd, IDOK );
                           return( TRUE );

                      case IDCANCEL:
                           EndDialog( hwnd, IDCANCEL );
                           return( TRUE );
                      }
              break;
           }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: BitMapToClipboard                                                   */
/*****************************************************************************/

VOID BitMapToClipboard( )
{
 HANDLE    CommandHandle;                        // Handle for Command Struct.
 LPMVWNCMD pmvwncmd;                             // Pointer to Commmand Struct.
 HBITMAP   hbmMVWN;                              // Video Bitmap Handle

 hbmMVWN = NULL;
 if  ( CommandHandle = MVWNCreateCmd( MVWN_CMD_CAPTUREBITMAP ) )
    { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
      pmvwncmd->data.dwCmdParam1 = ( LONG ) ( LPSTR ) &hbmMVWN;
      GlobalUnlock( CommandHandle );
      SendMessage( hwndVideoWindow, mvwnCommandMsg,
                   NULL, ( LONG ) CommandHandle );
      }

 if  ( hbmMVWN )
    { if  ( OpenClipboard( hwndFrame ) )            // Open the Clipboard
         { EmptyClipboard();                        // Empty the Clipboard
           SetClipboardData( CF_BITMAP, hbmMVWN );  // Send Bitmap to Clipboard
           CloseClipboard();                        // Close the Clipborad
           }
      }

 return;
 }
