/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  RANGE.C                                                          */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: All procedures which deal with frame ranges                      */
/*****************************************************************************/

/*****************************************************************************/
/* A Frame Range Entry Looks Like This:                                      */
/*                                                                           */
/*   KEY                    32 Characters Max                                */
/*                                                                           */
/*   FROM FRAME NUMBER      8 Characters, Padded Right With Blanks if Needed */
/*   TO FRAME NUMBER        8 Characters, Padded Right With Blanks if Needed */
/*   DESCRIPTION            Null Terminated, Max Length = 80                 */
/*****************************************************************************/

/*****************************************************************************/
/* Defines and Includes                                                       /
/*****************************************************************************/

#include <WINDOWS.H>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.

static  VOID   AddItem( HWND );
static  VOID   InitProfileListBox( HWND );
static  VOID   LoadRange( HWND );

VOID  ReadProfile( PCHAR, PCHAR, LONG *, LONG * );
VOID  WriteProfile( PCHAR, PCHAR, LONG, LONG );

static  CHAR   DialogTitle[128];         // Title of Dialog Window
static  CHAR   ProfileFileName[80];      // Current Profile File Name

static  CHAR   *DefaultEntry = "1       54000    ";


/*****************************************************************************/
/* Func: EditRange()                                                          /
/*                                                                            /
/* Parm: PSZ      pszDLLName;                                                 /
/*                                                                            /
/* Desc: Edit Frame Ranges                                                    /
/*****************************************************************************/

VOID EditRange( hwndOwner, ProfileName )
HWND hwndOwner;
PSZ  ProfileName;
{
 FARPROC lpEditProc;                     // Temp Dialog Procedure Instance Addr
                                               /******************************/
                                               /* Get and Save App. Name     */
                                               /******************************/

 LoadString( hInst, ID_EDITRANGETEXT,           /* Get Dialog's Title Text   */
             DialogTitle, sizeof( DialogTitle ));
 strcat( DialogTitle, ProfileName );            /* Add Profile Name to Title */
 DialogTitle[ strlen( DialogTitle ) - 4 ] = 0;  /* Truncate .BRZ Extension   */

 lpEditProc = MakeProcInstance( EditRangeDlgProc, hInst );
 DialogBox( hInst, "EDITRANGE", hwndFrame, lpEditProc );
 FreeProcInstance( lpEditProc );

 return;
 }


/*****************************************************************************/
/* Func: EditRangeDlgProc                                                     /
/*                                                                            /
/* Parm: HWND     hwnd                                                        /
/*       WORD     Message                                                     /
/*       WORD     wParam                                                      /
/*       DWORD    lParam                                                      /
/*                                                                            /
/* Desc: The Dialog procedure for Editing Frame Ranges                        /
/*****************************************************************************/

WORD  FAR PASCAL EditRangeDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
        CHAR    szErrorTitle[BUFFSIZE];
 static CHAR    szBuffer[BUFFSIZE + 1];
 static WORD    item = 0;                        /* Current Item Number      */
 static CHAR    ItemText[BUFFSIZE + 1];          /* Current Item Name        */
 static CHAR    ItemDescription[81];             /* Current Item Description */
 static WORD    TempItem;
        CHAR    TempBuffer[128];
        WORD    Answer;
 static HWND    hwndListBox;
 CHAR   Description[DESCSIZE];
 LONG   From;
 LONG   To;

 switch ( Message )
    {                               /*****************************************/
    case WM_INITDIALOG:             /* Initialize the list box with keywords */
         hwndListBox = GetDlgItem( hwnd, ID_LISTBOX );
         InitProfileListBox( hwnd );

                                            /*********************************/
                                            /* Set Size of Each Entry Field  */
                                            /*********************************/

         SendDlgItemMessage( hwnd, ID_DESCRIPTION, EM_LIMITTEXT, DESCSIZE, 0L );
         SendDlgItemMessage( hwnd, ID_KEYWORD, EM_LIMITTEXT, BUFFSIZE, 0L );
         SendDlgItemMessage( hwnd, ID_FROMFRAME, EM_LIMITTEXT, 6, 0L );
         SendDlgItemMessage( hwnd, ID_TOFRAME, EM_LIMITTEXT, 6, 0L );
         SendDlgItemMessage( hwnd, ID_DESCRIPTION, EM_LIMITTEXT, DESCSIZE, 0L );

                                            /*********************************/
                                            /* Select 1st Item and Set Focus */
                                            /*********************************/
         SendMessage( hwndListBox, LB_SETCURSEL, 0, 0L );

         SendMessage( hwndListBox, LB_GETTEXT, 0,
                      ( LONG ) ( LPSTR ) ItemText );
         ReadProfile( ItemText, Description, &From, &To );
         SetDlgItemText( hwnd, ID_KEYWORD, ItemText );
         wsprintf( TempBuffer, "%ld", From );
         SetDlgItemText( hwnd, ID_FROMFRAME, TempBuffer );
         wsprintf( TempBuffer, "%ld", To );
         SetDlgItemText( hwnd, ID_TOFRAME, TempBuffer );
         strcpy( ItemDescription, Description );
         SetDlgItemText( hwnd, ID_DESCRIPTION, ItemDescription );

         SetWindowText( hwnd, DialogTitle );
         SetFocus( GetDlgItem( hwnd, ID_KEYWORD ) );
         break;

      case WM_COMMAND:
         switch ( wParam )
            {
            case ID_LISTBOX:
               switch ( HIWORD( lParam ) )
                  {
                  case LBN_SELCHANGE:
                       item = ( WORD ) SendMessage( hwndListBox,
                                                    LB_GETCURSEL, 0, 0L );
                       SendMessage( hwndListBox, LB_GETTEXT,
                                    item, ( LONG ) ( LPSTR ) ItemText );

                       ReadProfile( ItemText, Description, &From, &To );

                       SetDlgItemText( hwnd, ID_KEYWORD, ItemText );

                       wsprintf( TempBuffer, "%ld", From );
                       SetDlgItemText( hwnd, ID_FROMFRAME, TempBuffer );

                       wsprintf( TempBuffer, "%ld", To );
                       SetDlgItemText( hwnd, ID_TOFRAME, TempBuffer );

                       strcpy( ItemDescription, Description );
                       SetDlgItemText( hwnd, ID_DESCRIPTION, ItemDescription );

                       break;

                  case LBN_DBLCLK:
                       LoadRange( hwnd );
                       break;
                  }
               break;

            case ID_LOAD:
                 LoadRange( hwnd );
                 return( TRUE );

            case ID_DELETE:
                 wsprintf( TempBuffer, " %s\n %s",
                           ( LPSTR ) ItemText,
                           ( LPSTR ) ItemDescription );
                 LoadString( hInst, ID_DELETEENTRY,
                             szErrorTitle, BUFFSIZE );
                 Answer = MessageBox( hwnd, TempBuffer, szErrorTitle,
                                     MB_ICONQUESTION | MB_YESNO | MB_APPLMODAL );
                 if  ( Answer == IDNO )
                     return( 0L );

                 WriteProfile( ItemText, NULL, 0, 0 );
                 SendMessage( hwndListBox, LB_DELETESTRING, item, 0L );
                 if  ( item )
                      item--;
                 SendMessage( hwndListBox, LB_SETCURSEL, item, 0L );
                 SendMessage( hwndListBox, LB_GETTEXT,
                            item, ( LONG ) ( LPSTR ) ItemText );
                 return( 0L );

            case ID_ADD:
                 GetDlgItemText( hwnd, ID_KEYWORD, szBuffer, BUFFSIZE );
                 if  ( ( szBuffer[0] == 0 ) ||
                      ( szBuffer[0] == ' ' ) )
                   { ShowMessage( hwnd, ID_BLANKNAME, ID_RANGENAME );
                     return( 0L );
                     }

                 TempItem = ( WORD ) SendMessage( hwndListBox, LB_FINDSTRING,
                                                  -1, ( LONG ) ( LPSTR ) szBuffer );
                 if  ( TempItem != LB_ERR )
                      ShowMessage( hwnd, ID_ALREADYEXISTS, ID_CANTADD );
                 else
                    { SendMessage( hwndListBox, LB_ADDSTRING,
                                   0, ( LONG ) ( LPSTR ) szBuffer );
                      AddItem( hwnd );
                      }
                 return( TRUE );

            case ID_CHANGE:
                 GetDlgItemText( hwnd, ID_KEYWORD, szBuffer, BUFFSIZE );
                 TempItem = ( WORD ) SendMessage( hwndListBox, LB_FINDSTRING,
                                                  -1, ( LONG ) ( LPSTR ) szBuffer );
                 if  ( TempItem == LB_ERR )
                      ShowMessage( hwnd, ID_DOESNTEXIST, ID_CANTADD );
                 else
                      AddItem( hwnd );
                 return( TRUE );

            case IDCANCEL:
                 EndDialog( hwnd, TRUE );
                 return( TRUE );
            }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: LoadRange()                                                         */
/*                                                                           */
/* Parm: HWND    Edit Frame Range Dialog Box Window Handle                   */
/*                                                                           */
/* Desc: Get the Current Values in the Edit Frame Range Dialog Box and       */
/*       Load the Frame Range                                                */
/*****************************************************************************/

static VOID LoadRange( hwnd )
HWND  hwnd;
{
 LONG from;
 LONG to;
 CHAR szBuffer[BUFFSIZE + 1];

 GetDlgItemText( hwnd, ID_FROMFRAME, szBuffer, BUFFSIZE );
 from = atol( szBuffer );
 GetDlgItemText( hwnd, ID_TOFRAME, szBuffer, BUFFSIZE );
 to = atol( szBuffer );
 MVPCCommand( MVPC_CMD_SETFRAMERANGE, from, to, NULL );

 return;
 }


/*****************************************************************************/
/* Func: AddItem                                                             */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*                                                                           */
/* Desc: Update the File with the New Item                                   */
/*****************************************************************************/

static VOID AddItem( hwnd )
HWND hwnd;
{
 CHAR  szBuffer[BUFFSIZE];
 CHAR  Key[BUFFSIZE];
 CHAR  Description[DESCSIZE];
 LONG  From;
 LONG  To;
 HWND  hwndList;
 WORD  TempItem;
                                               /******************************/
                                               /* Get Values Entered         */
                                               /******************************/

 GetDlgItemText( hwnd, ID_FROMFRAME, szBuffer, BUFFSIZE );
 From = atol( szBuffer );
 GetDlgItemText( hwnd, ID_TOFRAME, szBuffer, BUFFSIZE );
 To = atol( szBuffer );
 GetDlgItemText( hwnd, ID_DESCRIPTION, Description, DESCSIZE );
 GetDlgItemText( hwnd, ID_KEYWORD, Key, BUFFSIZE );

 WriteProfile( Key, Description, From, To );   /* Update Profile             */

                                               /******************************/
                                               /* Make Sure Item is Selected */
                                               /******************************/

 hwndList = GetDlgItem( hwnd, ID_LISTBOX );
 TempItem = ( WORD ) SendMessage( hwndList, LB_FINDSTRING,
                                  -1, ( LONG ) ( LPSTR ) szBuffer );
 if  ( TempItem != LB_ERR )
      SendMessage( hwndList, LB_SETCURSEL, TempItem, 0L );

 return;
 }


/*****************************************************************************/
/* Func: FrameRangeDlgProc                                                   */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: The Dialog procedure for letting the user set a new Videodisc       */
/*       Frame Range.                                                        */
/*****************************************************************************/

WORD  FAR PASCAL FrameRangeDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 static char szBuffer[7];

 switch ( Message )
    {
    case WM_INITDIALOG:
      SetFocus( GetDlgItem( hwnd, ID_FROMFRAME ) );
      break;

    case WM_COMMAND:
       switch ( wParam )
          {
          case ID_SETFRAMERANGE:      // user wants to set new Frame Range

             // FIRST, get the values entered and validate them
             // THEN, if valid, send the SETFRAMERANGE cmd to the Vdisc Control

             GetDlgItemText( hwnd, ID_FROMFRAME, szBuffer, sizeof( szBuffer ) );
             fromFrame = atol( szBuffer );

             GetDlgItemText( hwnd, ID_TOFRAME, szBuffer, sizeof( szBuffer ) );
             toFrame = atol( szBuffer );

             if  ( ( fromFrame >= 1 )  &&  ( fromFrame <=  MaxFrame )  &&
                   ( toFrame   >= 1 )  &&  ( toFrame   <= MaxFrame )  &&
                   ( fromFrame != toFrame ) )
                { MVPCCommand( MVPC_CMD_SETFRAMERANGE,
                               fromFrame, toFrame, NULL );
                  EndDialog( hwnd, 1 );
                  return( TRUE );
                  }
             else
                { ShowMessage( hwnd, ID_INVALIDFRAMERANGE, ID_ENTRYERROR );
                  fromFrame = 1L;
                  toFrame   = MaxFrame;
                  }
             break;

          case IDCANCEL:
             EndDialog( hwnd, 1 );
             return( TRUE );
          }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: SaveRangeDlgProc                                                    */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: The Dialog procedure for letting the user save the current          */
/*       Videodisc Frame Range.  This frame range can be later retrieved     */
/*       based on the keyword that the user supplies in this dialog          */
/*****************************************************************************/

WORD  FAR PASCAL SaveRangeDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 static char   szRangeText[DESCSIZE];
 static char   szBuffer[BUFFSIZE + 1];
 CHAR   Description[DESCSIZE];
 static LONG   From;
 static LONG   To;

 switch ( Message )
    {
    case WM_INITDIALOG:
         UpdatePlayerStatus();
         From = PlayerStatus.lLowerBound;
         To   = PlayerStatus.lUpperBound;
         LoadString( hInst, ID_CURRANGE, szBuffer, BUFFSIZE );
         wsprintf( szRangeText, szBuffer, From, To );
         SetDlgItemText( hwnd, ID_CURRENTRANGE, szRangeText );
         SendDlgItemMessage( hwnd, ID_KEYWORD, EM_LIMITTEXT, BUFFSIZE - 1, 0L );
         SendDlgItemMessage( hwnd, ID_DESCRIPTION, EM_LIMITTEXT, DESCSIZE, 0L );
         SetFocus( GetDlgItem( hwnd, ID_KEYWORD ) );
         break;

    case WM_COMMAND:
       switch ( wParam )
            {
            case ID_SAVEKEYWORD:
                  // Read the keyword from the dialog box the save the
                  //    data with this keyword in the current profile
                 GetDlgItemText( hwnd, ID_KEYWORD, szBuffer, BUFFSIZE );
                 if  ( ( szBuffer[0] == 0 ) ||
                       ( szBuffer[0] == ' ' ) )
                    { ShowMessage( hwnd, ID_BLANKNAME, ID_RANGENAME );
                      return( 0L );
                      }
                 GetDlgItemText( hwnd, ID_DESCRIPTION, Description, DESCSIZE );
                 WriteProfile( szBuffer, Description, From, To );

                 // Fall through to dismiss the dialog

            case IDCANCEL:
                 EndDialog( hwnd, TRUE );
                 return( TRUE );
            }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: OpenProfileDlgProc                                                  */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: The Dialog procedure that allows the user to open an existing       */
/*       Videodisc profile file.  All frame ranges are stored and retrieved  */
/*       from this file.                                                     */
/*****************************************************************************/

WORD  FAR PASCAL OpenProfileDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 WORD   i;
 WORD   Item;
 static HWND hwndOpenListBox;

 switch ( Message )
    {
    case WM_INITDIALOG:
       hwndOpenListBox = GetDlgItem( hwnd, ID_LISTBOX );
       DlgDirList( hwnd, "*.BRZ", ID_LISTBOX, 0, 0 );
       SendMessage( hwndOpenListBox, LB_SETCURSEL, 0, 0L );
       break;

    case WM_COMMAND:
       switch ( wParam )
          {
          case ID_LISTBOX:
             switch ( HIWORD( lParam ) )
                {
                case LBN_DBLCLK:
                   Item = ( WORD ) SendMessage( hwndOpenListBox,
                                                LB_GETCURSEL, 0, 0L );
                   SendMessage( hwndOpenListBox, LB_GETTEXT,
                                Item, ( LONG ) ( LPSTR ) szProfileName );

                   // Make the Fully Pathed Profile Name

                   strupr( szProfileName );
                   strcpy( ProfileFileName, CurrentDirectory );
                   strcat( ProfileFileName, szProfileName );

                   // Now Get the Profile Name Without the Extension

                   for  ( i = 0; i < sizeof( szShortProfileName ); i++ )
                        { if  ( ( szProfileName[i] == 0 )  ||
                                ( szProfileName[i] == '.' ) )
                             { szShortProfileName[i] = 0;
                               break;
                               }
                          szShortProfileName[i] = szProfileName[i];
                          }

                   EndDialog( hwnd, IDOK );
                   return( TRUE );
                }
             break;

          case IDOK:
             Item = ( WORD ) SendMessage( hwndOpenListBox,
                                          LB_GETCURSEL, 0, 0L );
             SendMessage( hwndOpenListBox, LB_GETTEXT,
                          Item, ( LONG ) ( LPSTR ) szProfileName );

             // Make the Fully Pathed Profile Name

             strupr( szProfileName );
             strcpy( ProfileFileName, CurrentDirectory );
             strcat( ProfileFileName, szProfileName );

             // Now Get the Profile Name Without the Extension

             for  ( i = 0; i < sizeof( szShortProfileName ); i++ )
                  { if  ( ( szProfileName[i] == 0 )  ||
                          ( szProfileName[i] == '.' ) )
                       { szShortProfileName[i] = 0;
                         break;
                         }
                    szShortProfileName[i] = szProfileName[i];
                    }

             EndDialog( hwnd, IDOK );
             return( TRUE );

          case IDCANCEL:
             EndDialog( hwnd, IDCANCEL );
             return( TRUE );
          }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: ProfileDlgProc                                                      */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: The Dialog procedure that allows the user to create and set a new   */
/*       Videodisc profile file.  All frame ranges are stored and retrieved  */
/*       from this file.                                                     */
/*****************************************************************************/

WORD  FAR PASCAL ProfileDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 switch ( Message )
    {
    case WM_INITDIALOG:
       SendDlgItemMessage( hwnd, ID_PROFILENAME, EM_LIMITTEXT, 8, 0L );
       SetFocus( GetDlgItem( hwnd, ID_PROFILENAME ) );
       break;

    case WM_COMMAND:
       switch ( wParam )
          {
          case IDOK:
             GetDlgItemText( hwnd, ID_PROFILENAME, szShortProfileName,
                             sizeof( szShortProfileName ) );
             if  ( szShortProfileName[0] != ' ' )
                { strupr( szShortProfileName );
                  strcpy( szProfileName, szShortProfileName );
                  strcat( szProfileName, ".BRZ" );
                  strcpy( ProfileFileName, CurrentDirectory );
                  strcat( ProfileFileName, szProfileName );
                  }
             else
                  szProfileName[0] = 0;

             // Fall through to dismiss the dialog

          case IDCANCEL:
             EndDialog( hwnd, wParam );
             return( TRUE );
          }
       break;
    }

 return( FALSE );
 }


/*****************************************************************************/
/* Func: LoadRangeDlgProc                                                    */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       DWORD    lParam                                                     */
/*                                                                           */
/* Desc: The Dialog procedure for letting the user select via keyword a      */
/*       saved Videodisc Frame Range.  This frame range will be sent to      */
/*       the videodisc control to remap the videodisc control frame range.   */
/*****************************************************************************/

WORD  FAR PASCAL LoadRangeDlgProc( hwnd, Message, wParam, lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 static char szBuffer[BUFFSIZE];
        WORD  item;
 static HWND hwndLoadListBox;
 CHAR   Description[DESCSIZE];

  switch ( Message )
       {
       case WM_INITDIALOG:       // Initialize the list box with the keywords of
          hwndLoadListBox = GetDlgItem( hwnd, ID_LISTBOX );
          InitProfileListBox( hwnd );
          SendMessage( hwndLoadListBox, LB_SETCURSEL, 0, 0L );
          SetFocus( hwnd );
          break;

       case WM_COMMAND:
          switch ( wParam )
             {
             case ID_LISTBOX:
                switch ( HIWORD( lParam ) )
                   {
                   case LBN_DBLCLK:
                        // When user selects an item, get the keyword, Query
                        //      the data for this keyword, then issue the
                        //      SETFRAMERANGE command to the videodisc control
                        item = ( WORD ) SendMessage( hwndLoadListBox,
                                                     LB_GETCURSEL, 0, 0L );
                        SendMessage( hwndLoadListBox, LB_GETTEXT,
                                     item, ( LONG ) ( LPSTR ) szBuffer );
                        ReadProfile( szBuffer, Description,
                                     &fromFrame, &toFrame );
                        MVPCCommand( MVPC_CMD_SETFRAMERANGE,
                                     fromFrame, toFrame, NULL );
                        EndDialog( hwnd, TRUE );
                        return( TRUE );
                   }
                break;

             case IDOK:
                  // When user selects an item, get the keyword,
                  //    query the data for this keyword, then issue the
                  //    SETFRAMERANGE command to the videodisc control
                  item = ( WORD ) SendMessage( hwndLoadListBox,
                                               LB_GETCURSEL, 0, 0L );
                  SendMessage( hwndLoadListBox, LB_GETTEXT,
                               item, ( LONG ) ( LPSTR ) szBuffer );
                  ReadProfile( szBuffer, Description, &fromFrame, &toFrame );
                  MVPCCommand( MVPC_CMD_SETFRAMERANGE,
                               fromFrame, toFrame, NULL );

                  /**** Intentional Fall-Thru to Next Case ****/

             case IDCANCEL:
                  EndDialog( hwnd, TRUE );
                  return( TRUE );
             }
          break;
       }

  return( FALSE );
  }


/*****************************************************************************/
/* Func: ShowMessage()                                                       */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     ID_Message    Resource ID of Message                       */
/*       WORD     ID_Title      Resource ID of Title                         */
/*                                                                           */
/* Desc: Show a Message Box                                                  */
/*****************************************************************************/

VOID  ShowMessage( hwnd, ID_Message, ID_Title )
HWND  hwnd;
WORD  ID_Message;
WORD  ID_Title;
{
  CHAR  szErrorTitle[80];
  CHAR  szErrorString[128];

  MessageBeep( 0 );
  LoadString( hInst, ID_Message,
              szErrorString, 80 );
  LoadString( hInst, ID_Title,
              szErrorTitle, 128 );
  MessageBox( hwnd, szErrorString,
              szErrorTitle, MB_ICONHAND );
  return;
  }


/*****************************************************************************/
/* FUNCTION: WriteProfile()                                                  */
/*                                                                           */
/* PARMS: PCHAR    Key              Key Index                                */
/*        PCHAR    Description      Description Text                         */
/*        LONG     From             From Frame Number                        */
/*        LONG     To               To Frame Number                          */
/*                                                                           */
/* DESCRIPTION: Write a Range Entry in a Profile                             */
/*****************************************************************************/

VOID  WriteProfile( Key, Description, From, To )
PCHAR Key;
PCHAR Description;
LONG  From;
LONG  To;
{
 CHAR  TempBuffer[9];
 CHAR  Entry[ DESCSIZE + 16 ];

 if  ( Description != NULL )
    { wsprintf( Entry, "%ld", From );
      while  ( strlen( Entry ) < 8 )
            strcat( Entry, " " );

      wsprintf( TempBuffer, "%ld", To );
      while  ( strlen( TempBuffer ) < 8 )
            strcat( TempBuffer, " " );

      strcat( Entry, TempBuffer );
      strcat( Entry, Description );

      WritePrivateProfileString( ( LPSTR ) szAppName, ( LPSTR ) Key,
                                 ( LPSTR ) Entry, ( LPSTR ) ProfileFileName );
      }
 else
      WritePrivateProfileString( ( LPSTR ) szAppName, ( LPSTR ) Key,
                                   NULL, ( LPSTR ) ProfileFileName );

 return;
 }


/*****************************************************************************/
/* FUNCTION: ReadProfile()                                                   */
/*                                                                           */
/* PARMS: PCHAR    Key              Key Index                                */
/*        PCHAR    Description      Description Text ( Returned )            */
/*        LONG     *FromPtr         From Frame Number ( Returned )           */
/*        LONG     *ToPtr           To Frame Number ( Returned )             */
/*                                                                           */
/* DESCRIPTION: Read a Range Entry in a Profile                              */
/*****************************************************************************/

VOID  ReadProfile( Key, Description, FromPtr, ToPtr )
PCHAR Key;
PCHAR Description;
LONG  *FromPtr;
LONG  *ToPtr;
{
 CHAR  Entry[128];
 int   rc;

 rc = GetPrivateProfileString( ( LPSTR ) szAppName, ( LPSTR ) Key,
                               ( LPSTR ) DefaultEntry, ( LPSTR ) Entry,
                               128, ( LPSTR ) ProfileFileName );
 if  ( rc )
    { *FromPtr = atol( Entry );
      *ToPtr   = atol( &Entry[8] );
      strcpy( Description, &Entry[16] );
      }
 else
    { *FromPtr = 1L;
      *ToPtr   = MaxFrame;
      memset( Description, 0, DESCSIZE );
      }
 return;
 }


/*****************************************************************************/
/* FUNCTION: InitProfileListBox()                                            */
/*                                                                           */
/* PARMS: HWND     hwnd             Dialog Box Handle                        */
/*                                                                           */
/* DESCRIPTION: Initialize the Profile List Box Entries                      */
/*****************************************************************************/

#define MEMSIZE 16000              // A Range Name Cannot Exceed 32 Characters,
                                   //   So This Size Holds at Least 500 Entries

static VOID InitProfileListBox( hwnd )
HWND hwnd;
{
 LPSTR  Next;                                 // Pointer to Returned Entry List
 HWND   hwndList;                             // List Box Handle
 HANDLE MemHandle;                            // Allocated Memory Handle
 int    ReturnChars;                          // Number of Characters Returned
 CHAR   NullString = 0;                       // Default Returned Entry

 hwndList = GetDlgItem( hwnd, ID_LISTBOX );   // Get List Box Handle

 MemHandle = GlobalAlloc( GMEM_FIXED | GMEM_ZEROINIT, ( DWORD ) MEMSIZE );
 if  ( MemHandle == NULL )                    // IF Memory Allocation Failed
      return;

 Next = GlobalLock( MemHandle );
 if  ( Next == NULL )                         // IF Can't Lock Down Memory
    { GlobalFree( MemHandle );                //    Free the Allocated memory
      return;
      }

 ReturnChars = GetPrivateProfileString( ( LPSTR ) szAppName,  NULL,
                                        ( LPSTR ) &NullString,  Next,
                                        MEMSIZE, ( LPSTR ) ProfileFileName );

 if  ( ReturnChars )                          // IF Entries Were Found
    { while ( *Next )                         //    Add Them as a List Box Item
           { SendMessage( hwndList, LB_ADDSTRING, 0, ( LONG ) Next );
             Next += lstrlen( Next ) + 1;
             }
      }

 GlobalUnlock( MemHandle );                   // Release the Allocated Memory
 GlobalFree( MemHandle );
 return;
 }
