/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  RESPONSE.C                                                       */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: Window procedures that handle feedback ( acknowledges, responses,*/
/*          and advises ) from the video window and the videodisc controls   */
/*****************************************************************************/

#include <WINDOWS.H>
#include <stdlib.h>

#define  EXTERN extern
#include "BROWDEFS.H"                    // This program's special defines etc.

static VOID PlayerErrorMsg( WORD, WORD, WORD );

/*****************************************************************************/
/* Func: MVPCResponseWndProc                                                 */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       LONG     lParam                                                     */
/*                                                                           */
/* Desc: This is the response wndproc for the videodisc control. We have     */
/*       requested that all acknowledges and responses from the control be   */
/*       sent here.  Basically, we'll just look for player errors.           */
/*****************************************************************************/

LONG FAR PASCAL MVPCResponseWndProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 WORD  Response,
       Player,
       Offset,
       Command,
       ReturnCode;
 LPMVPCCMD pmvpccmd;

 switch ( Message )
    {
    case MVPC_ACK:
       pmvpccmd   = ( LPMVPCCMD ) GlobalLock( ( HANDLE ) lParam );
       Command = pmvpccmd->command.wCommand;       // Get Original Command
       ReturnCode = pmvpccmd->command.wReturnCode;  // Get Return Code
       Response   = pmvpccmd->command.wResponse;    // Get Response Code
       GlobalUnlock( ( HANDLE ) lParam );
       if  ( ReturnCode != MVPC_ERR_NOERROR )       // IF There Was NO Error
          {                                         //  Display Error Condition
            for  ( Offset = 0; Offset < MAX_PLAYERS; Offset++ )
                { if  ( hwndMVPCs[ Offset ] == ( HWND ) wParam )
                       break;
                   }
            Player = Offset + 1;

            if  ( ReturnCode != MVPC_ERR_RETURNED )
                 PlayerErrorMsg( ReturnCode, ID_ACKERROR, Player );

            if  ( ReturnCode == MVPC_ERR_INVALIDRANGE )
               { fromFrame = 1L;
                 toFrame   = MaxFrame;
                 }

            if  ( ReturnCode == MVPC_ERR_NOTLOADED )
                 Loaded[ Offset ] = FALSE;

            }

       if  ( Response == MRESP_NORESPONSEPENDING )  // IF No Response Pending
            MVPCDestroyCmd( ( HANDLE ) lParam, ( HWND ) wParam, Message );

       return( TRUE );

    case MVPC_RESPONSE:
       pmvpccmd = ( LPMVPCCMD ) GlobalLock( ( HANDLE ) lParam );
       for  ( Offset = 0; Offset < MAX_PLAYERS; Offset++ )
           { if  ( hwndMVPCs[ Offset ] == ( HWND ) wParam )
                  break;
             }
       Command = pmvpccmd->command.wCommand;       // Get Original Command
       ReturnCode = pmvpccmd->command.wReturnCode; // Get Return Code
       GlobalUnlock( ( HANDLE ) lParam );
       switch ( Command )
          {
          case MVPC_CMD_INIT:                       // IF INIT went well
               if  ( ReturnCode == MVPC_ERR_NOERROR )
                  { if  ( ( HWND ) wParam == hwndMVPC )
                         MVWNCommand( MVWN_CMD_FREEZE,
                                      ( LONG ) FALSE, NULL );
                    Loaded[ Offset ] = TRUE;
                    }
               break;

          case MVPC_CMD_UNLOAD:
               if  ( ReturnCode == MVPC_ERR_NOERROR )
                    Loaded[ Offset ] = FALSE;
                                                       // Intentional Fall-Thru
          default:
               if  ( ReturnCode == MVPC_ERR_NOTLOADED )
                    Loaded[ Offset ] = FALSE;
               break;
          }

       MVPCDestroyCmd( ( HANDLE ) lParam, ( HWND ) wParam, Message );
       return( TRUE );
    }

 return( DefWindowProc( hwnd, Message, wParam, lParam ) );
 }


VOID DoCommand( WORD, LONG, LONG );

/*****************************************************************************/
/* Func: MVPCAdviseWndProc                                                   */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       LONG     lParam                                                     */
/*                                                                           */
/* Desc: We have told the videodisc control to send all advises ( for example*/
/*       seeks and activations ) here.  On Seeks, we'll freeze the video     */
/*       video on MVPC_ADV_SEEKBEGIN, unfreeze on MVPC_ADV_SEEKEND.          */
/*       This will give us 'seamless' video.                                 */
/*****************************************************************************/

LONG FAR PASCAL MVPCAdviseWndProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 LPMVPCADV pmvpcadv;
 HWND     hwndTemp;
 WORD     Advise, Error;

 if ( Message == MVPC_ADVISE )                  // IF a Control Advise Message
    {
    pmvpcadv = ( LPMVPCADV ) GlobalLock( ( HANDLE ) lParam );
    Advise = pmvpcadv->wAdvise;                 // Get the Advise Condition
    switch ( Advise )
       {
       case MVPC_ADV_LOSINGTOKEN:               // Control is Losing the Focus
          if  ( ( HWND ) wParam == hwndMVPC )
               MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) FALSE, NULL );
          break;

       case MVPC_ADV_TOKENCAPTURED:             // Control is Gaining the Focus
          if  ( ( HWND ) wParam == hwndMVPC )
               MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) TRUE, NULL );
          break;

       case MVPC_ADV_TOKENNOTCAPTURED:          // Try Again
          hwndTemp = hwndMVPC;
          hwndMVPC = ( HWND ) wParam;
          MVPCCommand( MVPC_CMD_ACTIVATE, ( LONG ) TRUE, NULL, NULL );
          hwndMVPC = hwndTemp;
          break;

       case MVPC_ADV_SEEKBEGIN:
          // We'll make an assumption here that IF the players and sources are
          // tied together,  THEN the video window is displaying the current
          // current player source and we will do the freeze/unfreeze for seeks
          if  ( ( HWND ) wParam == hwndMVPC )
             { if  ( fTiePlayersToSources )
                  { UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
                    if  ( VideoStatus.bActive )
                         MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) TRUE, NULL );
                    }
               }
          break;

       case MVPC_ADV_SEEKEND:
       case MVPC_ADV_SEEKFAILED:
          if  ( ( HWND ) wParam == hwndMVPC )
             { if  ( fTiePlayersToSources )
                  { UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
                    if  ( VideoStatus.bActive )
                         MVWNCommand( MVWN_CMD_FREEZE, ( LONG ) FALSE, NULL );
                    }
               }
          break;

       case MVPC_ADV_PLAYERERROR:
          Error = ( WORD ) ( pmvpcadv->dwAdvParam1 );
          PlayerErrorMsg( Error, ID_ADVISERROR, pmvpcadv->wPlayer );
          break;

       case VSB_ADV_USER:
          switch ( ( WORD ) ( pmvpcadv->dwAdvParam1 ) )
             {
               case VSB_ADV_SLIDERTRACK:
                    // Here's where you'd put any scrollbar slidertrack stuff
                    break;
               }
          break;
       }

    GlobalUnlock( ( HANDLE ) lParam );
    MVPCDestroyAdv( ( HANDLE ) lParam );

    return( TRUE );
    }

 return( DefWindowProc( hwnd, Message, wParam, lParam ) );
 }


/*****************************************************************************/
/* Func: MVWNResponseWndProc                                                 */
/*                                                                           */
/* Parm: HWND     hwnd                                                       */
/*       WORD     Message                                                    */
/*       WORD     wParam                                                     */
/*       LONG     lParam                                                     */
/*                                                                           */
/* Desc: This is the response proc for the video window.  We have requested  */
/*       that all acknowledges and responses from the control be sent here.  */
/*****************************************************************************/

LONG FAR PASCAL MVWNResponseWndProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 LPMVWNADV pmvwnadv;

 switch ( Message )
    {
      case MVWN_ACK:
           MVWNDestroyCmd( ( HANDLE ) lParam, ( HWND ) wParam, Message );
           return( TRUE );

      case MVWN_ADVISE:
          pmvwnadv = ( LPMVWNADV ) GlobalLock( ( HANDLE ) lParam );
          switch ( pmvwnadv->wAdvise )
                {
                  case MVWN_ADV_LOSINGTOKEN: // VideoWindow is Losing the Token
                       break;

                  case MVWN_ADV_TOKENCAPTURED:      // Control has Gained Focus
                       break;
                  }

          GlobalUnlock( ( HANDLE ) lParam );
          MVWNDestroyAdv( ( HANDLE ) lParam );
          return( TRUE );
      }

 return( DefWindowProc( hwnd, Message, wParam, lParam ) );
 }


/*****************************************************************************/
/* Func: PlayerErrorMsg()                                                    */
/*                                                                           */
/* Parm: WORD     ID_Message    Resource ID of Message                       */
/*       WORD     ID_Title      Resource ID of Title                         */
/*       WORD     Player        Player ( 1- 2 ) Inserted into Title String   */
/*                                                                           */
/* Desc: Show a Message Box Displaying a Player Error                        */
/*****************************************************************************/

static VOID PlayerErrorMsg( ID_Message, ID_Title, Player )
WORD  ID_Message;
WORD  ID_Title;
WORD  Player;
{
 CHAR  szErrorTitle[80];
 CHAR  szErrorString[80];

 Loaded[ Player - 1 ] = FALSE;
 LoadString( hInst, ID_Title, szErrorString, 80 );
 wsprintf( szErrorTitle, szErrorString, Player );
 LoadString( hInst, ID_Message, szErrorString, 80 );
 MessageBeep( 0 );
 MessageBox( hwndVideoWindow, szErrorString,
             szErrorTitle, MB_ICONHAND | MB_SYSTEMMODAL );
 return;
 }
