/*****************************************************************************/
/* (C) Copyright IBM Corporation 1991, 1992                                  */
/*                                                                           */
/* MODULE:  VIDEOCTL.C                                                       */
/* VERSION: M-Control Program / 2      Windows 3.0 ( TM ) Version            */
/*                                                                           */
/* PURPOSE: Functions Controlling the Video Window and Scroll Bar            */
/*****************************************************************************/

/*****************************************************************************/
/* Defines and Includes                                                      */
/*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  EXTERN  extern
#include "BROWDEFS.H"                  // This Program's Definitions

static FARPROC lpOldVideoProc;         // Original Video Window Procedure
static FARPROC lpNewVideoProc;         // New Video Window Procedure


/*****************************************************************************/
/* Func: SetIdealCoord()                                                     */
/*                                                                           */
/* Desc: Set Coordinates for Ideal Video Window Size.                        */
/*                                                                           */
/*       Ideal Coordinates is exactly half of the total scan lines and half  */
/*       of the width.  For NTSC inputs this is a client area of 328 X 232.  */
/*       Height would be 282 if a PAL input were used.                       */
/*****************************************************************************/

VOID SetIdealCoord()
{
 RECT  Rect;

 GetWindowRect( hwndFrame, &Rect );
 if  ( Rect.top < 0 )
      Rect.top = 0;
 if  ( Rect.left < 0 )
      Rect.left = 0;
 if  ( hwndMVPC )
      MoveWindow( hwndFrame, Rect.left, Rect.top,
                  IdealFrameWidth, IdealFrameHeight, TRUE );
 else
      MoveWindow( hwndFrame, Rect.left, Rect.top,
                  IdealFrameWidth, IdealFrameHeight - scrollbarheight, TRUE );
 return;
 }


/*****************************************************************************/
/* Func: ResizeVideoControls()                                               */
/*                                                                           */
/* Desc: ReSize VideoDisc Controls When Player or Video Source Changes       */
/*****************************************************************************/

VOID ResizeVideoControls()
{
 RECT  Rect;
 WORD  Height;
 WORD  Width;
 BOOL  ScrollBarWasVisible = FALSE;           // Is Current ScrollBar Visible ?
 BOOL  HadScrollBar;                          // Did Old Source Have Scroll Bar

 if  ( hwndMVPC )
    { HadScrollBar = TRUE;
      ScrollBarWasVisible = IsWindowVisible( hwndMVPC );
      if  ( hwndMVPC != hwndMVPCs[CurrentPlayer] )
           ShowWindow( hwndMVPC, SW_HIDE );
      }
 else
      HadScrollBar = FALSE;

 if  ( CurrentPlayer < MAX_PLAYERS )
      hwndMVPC = hwndMVPCs[CurrentPlayer];
 else
      hwndMVPC = NULL;

 if  ( hwndMVPC )                                // IF a New Scroll Bar Exists
    { if  ( ( ScrollBarWasVisible ) ||           //   IF Scroll Bar Was Visible
            ( !FullScreenVideo ) )               //     OR Not Full Screen
           ShowWindow( hwndMVPC, SW_SHOW );      //      Show New Scroll Bar
      }

 GetWindowRect( hwndFrame, &Rect );              // Get Height and Width of
 Width  = Rect.right - Rect.left;                //     the Frame Window
 Height = Rect.bottom - Rect.top;
 if  ( HadScrollBar == FALSE )                   // IF Prev. Had no Scroll Bar
      Height += scrollbarheight;                 //  Adjust Height as If it Did

 if  ( ( Width == IdealFrameWidth ) &&           // IF in Ideal Coordinates
       ( Height == IdealFrameHeight ) )          //    Keep it That Way !
      SetIdealCoord();

 if  ( !FullScreenVideo )                        // Size & Position Video Win
      SizeVideo();                               //      and Scroll Bar

 if  ( hwndMVPC )
    { MVPCCommand( MVPC_CMD_ACTIVATE, ( LONG ) TRUE, NULL, NULL );
      if  ( Loaded[ CurrentPlayer ] == FALSE )           // IF Not Loaded,
           MVPCCommand( MVPC_CMD_INIT, NULL, NULL, NULL ); //  Try to Load Plyr
      BringWindowToTop( hwndMVPC );
      UpdateWindow( hwndMVPC );
      }

 if  ( SaveSourceHlen[ usVideoSource ] == 0 )     // IF Source Defaults Not Set
    { UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
      SaveSourceHorg[ usVideoSource ] = VideoStatus.wSourceHorg;
      SaveSourceVorg[ usVideoSource ] = VideoStatus.wSourceVorg;
      SaveSourceHlen[ usVideoSource ] = VideoStatus.wSourceHlen;
      SaveSourceVlen[ usVideoSource ] = VideoStatus.wSourceVlen;
      }
 SetSourceCoordinates();

 return;
 }


/*****************************************************************************/
/* Func: CreateVideoControls                                                 */
/*                                                                           */
/* Desc: Creates the video controls.                                         */
/*****************************************************************************/

WORD CreateVideoControls()
{
 CHAR      ClassName[50];
 CHAR      ErrorMsg[50];
 RECT      ClientRect;
 LPMVWNCTL pmvwnctl;
 WORD      Width;
 WORD      Height;
 HANDLE    ControlHandle;
 DWORD     WinFlags;

  /************************************************************************/
  /* Init and register the Video Window class.                            */
  /* NOTE: This call tells US the message atoms for commands and queries; */
  /*    and WE tell the MVWN the message id's to use for Acknowledges,    */
  /*    Responses, and Advises.                                           */
  /************************************************************************/

 MVWNInitialize( &mvwnCommandMsg, &mvwnQueryMsg,
                 MVWN_ACK, MVWN_RESPONSE, MVWN_ADVISE );

  /********************************************************************/
  /* Load String, Register Class, Create Video Window Response Window */
  /********************************************************************/

 LoadString( hInst, ID_MVWNRESPONSECLASS, ClassName, sizeof( ClassName ) );
 hwndMVWNResponse = CreateWindow( ClassName, "", WS_POPUP, 0, 0,
                                  0, 0, hwndFrame, 0, hInst, NULL );

  /***********************************************************************/
  /* Now create a Control Structure to use for creating the Video Window */
  /***********************************************************************/

 ControlHandle = MVWNCreateCtl();
 if  ( ControlHandle )
    { pmvwnctl = ( LPMVWNCTL ) GlobalLock( ControlHandle );
      pmvwnctl->bUserActivate  = FALSE;   // We'll Do All Specific Activations
      pmvwnctl->hWndAdviseActive = hwndMVWNResponse;  // Set up Advise Window

      WinFlags = GetWinFlags();           // Windows Memory Configuration Flags
      if  ( ( WinFlags & WF_ENHANCED ) || // IF Standard or 386 Enhanced Mode
            ( WinFlags & WF_STANDARD ) )  //    Use Inactive Images
           pmvwnctl->wInactiveStyle = MVWN_INACTIVE_VIDEOIMAGE;
      else                                // ELSE in Real Mode
                                          //    Use Bitmap ( Uses Less Memory )
           pmvwnctl->wInactiveStyle = MVWN_INACTIVE_MONOBITMAP;

  /********************************************************************/
  /* Now check which of the three possible video sources are defined, */
  /*    and load active pointers for those that exist, and set the    */
  /*    MVWN CTL pointer fields accordingly                           */
  /********************************************************************/

      if  ( MVWNVideoSourceType( 1 ) )
         { hptrVid[0] = LoadCursor( hInst, MAKEINTRESOURCE( ID_VIDEO1 ) );
           pmvwnctl->hCursorSrc1 = hptrVid[0];
           }

      if  ( MVWNVideoSourceType( 2 ) )
         { hptrVid[1] = LoadCursor( hInst, MAKEINTRESOURCE( ID_VIDEO2 ) );
           pmvwnctl->hCursorSrc2 = hptrVid[1];
           }

      if  ( MVWNVideoSourceType( 3 ) )
         { hptrVid[2] = LoadCursor( hInst, MAKEINTRESOURCE( ID_VIDEO3 ) );
           pmvwnctl->hCursorSrc3 = hptrVid[2];
           }
      }

  /************************************************************/
  /* Create the Video Window.                                 */
  /* Get client rectangle in order to size the video window   */
  /*     which will be created as a child of the frame window */
  /*************************************************************/

 GetClientRect( hwndFrame, &ClientRect );
 Width  = ClientRect.right;
 Height = ClientRect.bottom;

 hwndVideoWindow = CreateWindow( WC_MVWN, NULL,
                                 WS_CHILD | WS_VISIBLE,
                                 0, 0, Width, Height,
                                 hwndFrame, 0, hInst,
                                 ( LPSTR ) pmvwnctl );

 GlobalUnlock( ControlHandle );
 MVWNDestroyCtl( ControlHandle );                      // Free the MVWN Control

 if  ( !hwndVideoWindow )
    { LoadString( hInst, ID_NOMULTIMEDIA, ErrorMsg, sizeof( ErrorMsg ) );
      MessageBox(  hwndFrame, ErrorMsg, "", MB_OK );
      return( FALSE );
      }

 MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) TRUE, NULL );  // Activate the Window

 SaveSourceHlen[2] = 0;                      // Sources 2 and 3 Not Initialized
 SaveSourceHlen[3] = 0;

 UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
 SaveSourceHorg[1] = VideoStatus.wSourceHorg;
 SaveSourceVorg[1] = VideoStatus.wSourceVorg;
 SaveSourceHlen[1] = VideoStatus.wSourceHlen;
 SaveSourceVlen[1] = VideoStatus.wSourceVlen;
 SetSourceCoordinates();

 /*****************************/
 /* Subclass the Video Window */
 /*****************************/

 lpOldVideoProc = ( FARPROC ) GetWindowLong( hwndVideoWindow, GWL_WNDPROC );
 lpNewVideoProc = MakeProcInstance( ( FARPROC ) VideoSubClassProc, hInst );
 SetWindowLong( hwndVideoWindow, GWL_WNDPROC, ( LONG ) lpNewVideoProc );

 return( TRUE );
 }


/*****************************************************************************/
/* Func: CreatePlayerControls                                                */
/*                                                                           */
/* Desc: Creates the VideoDisc Player Controls.                              */
/*****************************************************************************/

VOID CreatePlayerControls()
{
 LONG     lPlayerBits[MAX_PLAYERS];
 CHAR     szPlayerName[9];
 int      player;
 LPMVPCCTL pmvpcctl;
 HANDLE   ControlHandle;
 RECT     Rect;
 WORD     Width;
 CHAR     ClassName[50];
 CHAR     DLLName[50];

 GetClientRect( hwndFrame, &Rect );             /* Get Client Area Rectangle */
 Width = Rect.right;                            /* Initial Scroll Bar Width  */

 // Intialize the MVPC Response and Advise Windows

 LoadString( hInst, ID_MVPCRESPONSECLASS, ClassName, sizeof( ClassName ) );
 hwndMVPCResponse = CreateWindow( ClassName, "", WS_POPUP, 0, 0,
                                  0, 0, hwndFrame, 0, hInst, NULL );

 LoadString( hInst, ID_MVPCADVISECLASS, ClassName, sizeof( ClassName ) );
 hwndMVPCAdvise = CreateWindow( ClassName, "", WS_POPUP, 0, 0,
                                0, 0, hwndFrame, 0, hInst, NULL );

 // Intialize the MVPC DLL

 LoadString( hInst, ID_DEFAULTDLL, DLLName, sizeof( DLLName ) );
 MVPCInitialize( &mvpcCommandMsg, &mvpcQueryMsg,
                 MVPC_ACK, MVPC_RESPONSE, MVPC_ADVISE, DLLName );

 lPlayerBits[0] = WS_VDP1;
 lPlayerBits[1] = WS_VDP2;

 for ( player = MAX_PLAYERS - 1; player >= 0; player-- )
    { if  ( MVPCPlayerExists( player + 1, szPlayerName, sizeof( szPlayerName ) ) )
         { if ( ControlHandle = MVPCCreateCtl() )
             { pmvpcctl = ( LPMVPCCTL ) GlobalLock( ControlHandle );
               pmvpcctl->wStepIncrement   = 1;
               pmvpcctl->hWndAdviseFrame  = NULL;     // Don't Care About These
               pmvpcctl->hWndAdviseActive = hwndMVPCAdvise;
               pmvpcctl->hWndAdviseSeek   = hwndMVPCAdvise;
               pmvpcctl->hWndAdvisePlayerError = hwndMVPCAdvise;

               hwndMVPCs[player] = CreateWindow( WC_MVPC, NULL,
                                             WS_CHILD | lPlayerBits[player], 0,
                                             Rect.bottom - scrollbarheight,
                                             Width, scrollbarheight,
                                             hwndFrame, NULL, hInst,
                                             ( LPSTR ) pmvpcctl );

               GlobalUnlock( ControlHandle );
               MVPCDestroyCtl( ControlHandle );
               }

           // This is a special SCROLBAR user interface command that
           //     could be implemented by an custom user interface.
           //     VSB_CMD_SETPOINTERS sets an active and busy pointer,
           //     that will be used by the VSB DLL to set the pointer
           //     to reflect the current state ( available or busy ).

           hptrActive[player] = LoadCursor( hInst,
                             MAKEINTRESOURCE( ID_ACTIVE1 + ( player * 10 ) ) );
           hptrBusy[player] = LoadCursor( hInst,
                             MAKEINTRESOURCE( ID_BUSY1 + ( player * 10 ) ) );

           hwndMVPC = hwndMVPCs[player];
           MVPCCommand( VSB_CMD_SETPOINTERS,
                        ( LONG ) hptrActive[player],
                        ( LONG ) hptrBusy[player], NULL );
           }
      }

 hwndMVPC = hwndMVPCs[0];

 if  ( hwndMVPC )
    { ShowWindow( hwndMVPC, SW_SHOW );
      BringWindowToTop( hwndMVPC );
      UpdateWindow( hwndMVPC );
      }

 return;
 }

VOID ActivatePlayer( HWND );

/*****************************************************************************/
/* Func: BecomingActive                                                      */
/*                                                                           */
/* Desc: When Browser is re-gaining the focus, this function is called to    */
/*       ensure that the video window and the videodisc control are both     */
/*       active (i.e., hold the token) as well.                              */
/*****************************************************************************/

VOID BecomingActive()
{
 WORD  player;
 HWND  hwndTemp;
 BOOL  ActivateVideo = TRUE;       // Should We Activate the Video Window Now ?

 if  ( hwndMVPC )
    { UpdatePlayerStatus();
      if  ( !PlayerStatus.bActive )
           MVPCCommand( MVPC_CMD_ACTIVATE, ( LONG ) TRUE, NULL, NULL );
      BringWindowToTop( hwndMVPC );
      ActivateVideo = PlayerStatus.bActive;
      }

 hwndTemp = hwndMVPC;
 for  ( player = 0;  player != MAX_PLAYERS;  player++ )
     { if  ( hwndMVPCs[player] &&
           ( hwndMVPCs[player] != hwndTemp ) )
          { hwndMVPC = hwndMVPCs[player];
            UpdatePlayerStatus();
            if  ( !PlayerStatus.bActive )
                 MVPCCommand( MVPC_CMD_ACTIVATE, ( LONG ) TRUE, NULL, NULL );
            }
       }
 hwndMVPC = hwndTemp;

 if  ( ActivateVideo  &&  hwndVideoWindow )
    { UpdateVideoStatus( ( LPMVWNQRY ) &VideoStatus );
      if  ( !VideoStatus.bActive )
           MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) TRUE, NULL );
      }

 return;
 }


/*****************************************************************************/
/* FUNCTION: SizeVideo()                                                     */
/*                                                                           */
/* DESCRIPTION: Size the Video Control Windows                               */
/*****************************************************************************/

void SizeVideo()
{
 RECT  ClientRect;
 WORD  ClientHeight;
 WORD  ClientWidth;

 GetClientRect( hwndFrame, &ClientRect );      // Get Client Area Rectangle
 ClientHeight = ClientRect.bottom;
 ClientWidth  = ClientRect.right;

 if  ( hwndMVPC )                              // IF a Videodisc Control
    { if  ( IsWindowVisible ( hwndMVPC ) )
         { MoveWindow( hwndMVPC, 0,
                       ClientHeight - scrollbarheight,
                       ClientWidth, scrollbarheight, TRUE );
           if  ( !IsIconic( hwndFrame ) )
                ClientHeight -= scrollbarheight;
           }
      }

 if  ( ( hwndVideoWindow ) &&                  // IF a Video Window AND
       ( !FullScreenVideo ) )                  //    NOT Full Screen Video Mode
    { SetVideoWindowPos( hwndVideoWindow, 0, 0,
                         ClientWidth, ClientHeight );
      UpdateWindow( hwndVideoWindow );
      if  ( hwndRestoreImage )
           SetBoxPosition( hwndRestoreImage, TOP );
      if  ( hwndVideoAttributes )
           SetBoxPosition( hwndVideoAttributes, TOP );
      if  ( hwndAudioAttributes )
           SetBoxPosition( hwndAudioAttributes, BOTTM );
      }

 return;
 }


/*****************************************************************************/
/* Func: VideoSubClassProc()                                                 */
/*                                                                           */
/* Parm: HWND   hwnd          - window handle                                */
/*       WORD   Message       - the message                                  */
/*       WORD   wParam        - word length data for the message             */
/*       DWORD  lParam        - long length data for the message             */
/*                                                                           */
/* Desc: Handle Video Window SubClassed Messages                             */
/*****************************************************************************/

LONG  FAR PASCAL VideoSubClassProc( hwnd, Message, wParam,lParam )
HWND  hwnd;
WORD  Message;
WORD  wParam;
LONG  lParam;
{
 RECT Rect;
 WORD xpos;
 WORD ypos;
 HWND hwndSubMenu;

 switch ( Message )
     {
       case WM_ACTIVATE:
            if  ( wParam  &&  FullScreenVideo )
                 BecomingActive();
            break;

       case WM_SETCURSOR:
          /* In help mode it is necessary to reset the cursor in response  */
          /* to every WM_SETCURSOR message. Otherwise, by default, Windows */
          /* will reset the cursor to that of the window class.            */
          if  ( HelpPending )
             { SetCursor( HelpCursor );
               return( TRUE );
               }
          break;

      case WM_LBUTTONDBLCLK:                      // Full Screen Mode Toggling
           if  ( FullScreenVideo )
                DestroyFullScreenVideo();
           else
                DoFullScreenVideo();
           break;

      case WM_LBUTTONDOWN:
      case WM_RBUTTONDOWN:
           if  ( FullScreenVideo )
              { if  ( hwndRestoreImage )
                     BringWindowToTop( hwndRestoreImage );
                if  ( hwndVideoAttributes )
                     BringWindowToTop( hwndVideoAttributes );
                if  ( hwndAudioAttributes )
                     BringWindowToTop( hwndAudioAttributes );
                }
           break;

      case WM_RBUTTONUP:                          // Display Pop-Up Menu
           GetWindowRect( hwnd, &Rect );
           xpos = LOWORD( lParam ) + Rect.left;
           ypos = HIWORD( lParam ) + Rect.top;
           hwndSubMenu = GetSubMenu( hwndPopUpMenu, 0 );
           TrackPopupMenu( hwndSubMenu, 0, xpos, ypos, 0, hwnd, 0L );
           break;

      case WM_COMMAND:                     // Post Menu Items to the Frame Proc
           PostMessage( hwndFrame, Message, wParam, lParam );
           break;

       case WM_INITMENUPOPUP:                      // Initializating PopUp Menu
          hwndMenu = ( HWND ) wParam;
          switch ( LOWORD( lParam ) )
                { case 0:
                       HandleFileMenuInit();
                       break;
                  case 1:
                       HandleVideodiscMenuInit();
                       break;
                  case 2:
                       HandleVideoWindowMenuInit();
                       break;
                  }
          break;

      case WM_CLOSE:
          if  ( FullScreenVideo )        // IF Alt-F4 in Full Screen Video Mode
             { DestroyFullScreenVideo(); //    Destroy Full Screen Video Mode &
                                         //    Post the Message to the Frame
               PostMessage( hwndFrame, Message, wParam, lParam );
               return( 0L );
               }
          break;

      case WM_DESTROY:                   // Video Window is Being Destroyed
          hwndVideoWindow = NULL;        // Make Sure Command Will Not Send
          break;                         //      Any More Messages

      default:
           break;
      }

 return( CallWindowProc( lpOldVideoProc, hwnd, Message, wParam, lParam ) );
 }


/*****************************************************************************/
/* FUNCTION: UpdateVideoStatus()                                             */
/*                                                                           */
/* DESCRIPTION: Update the Global Video Status Structure                     */
/*                                                                           */
/* PARMS: Status Structure to Update                                         */
/*****************************************************************************/

VOID UpdateVideoStatus( StatusPtr )
LPMVWNQRY StatusPtr;
{
 LPMVWNQRY VideoQueryPtr;
 HANDLE    QueryHandle;

 QueryHandle = MVWNCreateQry( MVWN_QRY_STATUS );
 if  ( QueryHandle )
    { SendMessage( hwndVideoWindow, mvwnQueryMsg, NULL, ( LONG ) QueryHandle );
      VideoQueryPtr = ( LPMVWNQRY ) GlobalLock( QueryHandle );
      *StatusPtr = *VideoQueryPtr ;
      GlobalUnlock( QueryHandle );
      MVWNDestroyQry( QueryHandle );
      }
 return;
 }


/*****************************************************************************/
/* FUNCTION: UpdatePlayerStatus()                                            */
/*                                                                           */
/* DESCRIPTION: Update the Global Player Status Structure                    */
/*****************************************************************************/

VOID UpdatePlayerStatus()
{
 LPMVPCQRY pmvpcqry;
 HANDLE    QueryHandle;

 QueryHandle = MVPCCreateQry( MVPC_QRY_STATUS );
 if  ( QueryHandle )
    { SendMessage( hwndMVPC, mvpcQueryMsg, NULL, ( LONG ) QueryHandle );
      pmvpcqry = ( LPMVPCQRY ) GlobalLock( QueryHandle );
      * ( ( LPMVPCQRY ) &PlayerStatus ) = *pmvpcqry ;
      GlobalUnlock( QueryHandle );
      MVPCDestroyQry( QueryHandle );
      MaxFrame = PlayerStatus.lMaxPosition;        // Keep Max Position Current
      }
 return;
 }


/*****************************************************************************/
/* Func: DoFullScreenVideo()                                                 */
/*                                                                           */
/* Desc: Snap to Full Screen Video ( No Scroll Bar )                         */
/*****************************************************************************/

VOID DoFullScreenVideo()
{
 WORD  i;
 WORD  Height;
 WORD  Width;
 CHAR  TitleText[STRBUFLEN];

 if  ( FullScreenVideo )                      // IF Already in Full Screen Mode
      return;

 FullScreenVideo = 1;
 SetSourceCoordinates();
 Height = GetSystemMetrics( SM_CYSCREEN );
 Width  = GetSystemMetrics( SM_CXSCREEN );

 /************************************************************/
 /* Now Change the Video to A PopUp so it Can Go Full Screen */
 /************************************************************/

 SetParent( hwndVideoWindow, NULL );
 SetWindowLong( hwndVideoWindow, GWL_STYLE, WS_POPUP | WS_VISIBLE );
 LoadString( hInst, ID_FULLTITLETEXT, TitleText, STRBUFLEN );
 SetWindowText( hwndVideoWindow, TitleText );
 SetVideoWindowPos( hwndVideoWindow, 0, 0, Width, Height );
 BringWindowToTop( hwndVideoWindow );
 InvalidateRect( hwndVideoWindow, NULL, FALSE );
 UpdateWindow( hwndVideoWindow );

 /***************************************************************/
 /*  Make the Video Window the Parent of all ScrollBar Windows  */
 /*  so They Can Appear on Top of the Full Screen Video Window. */
 /*  Also, Size the ScrollBars to Full Screen Size.             */
 /***************************************************************/

 for  ( i = 0; i < MAX_PLAYERS; i++ )
     { if  ( hwndMVPCs[i] )
          { ShowWindow( hwndMVPCs[i], SW_HIDE );        // Hide All Scroll Bars
            SetParent( hwndMVPCs[i], hwndVideoWindow );
            MoveWindow( hwndMVPCs[i], 0, Height - scrollbarheight,
                        Width, scrollbarheight, FALSE );
            }
       }

 /*************************************************************/
 /*  Bring Any Active Dialogs to The Top of The Video Window  */
 /*************************************************************/

 if  ( hwndRestoreImage )
      SetBoxPosition( hwndRestoreImage, TOP );
 if  ( hwndVideoAttributes )
      SetBoxPosition( hwndVideoAttributes, TOP );
 if  ( hwndAudioAttributes )
      SetBoxPosition( hwndAudioAttributes, BOTTM );

 return;
 }


/*****************************************************************************/
/* FUNCTION: DestroyFullScreenVideo()                                        */
/*                                                                           */
/* DESCRIPTION: Get Rid of the Full Screen Video Window                      */
/*****************************************************************************/

VOID DestroyFullScreenVideo()
{
 WORD i;

 if  ( FullScreenVideo )                           // IF in Full Screen Video
    { FullScreenVideo = 0;
      SetSourceCoordinates();
      SetFocus( hwndFrame );

      /******************************************/
      /*  Restore Valid Parents to Scroll Bars  */
      /******************************************/

      for  ( i = 0; i < MAX_PLAYERS; i++ )         // Reset Scroll Bars Parent
          { if  ( hwndMVPCs[i] )
                 SetParent( hwndMVPCs[i], hwndFrame );
            }

      SetParent( hwndVideoWindow, hwndFrame );
      SetWindowLong( hwndVideoWindow, GWL_STYLE, WS_CHILD | WS_VISIBLE );

      ShowWindow( hwndFrame, SW_SHOW );
      if  ( hwndMVPC )
           ShowWindow( hwndMVPC, SW_SHOW );

      SizeVideo();
      UpdateWindow( hwndVideoWindow );
      SetFocus( hwndFrame );
      }
 return;
 }


/*****************************************************************************/
/* Func: SetSourceCoordinates()                                              */
/*                                                                           */
/* Desc: Set the Video Source Coordinates using different values for full    */
/*       screen video mode.                                                  */
/*       Its purpose is to try to elimate the user from seeing garbage       */
/*       at top and bottom of the screen since some video sources do not     */
/*       generate a complete signal.                                         */
/*****************************************************************************/

VOID SetSourceCoordinates()
{
 LPMVWNCMD pmvwncmd;
 HANDLE    CommandHandle;
 WORD      SourceType;
 BOOL      Pal = FALSE;

 if  ( hwndVideoWindow )
    { if  ( CommandHandle = MVWNCreateCmd( MVWN_CMD_SOURCEDIM ) )
         { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
           if  ( pmvwncmd )
              { pmvwncmd->data.dwCmdParam1 =
                                      ( LONG ) SaveSourceHorg[ usVideoSource ];
                pmvwncmd->data.dwCmdParam3 =
                                      ( LONG ) SaveSourceHlen[ usVideoSource ];

                SourceType = MVWNVideoSourceType( usVideoSource );
                if  ( ( SourceType == MVWN_VST_PAL ) ||
                      ( SourceType == MVWN_VST_SPAL ) )
                     Pal = TRUE;

                if  ( ( FullScreenVideo )  &&  ( Pal == FALSE ) )
                   { pmvwncmd->data.dwCmdParam2 = 1L;
                     pmvwncmd->data.dwCmdParam4 = 480L;
                     }
                else
                   { pmvwncmd->data.dwCmdParam2 =
                                      ( LONG ) SaveSourceVorg[ usVideoSource ];
                     pmvwncmd->data.dwCmdParam4 =
                                      ( LONG ) SaveSourceVlen[ usVideoSource ];
                     }
                GlobalUnlock( CommandHandle );
                PostMessage( hwndVideoWindow, mvwnCommandMsg,
                             NULL, ( LONG ) CommandHandle );
                }
           MVWNDestroyCmd( CommandHandle, hwndVideoWindow, MVWN_ACK );
           }
      }
 return;
 }
