/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       MCIVIDEO.C                                          */
/*                                                                    */
/*  VERSION:      M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*                                                                    */
/*  FUNCTIONS:    WndProc()                                           */
/*                About()                                             */
/*                SeekFrame()                                         */
/*                mciExecute()                                        */
/*                mciSendString()                                     */
/*                                                                    */
/*  DESCRIPTION:  MCI video window browser.                           */
/*                                                                    */
/**********************************************************************/
#include <windows.h>
#include "mcivideo.h"

//---------------------------------------------------------------------
// function prototypes

       long  FAR PASCAL  WndProc       (HWND, WORD, WORD, LONG);
       BOOL  FAR PASCAL  About         (HWND, WORD, WORD, LONG);
       BOOL  FAR PASCAL  SeekFrame     (HWND, WORD, WORD, LONG);
extern WORD  FAR PASCAL  mciExecute    (LPSTR);
extern DWORD FAR PASCAL  mciSendString (LPSTR, LPSTR, WORD, HWND);



//---------------------------------------------------------------------
// Program entry

int PASCAL WinMain ( hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR  lpCmdLine;
int    nCmdShow;

{
    char     szAppName[MAXSTRINGSIZE];
    char     szText[MAXSTRINGSIZE];
    WNDCLASS wc;
    MSG      msg;
    DWORD    dwRC;
    HMENU    hMenu;


    if (hPrevInstance)
       { // arbitrarily allow only one instance of this pgm to make it simple
       LoadString(hInstance, ID_ONEINST, szText, MAXSTRINGSIZE);
       MessageBox(NULL, szText, NULL,
                  MB_APPLMODAL | MB_ICONEXCLAMATION | MB_OK);
       return 0;
       }
    else
       {
       hHourglass = LoadCursor (NULL, IDC_WAIT);
       // register the window class
       LoadString(hInstance, ID_APPNAME, szAppName, MAXSTRINGSIZE);
       wc.style         = 0;
       wc.lpfnWndProc   = WndProc;
       wc.cbClsExtra    = 0;
       wc.cbWndExtra    = 0;
       wc.hInstance     = hInstance;
       // we use one icon to start the application, and then another one
       // when program is running.  This lets us show video in the icon.
       wc.hIcon         = LoadIcon (hInstance, szAppName);
       wc.hCursor       = LoadCursor ( NULL, IDC_ARROW );
       wc.hbrBackground = GetStockObject(WHITE_BRUSH);
       wc.lpszMenuName  = szAppName;
       wc.lpszClassName = szAppName;
       RegisterClass (&wc);
       }

    hInst = hInstance;

    // Create the program's main window, and display it.  We will
    // display video in this window.  See lines 108 - 109 below.
    hwndMain = CreateWindow (szAppName, "MCI Video Sample",
                             WS_OVERLAPPEDWINDOW,
                             150, 150, 300, 225,
                             NULL, NULL, hInstance, NULL);

    // We want to make sure that the cursor is set to an hourglass while
    // initialization and spin up of the videodisc occurs.  We capture
    // mouse input because normally, if the user puts the mouse over
    // another window, it will change the shape of the cursor.
    SetCapture( hwndMain );                    // capture mouse input
    SetCursor( hHourglass );                   // set cursor shape
    ShowWindow (hwndMain, SW_SHOWNORMAL);      // display window
    UpdateWindow (hwndMain);

    // following message is sent to the videomixer device
    dwRC = mciSendString( "open videomixer alias vm",
                           NULL, 0, NULL);
    // a side effect of opening the videomixer is that the cursor
    // changes shape; we change it back.
    SetCursor( hHourglass );

    if (dwRC != 0)
       {
       // if we can't display video, tell user and exit the application
       LoadString(hInstance, ID_NOVIDEO, szText, MAXSTRINGSIZE);
       MessageBox(hwndMain, szText, NULL,
                  MB_APPLMODAL | MB_ICONEXCLAMATION | MB_OK);
       return 0;
       }

    // following string is sent to the videodisc device
    dwRC = mciSendString( "open videodisc alias vd wait",
                              NULL, 0, NULL);
    if (dwRC != 0)
       {
       // We cannot control videodisc player, so tell user that he
       // must control the video source; don't quit application
       fHaveVideoDisc = FALSE;
       LoadString(hInstance, ID_NOVDISC, szText, MAXSTRINGSIZE);
       MessageBox(hwndMain, szText, NULL,
                  MB_APPLMODAL | MB_ICONINFORMATION | MB_OK);
       }

    // The next command is directed to the videomixer, and specifies
    // that video should be displayed in the window created by this program.
    // First, build the command.  Then send it.
    wsprintf( szCmd, (LPSTR) "window vm handle %d", hwndMain);
    mciSendString( szCmd, NULL, 0, NULL);

    // Enable the menu functions supported by all MCI videodisc devices.
    // Not all videodisc players can handle all of the commands we would
    // like to use, so query the disk player to see if we can reverse.

    if ( fHaveVideoDisc )
       {
       hMenu = GetMenu(hwndMain);
       // can videodisc player "reverse"?
       dwRC = mciSendString( "capability vd can reverse wait",
                             szRetString, MAXSTRINGSIZE, NULL);

       if ( !dwRC && (! lstrcmp(szRetString, (LPSTR)"true")))
          // if command was successful and response was "true", enable
          // the menu item
          EnableMenuItem( hMenu, IDM_REV,  MF_ENABLED | MF_BYCOMMAND);

       EnableMenuItem( hMenu, IDM_PLAY,   MF_ENABLED | MF_BYCOMMAND);
       EnableMenuItem( hMenu, IDM_PAUSE,  MF_ENABLED | MF_BYCOMMAND);
       EnableMenuItem( hMenu, IDM_FFWD,   MF_ENABLED | MF_BYCOMMAND);
       EnableMenuItem( hMenu, IDM_STOP,   MF_ENABLED | MF_BYCOMMAND);
       EnableMenuItem( hMenu, IDM_SEEK,   MF_ENABLED | MF_BYCOMMAND);
       // the index command may not be supported by all videodisc
       // player drivers, but we will assume that it is.
       EnableMenuItem( hMenu, IDM_INDEX,  MF_ENABLED | MF_BYCOMMAND);
       }

    ReleaseCapture( ); // Finished with initialization.  Now let mouse
                       // operation go back to normal.
    fReady = TRUE;     // flag used in WM_SETCURSOR processing below.

    // Message loop.
    while (GetMessage (&msg, NULL, 0 , 0))
       {
       TranslateMessage (&msg);
       DispatchMessage (&msg);
       }

    return msg.wParam;
}



//---------------------------------------------------------------------
// Window procedure for main window

long FAR PASCAL WndProc (HWND hwnd,
                         WORD message,
                         WORD wParam,
                         LONG lParam)

{

   FARPROC lpDlgProc;   /* pointer to dialog procedure */

   switch (message)
   {

   case WM_CLOSE :
      fReady = FALSE;
      SetCapture( hwnd );  // once again, do this so cursor won't change
      SetCursor( hHourglass );
      // It's good practice to close the devices
      mciSendString( "close vm", NULL, 0, NULL);
      if (fHaveVideoDisc)
         {
         // stop the videodisc by sending a WM_COMMAND with appropriate
         // parameter
         SendMessage(hwnd, WM_COMMAND, IDM_STOP, NULL);
         mciSendString("spin vd down wait", szRetString, MAXSTRINGSIZE, NULL);
         // now close the videodisc device
         mciSendString( "close vd wait", NULL, 0, NULL);
         }
      ReleaseCapture( );
      break;


   case WM_COMMAND:

      if ( ! fReady )
         return 0;

      switch (wParam)
      {
      case IDM_EXIT:
         // User asked to exit application.  We want to close the
         // devices, so we issue a WM_CLOSE message which causes
         // this to occur.
         PostMessage(hwnd, WM_CLOSE, 0, 0L);
         return 0;

      case IDM_PLAY:
         mciExecute((LPSTR)"play vd");
         return 0;

      case IDM_PAUSE:
         mciExecute((LPSTR)"pause vd");
         return 0;

      case IDM_FFWD:
         mciExecute((LPSTR)"play vd scan");
         return 0;

      case IDM_REV:
         mciExecute((LPSTR)"play vd reverse fast");
         return 0;

      case IDM_INDEX:
         fIndex = ! fIndex;
         if (fIndex)
            mciExecute((LPSTR)"index vd on");
         else
            mciExecute((LPSTR)"index vd off");
         return 0;

      case IDM_STOP:
         mciSendString("stop vd", szRetString, MAXSTRINGSIZE, NULL);
         return 0;

      case IDM_ABOUT:
         {
         // user asked to see "About..." information
         // so show a dialog box with that information
         lpDlgProc = MakeProcInstance(About, hInst);

         DialogBox(hInst,                 /* current instance         */
             "AboutBox",                  /* resource to use          */
             hwndMain,                    /* parent handle            */
             lpDlgProc);                  /* About() instance address */

         FreeProcInstance(lpDlgProc);
         return 0;
         }

      case IDM_SEEK:
         {
         // User asked to seek a specific frame number.  Display a
         // dialog box.  Dialog procedure handles the rest.
         lpDlgProc = MakeProcInstance(SeekFrame, hInst);

         DialogBox(hInst,                 // current instance
             "SeekFrame",                 // resource to use
             hwndMain,                    // parent handle
             lpDlgProc);                  // SeekFrame() instance address

         FreeProcInstance(lpDlgProc);
         return 0;
         }

      default:
         break;
      } /* endswitch */

   case WM_DESTROY :
      PostQuitMessage(0);                // exit application
      break;

   case WM_SETCURSOR:
      if ( ! fReady )
         {
         SetCursor( hHourglass );
         return TRUE;
         }
      break;

   default:
      return DefWindowProc (hwnd, message, wParam, lParam);

   }
   return DefWindowProc (hwnd, message, wParam, lParam);
}


//---------------------------------------------------------------------
// About Dialog Box

BOOL FAR PASCAL About(hDlg, wMessage, wParam, lParam)
HWND hDlg;
WORD wMessage;
WORD wParam;
LONG lParam;
{
   switch (wMessage)
   {

   case WM_INITDIALOG:
      return (TRUE);

   case WM_COMMAND:
      if (wParam == IDOK
          || wParam == IDCANCEL)
         {
         EndDialog(hDlg, TRUE);
         return (TRUE);
         }
      break;

   }
   return (FALSE);
}

//---------------------------------------------------------------------
// SeekFrame Dialog Box

BOOL FAR PASCAL SeekFrame(hDlg, wMessage, wParam, lParam)
HWND hDlg;
WORD wMessage;
WORD wParam;
LONG lParam;
{
   switch (wMessage)
   {

   case WM_INITDIALOG:
      return (TRUE);

   case WM_COMMAND:

      if (wParam == ID_SETFRAME)           // user entered a value
         {
         char    szBuffer[80];

         // Get the entered value
         GetDlgItemText( hDlg, ID_SEEKFRAME, (LPSTR) szBuffer, 80);
         // Build the command string
         wsprintf((LPSTR)szCmd, "seek vd to %s", (LPSTR) szBuffer);
         // Execute it.  Note that we leave it to the videodisc to
         // determine if the entered frame number is an error or not.
         // If it is, mciExecute will display an error message for us.
         // We arbitrarily exit the dialog box, even if there was an
         // error.
         mciExecute((LPSTR)szCmd);
         EndDialog(hDlg, TRUE);
         return (TRUE);
         }
      if (wParam == IDCANCEL)              // user cancelled
         {
         EndDialog(hDlg, TRUE);
         return (TRUE);
         }
      break;

   }
   return (FALSE);
}
