/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       AUDIO.C                                             */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    WinMain()                                           */
/*                InitApplication()                                   */
/*                MainWndProc()                                       */
/*                                                                    */
/*  DESCRIPTION:  Main proc for the digital audio sample application  */
/*                Read Chapter 9 of the M-Control Program Version 2.0 */
/*                of the User's Guide for additional information.     */
/*                                                                    */
/*                This sample uses the most basic RIFF (Resource      */
/*                Interchange File Format) WAVE file format.          */
/*                See AUDIO.H for the RIFF file format.               */
/*                                                                    */
/*                Standard Microsoft C 6.0 file handling functions    */
/*                were used in this application (versus using         */
/*                multimedia MMIO functions) to provide compati-      */
/*                bility with Windows and Multimedia Windows.         */
/*                                                                    */
/*                                                                    */
/*  M-MOTION/MCI FUNCTIONS USED:                                      */
/*                auxGetDevCaps                                       */
/*                auxGetNumDevs                                       */
/*                DrvClose                                            */
/*                DrvOpen                                             */
/*                DrvSendMessage                                      */
/*                waveInGetDevCaps                                    */
/*                waveInGetNumDevs                                    */
/*                waveInGetPosition                                   */
/*                waveInUnprepareHeader                               */
/*                waveOutGetDevCaps                                   */
/*                waveOutGetNumDevs                                   */
/*                waveOutGetPosition                                  */
/*                waveOutUnprepareHeader                              */
/**********************************************************************/
#include  "AUDIO.H"                    // sample application header file

/*******************/
/*   FILE  DATA    */
/*******************/

int    WaveFile     = -1;               // Audio File's DOS Handle
BOOL   FileExist    = FALSE;            // Does the file exist ?
BOOL   OldOpenFile  = FALSE;            // An old file that is open for record
BOOL   ProgTerminate= FALSE;            // Terminate Program ?
BOOL   NewFile      = FALSE;            // Flag if using NEWFILE dialog box
LONG   FileStart    = 0L;               // Offset of Start in File
LONG   Position     = 0L;               // Current Audio Position ( in Bytes )
LONG   WaveLength   = 0L;               // Audio File Data Length ( in Bytes )
LONG   EndofFile    = 0L;               // Check end of play audio file
LONG   CurPlayPos   = 0L;               // Current position in play buffer
char   FileName[32];                    // Currently Open Audio File Name
BOOL   Playing      = FALSE;            // Is Playing ? Flag
BOOL   Recording    = FALSE;            // Is Recording ? Flag
WORD   NeedsBuffers = 0;                // Means Started Loop, But No End
WORD   DigitalVolume;                   // Digital Output Volume
WORD   RecordLevel;                     // Digital Recording Level


HWND   hwndReset       = NULL;          // Reset Button Window
HWND   hwndPlay        = NULL;          // Play Button Window
HWND   hwndRecord      = NULL;          // Record Button Window
HWND   hwndPause       = NULL;          // Pause Button Window
HWND   hwndResume      = NULL;          // Resume Button Window
HWND   hwndStop        = NULL;          // Stop Button Window

HWND   hwndVolume      = NULL;          // Volume Scroll Bar
HWND   hwndBalance     = NULL;          // Balance Scroll Bar
HWND   hwndBass        = NULL;          // Bass   Scroll Bar
HWND   hwndTreble      = NULL;          // Treble Scroll Bar
HWND   hwndDigitalVol  = NULL;          // Digital Volume Scroll Bar
HWND   hwndRecordLevel = NULL;          // Record Level   Scroll Bar

HANDLE DriverHandle  = NULL;            // IBM M-Motion Driver Handle

HWND   hwndButtonsDlg  = NULL;          // Control Buttons Dialog Handle
HWND   hwndAuxDlg      = NULL;          // Auxiliary Scroll Bars Dialog Handle
HWND   hwndDigitalDlg  = NULL;          // Digital Scroll Bars Dialog Handle

WORD   AuxDeviceID;
WORD   waveOutDeviceID;
WORD   waveInDeviceID;

HWAVEOUT hWaveOut    = NULL;            // Handle to Wave Out Device
HWAVEOUT hWaveIn     = NULL;            // Handle to Wave In Device

BOOL   TimerOn         = FALSE;         // Timer On/Off Flag
WORD   TimerTics       = 0;             // Timer Tick Counter

BOOL   AuxMMotionExist     = FALSE;     // Flags if devices exists
BOOL   waveOutMMotionExist = FALSE;
BOOL   waveInMMotionExist  = FALSE;

WORD   RightVolume;                     // Right Channel Volume
WORD   LeftVolume;                      // Right Channel Volume
DWORD  Volume;                          // Line Volume Setting
WORD   CurrentVolume;                   // Current Volume
int    Balance;                         // Balance
int    Bass;                            // Bass
int    Treble;                          // Treble


/*******************/
/*   WAVE  DATA    */
/*******************/

WORD   WaveDevice;                      // Wave Device Number
WORD   NumWaveOutDevs;                  // Set Number of Wave Output Devices
WORD   NumberBuffers  = 1;              // Buffers Needed for "n" Seconds



PCMWAVEFORMAT     WaveFormat;           // Current Wave Format Data
static LPWAVEHDR  LastWaveHdrPtr;       // Last Buffer Used

static FARPROC lpButtonsDlg;            // Buttons Dialog Procedure
static FARPROC lpAuxDlg;                // Auxiliary Dialog Procedure
static FARPROC lpDigitalDlg;            // Digital Dialog Procedure
static FARPROC lpDeviceDlg;             // Device Dialog Procedure
static LONG    Factor       = 500L;     // Position to Scroll Bar Ratio Factor

char   *TitleText = "IBM Sample WAVE Audio";

/*******************/
/*    HANDLES      */
/*******************/

HANDLE hInst         = NULL;            // This Instance Handle

HWND hwndFrame       = NULL;            // Frame Window Handle


char FileDescText[80];
/**********************************************************************/
/* Function:     WinMain()                                            */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     InitApplication()                                    */
/*               InitInstance()                                       */
/*                                                                    */
/* Description:  Calls initialization function, processes message     */
/*               loop.                                                */
/**********************************************************************/

int FAR PASCAL WinMain( hInstance, hPrevInstance, lpCmdLine, nCmdShow )
HANDLE hInstance;                       // Current Instance
HANDLE hPrevInstance;                   // Previous Instance
LPSTR  lpCmdLine;                       // Command Line
int    nCmdShow;                        // Show-Window Type (Open/Icon)
{
 MSG msg;                               // Message

 if  ( !hPrevInstance )                 // Is This the First Instance ?
   { if (!InitApplication(hInstance))   // Initialize Shared Things
           return( FALSE );             // Exits IF Unable to Initialize
      }
 else return( FALSE );


 if  ( !InitInstance( hInstance, nCmdShow ) )
      return( FALSE );

 while ( GetMessage( &msg,              // Message Structure
                     NULL,              // Handle of Window Receiving Message
                     NULL,              // Lowest Message to Examine
                     NULL ) )           // Highest Message to Examine
      {
        if (
        ((hwndButtonsDlg == NULL) || !IsDialogMessage(hwndButtonsDlg,&msg)) &&
        ((hwndAuxDlg == NULL)     || !IsDialogMessage(hwndAuxDlg,&msg))     &&
        ((hwndDigitalDlg == NULL) || !IsDialogMessage(hwndDigitalDlg,&msg))
        )

       {TranslateMessage( &msg );       // Translates Virtual Key Codes
        DispatchMessage( &msg );        // Dispatches Message to Window
        }
      }

 return( msg.wParam );                  // Returns Value from PostQuitMessage
 }

/**********************************************************************/
/* Function:     InitApplication()                                    */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Initializes window data and registers window class   */
/**********************************************************************/

static BOOL InitApplication( hInstance )
HANDLE hInstance;                       // Current Instance
{
 WNDCLASS  wc;

 wc.style         = NULL;               // Class style(s)
 wc.lpfnWndProc   = MainWndProc;        // Window Procedure
 wc.cbClsExtra    = 0;                  // No per-class extra data
 wc.cbWndExtra    = 0;                  // No per-window extra data
 wc.hInstance     = hInstance;          // Application that owns class
 wc.hIcon         = LoadIcon( hInstance, "AudTestIcon" );
 wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
 wc.hbrBackground = GetStockObject( LTGRAY_BRUSH );
 wc.lpszMenuName  = "WinaudioMenu";     // Resource Name in .RC File
 wc.lpszClassName = "SampleClass";      // Name Used in CreateWindow()

 return( RegisterClass( &wc ) );
 }

/**********************************************************************/
/* Function:     InitInstance()                                       */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     AuxiliaryProc()                                      */
/*               ButtonsProc()                                        */
/*               DigitalProc()                                        */
/*                                                                    */
/* Description:  Creates the main window for this application         */
/*               instance.                                            */
/**********************************************************************/

static BOOL InitInstance( hInstance, nCmdShow )
HANDLE hInstance;                       // Current Instance Identifier
int    nCmdShow;                        // Param for First ShowWindow() Call
{
  SetMessageQueue( 32 );                // Try to Get a Bigger Message Queue

  hInst = hInstance;                    // Save Instance Handle in
                                        //   Static Variable

  //----------------------------------------------------
  // Create a Main Window for This Application Instance
  //----------------------------------------------------

  hwndFrame = CreateWindow(
                   "SampleClass",       // See RegisterClass() Call
                   TitleText,           // Text for Window Title Bar
                   WS_OVERLAPPEDWINDOW, // Window Style: Overlapped With
                   CW_USEDEFAULT,       // Default Horizontal Position
                   CW_USEDEFAULT,       // Default Vertical Position
                   360,                 // Width
                   408,                 // Height
                   NULL,                // Overlapped Window Has No Parent
                   NULL,                // Use the Window Class Menu
                   hInstance,           // This Instance Owns this Window
                   NULL );              // Pointer Not Needed

  if  ( !hwndFrame )                    // IF Window Not Created,
       return( FALSE );                 //    Return "Failure"

  //-------------------------------------------------
  // Make the Window Visible; Update its Client Area
  //-------------------------------------------------

  ShowWindow( hwndFrame, nCmdShow );    // Show the Window
  UpdateWindow( hwndFrame );            // Send WM_PAINT Message

  //-----------------------------------------
  // Create the "Scroll Bars" Control Dialog
  //-----------------------------------------

  lpAuxDlg   = MakeProcInstance( AuxiliaryProc, hInst );
  hwndAuxDlg = CreateDialog( hInst, "AuxDlg", hwndFrame, lpAuxDlg );

  //----------------------------------------------------------
  // Create the "Digital Controls Scroll Bars" Control Dialog
  //----------------------------------------------------------

  lpDigitalDlg   = MakeProcInstance( DigitalProc, hInst );
  hwndDigitalDlg = CreateDialog( hInst, "DigitalDlg", hwndFrame, lpDigitalDlg );

  //-------------------------------------
  // Create the "Buttons" Control Dialog
  //-------------------------------------

  lpButtonsDlg   = MakeProcInstance( ButtonsProc, hInst );
  hwndButtonsDlg = CreateDialog( hInst, "Buttons", hwndFrame, lpButtonsDlg );

  return( TRUE );
  }

/**********************************************************************/
/* Function:     MainWndProc()                                        */
/*                                                                    */
/* Called from:  None                                                 */
/*                                                                    */
/* Calls to:     AboutDlgProc()                                       */
/*               AddRecordBuffer()                                    */
/*               EnableButtons()                                      */
/*               CloseAll()                                           */
/*               DeleteFileProc()                                     */
/*               DigitalProc()                                        */
/*               GetWaveFormat()                                      */
/*               NewProc()                                            */
/*               OpenFileProc()                                       */
/*               SendNextBuffer()                                     */
/*               SetPosition()                                        */
/*               SetTitle                                             */
/*               StartTimer()                                         */
/*               StopTimer()                                          */
/*                                                                    */
/*                                                                    */
/* Description:  Creates the main window for this application         */
/*               instance.                                            */
/**********************************************************************/

long FAR PASCAL MainWndProc( hWnd, message, wParam, lParam )
HWND hWnd;                              // Window Handle
WORD message;                           // Type of Message
WORD wParam;
LONG lParam;
{
 char        ErrMsgText[120];
 char        ErrMCIText[120];
 WORD        NumWaveInDevs;
 WORD        NumAuxDevs;
 HANDLE      HeaderHandle;
 LPSTR       BufferPtr;
 AUXCAPS     AuxCaps;
 WAVEINCAPS  waveInCaps;
 WAVEOUTCAPS waveOutCaps;
 DWORD       LastBuffer;
 FARPROC     lpProcTemp;
 WORD        rc;
 RECT        FrameRect;
 RECT        Rect;
 LPWAVEHDR   WaveHdrPtr;
 HDC         hdc;
 PAINTSTRUCT ps;
 static int  CharHeight;
 TEXTMETRIC  tm;
 MMTIME      TimeInfo;
 char        OldFileName[32];
 WORD        i;
 char        string1[32];
 char        RightVolumeText[13];
 char        LeftVolumeText[12];

 switch ( message )

   {

     case  WM_CREATE:

        hdc = GetDC( hWnd );
        GetTextMetrics( hdc, ( LPTEXTMETRIC ) &tm );
        CharHeight = tm.tmHeight;
        ReleaseDC( hWnd, hdc );

        /********************************/
        /* Does WaveOut Devices Exist ? */
        /********************************/

        NumWaveOutDevs = waveOutGetNumDevs();
        if  ( NumWaveOutDevs == 0 )
           { LoadString ( hInst, IDS_NO_AUDIO_OUTPUT, ErrMsgText, STRBUFLEN );
             LoadString ( hInst, IDS_MCI_ERROR, ErrMCIText, STRBUFLEN );
             MessageBox ( hWnd, ErrMsgText, ErrMCIText,
                               MB_OK | MB_APPLMODAL | MB_ICONHAND );
           }
        else
           {
           // Check to see if the device drive supports the IBM M-Motion
           // Waveform Output
           // This code would need to be changed to check for other
           // adapters.

           for (i=0; i<NumWaveOutDevs; i++)
              { waveOutDeviceID = i ;
                waveOutGetDevCaps( waveOutDeviceID,
                      ( LPWAVEOUTCAPS ) &waveOutCaps, sizeof( waveOutCaps ) );

                LoadString ( hInst, IDS_MMOTION_OUTPUT, string1, STRLEN32 );
                if ( ( rc = strncmp ( waveOutCaps.szPname,
                                      string1, strlen(string1) ) ) == 0 )
                   { waveOutMMotionExist = TRUE;
                     break;
                   }
               }
           }



        /*******************************/
        /* Does a waveIn Device Exist? */
        /*******************************/


        NumWaveInDevs = waveInGetNumDevs();
        if  ( NumWaveInDevs == 0 )
           { LoadString ( hInst, IDS_NO_AUDIO_INPUT, ErrMsgText, STRBUFLEN );
             LoadString ( hInst, IDS_MCI_ERROR, ErrMCIText, STRBUFLEN );
             MessageBox ( hWnd, ErrMsgText, ErrMCIText,
                                MB_OK | MB_APPLMODAL | MB_ICONHAND );
           }
        else
           {
           // Check to see if the device drive supports the IBM M-Motion
           // Waveform Input
           // This code would need to be changed to check for other
           // adapters.
           for (i=0; i<NumWaveInDevs; i++)
              { waveInDeviceID = i;
                waveInGetDevCaps( waveInDeviceID, (LPWAVEINCAPS) &waveInCaps,
                                           sizeof(waveInCaps) );

                LoadString ( hInst, IDS_MMOTION_INPUT, string1, STRLEN32 );
                if ( ( rc = strncmp ( waveInCaps.szPname, string1,
                                          strlen(string1) ) ) == 0)
                   { waveInMMotionExist = TRUE;
                     break;
                   }
              }

           }


        /*****************************************************/
        /* Does an auxiliary device exist, i.e. the M-Motion */
        /* Video Adapter/A.                                  */
        /*****************************************************/

        NumAuxDevs = auxGetNumDevs();
        if  ( NumAuxDevs == 0 )
           { LoadString ( hInst, IDS_NO_MMOTION, ErrMsgText, STRBUFLEN );
             LoadString ( hInst, IDS_MCI_ERROR,  ErrMCIText, STRBUFLEN );
             MessageBox ( hWnd, ErrMsgText, ErrMCIText,
                                MB_OK | MB_APPLMODAL | MB_ICONHAND );
             break;
           }

        else
           {
           // Is it an IBM M-Motion Auxiliary Device ?
           for (i=0; i<NumAuxDevs; i++)
              { AuxDeviceID = i;
                auxGetDevCaps( AuxDeviceID, (LPAUXCAPS) &AuxCaps,
                               sizeof(AuxCaps) );
                LoadString ( hInst, IDS_MMOTION_AUX, string1, STRLEN32 );
                if ( ( rc = strncmp ( AuxCaps.szPname, string1,
                                              strlen(string1) ) ) == 0)
                   { AuxMMotionExist = TRUE;
                     break;
                   }
              }

           }


        /***************************************************/
        /*  For this sample, all devices must identify the */
        /*  M-Motion Video Adapter/A card.                 */
        /***************************************************/

        if (!AuxMMotionExist      ||
            !waveOutMMotionExist  ||
            !waveInMMotionExist)

            { LoadString ( hInst, IDS_NO_MMOTION, ErrMsgText, STRBUFLEN );
              LoadString ( hInst, IDS_MCI_ERROR,  ErrMCIText, STRBUFLEN );
              MessageBox ( hWnd, ErrMsgText, ErrMCIText,
                                 MB_OK | MB_APPLMODAL | MB_ICONHAND );
              break;
             }

        WaveDevice = AuxDeviceID;

        /************************************************************/
        /* Check if the auxiliary device supports volume controls   */
        /************************************************************/

        if  ( !( AuxCaps.dwSupport & AUXCAPS_VOLUME ) ||
              !( AuxCaps.dwSupport & AUXCAPS_LRVOLUME ) )
           { LoadString ( hInst, IDS_NO_VOL_CONTROL, ErrMsgText, STRBUFLEN );
             LoadString ( hInst, IDS_AUX_WARNING,    ErrMCIText, STRBUFLEN );
             MessageBox ( hWnd, ErrMsgText, ErrMCIText,
                                MB_OK | MB_APPLMODAL | MB_ICONHAND );
           }
        else
           { LoadString (hInst, IDS_LEFT_VOL, LeftVolumeText, 12);
             LeftVolume  = GetValue( LeftVolumeText, 0x7FFF );
             LoadString (hInst, IDS_RIGHT_VOL, RightVolumeText, 13);
             RightVolume = GetValue( RightVolumeText, 0x7FFF ) ;
           }


        /*****************************************************/
        /*   Open M-Motion driver and initialize MUTE to off */
        /*****************************************************/

        DriverHandle = DrvOpen( "AUX", "DRIVERS", NULL );
        DrvSendMessage( DriverHandle, AUX_MUTELINE, 0, NULL );

        break;

      case WM_COMMAND:

         switch ( wParam )

            {

              case IDM_NEW:

                /**********************************************************/
                /*  The NEW file dialog box has been selected.            */
                /*  Close any open files.                                 */
                /*  Call NewProc to setup the RIFF Header and setup the   */
                /*  RIFF Header, filename and enable the wave audio       */
                /*  buttons.                                              */
                /*  If the filename is invalid disable the wave audio     */
                /*  buttons and display an error message.                 */
                /**********************************************************/

                 FileExist = FALSE;
                 strcpy (OldFileName, FileName);
                 CloseAll();
                 strcpy (FileName, OldFileName);
                 lpProcTemp = MakeProcInstance( NewProc, hInst );
                 DialogBox( hInst, "NEWFILE", hwndFrame, lpProcTemp );
                 FreeProcInstance( lpProcTemp );
                 if (FileExist == FALSE)
                    { EnableButtons( FALSE, FALSE, FALSE, FALSE, FALSE, FALSE );
                      //           Reset, Play, Pause, Stop, Record, Resume
                      LoadString (hInst, IDS_BLANK, ErrMsgText, STRBUFLEN);
                      wsprintf( FileDescText, ErrMsgText );
                      SetTitle();
                      break;
                    }
                 EnableButtons( FALSE, FALSE, FALSE, FALSE, TRUE, FALSE );
                 //             Reset, Play, Pause, Stop, Record, Resume
                 break;


              case IDM_OPEN:

                /**********************************************************/
                /* The OPEN file dialog box has been selected.  Close any */
                /* open files.  Call OpenFileProc to display the listbox  */
                /* of existing .WAV files.  Select the file, check the    */
                /* RIFF Header, setup the window caption with the filename*/
                /* and enable the wave audio buttons for record.          */
                /* If the filename is invalid an error message will appear*/
                /* and the wave audio buttons will be disabled.           */
                /**********************************************************/

                 NewFile = FALSE;
                 strcpy (OldFileName, FileName);
                 CloseAll();
                 strcpy (FileName, OldFileName);
                 lpProcTemp = MakeProcInstance( OpenFileProc, hInst );
                 DialogBox( hInst, "OPENFILE", hwndFrame, lpProcTemp );
                 FreeProcInstance( lpProcTemp );
                 if (FileExist == FALSE)
                    { EnableButtons( FALSE, FALSE, FALSE, FALSE, FALSE, FALSE );
                      //             Reset, Play, Pause, Stop, Record, Resume
                      LoadString (hInst, IDS_BLANK, ErrMsgText, STRBUFLEN);
                      wsprintf( FileDescText, ErrMsgText );
                      SetTitle();
                      break;
                    }


                 if ( FileName[0] != NULL )
                    GetWaveFormat( hInst );
                 EnableButtons( FALSE, TRUE, FALSE, FALSE, TRUE, FALSE );
                 //             Reset, Play, Pause, Stop, Record, Resume
                 break;



              case IDM_CLOSE:

                /**********************************************************/
                /* Close is selected from the File menu item.  Stop the   */
                /* timer.  Close the opened file.  Clear the window       */
                /* caption.  Disable the wave audio buttons.  Reset the   */
                /* timer to zero.                                         */
                /**********************************************************/

                 StopTimer();
                 NumberBuffers = 1;
                 CloseAll();
                 LoadString (hInst, IDS_BLANK, ErrMsgText, STRBUFLEN);
                 wsprintf( FileDescText, ErrMsgText );
                 SetTitle();
                 EnableButtons( FALSE, FALSE, FALSE, FALSE, FALSE, FALSE );
                 //             Reset, Play, Pause, Stop, Record, Resume
                 Position = 0L;
                 SetPosition();
                 break;


              case IDM_DELETE:

                /**********************************************************/
                /* The DELETE dialog box has been selected from the FILE  */
                /* menu.  Close any opened files.  Call DeleteFileProc    */
                /* to display the listbox of all .WAV files within the    */
                /* current directory.  Disable the wave audio buttons.    */
                /**********************************************************/

                 CloseAll();
                 lpProcTemp = MakeProcInstance( DeleteFileProc, hInst );
                 DialogBox( hInst, "DELETEFILE", hwndFrame, lpProcTemp );
                 FreeProcInstance( lpProcTemp );
                 EnableButtons( FALSE, FALSE, FALSE, FALSE, FALSE, FALSE );
                 //             Reset, Play, Pause, Stop, Record, Resume
                 break;


              case IDM_ABOUT:

                /**********************************************************/
                /* The ABOUT dialog box has been selected from the FILE   */
                /* menu.  Call AboutDlgProc to display the dialog box.    */
                /**********************************************************/

                 lpProcTemp = MakeProcInstance( AboutDlgProc, hInst );
                 DialogBox( hInst, "ABOUTAUDIO", hwndFrame, lpProcTemp );
                 FreeProcInstance( lpProcTemp );
                 break;

              case IDM_EXIT:

                /**********************************************************/
                /* EXIT has been selected from the FILE menu.   Disable   */
                /* mute in case other applications can hear audio from    */
                /* the M-Motion adapter.  Close the M-Motion driver,      */
                /* any opened files, and the application.                 */
                /**********************************************************/

                 DrvSendMessage( DriverHandle, AUX_MUTELINE, 0, NULL );
                 DrvClose( DriverHandle, NULL, NULL );
                 CloseAll();
                 PostQuitMessage( 0 );
                 break;
            }

         break;


      case WIM_OPEN:                   /* WAVE input device is opened  */
      case WIM_CLOSE:                  /* WAVE input device is closed  */
      case WOM_OPEN:                   /* WAVE output device is opened */
      case WOM_CLOSE:                  /* WAVE output device is closed */
         /**********************************************************/
         /* Receive the open/close wave input/output messages but  */
         /* don't do anything with them.                           */
         /**********************************************************/
         break;

      case WOM_DONE:

         /***************************************************************/
         /* This message is sent to a waveform output callback function */
         /* when the output buffer is being returned to the application.*/
         /*                                                             */
         /* Check to see if this is the last buffer of the file.        */
         /* If playing back an audio file and there are still buffers   */
         /* to be played, get the next buffer.  If recording, set       */
         /* the last buffer switch.                                     */
         /*                                                             */
         /* Get the handle of the global memory for the wave header.    */
         /* Cleanup prepared waveform output data block after the device*/
         /* driver is finished with a data block.                       */
         /* Unlock and free the wave header                             */
         /*                                                             */
         /* If this is the last buffer, stop the timer, set the wave    */
         /* audio buttons to Reset, and turn off the playing flag.      */
         /***************************************************************/

         WaveHdrPtr = ( LPWAVEHDR ) lParam;
         LastBuffer = WaveHdrPtr->dwUser;    // check last buffer flag
         if ( !Playing )
              LastBuffer = 1;
         else
              SendNextBuffer();

         HeaderHandle = LOWORD( GlobalHandle( HIWORD( lParam ) ) );
         waveOutUnprepareHeader( hWaveOut, WaveHdrPtr, sizeof( WAVEHDR ) );
         GlobalUnlock( HeaderHandle );
         GlobalFree( HeaderHandle );

         if  ( LastBuffer )                       // IF This Was Last Buffer
            { StopTimer();
              TimeInfo.wType = TIME_SAMPLES;
              waveOutGetPosition ( hWaveOut, (LPMMTIME) &TimeInfo,
                                  sizeof (TimeInfo) );
              Position = TimeInfo.u.sample;
              SetPosition();
              EnableButtons( TRUE, FALSE, FALSE, FALSE, FALSE, FALSE );
              //             Reset, Play, Pause, Stop, Record, Resume
              Playing = FALSE;
            }

         break;

      case WIM_DATA:

         /***************************************************************/
         /* This message is sent when there is data in the input buffer.*/
         /* lParam contains the pointer to WAVEHDR structure            */
         /*                                                             */
         /* If recording, continue to get buffers and write to the file.*/
         /* Check for disk space errors.                                */
         /* When finished recording reset the timer reset the timer and */
         /* and cleanup the prepared waveform input data block.         */
         /***************************************************************/

         if (ProgTerminate)
              break;

         if  ( Recording )
              AddRecordBuffer();
         WaveHdrPtr = ( LPWAVEHDR ) lParam;

         BufferPtr    = WaveHdrPtr->lpData;
         Position += ( WORD ) WaveHdrPtr->dwBytesRecorded;
         rc = _lwrite( WaveFile, BufferPtr, ( WORD ) WaveHdrPtr->dwBytesRecorded );


         /****************************************************************/
         /* Check for disk error.  Most likely you may run out of disk   */
         /* space.  You may want to consider the following questions:    */
         /* Do you still want to write the file to the disk if it is a   */
         /* disk space problem?  How do you want to handle the file,     */
         /* assume some of it is still valid, or is it all invalid?      */
         /* More extensive error handling will need to be added for your */
         /* own applications.  We chose to reset the RIFF header, close  */
         /* the file, and exit the application.  Please note that this   */
         /* file may not be valid.                                       */
         /****************************************************************/

         if ( rc != ( WORD ) WaveHdrPtr->dwBytesRecorded)
            { LoadString ( hInst, IDS_DISK, ErrMsgText, STRBUFLEN );
              MessageBox( hwndFrame, ErrMsgText, NULL,
                          MB_OK | MB_SYSTEMMODAL | MB_ICONHAND );
              ResetRiffHeader();
              CloseAll();
              DrvSendMessage( DriverHandle,           // Turn off MUTE
                              AUX_MUTELINE, 0, NULL );
              DrvClose( DriverHandle, NULL, NULL );
              ProgTerminate = TRUE;
              PostQuitMessage( 0 );
            }

         /***************************/
         /* When finished recording */
         /***************************/

         if  ( !Recording )              // Finished Recording
            { TimeInfo.wType = TIME_SAMPLES;
              waveInGetPosition ( hWaveIn, (LPMMTIME) &TimeInfo,
                                            sizeof (TimeInfo));
              Position = TimeInfo.wType;
              SetPosition();
              ResetRiffHeader();
            }

         /**********************************************/
         /* Cleanup prepared waveform input data block */
         /**********************************************/
         waveInUnprepareHeader( hWaveIn, WaveHdrPtr, sizeof( WAVEHDR ) );

         HeaderHandle = LOWORD( GlobalHandle( HIWORD( lParam ) ) );
         GlobalUnlock( HeaderHandle );   // Release Wave Header
         GlobalFree( HeaderHandle );

         break;


      case WM_MOVE:

         if  ( IsWindow( hwndButtonsDlg ) )
            { GetWindowRect( hwndFrame, &FrameRect );

              GetWindowRect( hwndButtonsDlg, &Rect );
              MoveWindow( hwndButtonsDlg, FrameRect.left + 12,
                          FrameRect.top + 50, Rect.right - Rect.left,
                          Rect.bottom - Rect.top, TRUE );

              GetWindowRect( hwndAuxDlg, &Rect );
              MoveWindow( hwndAuxDlg, FrameRect.left + 134,
                          FrameRect.top + 50, Rect.right - Rect.left,
                          Rect.bottom - Rect.top, TRUE );

              GetWindowRect( hwndDigitalDlg, &Rect );
              MoveWindow( hwndDigitalDlg, FrameRect.left + 134,
                          FrameRect.top + 263, Rect.right - Rect.left,
                          Rect.bottom - Rect.top, TRUE );

            }
         break;

      case WM_TIMER:

         TimeInfo.wType = TIME_SAMPLES;

         if  ( hWaveOut )
            { waveOutGetPosition( hWaveOut, ( LPMMTIME ) &TimeInfo,
                                              sizeof( TimeInfo) );
              Position = TimeInfo.u.sample;
              if  ( NeedsBuffers )
                 { NeedsBuffers--;
                   SendNextBuffer();
                 }
            }

         if  ( hWaveIn )
            { waveInGetPosition( hWaveIn, ( LPMMTIME ) &TimeInfo,
                               sizeof( TimeInfo) );
              Position = TimeInfo.u.sample;
            }

         SetPosition();
         break;


      case WM_INITMENUPOPUP:

         /********************************************************/
         /*  Grey menu items during record and play so a file    */
         /*  will not closed or a new one opened.                */
         /********************************************************/

         if (Playing || Recording)
            { EnableMenuItem (wParam, IDM_NEW,    MF_DISABLED | MF_GRAYED );
              EnableMenuItem (wParam, IDM_OPEN,   MF_DISABLED | MF_GRAYED );
              EnableMenuItem (wParam, IDM_CLOSE,  MF_DISABLED | MF_GRAYED );
              EnableMenuItem (wParam, IDM_DELETE, MF_DISABLED | MF_GRAYED );
              EnableMenuItem (wParam, IDM_EXIT,   MF_DISABLED | MF_GRAYED );
              EnableMenuItem (wParam, IDM_ABOUT,  MF_DISABLED | MF_GRAYED );
            }
         else
            { EnableMenuItem (wParam, IDM_NEW,    MF_ENABLED );
              EnableMenuItem (wParam, IDM_OPEN,   MF_ENABLED );
              EnableMenuItem (wParam, IDM_CLOSE,  MF_ENABLED );
              EnableMenuItem (wParam, IDM_DELETE, MF_ENABLED );
              EnableMenuItem (wParam, IDM_EXIT,   MF_ENABLED );
              EnableMenuItem (wParam, IDM_ABOUT,  MF_ENABLED );
            }
         break;

      case WM_DESTROY:

         /***********************************************************/
         /*  When closing the application, you need to:             */
         /*    - close the M-Motion driver                          */
         /*    - close all open audio files                         */
         /*    - close the application window                       */
         /***********************************************************/

         DrvClose( DriverHandle, NULL, NULL );   // Close M-Motion driver
         CloseAll();                             // close all files
         PostQuitMessage( 0 );
         break;

      case WM_PAINT:

         InvalidateRect( hwndFrame, NULL, TRUE );
         hdc = BeginPaint( hWnd, &ps );
         if  ( WaveFile )
            {
            GetClientRect( hwndFrame, &Rect );
            Rect.top = Rect.bottom - CharHeight;
            FillRect( hdc, ( LPRECT ) &Rect,
                      GetStockObject( WHITE_BRUSH ) );
            TextOut( hdc, Rect.left, Rect.top,
                     FileDescText, strlen( FileDescText ) );
            }
         EndPaint( hWnd, &ps );
         break;

      default:
         return( DefWindowProc( hWnd, message, wParam, lParam ) );
     }

 return( NULL );
 }
