/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       AUDIO.H                                             */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  DESCRIPTION:  Header file                                         */
/*                                                                    */
/**********************************************************************/

#include "WINDOWS.H"

#ifdef NONMCI
  #include "MCPAUDIO.H"                  // M-Control Program / 2 header
#else
  #include "MMSYSTEM.H"                  // Multimedia Extensions header
#endif

#include "STRING.H"
#include "STDIO.H"
#include "IO.H"
#include "CTYPE.H"
#include "TYPES.H"
#include "STAT.H"
#include "FCNTL.H"
#include "MCONTROL.H"                   // IBM M-Control Program header

#include "AUDDEFS.H"                    // AUDIO definitions header


/*********************/
/*   EXTERNAL DATA   */
/*********************/

extern HANDLE hInst;                    // This Instance Handle
extern HANDLE DriverHandle;             // IBM M-Motion Driver Handle

extern PCMWAVEFORMAT WaveFormat;        // Current Wave Format Data
extern WORD  WaveDevice;                // Wave Device Number

extern DWORD RecordFormat;              // Current Recording Rate / Format
extern DWORD LoopCount;                 // Loop Count     
extern WORD  TimerTics;                 // Timer Tick Counter
extern HWND  hwndFrame;                 // Frame Window Handle

extern HWND  hwndReset;                 // Reset Button Window
extern HWND  hwndPlay;                  // Play Button Window
extern HWND  hwndRecord;                // Record Button Window
extern HWND  hwndPause;                 // Pause Button Window
extern HWND  hwndResume;                // Resume Button Window
extern HWND  hwndStop;                  // Stop Button Window

extern HWND  hwndVolume;                // Volume Scroll Bar
extern HWND  hwndBalance;               // Balance Scroll Bar
extern HWND  hwndBass;                  // Bass   Scroll Bar
extern HWND  hwndTreble;                // Treble Scroll Bar
extern HWND  hwndDigitalVol;            // Digital Volume Scroll Bar
extern HWND  hwndRecordLevel;           // Record Level   Scroll Bar


extern HWND  hwndButtonsDlg;            // Control Buttons Dialog Handle
extern HWND  hwndAuxDlg;                // Auxiliary Scroll Bars Dialog Handle
extern HWND  hwndDigitalDlg;            // Digital Scroll Bars Dialog Handle
extern HWND  hwndDeviceDlg;             // Choose Wave Device Dialog Handle

extern HWAVEOUT hWaveOut;               // Handle to Wave Out Device
extern HWAVEOUT hWaveIn;                // Handle to Wave In Device

extern WORD  AuxDeviceID;               // Auxiliary Device ID
extern WORD  waveOutDeviceID;           // waveOut Device ID
extern WORD  waveInDeviceID;            // waveIn Device ID

extern BOOL  Playing;                   // Is Playing ? Flag
extern BOOL  Recording;                 // Is Recording ? Flag
extern WORD  RecordLevel;               // Digital Recording Level
extern WORD  DigitalVolume;             // Digital Output Volume

extern WORD  NeedsBuffers;              // Means Started Loop, But No End

extern WORD  RightVolume;               // Right Channel Volume
extern WORD  LeftVolume;                // Right Channel Volume
extern DWORD Volume;                    // Line Volume Setting
extern WORD  CurrentVolume;             // Current Volume
extern int   Balance;                   // Balance
extern int   Bass;                      // Bass
extern int   Treble;                    // Treble


extern int   WaveFile;                  // Audio File's DOS Handle
extern WORD  NumberBuffers;             // Buffers Needed for "n" Seconds
extern LONG  FileStart;                 // Offset of Start in File
extern LONG  Position;                  // Current Audio Position ( in Bytes )
extern LONG  WaveLength;                // Audio File Data Length ( in Bytes )
extern char  FileName[];                // Currently Open Audio File Name
extern char  FileDescText[];            // Desc of audio file (sample rate)
extern char  *TitleText;                // Title of window
extern LONG  EndofFile;                 // End of Play File
extern LONG  CurPlayPos;                // Current position in play file
extern BOOL  FileExist;                 // Is there an open audio file ?
extern BOOL  OldOpenFile;               // File exists but want to record over?
extern BOOL  NewFile;                   // NEWFILE dialog box flag
extern BOOL  ProgTerminate;             // Terminating program flag
       LONG  atol( char * );


#define SECONDS         ( DWORD ) 4L    // How Many Seconds to Buffer
#define BUFFSIZE        ( WORD ) 0x8000 // Size of Our Read/Write Buffers
#define NOFILE          (-1)            // No file opened
#define STRBUFLEN       ( int ) 120     // message string buffer length of 40
#define STRLEN15        ( int ) 15      // message string buffer length of 15
#define STRLEN32        ( int ) 32      // message string buffer length of 32
#define MAXLEVEL         0xFFFF         // maximum volume/recording level


/*******************/
/*   TIMER DATA    */
/*******************/

#define  TIMER_INTERVAL  110            // SetTimer() Interval ( milliseconds )
extern  BOOL  TimerOn;                 // Timer On/Off Flag



/**********************/
/*   RIFF STRUCTURE   */
/**********************/

 typedef struct {
          char   RiffName[4] ;          // Four Character Code for RIFF ID
          DWORD  RiffLength ;           // Size of data field in chunk
          char   RiffType[4] ;          // Form Type

          char   FormatChunkType[4] ;   // Chunk Format Type
          DWORD  FormatChunkLength ;    // Length of Format Chunk
          char   WaveFormatData[16];    // PCMWAVEFORMAT

          char   DataChunkType[4] ;     // Chunk Data Type
          DWORD  DataChunkLength ;      // Length of Data Chunk

        } RIFFHEADER;

extern RIFFHEADER RiffHeader;

#define  RiffHdrLength ( LONG ) sizeof(RIFFHEADER)



/***************************/
/*   FUNCTION PROTOTYPES   */
/***************************/

VOID  AddRecordBuffer( VOID );
VOID  AudioSeek( VOID );
VOID  CalcRightLeftVolume( WORD );
VOID  CloseAll( VOID );
VOID  EnableButtons( BOOL, BOOL, BOOL, BOOL, BOOL, BOOL );
int   GetBalance ( WORD, WORD );
int   GetNegPercent( int );
WORD  GetPercent( WORD );
int   GetValue( char *, int );
VOID  GetVolume( VOID );
VOID  GetWaveFormat( HWND );
BOOL  InitApplication( HANDLE );
BOOL  InitInstance( HANDLE, int );
VOID  Play( VOID );
WORD  ReadRadioButtons ( HWND, WORD * );
VOID  ResetRiffHeader( VOID );
WORD  SendNextBuffer( VOID );
VOID  SetPosition( VOID );
VOID  SetTitle( VOID );
VOID  SetVolume( VOID );
VOID  StartTimer( VOID );
VOID  StopTimer( VOID );

WORD  FAR  PASCAL  AboutDlgProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  AuxiliaryProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  ButtonsProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  DeleteFileProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  DigitalProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  NewProc( HWND, WORD, WORD, LONG );
WORD  FAR  PASCAL  OpenFileProc( HWND, WORD, WORD, LONG );
LONG  FAR  PASCAL  MainWndProc( HWND, WORD, WORD, LONG );
int   FAR  PASCAL  WinMain( HANDLE, HANDLE, LPSTR, int );

