/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       DELETE.C                                            */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    DeleteFileProc()                                    */
/*                                                                    */
/*  DESCRIPTION:  Processes the DELETE dialog from the FILE menu      */
/*                                                                    */
/**********************************************************************/
#include "AUDIO.H"                     // sample application header file

/**********************************************************************/
/* Function:     DeleteFileProc()                                     */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Dialog procedure that allows the user to delete a    */
/*               ".WAV" file from the current directory.              */
/*               Cursor is set to the first entry.                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/**********************************************************************/


WORD FAR PASCAL DeleteFileProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 static HWND hwndDelBox;
 WORD   nIndex;                //Index into open file listbox
 WORD   nLength;               //Length of filename
 char   DelFname[13];          //Filename to be deleted
 char   ErrMsgText[120];
 char   ErrHdrText[120];
 int    rc;
 WORD   wrc;

 switch ( Message )
     {

      case WM_INITDIALOG:

         /*************************************************************/
         /* Display all files with the filetype of ".WAV" that are in */
         /* the current directory.  Set the cursor to the first file. */
         /*************************************************************/

         ShowWindow( hwnd, SW_SHOWNORMAL );
         hwndDelBox = GetDlgItem( hwnd, ID_DELBOX );
         DlgDirList( hwnd, "*.WAV", ID_DELBOX, 0, 0 );
         SendMessage( hwndDelBox, LB_SETCURSEL, 0, 0L );
         return( TRUE );

      case WM_COMMAND:
         switch ( wParam )
            {
              case ID_DELBOX:

                 /*****************************************************/
                 /* Within the listbox, when receiving a double-click */
                 /* on a filename, select the file and return to the  */
                 /* main dialog box.                                  */
                 /*****************************************************/

                 if  ( HIWORD( lParam ) == LBN_DBLCLK )
                    {

                      wrc = ( WORD ) SendMessage( hwndDelBox, LB_GETCOUNT, 0, 0L );
                      if ( wrc == 0 )
                         {
                          LoadString (hInst, IDS_NOFILES,
                                      ErrMsgText, STRBUFLEN);
                          LoadString (hInst, IDS_WARNING,
                                      ErrHdrText, STRBUFLEN );
                          MessageBox( hwnd, ErrMsgText, ErrHdrText,
                                      MB_OK          |
                                      MB_SYSTEMMODAL |
                                      MB_ICONEXCLAMATION );
                          EndDialog( hwnd, 1 );
                          SetFocus( hwndFrame );
                          return( TRUE );
                         }

                      nIndex  = (WORD) SendMessage ( hwndDelBox,
                                                     LB_GETCURSEL, 0, 0L );
                      nLength = (WORD) SendMessage ( hwndDelBox, LB_GETTEXT,
                                           nIndex, (LONG) (LPSTR) DelFname );


                      LoadString (hInst, IDS_DELETEFILE, ErrMsgText, STRBUFLEN);
                      LoadString (hInst, IDS_WARNING, ErrHdrText, STRBUFLEN );
                      rc = MessageBox( hwnd, ErrMsgText, ErrHdrText,
                                       MB_YESNO       |
                                       MB_SYSTEMMODAL |
                                       MB_ICONQUESTION );

                      if (rc == IDNO)
                          return( TRUE) ;
                       remove (DelFname);

                      SendMessage ( hwndDelBox, LB_DELETESTRING, nIndex, 0L );
                      EndDialog( hwnd, 1 );
                      SetFocus( hwndFrame );
                    }
                 return( TRUE );


              case IDCANCEL:
                 /*****************************************************/
                 /* Cancel delete dialog and return to main dialog    */
                 /*****************************************************/
                 EndDialog( hwnd, 0 );
                 SetFocus( hwndFrame );
                 return( TRUE );

             case ID_DELETE:
                 /*****************************************************/
                 /* Delete selected filename from listbox and current */
                 /* directory.  Stay within the Delete Dialog until   */
                 /* cancel is selected.                               */
                 /*****************************************************/

                 // Check if there are any files to be deleted in listbox
                 wrc = ( WORD ) SendMessage( hwndDelBox, LB_GETCOUNT, 0, 0L );
                 if ( wrc == 0 )
                    {
                     LoadString (hInst, IDS_NOFILES,
                                 ErrMsgText, STRBUFLEN);
                     LoadString (hInst, IDS_WARNING,
                                 ErrHdrText, STRBUFLEN );
                     MessageBox( hwnd, ErrMsgText, ErrHdrText,
                                 MB_OK          |
                                 MB_SYSTEMMODAL |
                                 MB_ICONEXCLAMATION );
                     EndDialog( hwnd, 1 );
                     SetFocus( hwndFrame );
                     return( TRUE );
                    }

                 nIndex  = (WORD) SendMessage ( hwndDelBox,
                                        LB_GETCURSEL, 0, 0L );
                 nLength = (WORD) SendMessage ( hwndDelBox, LB_GETTEXT,
                                        nIndex, (LONG) (LPSTR) DelFname );


                 LoadString (hInst, IDS_DELETEFILE, ErrMsgText, STRBUFLEN);
                 LoadString (hInst, IDS_WARNING, ErrHdrText, STRBUFLEN );
                 rc = MessageBox( hwndDelBox, ErrMsgText, ErrHdrText,
                                  MB_YESNO | MB_SYSTEMMODAL | MB_ICONQUESTION );

                 if (rc == IDNO)
                    return( TRUE) ;
                 remove (DelFname);

                 SendMessage ( hwndDelBox, LB_DELETESTRING, nIndex, 0L );
                 SendMessage ( hwndDelBox, LB_SETCURSEL, 0, NULL );
            }
         break;
     }

 return( FALSE );
 }

