/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       DIGITAL.C                                           */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    DigitalProc()                                       */
/*                GetValue()                                          */
/*                                                                    */
/*  DESCRIPTION:  Processes for the digital volume and record level   */
/*                controls                                            */
/*                                                                    */
/*  M-MOTION/MCI FUNCTIONS USED:                                      */
/*                DrvSendMessage                                      */
/*                waveOutSetVolume                                    */
/**********************************************************************/
#include  "AUDIO.H"                    // sample application header file

char RecordLevelText[13];
char DigitalVolumeText[15];

/**********************************************************************/
/* Func: DigitalProc()                                                */
/*                                                                    */
/*                                                                    */
/* Called From:  MainWinInst()                                        */
/*                                                                    */
/* Desc:    Dialog proc for the Digital Scroll Bars dialog box.       */
/**********************************************************************/

WORD FAR PASCAL DigitalProc( hwnd, msg, wParam, lParam )
HWND hwnd;
WORD msg;
WORD wParam;
LONG lParam;
{
 HWND   hwndScrollBar;
 static char   temp[32];
 int    Percent;
 LPSTR  ctlstr = "%d%%";


 switch ( msg )
    {
      case WM_INITDIALOG:

          //-------------------------------------------------------
          // Get Values from MMOTION.INI file
          //-------------------------------------------------------
          LoadString(hInst, IDS_REC_LEVEL, RecordLevelText, 13);
          LoadString(hInst, IDS_DIGITAL_VOL, DigitalVolumeText, 15);

          RecordLevel   = GetValue( RecordLevelText, 0xC000 );
          DigitalVolume = GetValue( DigitalVolumeText, 0xC000 );

          //--------------------------------------------
          // Initialize Digital Record Level Scroll Bar
          //--------------------------------------------

          hwndRecordLevel = GetDlgItem( hwnd, ID_RECORDLEVEL_SB );
          DrvSendMessage( DriverHandle,  WAVEIN_GETVOLUME,
                          ( DWORD ) ( LPWORD ) &RecordLevel,  NULL );
          Percent = GetPercent ( RecordLevel );
          SetScrollRange( hwndRecordLevel, SB_CTL, 0, 100, TRUE );
          SetScrollPos( hwndRecordLevel, SB_CTL, Percent, TRUE );
          wsprintf( ( LPSTR ) temp, ctlstr, Percent );
          SetDlgItemText( hwnd, ID_RECORDLEVEL, temp );

          //---------------------------------------
          // Initialize Digital Volume Scroll Bar
          //---------------------------------------

          hwndDigitalVol = GetDlgItem( hwnd, ID_DIGITALVOL_SB );
          SetScrollRange( hwndDigitalVol, SB_CTL, 0, 100, TRUE );
          Percent = GetPercent ( DigitalVolume );
          SetScrollPos( hwndDigitalVol, SB_CTL, Percent, TRUE );
          wsprintf( ( LPSTR ) temp, ctlstr, Percent );
          SetDlgItemText( hwnd, ID_DIGITALVOL, temp );

          ShowWindow( hwnd, SW_SHOWNORMAL );
          break;

      case WM_HSCROLL:
         hwndScrollBar = HIWORD( lParam );

         //---------------------------------
         // Record Level Scroll Bar Control
         //---------------------------------

         if  ( hwndScrollBar == hwndRecordLevel )
            { switch ( wParam )
                    { case SB_TOP:
                         RecordLevel = 0xFFFF;
                         break;

                      case SB_BOTTOM:
                         RecordLevel = 0;
                         break;

                      case SB_LINEDOWN:
                         if  ( RecordLevel < 64881 )
                              RecordLevel += 655;
                         else
                              RecordLevel = 0xFFFF;
                         break;

                      case SB_LINEUP:
                         if  ( RecordLevel > 655 )
                              RecordLevel -= 655;
                         else
                              RecordLevel = 0;
                         break;

                      case SB_PAGEUP:
                         if  ( RecordLevel > 6554 )
                              RecordLevel -= 6554;
                         else
                              RecordLevel = 0;
                         break;

                      case SB_PAGEDOWN:
                         if  ( RecordLevel < 58982 )
                              RecordLevel += 6554;
                         else
                              RecordLevel = 0xFFFF;
                         break;

                      case SB_THUMBTRACK:
                      case SB_THUMBPOSITION:
                         RecordLevel = ( WORD )
                             ( ( ( LONG ) LOWORD( lParam ) << 16 ) / 100L );
                         break;

                      case SB_ENDSCROLL:
                         break;
                      }

              if  ( wParam != SB_THUMBTRACK )
                 { Percent = GetPercent ( RecordLevel );
                   RecordLevel = ( WORD )
                       ( ( ( DWORD ) Percent * ( DWORD ) 0xFFFF ) / 100L ) ;
                   SetScrollPos( hwndRecordLevel, SB_CTL, Percent, TRUE );
                   wsprintf( ( LPSTR ) temp, ctlstr, Percent );
                   SetDlgItemText( hwnd, ID_RECORDLEVEL, temp );
                   DrvSendMessage( DriverHandle,  WAVEIN_SETVOLUME,
                                   ( DWORD ) RecordLevel,  NULL );
                 }

              return( TRUE );
              }

         //-----------------------------------
         // Digital Volume Scroll Bar Control
         //-----------------------------------

         if  ( hwndScrollBar == hwndDigitalVol )
            { switch ( wParam )
                    { case SB_TOP:
                         DigitalVolume = 0xFFFF;
                         break;

                      case SB_BOTTOM:
                           DigitalVolume = 0;
                         break;

                      case SB_LINEDOWN:
                         if  ( DigitalVolume < 64881 )
                              DigitalVolume += 655;
                         else
                              DigitalVolume = 0xFFFF;
                         break;

                      case SB_LINEUP:
                     //  DigitalVolume --;
                         if  ( DigitalVolume > 655 )
                              DigitalVolume -= 655;
                         else
                              DigitalVolume = 0;
                         break;

                      case SB_PAGEUP:
                         if  ( DigitalVolume > 6554 )
                              DigitalVolume -= 6554;
                         else
                              DigitalVolume = 0;
                         break;

                      case SB_PAGEDOWN:
                         if  ( DigitalVolume < 58982 )
                              DigitalVolume += 6554;
                         else
                              DigitalVolume = 0xFFFF;
                         break;

                      case SB_THUMBTRACK:
                      case SB_THUMBPOSITION:
                         DigitalVolume = ( WORD )
                            ( ( ( LONG ) LOWORD( lParam ) << 16 ) / 100L );
                         break;

                      case SB_ENDSCROLL:
                         break;
                      }

              if  ( wParam != SB_THUMBTRACK )
                 { Percent = GetPercent ( DigitalVolume );
                   DigitalVolume = ( WORD )
                         ( ( ( DWORD ) Percent * ( DWORD ) 0xFFFF ) / 100L );
                   SetScrollPos( hwndDigitalVol, SB_CTL, Percent, TRUE );
                   wsprintf( ( LPSTR ) temp, ctlstr, Percent );
                   SetDlgItemText( hwnd, ID_DIGITALVOL, temp );
                   if  ( hWaveOut )
                        waveOutSetVolume( AuxDeviceID, ( DWORD ) DigitalVolume );
                   return( TRUE );
                 }
              }
              return( TRUE );

      default:
          return( FALSE );
      }

 return( TRUE );
 }

/**********************************************************************/
/* Func:  GetValue()                                                  */
/*                                                                    */
/* Called from:  DigitalProc()                                        */
/*                                                                    */
/* Parm1  char *  String    Key Name                                  */
/* Parm2  int     Default   Return Value if                           */
/*                          GetPrivateProfileString() Fails           */
/*                                                                    */
/* Desc:  Read the Default Values from the MMOTION.INI File for the   */
/*        mmotion.dll application                                     */
/*                                                                    */
/* RETURNS: int Value (value in MMOTION.INI file)                     */
/**********************************************************************/

int GetValue( String, Default )
char *String;
int  Default;
{
 char    DriverText[12];                 // App name [M-MOTION]
 char    MMotionIniText[12];             // System filename MMOTION.INI
 static  char  temp[16];                //
 int     n;                             // value
 char    *p;                            // pointer to value in file
 char    c;                             // a character value from the file
 WORD    Sign = 1;                      // change the sign
 int     rc;                            // return code


 //----------------------------------------------------
 // Checking MMOTION.INI file for the M-MOTION settings
 //----------------------------------------------------

 LoadString (hInst, IDS_DRIVER, DriverText, STRLEN15);
 LoadString (hInst, IDS_MMOTIONINI, MMotionIniText, STRLEN15);

 rc = GetPrivateProfileString( ( LPSTR ) DriverText,
                               ( LPSTR ) String,
                               ( LPSTR ) "",
                               ( LPSTR ) temp,
                               sizeof( temp ),
                               ( LPSTR ) MMotionIniText );

 // Didn't find the [M-MOTION] values, set them to the default values
 if  ( rc <= 0 )
      n = Default;
 // Found [M-MOTION] setting, get the values
 else
    { n = 0;
      p = &temp[0];

      while ( !isdigit( *p )  &&  ( *p ) )
           { if  ( *p == '-' )
                  Sign = -1;
             p++;
             }

      while ( isdigit( *p ) )
           { c = *p++;
             c = c - ( char ) '0';
             n = n * 10 + ( int ) c;
             }

      n *= Sign;
      }

 return( n );
 }

