/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       NEW.C                                               */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    NewProc()                                           */
/*                GetWaveFormat()                                     */
/*                ReadRadioButtons()                                  */
/*                                                                    */
/*  DESCRIPTION:  Processes the NEW dialog from the FILE menu item    */
/*                                                                    */
/**********************************************************************/
#include "AUDIO.H"                     // Sample application header file


/*  Initialize RIFF Header  */

RIFFHEADER RiffHeader = { 'R', 'I', 'F', 'F',
                          0L,
                          'W', 'A', 'V', 'E',
                          'f', 'm', 't', ' ',
                          16L,
                          0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                          'd', 'a', 't', 'a' ,
                          0L };

/* Initialize new dialog proc radio buttons array */

WORD WaveChannel[]    = { ID_MONO, ID_STEREO };
WORD WaveSampleSize[] = { ID_8BIT, ID_16BIT };
WORD WaveFrequency[]  = { ID_31KHZ, ID_78KHZ };

/**********************************************************************/
/* Function:     NewProc()                                            */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     GetWaveFormat()                                      */
/*               SetTitle()                                           */
/*                                                                    */
/* Description:  Dialog procedure for creating a NEW audio file name  */
/**********************************************************************/
WORD FAR PASCAL NewProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 char   Name[10];
 HWND   hTemp;

 switch ( Message )
     {
      case WM_INITDIALOG:

         /*************************************************/
         /*  Set default buttons on NEWFILE dialog box to */
         /*  Stereo, 8-bit, 31.50 sample rate             */
         /*************************************************/

         CheckDlgButton ( hwnd, ID_STEREO, 1);
         CheckDlgButton ( hwnd, ID_31KHZ, 1);
         CheckDlgButton ( hwnd, ID_8BIT, 1);
         SendDlgItemMessage( hwnd, ID_FNAME, EM_LIMITTEXT, 8, 0L );
         SetDlgItemText( hwnd, ID_FNAME, "" );
         return( TRUE );

      case WM_COMMAND:

         switch ( wParam )

               {
                 case ID_78KHZ:

                   /*******************************************************/
                   /* Frequencey Rate:  7.875 selected                    */
                   /* Disable stereo, select mono button, deselect stereo.*/
                   /* You cannot record stereo at this frequency.         */
                   /*******************************************************/

                   hTemp = GetDlgItem ( hwnd, ID_STEREO );
                   EnableWindow (hTemp, 0);
                   CheckDlgButton( hwnd, ID_MONO, 1);
                   CheckDlgButton( hwnd, ID_STEREO, 0);
                   break;

                 case ID_31KHZ:

                   /***********************************/
                   /* Frequencey Rate:  31.5 selected */
                   /* Enable stereo                   */
                   /***********************************/

                   hTemp = GetDlgItem ( hwnd, ID_STEREO );
                   EnableWindow ( hTemp, 1 );
                   break;

                 case IDOK:

                   /********************************************************/
                   /* Setup the filename , RIFF Header, and window caption */
                   /* for the new audio file.                              */
                   /********************************************************/

                    NewFile = FALSE;
                    GetDlgItemText( hwnd, ID_FNAME, Name, sizeof( Name ) );
                    if  ( Name[0] )
                       { strcpy( FileName, Name );
                         strcat( FileName, ".WAV" );
                         NewFile = TRUE;
                         GetWaveFormat( hwnd );
                         SetTitle();
                       }
                    EndDialog( hwnd, IDOK );
                    return( TRUE );

                 case IDCANCEL:

                    /*****************************************/
                    /* Reset the filename to null and return */
                    /*****************************************/

                    FileName[0] = NULL;
                    FileExist = FALSE;
                    EndDialog( hwnd, 0 );
                    SetFocus( hwndFrame );
                    return( TRUE );

                 }
         break;
      }

 return( FALSE );
 }

/*****************************************************************************/
/* Function:     GetWaveFormat()                                             */
/*                                                                           */
/* Called from:  MainWinProc()                                               */
/*               NewProc()                                                   */
/*                                                                           */
/* Calls to:     AudioSeek()                                                 */
/*               CloseAll()                                                  */
/*               ReadRadioButtons()                                          */
/*               SetTitle()                                                  */
/*               SetPosition()                                               */
/*                                                                           */
/* Description:  Setup the RIFF Header format for the WAVE file              */
/*                                                                           */
/*****************************************************************************/
VOID GetWaveFormat( HWND  hwnd )
{
 char  ErrMsgText[120];
 char  CaptionText[120];
 BOOL  New = FALSE;
 DWORD BytesSecs;
 WORD  BytesPerSample;
 WORD  wChannel;
 WORD  wSampleSize;
 WORD  wFreq;
 WORD  TwoBuffers  = 2;
 int   rc;

 /***********************************************************************/
 /*  Open audio file.  We used a 'C' open file command here rather      */
 /*  than the MCI commands. This allows this program to either be     ` */
 /*  linked with the Multimedia Extensions MMSYSTEM.LIB (using the      */
 /*  MMSYSTEM.H header file) or the M-Control Program's MCPAUDIO.LIB    */
 /*  file which is a non-MCI interface.                                 */
 /***********************************************************************/

 WaveFile = open( FileName, O_BINARY | O_RDWR );

 /********************************************************/
 /*  If the file exists, do you want to record over it ? */
 /********************************************************/

 if ( WaveFile != NOFILE  &  NewFile == TRUE )
    { LoadString (hInst, IDS_RECORDOVERFILE, ErrMsgText, STRBUFLEN);
      LoadString (hInst, IDS_WARNING, CaptionText, STRBUFLEN );
      rc = MessageBox( hwnd, ErrMsgText, CaptionText,
                           MB_YESNO       |
                           MB_SYSTEMMODAL |
                           MB_ICONQUESTION );
      if ( rc == IDNO )
         { CloseAll();
           FileExist = FALSE;
           return;
         }
      else
         New = TRUE;
    }

 /*****************************************/
 /*  If the file doesn't exist, create it */
 /*****************************************/

 if  ( WaveFile == NOFILE )
    { WaveFile = open( FileName, O_BINARY | O_RDWR | O_CREAT, S_IWRITE );
      New = TRUE;
      FileExist = TRUE;
    }

 /****************************************************/
 /* If it can't be created, display an error message */
 /****************************************************/

 if  ( WaveFile == NOFILE )              /* still no file, error message */
    { LoadString (hInst, IDS_LOCATEFILE, ErrMsgText, STRBUFLEN);
      MessageBox( hwndFrame, ErrMsgText, NULL,
                             MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
      FileExist = FALSE;
      return;
    }

 /*****************************************************************/
 /* If this is a brand new file, setup the waveformat/RIFF Header */
 /* information                                                   */
 /*****************************************************************/

 if (New)
    { WaveFormat.wf.wFormatTag = WAVE_FORMAT_PCM;

      /* Read the readion buttons */

      wChannel    = ReadRadioButtons ( hwnd, WaveChannel );
      wSampleSize = ReadRadioButtons ( hwnd, WaveSampleSize );
      wFreq       = ReadRadioButtons ( hwnd, WaveFrequency );

      /* Map the user setting into the corresponding values in  */
      /* the PCMWAVEFORMAT structure.                           */
      /* The physical capacity of the M-Motion Video Adapter/A  */
      /* is 12 bits.  Padding of 4 bits maintains compatibility */
      /* with standard (RIFF) file formats of 8 and 16 bits.    */
      /* See Chapter 9 of the M-Control Program Version 2.0     */
      /* User's Guide                                           */

      WaveFormat.wf.nChannels   = (wChannel == ID_STEREO) ? 2 : 1;
      WaveFormat.wBitsPerSample = (wSampleSize == ID_8BIT) ? 8 : 16;

      /* Check sample rate */

      switch (wFreq)
         {
         case ID_31KHZ:
            WaveFormat.wf.nSamplesPerSec  = 31500L;
            break;
         case ID_78KHZ:
            WaveFormat.wf.nSamplesPerSec  = 7875L;
            break;
         default:
            break;
         }

      /* Calculate number of bytes per sample */

      BytesPerSample = (WaveFormat.wBitsPerSample / 8);

      /* Calculate number of units of data that can be passed to a */
      /* driver.  Number of channels * bytes per sample            */

      WaveFormat.wf.nBlockAlign = WaveFormat.wf.nChannels * BytesPerSample;

      /* Calculate average data rate in bytes per second.               */
      /* Number of channels * bytes per sample * sample rate in samples */
      /* per second                                                     */

      WaveFormat.wf.nAvgBytesPerSec = (WaveFormat.wf.nChannels *
                                    BytesPerSample *
                                    WaveFormat.wf.nSamplesPerSec);

      /*  Create RIFF Header */

      RiffHeader.RiffLength      = ( LONG ) sizeof( RiffHeader );
      RiffHeader.DataChunkLength = 0L;
      memcpy( &(RiffHeader.WaveFormatData), &WaveFormat, sizeof( PCMWAVEFORMAT ) );
      write( WaveFile, ( char * ) &RiffHeader, sizeof( RiffHeader ) );
      lseek( WaveFile, sizeof(RiffHeader), SEEK_SET);
      New = FALSE;
    }
 else
    {
      /* If it is an existing file, check to see if the header of the    */
      /* file is in the RIFF format, if not display a message and return */

      lseek( WaveFile, 0L, SEEK_SET );
      read( WaveFile, &RiffHeader, sizeof( RiffHeader ) );

      if (strncmp (RiffHeader.RiffName, "RIFF", 4 )           ||
          strncmp (RiffHeader.RiffType, "WAVE", 4 )           ||
          strncmp (RiffHeader.FormatChunkType, "fmt ",  4 )   ||
          strncmp (RiffHeader.DataChunkType, "data", 4 )      )
         {  // if this is a -not- RIFF WAVE file
          LoadString (hInst, IDS_NOTRIFF, ErrMsgText, STRBUFLEN);
          MessageBox( hwndFrame, ErrMsgText, NULL,
                                 MB_OK | MB_APPLMODAL | MB_ICONEXCLAMATION );
          FileName[0] = 0;
          SetTitle();
          return;
         }


      lseek( WaveFile, 20L, SEEK_SET );
      read( WaveFile, &WaveFormat, sizeof( WaveFormat ) );
    }


 FileExist = TRUE;
 Position  = 0L;
 EndofFile = lseek( WaveFile, 0L, SEEK_END);

 AudioSeek();             /* seek to the beginning of the file */

 /***********************************************************/
 /* Write the bottom line of the parent dialog box with the */
 /* rate information                                        */
 /***********************************************************/

 LoadString (hInst, IDS_SAMPLERATE, ErrMsgText, STRBUFLEN);
 wsprintf( FileDescText, ErrMsgText, WaveFormat.wf.nSamplesPerSec,
                                  WaveFormat.wBitsPerSample );

 if  ( WaveFormat.wf.nChannels == 2 )
      LoadString (hInst, IDS_STEREO, ErrMsgText, STRBUFLEN);
 else
      LoadString (hInst, IDS_MONO, ErrMsgText, STRBUFLEN);
      strcat( FileDescText, ErrMsgText );

 /* Calculate Number of Buffers Needed for "n" Seconds */

 BytesSecs     = WaveFormat.wf.nAvgBytesPerSec * SECONDS;
 NumberBuffers = ( WORD ) ( BytesSecs / BUFFSIZE );

 /* You will need at least 2 buffers if recording less than 2 buffers, */
 /* otherwise there will be a pop sound in the playback.               */

 if (NumberBuffers < TwoBuffers)
    NumberBuffers = TwoBuffers;

 if  ( ( BytesSecs % BUFFSIZE ) >= ( BUFFSIZE / 2L ) )
       NumberBuffers++;

 SetTitle();

 return;
}

/*****************************************************************************/
/* Func: ReadRadioButtons()                                                  */
/*                                                                           */
/* Called from:  GetWaveForamt()                                             */
/*                                                                           */
/* Calls to:     None                                                        */
/*                                                                           */
/* Desc:  Checks which buttons have been selected for the Channel, Sample    */
/*        Size and Frequency options when creating a new audio file.         */
/*                                                                           */
/*****************************************************************************/
WORD ReadRadioButtons ( HWND hwnd, WORD wIds[] )
{
   WORD i;
   for (i=0; wIds[i]; i++)
      { if (SendDlgItemMessage ( hwnd, wIds[i], BM_GETCHECK, 0, 0L))
           return ( wIds[i] );
      }
   return (0);
}
