/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       OPEN.C                                              */
/*                                                                    */
/*  VERSION:      Digital Audio Sample Application for the            */
/*                M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  FUNCTIONS:    OpenFileProc()                                      */
/*                GetWaveFormat()                                     */
/*                ReadRadioButtons()                                  */
/*                                                                    */
/*  DESCRIPTION:  Processes the OPEN dialog from the FILE menu item   */
/*                                                                    */
/**********************************************************************/
#include "AUDIO.H"                     // sample application header file

/**********************************************************************/
/*                                                                    */
/* Function:     OpenFileProc()                                       */
/*                                                                    */
/* Called from:  MainWndProc()                                        */
/*                                                                    */
/* Calls to:     None                                                 */
/*                                                                    */
/* Description:  Dialog proc called when OPEN is selected from the    */
/*               File menu item.  This module will only list those    */
/*               files with the filetype of ".WAV" that are within    */
/*               the same directory as SAMPAUD.EXE.                   */
/*                                                                    */
/**********************************************************************/

WORD FAR PASCAL OpenFileProc( hwnd, Message, wParam, lParam )
HWND hwnd;
WORD Message;
WORD wParam;
LONG lParam;
{
 static HWND hwndOpenBox;
 char   Choice[16];                     // Filename selected within listbox
 WORD   listcount;                      // Number of files in the listbox
 char   ErrMsgText[120];
 char   ErrHdrText[120];

 switch ( Message )
     {
      case WM_INITDIALOG:

         /**************************************************************/
         /*  Gets the handle to the child window of hwndOpenBox        */
         /*  Fills the listbox with all files with the filetype ".WAV" */
         /*  that are in the same directory as the SAMPAUD.EXE.        */
         /*  Sets the cursor to the first item in the listbox          */
         /**************************************************************/

         ShowWindow( hwnd, SW_SHOWNORMAL );
         hwndOpenBox = GetDlgItem( hwnd, ID_OPENBOX );
         DlgDirList( hwnd, "*.WAV", ID_OPENBOX, 0, 0 );
         SendMessage( hwndOpenBox, LB_SETCURSEL, 0, 0L );
         return( TRUE );

      case WM_COMMAND:

         switch ( wParam )

               {
                 case IDOK:

                   /***************************************************/
                   /* Check to see if any files exist in the listbox. */
                   /* Get the selected filename, and return to the    */
                   /* parent dialog.                                  */
                   /***************************************************/

                   listcount = ( WORD ) SendMessage( hwndOpenBox,
                                                     LB_GETCOUNT, 0, 0L );
                   if (listcount == 0)
                      { LoadString (hInst, IDS_NOFILES, ErrMsgText, STRBUFLEN);
                        LoadString (hInst, IDS_WARNING, ErrHdrText, STRBUFLEN );
                        MessageBox( hwnd, ErrMsgText, ErrHdrText,
                                          MB_OK          |
                                          MB_SYSTEMMODAL |
                                          MB_ICONEXCLAMATION );
                        EndDialog( hwnd, 1);
                        return(FALSE);
                      }

                   DlgDirSelect( hwnd, Choice, ID_OPENBOX );
                   if  ( Choice[0] )
                        strcpy( FileName, Choice );

                   FileExist = TRUE;
                   OldOpenFile = TRUE;
                   EndDialog( hwnd, 1 );
                   return( TRUE );


                 case IDCANCEL:

                   /**************************************************/
                   /* Cancel the OPEN child dialog and return to the */
                   /* parent dialog                                  */
                   /* Reset switches                                 */
                   /**************************************************/

                   FileExist = FALSE;
                   OldOpenFile = FALSE;
                   EndDialog( hwnd, 0 );
                   return( TRUE );
               }
         break;
     }
 return( FALSE );
}
