#**************************************************************************
#  (C) Copyright IBM Corporation 1992
#
#  MODULE:       SAMPAUD.MAK     Sample Audio Make File
#
#  DESCRIPTION:  Sample MAKE file for SAMPAUD.EXE
#                IBM M-Control Program / 2  Version 2.0
#
#                You will need the following to compile/link this
#                this digital audio sample:
#
#                   Microsoft C (TM) 6.0
#                   Microsoft Windows (TM) Software Development Toolkit
#                   Non-MCI:  M-Control Program
#                             (MCPAUDIO.H & MCPAUDIO.LIB)
#                   MCI:      Multimedia Extensions
#                             (MMSYSTEM.H & MMSYSTEM.LIB)
#
#                SAMPAUD.MSG will contain the compile/link messages
#**************************************************************************
.SUFFIXES: .obj .exe .c .h .lib


#**************************************************************************
# COMPILE INSTRUCTIONS:
#**************************************************************************
# This sample can be compiled for use on an MCI system or a Non-MCI system.
#
# To compile on an MCI system do the following:
#    1. Delete any old ".obj", ".exe", or ".res" files from the directory.
#    2. Remove the comment (#) from the line with the "MCI_FLAG" definition.
#    3. Execute "NMAKE SAMPAUD.MAK"
#
# To compile on a Non-MCI system do the following:
#    1. Delete any old ".obj", ".exe", or ".res" files from the directory.
#    2. Add a comment (#) in column 1 to the line with the
#       "MCI_FLAG" definition.
#    3. Execute "NMAKE SAMPAUD.MAK"
#
#**************************************************************************
#MCI_FLAG = Delete the comment symbol (#) from this line to compile for MCI


#**************************************************************************
# VARIABLE ASSIGNMENTS:
#**************************************************************************
CC  = cl
LNK = link
ERR = sampaud.msg


#**************************************************************************
# DEPENDENT HEADERS, OBJECTS, and LIBRARIES:
#**************************************************************************
DEP_INC = { $(INCLUDE) } MCONTROL.H    \
          { $(INCLUDE) } MCPAUDIO.H    \
          AUDDEFS.H                    \
          AUDIO.H

DEP_OBJ = AUDIO.OBJ     \
          ABOUT.OBJ     \
          AUXIL.OBJ     \
          BUTTONS.OBJ   \
          DIGITAL.OBJ   \
          DELETE.OBJ    \
          MISC.OBJ      \
          NEW.OBJ       \
          OPEN.OBJ      \
          AUDIO.DEF


#**************************************************************************
# COMPILE ARGUMENTS:
#**************************************************************************
CFLAGS = -AM -c -FPa -G2sw -Od -W3 -Zpe


#**************************************************************************
# INFERENCE RULES
#**************************************************************************
.c.obj  :
!  IFDEF MCI_FLAG
      $(CC) $(CFLAGS)  $*.c  >>$(ERR)
!  ELSE
      $(CC) $(CFLAGS) -DNONMCI  $*.c  >>$(ERR)
!  ENDIF


#**************************************************************************
# "OBJ"'s -> "DLL", "LIB", "EXE" Section
#**************************************************************************
all  :  sampaud.exe

sampaud.exe  :  $(DEP_OBJ)  sampmci.lnk  sampaud.lnk  sampaud.res
!  IFDEF MCI_FLAG
      $(LNK) @sampmci.lnk  >>$(ERR)
!  ELSE
      $(LNK) @sampaud.lnk  >>$(ERR)
!  ENDIF
      rc  sampaud.res  sampaud.exe  >>$(ERR)

audio.obj    :  audio.c    $(DEP_H)

about.obj    :  about.c    $(DEP_H)

auxil.obj    :  auxil.c    $(DEP_H)

buttons.obj  :  buttons.c  $(DEP_H)

digital.obj  :  digital.c  $(DEP_H)

delete.obj   :  delete.c   $(DEP_H)

misc.obj     :  misc.c     $(DEP_H)

new.obj      :  new.c      $(DEP_H)

open.obj     :  open.c     $(DEP_H)


#**************************************************************************
# "RC" -> "RES"
#**************************************************************************
sampaud.res  :  sampaud.rc
!  IFDEF MCI_FLAG
      rc  -r  sampaud.rc  sampaud.res  >>$(ERR)
!  ELSE
      rc  -r  -DNONMCI  sampaud.rc  sampaud.res  >>$(ERR)
!  ENDIF
