/**********************************************************************/
/*                                                                    */
/* (C) Copyright IBM Corporation 1992                                 */
/*                                                                    */
/*  MODULE:       SAMPVID.C                                           */
/*                                                                    */
/*  VERSION:      M-Control Program /2   Version 2.0                  */
/*                using Microsoft Windows 3.0 ( TM )                  */
/*                                                                    */
/*  DESCRIPTION:  A video application with limited keyboard control   */
/*                of the video attributes.  No player control         */
/*                support is provided.                                */
/*                                                                    */
/**********************************************************************/

#include <windows.h>
#include <mvwnwin.h>


#define  MVWN_ACK        WM_USER + 0x100
#define  MVWN_RESPONSE   WM_USER + 0x101
#define  MVWN_ADVISE     WM_USER + 0x102

//---------------------------------------------------------------------
// GLOBAL VARIABLES
//---------------------------------------------------------------------

typedef long ( FAR PASCAL *PFNWP ) ( HWND, WORD, WORD, LONG );

static  HWND    hwndVideo;                    // Video Window
static  ATOM    CMD_vw, QUERY_vw;
static  PFNWP   pfnVideoProc;
static  HANDLE  hInst;                        // This Instance
static  HANDLE  hPenHandle;

static  WORD    usVolume =  40;               // Current Volume Setting

//---------------------------------------------------------------------
// TRACKING RECTANGLE DATA
//---------------------------------------------------------------------

static  BOOL  Tracking  =  FALSE;   // Are We Currently Tracking a Move?
static  RECT  TrackingRect;
static  POINT TrackingPoint;
static  HPEN  hPen;
static  HDC   hDC;


//---------------------------------------------------------------------
// FUNCTION PROTOTYPES
//---------------------------------------------------------------------

long    FAR   PASCAL VideoSubclassProc( HWND , WORD, WORD, LONG );

static  VOID  BeginTrackRect( VOID );
static  BOOL  CreateVideoControls( VOID );
static  BOOL  InitInstance( HANDLE );
static  VOID  MoveTrackRect( VOID );
static  VOID  EndTrackRect( VOID );
static  VOID  MVWNCommand( WORD, LONG );


/**********************************************************************/
/*                                                                    */
/* Function:     WinMain()                                            */
/*                                                                    */
/* Description:  Initialize application instance, set up video        */
/*               window, and set up message processing loop.          */
/*                                                                    */
/**********************************************************************/

int PASCAL WinMain( HANDLE hInstance,  HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow )
{
 MSG  msg;
 WORD VideoSource;

 if  ( !InitInstance( hInstance ) )             // Initial This Instance
      return( FALSE );

 if  ( hwndVideo )
    { if  ( lpszCmdLine )
         { switch ( *lpszCmdLine )
                 { case '2':
                        VideoSource = 2;
                        break;
                   case '3':
                        VideoSource = 3;
                        break;
                   default:
                        VideoSource = 1;
                        break;
                 }
           MVWNCommand( MVWN_CMD_VIDEOSOURCE, VideoSource );
           }
      }

 ShowWindow( hwndVideo, nCmdShow );
 UpdateWindow( hwndVideo );
 SetFocus( hwndVideo );

 while  ( GetMessage( &msg, NULL, NULL, NULL ) )
       { TranslateMessage( &msg );
         DispatchMessage( &msg );
         }

 MVWNTerminate();
 return( 0 );
 }


/**********************************************************************/
/*                                                                    */
/* Function:     InitInstance()                                       */
/*                                                                    */
/* Description:  Initialize information for this instance.            */
/*                                                                    */
/**********************************************************************/

static BOOL InitInstance( hInstance )
HANDLE hInstance;
{
 hInst = hInstance;                         // Save This Instance Handle
 return( CreateVideoControls() );
 }


/**********************************************************************/
/*                                                                    */
/* Function:     CreateVideoControls()                                */
/*                                                                    */
/* Description:  Creates the video controls.                          */
/*                                                                    */
/**********************************************************************/

static BOOL CreateVideoControls()
{
 LPMVWNCTL pmvwnctl;
 WORD      Width;
 WORD      Height;
 HANDLE    ControlHandle;
 DWORD     WinFlags;
 FARPROC   lpNewVideoProc;
 BOOL      bSuccess = FALSE;

  // Query system for screen size information to be used
  // in determining small video size ( 1/16 screen area )

  Width  = GetSystemMetrics( SM_CXSCREEN ) / 4;
  Height = GetSystemMetrics( SM_CYSCREEN ) / 4;

  //-------------------------------------------------------------
  // Init and register the Video Window class.
  // NOTE: This call tells US the message atoms for commands and
  // queries; and WE tell the MVWN the message id's to use for
  // Acknowledges, Responses, and Advises.
  //-------------------------------------------------------------

  if  ( !MVWNInitialize( &CMD_vw,  &QUERY_vw,  MVWN_ACK,
                         MVWN_RESPONSE,  MVWN_ADVISE ) )
       return( FALSE );

  //---------------------------------------------------------------
  // Now create Control Structure to use for creating Video Window
  //---------------------------------------------------------------

  ControlHandle = MVWNCreateCtl();
  if  ( ControlHandle )
     { pmvwnctl = ( LPMVWNCTL ) GlobalLock( ControlHandle );
       pmvwnctl->bUserActivate    = TRUE;      // We Do All Specific Activates
       pmvwnctl->hWndAdviseActive = hwndVideo; // Set up Advise Window

       WinFlags = GetWinFlags();          // Windows Memory Configuration Flags
       if  ( ( WinFlags & WF_ENHANCED ) ||  // IF Standard or 386 Enhanced Mode
             ( WinFlags & WF_STANDARD ) )   //    Use Inactive Images
            pmvwnctl->wInactiveStyle = MVWN_INACTIVE_VIDEOIMAGE;
       else                                 // ELSE in Real Mode
                                            //   Use Bitmap (Uses Less Memory)
            pmvwnctl->wInactiveStyle = MVWN_INACTIVE_MONOBITMAP;
       }

  //--------------------------------------------------------
  // Create the Video Window.
  //--------------------------------------------------------

  hwndVideo = CreateWindow( WC_MVWN,
                            "SAMPVID",
                            WS_POPUP | WS_BORDER | WS_THICKFRAME,
                            Width,
                            Height,
                            Width,
                            Height,
                            NULL,
                            0,
                            hInst,
                            ( LPSTR ) pmvwnctl );

  GlobalUnlock( ControlHandle );
  MVWNDestroyCtl( ControlHandle );     // Free the MVWN Control

  if  ( !hwndVideo )
       return( FALSE );

  MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) TRUE );       // Activate the Window
  pfnVideoProc   = ( PFNWP ) GetWindowLong( hwndVideo, GWL_WNDPROC );
  lpNewVideoProc = MakeProcInstance( ( FARPROC ) VideoSubclassProc, hInst );
  SetWindowLong( hwndVideo, GWL_WNDPROC, ( LONG ) lpNewVideoProc );
  return( TRUE );
  }

/**********************************************************************/
/*                                                                    */
/* Function:     MVWNCommand()                                        */
/*                                                                    */
/* Description:  Video Window generic command function.  All commands */
/*               to the video window get funneled through here.       */
/*                                                                    */
/**********************************************************************/

static VOID MVWNCommand( WORD Command, LONG mp1 )
{
 LPMVWNCMD pmvwncmd;
 HANDLE    CommandHandle;

 if  ( IsWindow( hwndVideo ) )                      // IF Video Window is Valid
    { if  ( CommandHandle = MVWNCreateCmd( Command ) ) // Create the Command
         { pmvwncmd = ( LPMVWNCMD ) GlobalLock( CommandHandle );
           if  ( pmvwncmd )                            // IF Memory Locked
              { pmvwncmd->data.dwCmdParam1 = mp1;      //    Set Command Parm 1
                pmvwncmd->data.dwCmdParam2 = 0L;       //    Parm 2 is Always 0
                GlobalUnlock( CommandHandle );         //    Release Memory
                if  ( Command == MVWN_CMD_ACTIVATE )
                     SendMessage( hwndVideo, CMD_vw, hwndVideo,
                                  ( LONG ) CommandHandle );
                else
                     PostMessage( hwndVideo, CMD_vw, hwndVideo,
                                  ( LONG ) CommandHandle );
                }
           }
      }
 return;
 }


/**********************************************************************/
/*                                                                    */
/* Function:     VideoSubclassProc()                                  */
/*                                                                    */
/* Description:  Allow keyboard and mouse input to the video window   */
/*               to be intercepted and used to move the video window  */
/*               and control various display attributes.              */
/*                                                                    */
/*                                                                    */
/**********************************************************************/

long FAR PASCAL VideoSubclassProc( HWND hwnd,  WORD msg,
                                   WORD wParam,  LONG lParam )
{
 LPMVWNQRY  pqry;
 HANDLE     hQuery;

 switch ( msg )
    {

    case WM_ACTIVATE:
         if (wParam)
            MVWNCommand( MVWN_CMD_ACTIVATE, ( LONG ) TRUE );
         break;

    case WM_LBUTTONDOWN:
         //----------------------------------
         // Begin drawing tracking rectangle
         //----------------------------------
         BeginTrackRect();
         SetCapture( hwnd );
         break;

    case WM_MOUSEMOVE:
         //------------------------------------------------------------
         // Move tracking rectangle with the mouse if we are tracking
         //------------------------------------------------------------
         if  ( Tracking )
              MoveTrackRect();
         break;

    case WM_LBUTTONUP:
         //---------------------------------------------------------
         // Erase the tracking rectangle and move the window to the
         // final position of the rectangle
         //---------------------------------------------------------

         if  ( Tracking )
            { EndTrackRect();
              SetWindowPos( hwndVideo,  NULL,  TrackingRect.left,
                            TrackingRect.top,  NULL,  NULL,  SWP_NOSIZE );
              ReleaseCapture();
              }
         break;

    case WM_SETCURSOR:
       // We manage cursor shape since we are a popup window.
       switch (LOWORD(lParam))
          {
          case HTBOTTOM:
          case HTTOP:
             SetCursor(LoadCursor(NULL, IDC_SIZENS));
             break;
          case HTLEFT:
          case HTRIGHT:
             SetCursor(LoadCursor(NULL, IDC_SIZEWE));
             break;
          case HTBOTTOMLEFT:
          case HTTOPRIGHT:
             SetCursor(LoadCursor(NULL, IDC_SIZENESW));
             break;
          case HTBOTTOMRIGHT:
          case HTTOPLEFT:
             SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
             break;
          default:
             SetCursor(LoadCursor(NULL, IDC_ARROW));
             break;
          }
       return TRUE;

    case WM_CHAR:
            {
            switch ( wParam )
               {
               // Quit
               case 'q':
               case 'Q':
                    PostMessage( hwndVideo, WM_CLOSE, 0, 0L );
                    return( ( *pfnVideoProc ) ( hwnd, msg, wParam, lParam ) );
                    break;

               // Thaw video
               case 't':
               case 'T':
                    MVWNCommand( MVWN_CMD_FREEZE, FALSE );
                    break;

               // Freeze video
               case 'f':
               case 'F':
                    MVWNCommand( MVWN_CMD_FREEZE, TRUE );
                    break;

               // Switch video to black and white
               case 'b':
               case 'B':
                    MVWNCommand( MVWN_CMD_BLACKWHITE, TRUE );
                    break;

               // Switch video to color
               case 'c':
               case 'C':
                    MVWNCommand( MVWN_CMD_BLACKWHITE, FALSE );
                    break;

               // Increase (up) volume
               case 'u':
               case 'U':
                    if  ( hQuery = MVWNCreateQry( MVWN_QRY_STATUS ) )
                       { pqry = ( LPMVWNQRY ) GlobalLock( hQuery );
                         SendMessage( hwnd, QUERY_vw, NULL, ( LONG ) hQuery );
                         usVolume = pqry->maudst.wVolume;
                         GlobalUnlock( hQuery );
                         MVWNDestroyQry( hQuery );
                         usVolume = ( usVolume <= 95 )  ?  usVolume + 5 : 100;
                         MVWNCommand( MVWN_CMD_VOLUME, usVolume );
                         }
                    break;

               // Decrease (down) volume
               case 'd':
               case 'D':
                    if  ( hQuery = MVWNCreateQry( MVWN_QRY_STATUS ) )
                       { pqry = ( LPMVWNQRY ) GlobalLock( hQuery );
                         SendMessage( hwnd, QUERY_vw, NULL, ( LONG ) hQuery );
                         usVolume = pqry->maudst.wVolume;
                         GlobalUnlock( hQuery );
                         MVWNDestroyQry( hQuery );
                         usVolume = ( usVolume >= 5 )  ?  usVolume - 5 : 0;
                         MVWNCommand( MVWN_CMD_VOLUME, usVolume );
                         }
                    break;

               // Set video scaling off
               case 'S':
                    MVWNCommand( MVWN_CMD_SCALE, FALSE );
                    break;

               // Set video scaling on
               case 's':
                    MVWNCommand( MVWN_CMD_SCALE, TRUE );
                    break;

               // Video source 1
               case '1':
                    MVWNCommand( MVWN_CMD_VIDEOSOURCE, 1 );
                    break;

               // Video source 2
               case '2':
                    MVWNCommand( MVWN_CMD_VIDEOSOURCE, 2 );
                    break;

               // Video source 3
               case '3':
                    MVWNCommand( MVWN_CMD_VIDEOSOURCE, 3 );
                    break;

               default:
                    return( ( *pfnVideoProc ) ( hwnd, msg, wParam, lParam ) );
               }    // WM_CHAR switch
            }       // if KC_KEYUP
         break;


    case WM_KEYDOWN:
            switch ( wParam )
               {
               // F3 to quit application
               case VK_F3:
                    PostMessage( hwndVideo, WM_CLOSE, 0, 0L );
                    return( ( *pfnVideoProc ) ( hwnd, msg, wParam, lParam ) );
                    break;

               default:
                    break;
               }
            break;

    case MVWN_ACK:
         MVWNDestroyCmd( ( HANDLE ) lParam, ( HWND ) wParam, msg );
         return( TRUE );

    case MVWN_ADVISE:
         MVWNDestroyAdv( ( HANDLE ) lParam );
         return( TRUE );

    case WM_DESTROY:
         PostQuitMessage( 0 );
         break;

    default:
         break;
    }

 return( ( *pfnVideoProc ) ( hwnd, msg, wParam, lParam ) );
 }


/**********************************************************************/
/*                                                                    */
/* Function:     BeginTrackRect()                                     */
/*                                                                    */
/* Description:  1) Inialize Rectangle Data                           */
/*               2) Draw the initial point                            */
/*                                                                    */
/**********************************************************************/

static VOID BeginTrackRect()
{
 HANDLE hNewBrush;
 short  nPenStyle = 2;

 hDC = CreateDC( "DISPLAY", NULL, NULL, NULL );
 GetWindowRect( hwndVideo, &TrackingRect );
 GetCursorPos( &TrackingPoint );
 hNewBrush = GetStockObject( NULL_BRUSH );
 SelectObject( hDC, hNewBrush );
 hPen = CreatePen( nPenStyle, 1, RGB( 0, 0, 0 ) );      // Pen With Dots
 SetROP2( hDC, R2_XORPEN );                             // Set Drawing Mode
 hPenHandle = SelectObject( hDC, hPen );
 Rectangle( hDC, TrackingRect.left, TrackingRect.top,   // Draw Tracking Rect
            TrackingRect.right, TrackingRect.bottom);

 Tracking = TRUE;
 return;
 }


/**********************************************************************/
/*                                                                    */
/* Function:     MoveTrackRect()                                      */
/*                                                                    */
/* Description:  1) Erase previous track rectangle                    */
/*               2) Update rectangle data                             */
/*               3) Draw new track rectangle                          */
/*                                                                    */
/**********************************************************************/

static VOID MoveTrackRect()
{
 POINT  NewPt;
 short  dx, dy;

 GetCursorPos( &NewPt );
 Rectangle( hDC,  TrackingRect.left,  TrackingRect.top,  // Remove Current Rect
            TrackingRect.right,  TrackingRect.bottom );
 dx = TrackingPoint.x - NewPt.x;
 dy = TrackingPoint.y - NewPt.y;
 TrackingPoint = NewPt;
 TrackingRect.left   -= dx;
 TrackingRect.right  -= dx;
 TrackingRect.top    -= dy;
 TrackingRect.bottom -= dy;
 Rectangle( hDC,  TrackingRect.left,  TrackingRect.top,  // Draw New Rectangle
            TrackingRect.right,  TrackingRect.bottom );
 return;
 }

/**********************************************************************/
/*                                                                    */
/* Function:     EndTrackRect()                                       */
/*                                                                    */
/* Description:  1) Erase old track rectangle                         */
/*               2) Update array of POINT's                           */
/*               3) Copy data into tracking rectangle                 */
/*                                                                    */
/**********************************************************************/

static VOID EndTrackRect()
{
 POINT  NewPt;
 short  dx, dy;

 GetCursorPos( &NewPt );
 Rectangle( hDC,  TrackingRect.left, TrackingRect.top,  // Remove Tracking Rect
                  TrackingRect.right, TrackingRect.bottom );
 dx = TrackingPoint.x - NewPt.x;
 dy = TrackingPoint.y - NewPt.y;
 TrackingRect.top    -= dy;
 TrackingRect.bottom -= dy;
 TrackingRect.left   -= dx;
 TrackingRect.right  -= dx;
 DeleteObject( SelectObject( hDC, hPenHandle ));
 DeleteDC( hDC );
 Tracking = FALSE;
 return;
 }

