// Private WINDOWS header
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSB2WIN.H
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This file imbeds the internal include files used by the
//          Videodisc Scroll Bar (VSB) User Interface (UI).
//
//          The VSB is a sample UI for the MVPC (Videodisc Player Control
//          Class).  See the VSB.H include file for more details about
//          this UI.
//
//          This file should be included in conjunction with the VSB.H
//          include file:
//
//             #include "mvpc.h"    // If needed
//             #include "vsb.h"     // External (Public)
//             #include "vsb2.h"    // Internal (Private)
//
//*****************************************************************************

//--- VSB Window Word Size ----------------------------------------------------

#define VSB_WWSIZE         sizeof (LPVSBDATA)

//--- VSB Window Word Offset(s) -----------------------------------------------

#define VSB_WWOFF_PVSB     0

//--- Value Ranges for the "Real" scroll bar ----------------------------------

#define SB_LOW             1
#define SB_HIGH            32767
#define SB_RANGE           32767

//--- CmdStep () --------------------------------------------------------------

#define BACKWARD           1
#define FORWARD            2

//--- Lengths ------------------------------------------------------------------

#define MAX_LEN_VSBCLASS   4
#define MAX_LEN_LOADSTRING 80

//--- RC-related DEFINEs ------------------------------------------------------

#define IDS_VSB                  0x0001

// Internal: DEFINES for Log Facility
//*****************************************************************************
// DEFINES
//*****************************************************************************

#define MAX_LOGMESSAGES    250

//--- LENGTHS -----------------------------------------------------------------

#define MAX_LEN_LOGMESSAGE 100
#define MAX_LEN_LINESTRING 6
#define MAX_LEN_LOGMASK    15
#define MAX_LEN_LOGENTRY   120

#define MAX_LEN_LOGTITLE   60
#define MAX_LEN_MARK       60

//--- TITLES ------------------------------------------------------------------

#define IDS_LOG_TITLE            0x0800
#define IDS_LOG_PLAYER1          0x0801
#define IDS_LOG_PLAYER2          0x0802
#define IDS_LOG_PLAYER3          0x0803

#define IDS_LOG_MSGMASK          0x0810

//--- MESSAGE STRING IDs ------------------------------------------------------

#define IDS_LM_TOPOFLOG          0x0900

#define IDS_LM_BUSY              0x0910
#define IDS_LM_READY             0x0911

#define IDS_LM_FOCUSLOST         0x0920
#define IDS_LM_FOCUSREQUESTED    0x0921
#define IDS_LM_FOCUSCAPTURED     0x0922
#define IDS_LM_FOCUSNOTCAPTURED  0x0923

#define IDS_LM_INVALIDFRAME      0x0930
#define IDS_LM_PLAYERERROR       0x0931

#define IDS_LM_PLAYERLOADED      0x0940
#define IDS_LM_PLAYERUNLOADED    0x0941

#define IDS_LM_SEEKBEGIN         0x0950
#define IDS_LM_SEEKEND           0x0951
#define IDS_LM_SEEKFAILED        0x0952

#define IDS_LM_PLAYSTOPPED       0x0960
#define IDS_LM_PLAYFORWARD       0x0961
#define IDS_LM_PLAYBACKWARD      0x0962

#define IDS_LM_AUDIO             0x0970
#define IDS_LM_FRAMERANGE        0x0971
#define IDS_LM_INFOOFF           0x0972
#define IDS_LM_INFOON            0x0973
#define IDS_LM_SPEED             0x0974
#define IDS_LM_STEPINCREMENT     0x0975
#define IDS_LM_FRAMECHANGE       0x0976

//--- DIALOG BOX & CONTROL IDs ------------------------------------------------

// #define IDD_VSBLOG               0x0100   // Dialog ID
// #define IDD_VSBLOGMARK           0x0200   // Dialog ID

#define IDC_LOGBOX               0x0101   // Control ID
#define IDC_OK                   0x0102   // Control ID
#define IDC_MARK                 0x0103   // Control ID
#define IDC_CLEAR                0x0104   // Control ID

#define IDC_EDITBOX              0x0201   // Control ID
#define IDC_MARKADD              0x0202   // Control ID
#define IDC_MARKCANCEL           0x0203   // Control ID


#ifndef RC_INCLUDES_ONLY
#ifdef  DEBUG
//**************************************************************************<f>
// TYPEDEFS
//*****************************************************************************

typedef LONG (FAR PASCAL *PFNWP)(HWND, WORD, WORD, DWORD);

//-----------------------------------------------------------------------------
typedef struct _LOGINFO          // Log Information
   {
   HWND     hWndVSBLOG;          // Hwnd to the VSBLOG Dlg Box
   PFNWP    pfnSBWndProc;        // Used to trap Double clicks
   HWND     hWndLOGBOX;          // LB hwnd to post debug statements
   LONG     lBusyCount;          // Semaphore count for busy status
   } LOGINFO;

typedef LOGINFO FAR *LPLOGINFO;


//-----------------------------------------------------------------------------
typedef struct _LOGCREATEPARAMS
   {
   WORD     wData;        // Suggested for future compliance with SAA
   WORD     wType;        // Suggested for future compliance with SAA
   HWND     hWndVSB;
   WORD     wPlayer;
   } LOGCREATEPARAMS;

typedef LOGCREATEPARAMS FAR *LPLOGCREATEPARAMS;


//*****************************************************************************
// FUNCTION PROTOTYPES
//*****************************************************************************

//--- VSBLog.c   --------------------------------------------------------------
VOID        LogInitialize (VOID);
VOID        LogTerminate  (VOID);
VOID        LogCreate  (HWND hWndVSB);
VOID        LogDestroy (HWND hWndVSB);
VOID        LogLock (BOOL bLock);
VOID cdecl  LogMessage (HWND hWndVSB, WORD wLogMsgID, ...);

LONG FAR PASCAL  VSBLogSubclassWndProc (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam);
LONG FAR PASCAL  VSBLogDlgProc         (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam);
LONG FAR PASCAL  VSBLogMarkDlgProc     (HWND hWnd, WORD msg,
                                        WORD wparam, DWORD lparam);
#endif

// Internal: TYPEDEFS
//*****************************************************************************
// VSBDATA = VSB window word structure
typedef struct _VSBDATA
   {
   WORD      wData;               // Size of this structure

   HWND      hWndOwner;           // The MVPC owner of this user interface
   HANDLE    hmvpcqry;            // -> Query structure for the MVPC object

   HWND      hWndScrollBar;       // The "real" scroll bar control
   BOOL      bInSliderTrack;      // TRUE - "real" scroll bar is tracking

   HWND      hWndAdviseUser;      // Hwnd to advise of user interaction
   WORD      wPlayer;            // Videodisc player association

   LONG      lFromFrame;          // Frame range (from set frame range advises)
   LONG      lToFrame;            // Frame range
   LONG      lCurrentFrame;       // Current frame number
   HPOINTER  hptrActive;          // Active pointer
   HPOINTER  hptrBusy;            // Busy pointer

#ifdef DEBUG
   HANDLE    hLogInfo;            // Used to store log information
   LPVOID    lpReserved [2];      // Reserved for future use
#else
   LPVOID    lpReserved [3];      // Reserved for future use
#endif
   } VSBDATA;

typedef VSBDATA FAR *LPVSBDATA;


//*****************************************************************************
// External data declarations:
//*****************************************************************************

// Handle of this DLL's module:
extern WORD    hmodDLL;             // DLLINIT.ASM: Filled in by DLLInit ()
                                    //              at load time.

extern WORD    msgMVPC_COMMAND;
extern WORD    msgMVPC_ACK;
extern WORD    msgMVPC_RESP;
extern WORD    msgMVPC_QUERY;
extern WORD    msgMVPC_ADVISE;


//*****************************************************************************
// Function Prototypes:
//*****************************************************************************

//--- VSBInit.c   -------------------------------------------------------------
// Exported: VSBInitialize ()
// Exported: VSBTerminate ()

//--- VSBWP.c   -------------------------------------------------------------
LONG FAR PASCAL  VSBWndProc (HWND hWnd, WORD msg, WORD wparam, DWORD lparam);

//--- VSBWM.c   -------------------------------------------------------------
LONG  ProcessCREATE         (HWND hWnd, DWORD lparam);
LONG  ProcessDESTROY        (HWND hWnd, WORD msg, WORD wparam, DWORD lparam);
LONG  ProcessSIZE           (HWND hWnd, DWORD lparam);
LONG  ProcessSETCURSOR      (HWND hWnd);
LONG  ProcessHSCROLL        (HWND hWnd, WORD wparam, DWORD lparam);

//--- VSBAdv.c   -------------------------------------------------------------
LONG  ProcessADVISE (HWND hWnd, WORD wparam, DWORD lparam);
VOID  AdviseUserInteraction (LPVSBDATA lpvsbdata, DWORD dwAdvParam1,
                                                  DWORD dwAdvParam2,
                                                  DWORD dwAdvParam3);

//--- VSBCmd.c   -------------------------------------------------------------
VOID  ProcessACK     (HWND hWnd, WORD msg, WORD wparam, DWORD lparam);
VOID  ProcessRESP    (HWND hWnd, WORD msg, WORD wparam, DWORD lparam);
VOID  ProcessCOMMAND (HWND hWnd, DWORD lparam);

BOOL     CmdGetToken   (HWND hWndMVPC, HWND hWndVSB);
BOOL     CmdPlayFromTo (HWND hWndMVPC, HWND hWndVSB, LONG lFrom, LONG lTo,
                        BOOL bNoReMap);
BOOL     CmdSeek       (HWND hWndMVPC, HWND hWndVSB, LONG lFrame);
BOOL     CmdStep       (HWND hWndMVPC, HWND hWndVSB, WORD wDirection);
BOOL     CmdStop       (HWND hWndMVPC, HWND hWndVSB);

//--- VSBQry.c   -------------------------------------------------------------
// Exported: VSBCreateQry ()
// Exported: VSBDestroyQry ()
LONG  ProcessQUERY (HWND hWnd, DWORD lparam);

//--- VSBFunc.c   -------------------------------------------------------------
LPVSBDATA  CreateVSBDATA (VOID);
WORD       PosSlider (LONG lLow, LONG lHigh, LONG lCurrent);

#endif
//*****************************************************************************
// END OF FILE
//*****************************************************************************
