// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBAdv.c          (VSB Advise Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that process advises.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>
#include <string.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes


BOOL isScrollBar (HWND hwnd)

   {
   // function to tell if hwnd is a SCROLLBAR or not
   char classname[11];

   if (!hwnd) {
     return FALSE;
   }

   if (GetClassName (hwnd, classname, sizeof(classname)) == 9)
      {
      _fstrupr (classname);
      if (lstrcmp(classname, "SCROLLBAR") == 0)
         {
         return TRUE;
         }
      }

   return FALSE;
   }




//**************************************************************************<f>
// Func: ProcessADVISE                                Author: Steve Henson
//
// Parm: HWND     hWnd;          --
//       WORD     wparam;        --
//       DWORD    lparam;        --
//
// Desc: Processes ADVISES from the MVPC
//
// Retn: Returns through WinDefWindowProc ()
//
//*****************************************************************************

LONG ProcessADVISE (HWND hWnd, WORD wparam, DWORD lparam)
   {
   int         posSlider;
   HANDLE      hvsbdata;
   LPVSBDATA   lpvsbdata;
   LONG        lRange;
   int         sSBRange;
   LPUIADVISE  lpuiadvise;

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
   LPLOGINFO   lpLogInfo;
#endif
///////////////////////////////////////////////////////////////////////////////

   lpuiadvise = (LPUIADVISE)GlobalLock (LOWORD (lparam));

   // Address this instance's data area
   if (!(hvsbdata = (HANDLE)GetWindowWord (hWnd, VSB_WWOFF_PVSB)))
      return 0;

   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return 0;

   switch (wparam)         // switch on the specific advise
      {
      //-----------------------------------------------------------------------
      case MVPC_ADV_FRAMERANGE:        // frame range change advise
         if (!lpvsbdata->hWndScrollBar) {
           break;
         }

         // Remember new bounds & current frame
         lpvsbdata->lFromFrame    = lpuiadvise->lLowerBound;
         lpvsbdata->lToFrame      = lpuiadvise->lUpperBound;
         lpvsbdata->lCurrentFrame = lpuiadvise->lCurrentFrame;

         lRange = lpvsbdata->lToFrame - lpvsbdata->lFromFrame;
         if (lRange & 0xFFFF8000)   // If range > 32767, then it won't matter
            sSBRange = SB_RANGE;
         else
            sSBRange = (int) lRange;

         // Set the range of the scroll bar
         SetScrollRange (lpvsbdata->hWndScrollBar, SB_CTL, 1, sSBRange, TRUE);

         // Then set the position of the Scrollbar thumb to this frame
         posSlider = PosSlider (lpvsbdata->lFromFrame, lpvsbdata->lToFrame,
                                lpvsbdata->lCurrentFrame);
         SetScrollPos (lpvsbdata->hWndScrollBar, SB_CTL, posSlider, TRUE);

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
         LogMessage (hWnd, IDS_LM_FRAMERANGE,
                     lpvsbdata->lFromFrame,
                     lpvsbdata->lToFrame,
                     lpvsbdata->lCurrentFrame);
#endif
///////////////////////////////////////////////////////////////////////////////
         break;

      //-----------------------------------------------------------------------
    case MVPC_ADV_FRAMECHANGE:     // frame change advise
         if (!lpvsbdata->hWndScrollBar) {
           break;
         }

         // remember new frame
         lpvsbdata->lCurrentFrame = lpuiadvise->lCurrentFrame;

         // if we are already tracking the slider, skip the update
         if (lpvsbdata->bInSliderTrack)
            {
            break;
            }

         // and update scrollbar accordingly
         posSlider = PosSlider (lpvsbdata->lFromFrame, lpvsbdata->lToFrame, lpvsbdata->lCurrentFrame);

         {
         RECT rectScroll;
         WORD wScrollWidth;
         int  wFramePos;
         int  wSliderPos;
         int  minPos;
         int  maxPos;
         WORD range;
         WORD ratio;

         GetWindowRect (lpvsbdata->hWndScrollBar, &rectScroll);
         GetScrollRange (lpvsbdata->hWndScrollBar, SB_CTL, &minPos, &maxPos);
         range = (WORD)(maxPos - minPos + 1);
         wScrollWidth = rectScroll.right - rectScroll.left;
         wScrollWidth -= 2 * GetSystemMetrics (SM_CXHSCROLL); // less arrows size
         ratio = range / wScrollWidth;
         if (ratio)
            {
            wFramePos = posSlider / ratio;
            wSliderPos = (WORD)GetScrollPos (lpvsbdata->hWndScrollBar, SB_CTL)
                               / ratio;
            }

         if ((wFramePos != wSliderPos) || !ratio )
            {
            if ((posSlider <= maxPos) && (posSlider >= minPos))
               {
               HWND focus;
               focus = GetFocus ();
               if ((!isScrollBar(focus)) || (focus == lpvsbdata->hWndScrollBar))
                  {
                  SetScrollPos (lpvsbdata->hWndScrollBar, SB_CTL,
                                posSlider, TRUE);
                  }
               }
            }
         }
///////////////////////////////////////////////////////////////////////////////
#if 0
#ifdef DEBUG
         LogMessage (hWnd, IDS_LM_FRAMECHANGE,
                     lpvsbdata->lCurrentFrame,
                     posSlider);
#endif
#endif
///////////////////////////////////////////////////////////////////////////////

         break;

      //-----------------------------------------------------------------------
      case MVPC_ADV_BUSY:
         if (!lpvsbdata->hWndScrollBar) {
           break;
         }

         EnableWindow (lpvsbdata->hWndScrollBar,
                       (lpuiadvise->bBusy) ? FALSE : TRUE);

         // wiggle the cursor to get the hptr updated
         {
         POINT pt;
         GetCursorPos (&pt);
         SetCursorPos (pt.x+1,pt.y);
         SetCursorPos (pt.x,pt.y);
         }

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG

         lpLogInfo = (LPLOGINFO)GlobalLock (lpvsbdata->hLogInfo);
         if (lpLogInfo)
            {
            if (lpuiadvise->bBusy)
               {
               lpLogInfo->lBusyCount++;
               LogMessage (hWnd, IDS_LM_BUSY, lpLogInfo->lBusyCount);
               }
            else
               {
               if (lpLogInfo->lBusyCount)
                  lpLogInfo->lBusyCount--;

               LogMessage (hWnd, IDS_LM_READY, lpLogInfo->lBusyCount);
               }
            GlobalUnlock (lpvsbdata->hLogInfo);
            }
#endif
///////////////////////////////////////////////////////////////////////////////
         break;

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_AUDIO:
         LogMessage (hWnd, IDS_LM_AUDIO, (LONG) lpuiadvise->wAudio);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_INFODISPLAY:
         if (lpuiadvise->wInfoDisplay)
            LogMessage (hWnd, IDS_LM_INFOON);
         else
            LogMessage (hWnd, IDS_LM_INFOOFF);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_SPEED:
         LogMessage (hWnd, IDS_LM_SPEED, (LONG)lpuiadvise->wSpeed);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_STEPINCREMENT:
         LogMessage (hWnd, IDS_LM_STEPINCREMENT, (LONG)lpuiadvise->wStepIncrement);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_SEEK:
         switch (lpuiadvise->wSeek)
            {
            case MVPC_ADV_SEEKBEGIN:
               LogMessage (hWnd, IDS_LM_SEEKBEGIN);
               break;

            case MVPC_ADV_SEEKEND:
               LogMessage (hWnd, IDS_LM_SEEKEND);
               break;

            case MVPC_ADV_SEEKFAILED:
               LogMessage (hWnd, IDS_LM_SEEKFAILED);
               break;
            }
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_MOTION:
         if (!lpuiadvise->wPlayDirection)
            LogMessage (hWnd, IDS_LM_PLAYSTOPPED);
         else
            LogMessage (hWnd, (lpuiadvise->wPlayDirection == MVDP_PLAYFORWARD)
                              ? IDS_LM_PLAYFORWARD
                              : IDS_LM_PLAYBACKWARD);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_LOADED:
         if (lpuiadvise->bLoaded)
            LogMessage (hWnd, IDS_LM_PLAYERLOADED);
         else
            LogMessage (hWnd, IDS_LM_PLAYERUNLOADED);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_PLAYERERROR:
         if (lpuiadvise->wError == MVPC_ERR_INVALIDFRAME)
            LogMessage (hWnd, IDS_LM_INVALIDFRAME);
         else
            LogMessage (hWnd, IDS_LM_PLAYERERROR);
         break;
#endif
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      //-----------------------------------------------------------------------
      case MVPC_ADV_TOKENCHANGE:
         switch (lpuiadvise->wFocus)
            {
            case MVPC_ADV_LOSINGTOKEN:
               LogMessage (hWnd, IDS_LM_FOCUSLOST);
               break;

            case MVPC_ADV_REQUESTINGTOKEN:
               LogMessage (hWnd, IDS_LM_FOCUSREQUESTED);
               break;

            case MVPC_ADV_TOKENCAPTURED:
               LogMessage (hWnd, IDS_LM_FOCUSCAPTURED);
               break;

            case MVPC_ADV_TOKENNOTCAPTURED:
               LogMessage (hWnd, IDS_LM_FOCUSNOTCAPTURED);
               break;
            }
         break;
#endif
///////////////////////////////////////////////////////////////////////////////
      }

   GlobalUnlock (LOWORD (lparam));
   GlobalUnlock (hvsbdata);
   return ((LONG)0);
   }


//**************************************************************************<f>
// Func: AdviseUserInteraction                        Author: Steve Henson
//
// Desc: This function passes an advise back to the application window
//       set with a VSB_CMD_ADVISEUSER command.
//
// Retn: VOID
//
//*****************************************************************************

VOID AdviseUserInteraction (LPVSBDATA lpvsbdata, DWORD dwAdvParam1,
                                                 DWORD dwAdvParam2,
                                                 DWORD dwAdvParam3)
   {
   HANDLE    hmvpcadv;
   LPMVPCADV lpmvpcadv;

   if (hmvpcadv = MVPCCreateAdv (VSB_ADV_USER, lpvsbdata->wPlayer,
                                 lpvsbdata->hWndAdviseUser))
      {
      if (lpmvpcadv = (LPMVPCADV)GlobalLock (hmvpcadv))
         {
         lpmvpcadv->dwAdvParam1 = dwAdvParam1;
         lpmvpcadv->dwAdvParam2 = dwAdvParam2;
         lpmvpcadv->dwAdvParam3 = dwAdvParam3;

         GlobalUnlock (hmvpcadv);
         PostMessage (lpvsbdata->hWndAdviseUser,
                      msgMVPC_ADVISE,
                      lpvsbdata->hWndOwner,
                      (DWORD)hmvpcadv);
         }
      }
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
