// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBInit.c         (VSB Initialize/Terminate)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions that initialize and terminate
//          the Videodisc Scroll Bar (VSB) object.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes

int far pascal LibMain ( HANDLE hInstance,
                         WORD   wDataSeg,
                         WORD   wHeapSize,
                         LPSTR  lpszCmdLine )
   {

   hmodDLL = hInstance;

   if ( wHeapSize != 0 )
      {
      UnlockData( 0 );
      }

   return (1);
   }


BOOL FAR PASCAL WEP (int nParameter)
   {
     return (1);
   }


//**************************************************************************<f>
// Func: VSBInitialize     (@1 function of DLL)       Author: Steve Henson
//
// Desc: This function initializes the User Interface.  It is automatically
//       called by MVPC in MVPCInitialize.
//
//       This function must be exported in the @1 position. ie:
//             EXPORTS
//                   VSBInitialize  @1
//
//*****************************************************************************

BOOL FAR PASCAL VSBInitialize (WORD  msgCmd,
                               WORD  msgAck,
                               WORD  msgResp,
                               WORD  msgQuery,
                               WORD  msgAdvise,
                               LPSTR pszUIClassName)
   {
   char        szVSBClass [MAX_LEN_VSBCLASS];
   BOOL        bReturnCode;
   WNDCLASS    wc;

   // Load the name of our User Interface class and copy it
   LoadString (hmodDLL, IDS_VSB, szVSBClass, sizeof (szVSBClass));
   lstrcpy (pszUIClassName, szVSBClass);

   // If the class is already registered, return TRUE
   if (GetClassInfo (hmodDLL, szVSBClass, &wc))
      return (TRUE);

   // This is the first time.  Save the message values.
   msgMVPC_COMMAND = msgCmd;
   msgMVPC_ACK     = msgAck;
   msgMVPC_RESP    = msgResp;
   msgMVPC_QUERY   = msgQuery;
   msgMVPC_ADVISE  = msgAdvise;

   // Otherwise, return the result from RegisterClass
   wc.style = CS_GLOBALCLASS |
              CS_HREDRAW |
              CS_VREDRAW |
              CS_PARENTDC;
   wc.lpfnWndProc = VSBWndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = VSB_WWSIZE;
   wc.hInstance = hmodDLL;
   wc.hIcon = NULL;
   wc.hCursor = NULL;
   wc.hbrBackground = NULL;
   wc.lpszMenuName = NULL;
   wc.lpszClassName = szVSBClass;

   bReturnCode = RegisterClass (&wc);

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
   // Initialize the Log System
   LogInitialize ();
#endif
///////////////////////////////////////////////////////////////////////////////

   return (bReturnCode);
   }


//**************************************************************************<f>
// Func: VSBTerminate      (@2 function of DLL)       Author: Daryl Warinner
//
//
// Desc: This function perform exit processing for the UI.
//       None is required for this control.  But it is provided
//       for your convenience.
//
// Retn: VOID
//
//*****************************************************************************

VOID FAR PASCAL VSBTerminate ()
   {

   char        szVSBClass [MAX_LEN_VSBCLASS];

   // Load the name of our User Interface class and copy it
   LoadString (hmodDLL, IDS_VSB, szVSBClass, sizeof (szVSBClass));

   UnregisterClass (szVSBClass, hmodDLL);

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
   // Shut down the Log System
   LogTerminate ();
#endif
///////////////////////////////////////////////////////////////////////////////
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
