// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBQry.c          (VSB Query Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module include functions that provide support for
//          the Query operations of this User Interface.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes


//**************************************************************************<f>
// Func: VSBCreateQry                             Author: Steve Henson
//
// Parm:
//       WORD           wQuery     -- The MVPC_QRY_* value for the query
//
// Desc: This function will allocate and initialize a MVPCQRY structure.
//
// Retn: HANDLE of  the MVPCQRY structure was allocated and initialized.
//       NULL = Failed in allocation or initialization.
//
//*****************************************************************************

HANDLE FAR PASCAL VSBCreateQry (WORD wQuery)
   {
   HANDLE    hmvpcqry = (HANDLE)NULL;
   LPMVPCQRY lpmvpcqry;                     // Ptr to the MVPCQRY structure

   if (hmvpcqry = GlobalAlloc (GMEM_MOVEABLE |
                               GMEM_ZEROINIT |
                               GMEM_DDESHARE |
                               GMEM_NODISCARD,
                               sizeof (VSBQRY)))
      {
      if (lpmvpcqry = (LPMVPCQRY)GlobalLock (hmvpcqry))    // Make into a pointer
        {
         lpmvpcqry->dwData  = sizeof (VSBQRY);            // Required
         lpmvpcqry->wQuery = wQuery;                      // Required

         GlobalUnlock (hmvpcqry);
         }
      }

   return (hmvpcqry);
   }


//**************************************************************************<f>
// Func: VSBDestroyQry                            Author: Steve Henson
//
// Parm: HANDLE hmvpcqry -- handle of the MVPCQRY structure.
//
// Desc: This function will free the MVPCQRY structure created by a previous
//       call to CreateQry.  Since all Query calls are SENDMSGs, there
//       is no further processing that is necessary to be done.
//
// Retn: TRUE if success
//
//*****************************************************************************

BOOL FAR PASCAL VSBDestroyQry (HANDLE hmvpcqry)
   {

   // flush any remaining locks on the memory
   do
      {
      if (GlobalFlags (hmvpcqry) & GMEM_LOCKCOUNT)
         {
         GlobalUnlock (hmvpcqry);
         }
      else
         {
         break;
         }
      } while (TRUE);

   // free the memory
   if (GlobalFree (hmvpcqry))
     return TRUE;

   return FALSE;
   }


//**************************************************************************<f>
// Func: ProcessQUERY                                  Author: Steve Henson
//
// Desc: Processes QUERYS from the MVPC
//
// Retn: Returns MRESULT 0
//
//*****************************************************************************
LONG ProcessQUERY (HWND hWnd, DWORD lparam)
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;
   LPVSBQRY  lpvsbqry;

   // address this instance's data area
   hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB);
   lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata);

   // Convert handle in the LOWORD of lparam to the MVPCCMD ptr:
   lpvsbqry = (LPVSBQRY)GlobalLock (LOWORD (lparam));

   switch (lpvsbqry->wQuery)
      {
      case VSB_QRY_STATUS:
         lpvsbqry->hptrActive   = lpvsbdata->hptrActive;
         lpvsbqry->hptrBusy     = lpvsbdata->hptrBusy;
         lpvsbqry->wReturnCode  = VSB_OKQRY;
         break;

      default:
         lpvsbqry->wReturnCode  = VSB_INVALIDQRY;
         GlobalUnlock (LOWORD (lparam));
         GlobalUnlock (hvsbdata);
         return ((LONG)TRUE);
      }

   // Unlock the memory
   GlobalUnlock (LOWORD (lparam));
   GlobalUnlock (hvsbdata);
   return ((LONG)FALSE);
   }


//*****************************************************************************
// END OF FILE
//*****************************************************************************
