// WINDOWS version
//*****************************************************************************
// (C) Copyright IBM Corporation 1989,1992
//
// MODULE:  VSBWM.c           (VSB Window Message Processing)
// VERSION: 1.00
//
// DESCRIPTION:
//
//          This module contains functions for processing many of the
//          window messages received by the Videodisc Scroll Bar (VSB).
//
//          Other window message functions can be found in VSBADV.C,
//          VSBCMD.C, and VSBQRY.C.
//
//*****************************************************************************

//*****************************************************************************
// DEFINES & INCLUDES
//*****************************************************************************

#include <windows.h>

#include "mvpcwin.h"       // Include for the MVPC object
#include "vsbwin.h"        // Public  includes
#include "vsb2win.h"       // Private includes


//**************************************************************************<f>
// Func: ProcessCREATE                                Author: Steve Henson
//
// Desc: This function creates and initializes the Videodisc Scroll Bar.
//       Each VSB contains a window word data structure that contains
//       pertinent state information about itself.  This VSB creates a
//       "real" scroll bar for its visible representation and manipulates
//       it as required.
//
//*****************************************************************************

LONG ProcessCREATE (HWND hWnd, DWORD lParam)
   {
   LPCREATESTRUCT lpCreateStruct;   // Pointer to the CREATESTRUCT structure
   int            xWidth;           // Width of ScrollBar interface
   int            yHeigth;          // Heigth of ScrollBar interface
   HWND           hWndScrollBar;    // the ScrollBar handle
   HANDLE         hvsbdata;
   LPVSBDATA      lpvsbdata;        // pointer to user interface instance data

   lpCreateStruct = (LPCREATESTRUCT)lParam;
   xWidth  = lpCreateStruct->cx;           // Get the Width of the SB
   yHeigth = lpCreateStruct->cy;           // Get the Heigth of the SB

   if (lpvsbdata = CreateVSBDATA ())       // create an instance data structure
      {
      hvsbdata = LOWORD (GlobalHandle (HIWORD (lpvsbdata)));
      SetWindowWord (hWnd, VSB_WWOFF_PVSB, hvsbdata); // remember this handle

      lpvsbdata->lFromFrame = 1L;        // initialize frame range
      lpvsbdata->lToFrame = 54000L;
      lpvsbdata->wPlayer = (WORD) ((lpCreateStruct->style & WS_PLAYERMASK) + 1);

      // Create a permanent query structure
      lpvsbdata->hmvpcqry = MVPCCreateQry (MVPC_QRY_STATUS);

      hWndScrollBar = CreateWindow ("SCROLLBAR",
                                    NULL,
                                    WS_VISIBLE |
                                    WS_CHILD |
                                    SBS_HORZ,
                                    0,0,
                                    xWidth,
                                    yHeigth,
                                    hWnd,
                                    0,
                                    lpCreateStruct->hInstance,
                                    NULL);

      SetScrollRange (hWndScrollBar, SB_CTL, SB_LOW, SB_HIGH, FALSE);
      SetScrollPos (hWndScrollBar, SB_CTL, SB_LOW, TRUE);

      // Get and remember handle of the MVPC
      // this is who gets our commands!
      lpvsbdata->hWndOwner = GetWindowWord (hWnd, GWW_HWNDPARENT);

      // Remember handle of physical scrollbar
      lpvsbdata->hWndScrollBar = hWndScrollBar;

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
      // Create the VSBLog Dialog box (modeless)
      LogCreate (hWnd);
#endif
///////////////////////////////////////////////////////////////////////////////

      GlobalUnlock (hvsbdata);
      return ((LONG)FALSE);   // return all OK
      }

   GlobalUnlock (hvsbdata);
   return ((LONG)TRUE);       // return unsuccessful
   }


//**************************************************************************<f>
// Func: ProcessDESTROY                               Author: Steve Henson
//
// Desc: This function frees the VSBDATA window word structure and
//       destroys the query structure.
//
// Retn: Returns through DefWindowProc ()
//
//*****************************************************************************

LONG ProcessDESTROY (HWND hWnd, WORD msg, WORD wParam, DWORD lParam)
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;         // Pointer to the VSBDATA structure

   if (!(hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB)))
     return (DefWindowProc (hWnd, msg, wParam, lParam));

   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
     return (DefWindowProc (hWnd, msg, wParam, lParam));

   MVPCDestroyQry (lpvsbdata->hmvpcqry);

///////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
   // Destroy the Log Dialog box (modeless)
   LogDestroy (hWnd);
#endif
///////////////////////////////////////////////////////////////////////////////

   GlobalUnlock (hvsbdata);
   GlobalFree (hvsbdata);                   // and release it

   SetWindowWord (hWnd, VSB_WWOFF_PVSB, 0);

   return (DefWindowProc (hWnd, msg, wParam, lParam));
   }


//**************************************************************************<f>
// Func: ProcessSIZE                                  Author: Steve Henson
//
// Desc: This function sets the size of the "real" scroll bar.
//
// Retn: Returns through DefWindowProc ()
//       (LONG)0 = means we processed this message
//
//*****************************************************************************

LONG ProcessSIZE (HWND hWnd, DWORD lParam)
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;

   // Address this instance's data area in order to scrollbar handle
   if (!(hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB)))
      return 0L;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return 0L;

   // Translate the WM_SIZE message into a WinSetWindowPos () call:
   SetWindowPos (lpvsbdata->hWndScrollBar,
                 NULL,
                 0,0,
                 LOWORD (lParam),
                 HIWORD (lParam),
                 SWP_NOACTIVATE);

   GlobalUnlock (hvsbdata);
   return ((LONG)0);
   }


//**************************************************************************<f>
// Func: ProcessSETCURSOR                               Author: Steve Henson
//
// Parm: HWND     hWnd;    -- The MVPC scroll bar window handle
//       DWORD    lParam;  -- handle to mouse pointer
//
// Desc: This function process the WM_CONTROLPOINTER message for the MVPC
//       scrollbar. In processing this message, I set the handle to the mouse
//       pointer to that specified by the application if so requested, otherwise
//       return through handle given to this function in MP2.
//
// Retn: Returns through DefWindowProc ()
//
//***************************************************************************** */

LONG ProcessSETCURSOR (HWND hWnd)
   {
   HPOINTER  hptrPointer;            // Handle to the pointer to use
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;

   // address this instance's data area

   if (!(hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB)))
      return ((LONG) FALSE);
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return ((LONG) FALSE);

   if (lpvsbdata->bInSliderTrack)
      {
      GlobalUnlock (hvsbdata);
      return ((LONG) FALSE);
      }

   if (IsWindowEnabled (lpvsbdata->hWndScrollBar))
      hptrPointer = lpvsbdata->hptrActive;       // Enabled
   else
      hptrPointer = lpvsbdata->hptrBusy;         // Disabled

   if (hptrPointer) {
     SetCursor (hptrPointer);
   }

   GlobalUnlock (hvsbdata);
   return (hptrPointer ? (LONG)TRUE : (LONG)FALSE);
   }


//**************************************************************************<f>
// Func: ProcessHSCROLL                                Author: Steve Henson
//
// Desc: processes the user interactions with the scrollbar
//
//       Action            If Stopped        If Moving
//
//       LINELEFT          Step Backward     Stop
//       LINERIGHT         Step Forward      Stop
//       PAGELEFT          Play Backward     Play Backward
//       PAGERIGHT         Play Forward      Play Forward
//       SLIDERPOSITION    Still @ position  Resume play from position
//
// Retn: Returns through DefWindowProc ()
//       (LONG)0 = means we processed this message
//
//*****************************************************************************

LONG ProcessHSCROLL (HWND hWnd, WORD wParam, DWORD lParam)
   {
   HANDLE    hvsbdata;
   LPVSBDATA lpvsbdata;
   LONG      lFrame;
   int       rcPlaying;
   HWND      hWndMVPCOwner;
   WORD      wCmd;
   WORD      posSlider;
   LPMVPCQRY lpmvpcqry = NULL;
   int       MinPos;
   int       MaxPos;


   // Address the instance data
   if (!(hvsbdata = GetWindowWord (hWnd, VSB_WWOFF_PVSB)))
      return 0;
   if (!(lpvsbdata = (LPVSBDATA)GlobalLock (hvsbdata)))
      return 0;

   // Get handle to MVPC owner
   hWndMVPCOwner = lpvsbdata->hWndOwner;

   // get the player state if required for this message
   switch (wParam)
      {
      case SB_THUMBPOSITION:
      case SB_PAGEUP:
      case SB_PAGEDOWN:
      case SB_LINEUP:
      case SB_LINEDOWN:
         // Determine player state
         SendMessage (hWndMVPCOwner, msgMVPC_QUERY,
                      NULL, (DWORD)lpvsbdata->hmvpcqry);

         lpmvpcqry = (LPMVPCQRY)GlobalLock (lpvsbdata->hmvpcqry);
         rcPlaying = lpmvpcqry->wPlayStatus;
         break;
      default:
         break;
      }

   switch (wCmd = wParam)
      {
      //-----------------------------------------------------------------------
      case SB_THUMBTRACK:
         lpvsbdata->bInSliderTrack = TRUE;
         if (lpvsbdata->hWndAdviseUser)
            {
            posSlider = LOWORD (lParam);
            GetScrollRange (HIWORD (lParam), SB_CTL, &MinPos, &MaxPos);
            lFrame = MVPCUserRangeToFrame (hWndMVPCOwner, MinPos, MaxPos, posSlider);
            AdviseUserInteraction (lpvsbdata,
                                   (DWORD)VSB_ADV_SLIDERTRACK,
                                   (DWORD)lFrame,
                                   (DWORD)NULL);
            }
         break;

      //-----------------------------------------------------------------------
      case SB_THUMBPOSITION:   // user has repositioned slider
         lpvsbdata->bInSliderTrack = FALSE;
         posSlider = LOWORD (lParam);
         GetScrollRange (HIWORD (lParam), SB_CTL, &MinPos, &MaxPos);
         lFrame = MVPCUserRangeToFrame (hWndMVPCOwner, MinPos, MaxPos, posSlider);
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_STILL,
                                         (DWORD)lFrame,
                                         (DWORD)NULL);

               CmdSeek (hWndMVPCOwner, hWnd, lFrame);
               break;

            case MVDP_PLAYFORWARD:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_PLAYFT,
                                         (DWORD)lFrame,
                                         (DWORD)lpvsbdata->lToFrame);

               CmdPlayFromTo (hWndMVPCOwner, hWnd,
                              lFrame, lpvsbdata->lToFrame, TRUE);
               break;

            case MVDP_PLAYREVERSE:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_PLAYFT,
                                         (DWORD)lFrame,
                                         (DWORD)lpvsbdata->lFromFrame);

               CmdPlayFromTo (hWndMVPCOwner, hWnd,
                              lFrame, lpvsbdata->lFromFrame, TRUE);
               break;
            }
         break;

      //-----------------------------------------------------------------------
      case SB_PAGEUP:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
            case MVDP_PLAYFORWARD:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_PLAYFT,
                                         (DWORD)(-1L),
                                         (DWORD)lpvsbdata->lFromFrame);

               CmdPlayFromTo (hWndMVPCOwner, hWnd,
                              -1L, lpvsbdata->lFromFrame, TRUE);
               break;

            case MVDP_PLAYREVERSE:     // already playing backwards
               if (!lpmvpcqry->bActive)  // do nothing unless not active
                  CmdGetToken (hWndMVPCOwner, hWnd);
               break;
            }
         break;

      //-----------------------------------------------------------------------
      case SB_PAGEDOWN:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
            case MVDP_PLAYREVERSE:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_PLAYFT,
                                         (DWORD)(-1L),
                                         (DWORD)lpvsbdata->lToFrame);

               CmdPlayFromTo (hWndMVPCOwner, hWnd,
                              -1L, lpvsbdata->lToFrame, TRUE);
               break;

            case MVDP_PLAYFORWARD:  // already playing forwards
               if (!lpmvpcqry->bActive)  // do nothing unless not active
                 CmdGetToken (hWndMVPCOwner, hWnd);
               break;
            }
         break;

      //-----------------------------------------------------------------------
      case SB_LINEUP:
      case SB_LINEDOWN:
         switch (rcPlaying)
            {
            case MVDP_NOTPLAYING:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)((wCmd == SB_LINEUP) ?
                                                  MVDP_STEPB : MVDP_STEPF),
                                         NULL,
                                         NULL);

               CmdStep (hWndMVPCOwner, hWnd,
                        (wCmd == SB_LINEUP) ? BACKWARD : FORWARD);
               break;

            case MVDP_PLAYFORWARD:
            case MVDP_PLAYREVERSE:
               if (lpvsbdata->hWndAdviseUser)
                  AdviseUserInteraction (lpvsbdata,
                                         (DWORD)MVDP_STOP,
                                         NULL,
                                         NULL);

               CmdStop (hWndMVPCOwner, hWnd);
               break;
            }
         break;
      default:
         break;
      }

   if (lpmvpcqry)
      {
      GlobalUnlock (lpvsbdata->hmvpcqry);
      }
   GlobalUnlock (hvsbdata);
   return ((LONG) 0);
   }

//*****************************************************************************
// END OF FILE
//*****************************************************************************
